/* SourceList.c generated by valac 0.56.18, the Vala compiler
 * generated from SourceList.vala, do not modify */

/*
 * Copyright 2019 elementary, Inc. (https://elementary.io)
 * Copyright 2012-2014 Victor Martinez <victoreduardm@gmail.com>
 * SPDX-License-Identifier: LGPL-3.0-or-later
 */
/**
     * = WORKING INTERNALS =
     *
     * In order to offer a transparent Item-based API, and avoid the need of providing methods
     * to deal with items directly on the SourceList widget, it was decided to follow a monitor-like
     * implementation, where the source list permanently monitors its root item and any other
     * child item added to it. The task of monitoring the properties of the items has been
     * divided among different objects, as shown below:
     *
     * Monitored by: Object::method that receives the signals indicating the property change.
     * Applied by: Object::method that actually updates the tree to reflect the property changes
     *             (directly or indirectly, as in the case of the tree data model).
     *
     * ---------------------------------------------------------------------------------------------
     *   PROPERTY        |  MONITORED BY                     |  APPLIED BY
     * ---------------------------------------------------------------------------------------------
     * + Item            |                                   |
     *   - parent        | Not monitored                     | N/A
     *   - name          | DataModel::on_item_prop_changed   | Tree::name_cell_data_func
     *   - editable      | DataModel::on_item_prop_changed   | Queried when needed (See Tree::start_editing_item)
     *   - visible       | DataModel::on_item_prop_changed   | DataModel::filter_visible_func
     *   - icon          | DataModel::on_item_prop_changed   | Tree::icon_cell_data_func
     *   - activatable   | Same as @icon                     | Same as @icon
     * + ExpandableItem  |                                   |
     *   - collapsible   | DataModel::on_item_prop_changed   | Tree::update_expansion
     *                   |                                   | Tree::expander_cell_data_func
     *   - expanded      | Same as @collapsible              | Same as @collapsible
     * ---------------------------------------------------------------------------------------------
     * * Only automatic properties are monitored. ExpandableItem's additions/removals are handled by
     *   DataModel::add_item() and DataModel::remove_item()
     *
     * Other features:
     * - Sorting: this happens on the tree-model level (DataModel).
     */

#include <glib-object.h>
#include <glib.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <float.h>
#include <math.h>
#include <pango/pango.h>
#include <stdlib.h>
#include <string.h>
#include <gio/gio.h>
#include <gee.h>
#include <gobject/gvaluecollector.h>
#include <cairo-gobject.h>

#define CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_ITEM_PARENT_NEEDS_UPDATE "item-parent-needs-update"
#define CODE_WIDGETS_SOURCE_LIST_CELL_RENDERER_ICON_ICON_SIZE GTK_ICON_SIZE_MENU
#define CODE_WIDGETS_SOURCE_LIST_TREE_DEFAULT_STYLESHEET "\n" \
"            .sidebar.badge {\n" \
"                border-radius: 10px;\n" \
"                border-width: 0;\n" \
"                padding: 1px 2px 1px 2px;\n" \
"                font-weight: bold;\n" \
"            }\n" \
"        "
#define CODE_WIDGETS_SOURCE_LIST_TREE_STYLE_PROP_LEVEL_INDENTATION "level-indentation"
#define CODE_WIDGETS_SOURCE_LIST_TREE_STYLE_PROP_LEFT_PADDING "left-padding"
#define CODE_WIDGETS_SOURCE_LIST_TREE_STYLE_PROP_EXPANDER_SPACING "expander-spacing"
#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif
#if !defined(VALA_EXTERN)
#if defined(_MSC_VER)
#define VALA_EXTERN __declspec(dllexport) extern
#elif __GNUC__ >= 4
#define VALA_EXTERN __attribute__((visibility("default"))) extern
#else
#define VALA_EXTERN extern
#endif
#endif

#define CODE_WIDGETS_TYPE_SOURCE_LIST_SORTABLE (code_widgets_source_list_sortable_get_type ())
#define CODE_WIDGETS_SOURCE_LIST_SORTABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), CODE_WIDGETS_TYPE_SOURCE_LIST_SORTABLE, CodeWidgetsSourceListSortable))
#define CODE_WIDGETS_IS_SOURCE_LIST_SORTABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CODE_WIDGETS_TYPE_SOURCE_LIST_SORTABLE))
#define CODE_WIDGETS_SOURCE_LIST_SORTABLE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), CODE_WIDGETS_TYPE_SOURCE_LIST_SORTABLE, CodeWidgetsSourceListSortableIface))

typedef struct _CodeWidgetsSourceListSortable CodeWidgetsSourceListSortable;
typedef struct _CodeWidgetsSourceListSortableIface CodeWidgetsSourceListSortableIface;

#define CODE_WIDGETS_SOURCE_LIST_TYPE_ITEM (code_widgets_source_list_item_get_type ())
#define CODE_WIDGETS_SOURCE_LIST_ITEM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), CODE_WIDGETS_SOURCE_LIST_TYPE_ITEM, CodeWidgetsSourceListItem))
#define CODE_WIDGETS_SOURCE_LIST_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), CODE_WIDGETS_SOURCE_LIST_TYPE_ITEM, CodeWidgetsSourceListItemClass))
#define CODE_WIDGETS_SOURCE_LIST_IS_ITEM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CODE_WIDGETS_SOURCE_LIST_TYPE_ITEM))
#define CODE_WIDGETS_SOURCE_LIST_IS_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), CODE_WIDGETS_SOURCE_LIST_TYPE_ITEM))
#define CODE_WIDGETS_SOURCE_LIST_ITEM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), CODE_WIDGETS_SOURCE_LIST_TYPE_ITEM, CodeWidgetsSourceListItemClass))

typedef struct _CodeWidgetsSourceListItem CodeWidgetsSourceListItem;
typedef struct _CodeWidgetsSourceListItemClass CodeWidgetsSourceListItemClass;

#define CODE_WIDGETS_SOURCE_LIST_TYPE_EXPANDABLE_ITEM (code_widgets_source_list_expandable_item_get_type ())
#define CODE_WIDGETS_SOURCE_LIST_EXPANDABLE_ITEM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), CODE_WIDGETS_SOURCE_LIST_TYPE_EXPANDABLE_ITEM, CodeWidgetsSourceListExpandableItem))
#define CODE_WIDGETS_SOURCE_LIST_EXPANDABLE_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), CODE_WIDGETS_SOURCE_LIST_TYPE_EXPANDABLE_ITEM, CodeWidgetsSourceListExpandableItemClass))
#define CODE_WIDGETS_SOURCE_LIST_IS_EXPANDABLE_ITEM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CODE_WIDGETS_SOURCE_LIST_TYPE_EXPANDABLE_ITEM))
#define CODE_WIDGETS_SOURCE_LIST_IS_EXPANDABLE_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), CODE_WIDGETS_SOURCE_LIST_TYPE_EXPANDABLE_ITEM))
#define CODE_WIDGETS_SOURCE_LIST_EXPANDABLE_ITEM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), CODE_WIDGETS_SOURCE_LIST_TYPE_EXPANDABLE_ITEM, CodeWidgetsSourceListExpandableItemClass))

typedef struct _CodeWidgetsSourceListExpandableItem CodeWidgetsSourceListExpandableItem;
typedef struct _CodeWidgetsSourceListExpandableItemClass CodeWidgetsSourceListExpandableItemClass;
enum  {
	CODE_WIDGETS_SOURCE_LIST_SORTABLE_USER_MOVED_ITEM_SIGNAL,
	CODE_WIDGETS_SOURCE_LIST_SORTABLE_NUM_SIGNALS
};
static guint code_widgets_source_list_sortable_signals[CODE_WIDGETS_SOURCE_LIST_SORTABLE_NUM_SIGNALS] = {0};

#define CODE_WIDGETS_TYPE_SOURCE_LIST_DRAG_SOURCE (code_widgets_source_list_drag_source_get_type ())
#define CODE_WIDGETS_SOURCE_LIST_DRAG_SOURCE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), CODE_WIDGETS_TYPE_SOURCE_LIST_DRAG_SOURCE, CodeWidgetsSourceListDragSource))
#define CODE_WIDGETS_IS_SOURCE_LIST_DRAG_SOURCE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CODE_WIDGETS_TYPE_SOURCE_LIST_DRAG_SOURCE))
#define CODE_WIDGETS_SOURCE_LIST_DRAG_SOURCE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), CODE_WIDGETS_TYPE_SOURCE_LIST_DRAG_SOURCE, CodeWidgetsSourceListDragSourceIface))

typedef struct _CodeWidgetsSourceListDragSource CodeWidgetsSourceListDragSource;
typedef struct _CodeWidgetsSourceListDragSourceIface CodeWidgetsSourceListDragSourceIface;

#define CODE_WIDGETS_TYPE_SOURCE_LIST_DRAG_DEST (code_widgets_source_list_drag_dest_get_type ())
#define CODE_WIDGETS_SOURCE_LIST_DRAG_DEST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), CODE_WIDGETS_TYPE_SOURCE_LIST_DRAG_DEST, CodeWidgetsSourceListDragDest))
#define CODE_WIDGETS_IS_SOURCE_LIST_DRAG_DEST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CODE_WIDGETS_TYPE_SOURCE_LIST_DRAG_DEST))
#define CODE_WIDGETS_SOURCE_LIST_DRAG_DEST_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), CODE_WIDGETS_TYPE_SOURCE_LIST_DRAG_DEST, CodeWidgetsSourceListDragDestIface))

typedef struct _CodeWidgetsSourceListDragDest CodeWidgetsSourceListDragDest;
typedef struct _CodeWidgetsSourceListDragDestIface CodeWidgetsSourceListDragDestIface;

#define CODE_WIDGETS_TYPE_SOURCE_LIST (code_widgets_source_list_get_type ())
#define CODE_WIDGETS_SOURCE_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), CODE_WIDGETS_TYPE_SOURCE_LIST, CodeWidgetsSourceList))
#define CODE_WIDGETS_SOURCE_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), CODE_WIDGETS_TYPE_SOURCE_LIST, CodeWidgetsSourceListClass))
#define CODE_WIDGETS_IS_SOURCE_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CODE_WIDGETS_TYPE_SOURCE_LIST))
#define CODE_WIDGETS_IS_SOURCE_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), CODE_WIDGETS_TYPE_SOURCE_LIST))
#define CODE_WIDGETS_SOURCE_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), CODE_WIDGETS_TYPE_SOURCE_LIST, CodeWidgetsSourceListClass))

typedef struct _CodeWidgetsSourceList CodeWidgetsSourceList;
typedef struct _CodeWidgetsSourceListClass CodeWidgetsSourceListClass;
typedef struct _CodeWidgetsSourceListPrivate CodeWidgetsSourceListPrivate;

#define CODE_WIDGETS_SOURCE_LIST_TYPE_TREE (code_widgets_source_list_tree_get_type ())
#define CODE_WIDGETS_SOURCE_LIST_TREE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), CODE_WIDGETS_SOURCE_LIST_TYPE_TREE, CodeWidgetsSourceListTree))
#define CODE_WIDGETS_SOURCE_LIST_TREE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), CODE_WIDGETS_SOURCE_LIST_TYPE_TREE, CodeWidgetsSourceListTreeClass))
#define CODE_WIDGETS_SOURCE_LIST_IS_TREE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CODE_WIDGETS_SOURCE_LIST_TYPE_TREE))
#define CODE_WIDGETS_SOURCE_LIST_IS_TREE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), CODE_WIDGETS_SOURCE_LIST_TYPE_TREE))
#define CODE_WIDGETS_SOURCE_LIST_TREE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), CODE_WIDGETS_SOURCE_LIST_TYPE_TREE, CodeWidgetsSourceListTreeClass))

typedef struct _CodeWidgetsSourceListTree CodeWidgetsSourceListTree;
typedef struct _CodeWidgetsSourceListTreeClass CodeWidgetsSourceListTreeClass;

#define CODE_WIDGETS_SOURCE_LIST_TYPE_DATA_MODEL (code_widgets_source_list_data_model_get_type ())
#define CODE_WIDGETS_SOURCE_LIST_DATA_MODEL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), CODE_WIDGETS_SOURCE_LIST_TYPE_DATA_MODEL, CodeWidgetsSourceListDataModel))
#define CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), CODE_WIDGETS_SOURCE_LIST_TYPE_DATA_MODEL, CodeWidgetsSourceListDataModelClass))
#define CODE_WIDGETS_SOURCE_LIST_IS_DATA_MODEL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CODE_WIDGETS_SOURCE_LIST_TYPE_DATA_MODEL))
#define CODE_WIDGETS_SOURCE_LIST_IS_DATA_MODEL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), CODE_WIDGETS_SOURCE_LIST_TYPE_DATA_MODEL))
#define CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), CODE_WIDGETS_SOURCE_LIST_TYPE_DATA_MODEL, CodeWidgetsSourceListDataModelClass))

typedef struct _CodeWidgetsSourceListDataModel CodeWidgetsSourceListDataModel;
typedef struct _CodeWidgetsSourceListDataModelClass CodeWidgetsSourceListDataModelClass;
enum  {
	CODE_WIDGETS_SOURCE_LIST_0_PROPERTY,
	CODE_WIDGETS_SOURCE_LIST_ROOT_PROPERTY,
	CODE_WIDGETS_SOURCE_LIST_SELECTED_PROPERTY,
	CODE_WIDGETS_SOURCE_LIST_ELLIPSIZE_MODE_PROPERTY,
	CODE_WIDGETS_SOURCE_LIST_EDITING_PROPERTY,
	CODE_WIDGETS_SOURCE_LIST_NUM_PROPERTIES
};
static GParamSpec* code_widgets_source_list_properties[CODE_WIDGETS_SOURCE_LIST_NUM_PROPERTIES];
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
typedef gboolean (*CodeWidgetsSourceListVisibleFunc) (CodeWidgetsSourceListItem* item, gpointer user_data);
#define __vala_GtkTreePath_free0(var) ((var == NULL) ? NULL : (var = (_vala_GtkTreePath_free (var), NULL)))
#define __vala_GtkTreeIter_free0(var) ((var == NULL) ? NULL : (var = (_vala_GtkTreeIter_free (var), NULL)))
typedef struct _CodeWidgetsSourceListItemPrivate CodeWidgetsSourceListItemPrivate;
enum  {
	CODE_WIDGETS_SOURCE_LIST_ITEM_0_PROPERTY,
	CODE_WIDGETS_SOURCE_LIST_ITEM_PARENT_PROPERTY,
	CODE_WIDGETS_SOURCE_LIST_ITEM_NAME_PROPERTY,
	CODE_WIDGETS_SOURCE_LIST_ITEM_TOOLTIP_PROPERTY,
	CODE_WIDGETS_SOURCE_LIST_ITEM_MARKUP_PROPERTY,
	CODE_WIDGETS_SOURCE_LIST_ITEM_BADGE_PROPERTY,
	CODE_WIDGETS_SOURCE_LIST_ITEM_EDITABLE_PROPERTY,
	CODE_WIDGETS_SOURCE_LIST_ITEM_VISIBLE_PROPERTY,
	CODE_WIDGETS_SOURCE_LIST_ITEM_SELECTABLE_PROPERTY,
	CODE_WIDGETS_SOURCE_LIST_ITEM_ICON_PROPERTY,
	CODE_WIDGETS_SOURCE_LIST_ITEM_ACTIVATABLE_PROPERTY,
	CODE_WIDGETS_SOURCE_LIST_ITEM_ACTIVATABLE_TOOLTIP_PROPERTY,
	CODE_WIDGETS_SOURCE_LIST_ITEM_NUM_PROPERTIES
};
static GParamSpec* code_widgets_source_list_item_properties[CODE_WIDGETS_SOURCE_LIST_ITEM_NUM_PROPERTIES];
#define _g_free0(var) (var = (g_free (var), NULL))
enum  {
	CODE_WIDGETS_SOURCE_LIST_ITEM_EDITED_SIGNAL,
	CODE_WIDGETS_SOURCE_LIST_ITEM_ACTION_ACTIVATED_SIGNAL,
	CODE_WIDGETS_SOURCE_LIST_ITEM_ACTIVATED_SIGNAL,
	CODE_WIDGETS_SOURCE_LIST_ITEM_NUM_SIGNALS
};
static guint code_widgets_source_list_item_signals[CODE_WIDGETS_SOURCE_LIST_ITEM_NUM_SIGNALS] = {0};
typedef struct _CodeWidgetsSourceListExpandableItemPrivate CodeWidgetsSourceListExpandableItemPrivate;
enum  {
	CODE_WIDGETS_SOURCE_LIST_EXPANDABLE_ITEM_0_PROPERTY,
	CODE_WIDGETS_SOURCE_LIST_EXPANDABLE_ITEM_COLLAPSIBLE_PROPERTY,
	CODE_WIDGETS_SOURCE_LIST_EXPANDABLE_ITEM_EXPANDED_PROPERTY,
	CODE_WIDGETS_SOURCE_LIST_EXPANDABLE_ITEM_N_CHILDREN_PROPERTY,
	CODE_WIDGETS_SOURCE_LIST_EXPANDABLE_ITEM_CHILDREN_PROPERTY,
	CODE_WIDGETS_SOURCE_LIST_EXPANDABLE_ITEM_NUM_PROPERTIES
};
static GParamSpec* code_widgets_source_list_expandable_item_properties[CODE_WIDGETS_SOURCE_LIST_EXPANDABLE_ITEM_NUM_PROPERTIES];
enum  {
	CODE_WIDGETS_SOURCE_LIST_EXPANDABLE_ITEM_CHILD_ADDED_SIGNAL,
	CODE_WIDGETS_SOURCE_LIST_EXPANDABLE_ITEM_CHILD_REMOVED_SIGNAL,
	CODE_WIDGETS_SOURCE_LIST_EXPANDABLE_ITEM_TOGGLED_SIGNAL,
	CODE_WIDGETS_SOURCE_LIST_EXPANDABLE_ITEM_NUM_SIGNALS
};
static guint code_widgets_source_list_expandable_item_signals[CODE_WIDGETS_SOURCE_LIST_EXPANDABLE_ITEM_NUM_SIGNALS] = {0};
typedef struct _CodeWidgetsSourceListDataModelPrivate CodeWidgetsSourceListDataModelPrivate;

#define CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_TYPE_NODE_WRAPPER (code_widgets_source_list_data_model_node_wrapper_get_type ())
#define CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_NODE_WRAPPER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_TYPE_NODE_WRAPPER, CodeWidgetsSourceListDataModelNodeWrapper))
#define CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_NODE_WRAPPER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_TYPE_NODE_WRAPPER, CodeWidgetsSourceListDataModelNodeWrapperClass))
#define CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_IS_NODE_WRAPPER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_TYPE_NODE_WRAPPER))
#define CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_IS_NODE_WRAPPER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_TYPE_NODE_WRAPPER))
#define CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_NODE_WRAPPER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_TYPE_NODE_WRAPPER, CodeWidgetsSourceListDataModelNodeWrapperClass))

typedef struct _CodeWidgetsSourceListDataModelNodeWrapper CodeWidgetsSourceListDataModelNodeWrapper;
typedef struct _CodeWidgetsSourceListDataModelNodeWrapperClass CodeWidgetsSourceListDataModelNodeWrapperClass;

#define CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_TYPE_ITEM_MONITOR (code_widgets_source_list_data_model_item_monitor_get_type ())
#define CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_ITEM_MONITOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_TYPE_ITEM_MONITOR, CodeWidgetsSourceListDataModelItemMonitor))
#define CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_ITEM_MONITOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_TYPE_ITEM_MONITOR, CodeWidgetsSourceListDataModelItemMonitorClass))
#define CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_IS_ITEM_MONITOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_TYPE_ITEM_MONITOR))
#define CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_IS_ITEM_MONITOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_TYPE_ITEM_MONITOR))
#define CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_ITEM_MONITOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_TYPE_ITEM_MONITOR, CodeWidgetsSourceListDataModelItemMonitorClass))

typedef struct _CodeWidgetsSourceListDataModelItemMonitor CodeWidgetsSourceListDataModelItemMonitor;
typedef struct _CodeWidgetsSourceListDataModelItemMonitorClass CodeWidgetsSourceListDataModelItemMonitorClass;
enum  {
	CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_0_PROPERTY,
	CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_ROOT_PROPERTY,
	CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_NUM_PROPERTIES
};
static GParamSpec* code_widgets_source_list_data_model_properties[CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_NUM_PROPERTIES];
typedef enum  {
	CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_COLUMN_ITEM,
	CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_COLUMN_N_COLUMNS
} CodeWidgetsSourceListDataModelColumn;

#define CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_TYPE_COLUMN (code_widgets_source_list_data_model_column_get_type ())
#define _code_widgets_source_list_data_model_node_wrapper_unref0(var) ((var == NULL) ? NULL : (var = (code_widgets_source_list_data_model_node_wrapper_unref (var), NULL)))
#define _code_widgets_source_list_data_model_item_monitor_unref0(var) ((var == NULL) ? NULL : (var = (code_widgets_source_list_data_model_item_monitor_unref (var), NULL)))
typedef struct _Block37Data Block37Data;
typedef struct _CodeWidgetsSourceListDataModelNodeWrapperPrivate CodeWidgetsSourceListDataModelNodeWrapperPrivate;
#define __vala_GtkTreeRowReference_free0(var) ((var == NULL) ? NULL : (var = (_vala_GtkTreeRowReference_free (var), NULL)))
typedef struct _CodeWidgetsSourceListDataModelParamSpecNodeWrapper CodeWidgetsSourceListDataModelParamSpecNodeWrapper;
typedef struct _CodeWidgetsSourceListDataModelItemMonitorPrivate CodeWidgetsSourceListDataModelItemMonitorPrivate;
typedef struct _CodeWidgetsSourceListDataModelParamSpecItemMonitor CodeWidgetsSourceListDataModelParamSpecItemMonitor;
enum  {
	CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_ITEM_MONITOR_CHANGED_SIGNAL,
	CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_ITEM_MONITOR_NUM_SIGNALS
};
static guint code_widgets_source_list_data_model_item_monitor_signals[CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_ITEM_MONITOR_NUM_SIGNALS] = {0};
enum  {
	CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_ITEM_UPDATED_SIGNAL,
	CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_NUM_SIGNALS
};
static guint code_widgets_source_list_data_model_signals[CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_NUM_SIGNALS] = {0};

#define CODE_WIDGETS_SOURCE_LIST_TYPE_CELL_RENDERER_ICON (code_widgets_source_list_cell_renderer_icon_get_type ())
#define CODE_WIDGETS_SOURCE_LIST_CELL_RENDERER_ICON(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), CODE_WIDGETS_SOURCE_LIST_TYPE_CELL_RENDERER_ICON, CodeWidgetsSourceListCellRendererIcon))
#define CODE_WIDGETS_SOURCE_LIST_CELL_RENDERER_ICON_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), CODE_WIDGETS_SOURCE_LIST_TYPE_CELL_RENDERER_ICON, CodeWidgetsSourceListCellRendererIconClass))
#define CODE_WIDGETS_SOURCE_LIST_IS_CELL_RENDERER_ICON(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CODE_WIDGETS_SOURCE_LIST_TYPE_CELL_RENDERER_ICON))
#define CODE_WIDGETS_SOURCE_LIST_IS_CELL_RENDERER_ICON_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), CODE_WIDGETS_SOURCE_LIST_TYPE_CELL_RENDERER_ICON))
#define CODE_WIDGETS_SOURCE_LIST_CELL_RENDERER_ICON_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), CODE_WIDGETS_SOURCE_LIST_TYPE_CELL_RENDERER_ICON, CodeWidgetsSourceListCellRendererIconClass))

typedef struct _CodeWidgetsSourceListCellRendererIcon CodeWidgetsSourceListCellRendererIcon;
typedef struct _CodeWidgetsSourceListCellRendererIconClass CodeWidgetsSourceListCellRendererIconClass;
typedef struct _CodeWidgetsSourceListCellRendererIconPrivate CodeWidgetsSourceListCellRendererIconPrivate;
enum  {
	CODE_WIDGETS_SOURCE_LIST_CELL_RENDERER_ICON_0_PROPERTY,
	CODE_WIDGETS_SOURCE_LIST_CELL_RENDERER_ICON_NUM_PROPERTIES
};
static GParamSpec* code_widgets_source_list_cell_renderer_icon_properties[CODE_WIDGETS_SOURCE_LIST_CELL_RENDERER_ICON_NUM_PROPERTIES];
enum  {
	CODE_WIDGETS_SOURCE_LIST_CELL_RENDERER_ICON_ACTIVATED_SIGNAL,
	CODE_WIDGETS_SOURCE_LIST_CELL_RENDERER_ICON_NUM_SIGNALS
};
static guint code_widgets_source_list_cell_renderer_icon_signals[CODE_WIDGETS_SOURCE_LIST_CELL_RENDERER_ICON_NUM_SIGNALS] = {0};

#define CODE_WIDGETS_SOURCE_LIST_TYPE_CELL_RENDERER_SPACER (code_widgets_source_list_cell_renderer_spacer_get_type ())
#define CODE_WIDGETS_SOURCE_LIST_CELL_RENDERER_SPACER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), CODE_WIDGETS_SOURCE_LIST_TYPE_CELL_RENDERER_SPACER, CodeWidgetsSourceListCellRendererSpacer))
#define CODE_WIDGETS_SOURCE_LIST_CELL_RENDERER_SPACER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), CODE_WIDGETS_SOURCE_LIST_TYPE_CELL_RENDERER_SPACER, CodeWidgetsSourceListCellRendererSpacerClass))
#define CODE_WIDGETS_SOURCE_LIST_IS_CELL_RENDERER_SPACER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CODE_WIDGETS_SOURCE_LIST_TYPE_CELL_RENDERER_SPACER))
#define CODE_WIDGETS_SOURCE_LIST_IS_CELL_RENDERER_SPACER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), CODE_WIDGETS_SOURCE_LIST_TYPE_CELL_RENDERER_SPACER))
#define CODE_WIDGETS_SOURCE_LIST_CELL_RENDERER_SPACER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), CODE_WIDGETS_SOURCE_LIST_TYPE_CELL_RENDERER_SPACER, CodeWidgetsSourceListCellRendererSpacerClass))

typedef struct _CodeWidgetsSourceListCellRendererSpacer CodeWidgetsSourceListCellRendererSpacer;
typedef struct _CodeWidgetsSourceListCellRendererSpacerClass CodeWidgetsSourceListCellRendererSpacerClass;
typedef struct _CodeWidgetsSourceListCellRendererSpacerPrivate CodeWidgetsSourceListCellRendererSpacerPrivate;
enum  {
	CODE_WIDGETS_SOURCE_LIST_CELL_RENDERER_SPACER_0_PROPERTY,
	CODE_WIDGETS_SOURCE_LIST_CELL_RENDERER_SPACER_LEVEL_PROPERTY,
	CODE_WIDGETS_SOURCE_LIST_CELL_RENDERER_SPACER_NUM_PROPERTIES
};
static GParamSpec* code_widgets_source_list_cell_renderer_spacer_properties[CODE_WIDGETS_SOURCE_LIST_CELL_RENDERER_SPACER_NUM_PROPERTIES];
typedef struct _CodeWidgetsSourceListTreePrivate CodeWidgetsSourceListTreePrivate;

#define CODE_WIDGETS_TYPE_CELL_RENDERER_BADGE (code_widgets_cell_renderer_badge_get_type ())
#define CODE_WIDGETS_CELL_RENDERER_BADGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), CODE_WIDGETS_TYPE_CELL_RENDERER_BADGE, CodeWidgetsCellRendererBadge))
#define CODE_WIDGETS_CELL_RENDERER_BADGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), CODE_WIDGETS_TYPE_CELL_RENDERER_BADGE, CodeWidgetsCellRendererBadgeClass))
#define CODE_WIDGETS_IS_CELL_RENDERER_BADGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CODE_WIDGETS_TYPE_CELL_RENDERER_BADGE))
#define CODE_WIDGETS_IS_CELL_RENDERER_BADGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), CODE_WIDGETS_TYPE_CELL_RENDERER_BADGE))
#define CODE_WIDGETS_CELL_RENDERER_BADGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), CODE_WIDGETS_TYPE_CELL_RENDERER_BADGE, CodeWidgetsCellRendererBadgeClass))

typedef struct _CodeWidgetsCellRendererBadge CodeWidgetsCellRendererBadge;
typedef struct _CodeWidgetsCellRendererBadgeClass CodeWidgetsCellRendererBadgeClass;

#define CODE_WIDGETS_TYPE_CELL_RENDERER_EXPANDER (code_widgets_cell_renderer_expander_get_type ())
#define CODE_WIDGETS_CELL_RENDERER_EXPANDER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), CODE_WIDGETS_TYPE_CELL_RENDERER_EXPANDER, CodeWidgetsCellRendererExpander))
#define CODE_WIDGETS_CELL_RENDERER_EXPANDER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), CODE_WIDGETS_TYPE_CELL_RENDERER_EXPANDER, CodeWidgetsCellRendererExpanderClass))
#define CODE_WIDGETS_IS_CELL_RENDERER_EXPANDER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CODE_WIDGETS_TYPE_CELL_RENDERER_EXPANDER))
#define CODE_WIDGETS_IS_CELL_RENDERER_EXPANDER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), CODE_WIDGETS_TYPE_CELL_RENDERER_EXPANDER))
#define CODE_WIDGETS_CELL_RENDERER_EXPANDER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), CODE_WIDGETS_TYPE_CELL_RENDERER_EXPANDER, CodeWidgetsCellRendererExpanderClass))

typedef struct _CodeWidgetsCellRendererExpander CodeWidgetsCellRendererExpander;
typedef struct _CodeWidgetsCellRendererExpanderClass CodeWidgetsCellRendererExpanderClass;
enum  {
	CODE_WIDGETS_SOURCE_LIST_TREE_0_PROPERTY,
	CODE_WIDGETS_SOURCE_LIST_TREE_DATA_MODEL_PROPERTY,
	CODE_WIDGETS_SOURCE_LIST_TREE_SELECTED_ITEM_PROPERTY,
	CODE_WIDGETS_SOURCE_LIST_TREE_EDITING_PROPERTY,
	CODE_WIDGETS_SOURCE_LIST_TREE_ELLIPSIZE_MODE_PROPERTY,
	CODE_WIDGETS_SOURCE_LIST_TREE_NUM_PROPERTIES
};
static GParamSpec* code_widgets_source_list_tree_properties[CODE_WIDGETS_SOURCE_LIST_TREE_NUM_PROPERTIES];
typedef enum  {
	CODE_WIDGETS_SOURCE_LIST_TREE_COLUMN_ITEM,
	CODE_WIDGETS_SOURCE_LIST_TREE_COLUMN_N_COLS
} CodeWidgetsSourceListTreeColumn;

#define CODE_WIDGETS_SOURCE_LIST_TREE_TYPE_COLUMN (code_widgets_source_list_tree_column_get_type ())
#define _gtk_target_list_unref0(var) ((var == NULL) ? NULL : (var = (gtk_target_list_unref (var), NULL)))
#define _g_string_free0(var) ((var == NULL) ? NULL : (var = (g_string_free (var, TRUE), NULL)))
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))
#define _g_param_spec_unref0(var) ((var == NULL) ? NULL : (var = (g_param_spec_unref (var), NULL)))
enum  {
	CODE_WIDGETS_SOURCE_LIST_TREE_ITEM_SELECTED_SIGNAL,
	CODE_WIDGETS_SOURCE_LIST_TREE_NUM_SIGNALS
};
static guint code_widgets_source_list_tree_signals[CODE_WIDGETS_SOURCE_LIST_TREE_NUM_SIGNALS] = {0};
enum  {
	CODE_WIDGETS_SOURCE_LIST_ITEM_SELECTED_SIGNAL,
	CODE_WIDGETS_SOURCE_LIST_NUM_SIGNALS
};
static guint code_widgets_source_list_signals[CODE_WIDGETS_SOURCE_LIST_NUM_SIGNALS] = {0};
#define _vala_assert(expr, msg) if G_LIKELY (expr) ; else g_assertion_message_expr (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);
#define _vala_return_if_fail(expr, msg) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return; }
#define _vala_return_val_if_fail(expr, msg, val) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return val; }
#define _vala_warn_if_fail(expr, msg) if G_LIKELY (expr) ; else g_warn_message (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);

struct _CodeWidgetsSourceListSortableIface {
	GTypeInterface parent_iface;
	gboolean (*allow_dnd_sorting) (CodeWidgetsSourceListSortable* self);
	gint (*compare) (CodeWidgetsSourceListSortable* self, CodeWidgetsSourceListItem* a, CodeWidgetsSourceListItem* b);
};

struct _CodeWidgetsSourceListDragSourceIface {
	GTypeInterface parent_iface;
	gboolean (*draggable) (CodeWidgetsSourceListDragSource* self);
	void (*prepare_selection_data) (CodeWidgetsSourceListDragSource* self, GtkSelectionData* selection_data);
};

struct _CodeWidgetsSourceListDragDestIface {
	GTypeInterface parent_iface;
	gboolean (*data_drop_possible) (CodeWidgetsSourceListDragDest* self, GdkDragContext* context, GtkSelectionData* data);
	GdkDragAction (*data_received) (CodeWidgetsSourceListDragDest* self, GdkDragContext* context, GtkSelectionData* data);
};

struct _CodeWidgetsSourceList {
	GtkScrolledWindow parent_instance;
	CodeWidgetsSourceListPrivate * priv;
};

struct _CodeWidgetsSourceListClass {
	GtkScrolledWindowClass parent_class;
	void (*item_selected) (CodeWidgetsSourceList* self, CodeWidgetsSourceListItem* item);
};

struct _CodeWidgetsSourceListPrivate {
	CodeWidgetsSourceListTree* tree;
	CodeWidgetsSourceListDataModel* data_model;
};

struct _CodeWidgetsSourceListItem {
	GObject parent_instance;
	CodeWidgetsSourceListItemPrivate * priv;
};

struct _CodeWidgetsSourceListItemClass {
	GObjectClass parent_class;
	GtkMenu* (*get_context_menu) (CodeWidgetsSourceListItem* self);
	void (*edited) (CodeWidgetsSourceListItem* self, const gchar* new_name);
	void (*action_activated) (CodeWidgetsSourceListItem* self);
	void (*activated) (CodeWidgetsSourceListItem* self);
};

struct _CodeWidgetsSourceListItemPrivate {
	CodeWidgetsSourceListExpandableItem* _parent;
	gchar* _name;
	gchar* _tooltip;
	gchar* _markup;
	gchar* _badge;
	gboolean _editable;
	gboolean _visible;
	gboolean _selectable;
	GIcon* _icon;
	GIcon* _activatable;
	gchar* _activatable_tooltip;
};

struct _CodeWidgetsSourceListExpandableItem {
	CodeWidgetsSourceListItem parent_instance;
	CodeWidgetsSourceListExpandableItemPrivate * priv;
};

struct _CodeWidgetsSourceListExpandableItemClass {
	CodeWidgetsSourceListItemClass parent_class;
	void (*toggled) (CodeWidgetsSourceListExpandableItem* self);
};

struct _CodeWidgetsSourceListExpandableItemPrivate {
	gboolean _collapsible;
	gboolean _expanded;
	GeeCollection* children_list;
};

struct _CodeWidgetsSourceListDataModel {
	GtkTreeModelFilter parent_instance;
	CodeWidgetsSourceListDataModelPrivate * priv;
};

struct _CodeWidgetsSourceListDataModelClass {
	GtkTreeModelFilterClass parent_class;
};

struct _CodeWidgetsSourceListDataModelPrivate {
	CodeWidgetsSourceListExpandableItem* _root;
	GeeHashMap* items;
	GeeHashMap* monitors;
	GtkTreeStore* child_tree;
	CodeWidgetsSourceListVisibleFunc filter_func;
	gpointer filter_func_target;
};

struct _Block37Data {
	int _ref_count_;
	CodeWidgetsSourceListDataModel* self;
	CodeWidgetsSourceListExpandableItem* parent;
};

struct _CodeWidgetsSourceListDataModelNodeWrapper {
	GTypeInstance parent_instance;
	volatile int ref_count;
	CodeWidgetsSourceListDataModelNodeWrapperPrivate * priv;
};

struct _CodeWidgetsSourceListDataModelNodeWrapperClass {
	GTypeClass parent_class;
	void (*finalize) (CodeWidgetsSourceListDataModelNodeWrapper *self);
};

struct _CodeWidgetsSourceListDataModelNodeWrapperPrivate {
	GtkTreeRowReference* row_reference;
};

struct _CodeWidgetsSourceListDataModelParamSpecNodeWrapper {
	GParamSpec parent_instance;
};

struct _CodeWidgetsSourceListDataModelItemMonitor {
	GTypeInstance parent_instance;
	volatile int ref_count;
	CodeWidgetsSourceListDataModelItemMonitorPrivate * priv;
};

struct _CodeWidgetsSourceListDataModelItemMonitorClass {
	GTypeClass parent_class;
	void (*finalize) (CodeWidgetsSourceListDataModelItemMonitor *self);
};

struct _CodeWidgetsSourceListDataModelItemMonitorPrivate {
	CodeWidgetsSourceListItem* item;
};

struct _CodeWidgetsSourceListDataModelParamSpecItemMonitor {
	GParamSpec parent_instance;
};

struct _CodeWidgetsSourceListCellRendererIcon {
	GtkCellRendererPixbuf parent_instance;
	CodeWidgetsSourceListCellRendererIconPrivate * priv;
};

struct _CodeWidgetsSourceListCellRendererIconClass {
	GtkCellRendererPixbufClass parent_class;
};

struct _CodeWidgetsSourceListCellRendererSpacer {
	GtkCellRenderer parent_instance;
	CodeWidgetsSourceListCellRendererSpacerPrivate * priv;
};

struct _CodeWidgetsSourceListCellRendererSpacerClass {
	GtkCellRendererClass parent_class;
};

struct _CodeWidgetsSourceListCellRendererSpacerPrivate {
	gint _level;
};

struct _CodeWidgetsSourceListTree {
	GtkTreeView parent_instance;
	CodeWidgetsSourceListTreePrivate * priv;
};

struct _CodeWidgetsSourceListTreeClass {
	GtkTreeViewClass parent_class;
};

struct _CodeWidgetsSourceListTreePrivate {
	CodeWidgetsSourceListDataModel* _data_model;
	CodeWidgetsSourceListItem* selected;
	CodeWidgetsSourceListItem* edited;
	GtkEntry* editable_entry;
	GtkCellRendererText* text_cell;
	CodeWidgetsSourceListCellRendererIcon* icon_cell;
	CodeWidgetsSourceListCellRendererIcon* activatable_cell;
	CodeWidgetsCellRendererBadge* badge_cell;
	CodeWidgetsCellRendererExpander* primary_expander_cell;
	CodeWidgetsCellRendererExpander* secondary_expander_cell;
	GeeHashMap* spacer_cells;
	gboolean unselectable_item_clicked;
};

static gint CodeWidgetsSourceList_private_offset;
static gpointer code_widgets_source_list_parent_class = NULL;
static gint CodeWidgetsSourceListItem_private_offset;
static gpointer code_widgets_source_list_item_parent_class = NULL;
static gint CodeWidgetsSourceListExpandableItem_private_offset;
static gpointer code_widgets_source_list_expandable_item_parent_class = NULL;
static gint CodeWidgetsSourceListDataModel_private_offset;
static gpointer code_widgets_source_list_data_model_parent_class = NULL;
static gint CodeWidgetsSourceListDataModelNodeWrapper_private_offset;
static gpointer code_widgets_source_list_data_model_node_wrapper_parent_class = NULL;
static gint CodeWidgetsSourceListDataModelItemMonitor_private_offset;
static gpointer code_widgets_source_list_data_model_item_monitor_parent_class = NULL;
static GtkTreeDragSourceIface * code_widgets_source_list_data_model_gtk_tree_drag_source_parent_iface = NULL;
static GtkTreeDragDestIface * code_widgets_source_list_data_model_gtk_tree_drag_dest_parent_iface = NULL;
static gpointer code_widgets_source_list_cell_renderer_icon_parent_class = NULL;
static gint CodeWidgetsSourceListCellRendererSpacer_private_offset;
static gpointer code_widgets_source_list_cell_renderer_spacer_parent_class = NULL;
static gint CodeWidgetsSourceListTree_private_offset;
static gpointer code_widgets_source_list_tree_parent_class = NULL;

VALA_EXTERN GType code_widgets_source_list_item_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (CodeWidgetsSourceListItem, g_object_unref)
VALA_EXTERN GType code_widgets_source_list_expandable_item_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (CodeWidgetsSourceListExpandableItem, g_object_unref)
VALA_EXTERN GType code_widgets_source_list_sortable_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (CodeWidgetsSourceListSortable, g_object_unref)
VALA_EXTERN gboolean code_widgets_source_list_sortable_allow_dnd_sorting (CodeWidgetsSourceListSortable* self);
VALA_EXTERN gint code_widgets_source_list_sortable_compare (CodeWidgetsSourceListSortable* self,
                                                CodeWidgetsSourceListItem* a,
                                                CodeWidgetsSourceListItem* b);
static GType code_widgets_source_list_sortable_get_type_once (void);
VALA_EXTERN GType code_widgets_source_list_drag_source_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (CodeWidgetsSourceListDragSource, g_object_unref)
VALA_EXTERN gboolean code_widgets_source_list_drag_source_draggable (CodeWidgetsSourceListDragSource* self);
VALA_EXTERN void code_widgets_source_list_drag_source_prepare_selection_data (CodeWidgetsSourceListDragSource* self,
                                                                  GtkSelectionData* selection_data);
static GType code_widgets_source_list_drag_source_get_type_once (void);
VALA_EXTERN GType code_widgets_source_list_drag_dest_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (CodeWidgetsSourceListDragDest, g_object_unref)
VALA_EXTERN gboolean code_widgets_source_list_drag_dest_data_drop_possible (CodeWidgetsSourceListDragDest* self,
                                                                GdkDragContext* context,
                                                                GtkSelectionData* data);
VALA_EXTERN GdkDragAction code_widgets_source_list_drag_dest_data_received (CodeWidgetsSourceListDragDest* self,
                                                                GdkDragContext* context,
                                                                GtkSelectionData* data);
static GType code_widgets_source_list_drag_dest_get_type_once (void);
VALA_EXTERN GType code_widgets_source_list_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (CodeWidgetsSourceList, g_object_unref)
static GType code_widgets_source_list_tree_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (CodeWidgetsSourceListTree, g_object_unref)
static GType code_widgets_source_list_data_model_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (CodeWidgetsSourceListDataModel, g_object_unref)
static CodeWidgetsSourceListDataModel* code_widgets_source_list_data_model_new (void);
static CodeWidgetsSourceListDataModel* code_widgets_source_list_data_model_construct (GType object_type);
VALA_EXTERN CodeWidgetsSourceList* code_widgets_source_list_new (CodeWidgetsSourceListExpandableItem* root);
VALA_EXTERN CodeWidgetsSourceList* code_widgets_source_list_construct (GType object_type,
                                                           CodeWidgetsSourceListExpandableItem* root);
VALA_EXTERN void code_widgets_source_list_set_root (CodeWidgetsSourceList* self,
                                        CodeWidgetsSourceListExpandableItem* value);
VALA_EXTERN gboolean code_widgets_source_list_has_item (CodeWidgetsSourceList* self,
                                            CodeWidgetsSourceListItem* item);
static gboolean code_widgets_source_list_data_model_has_item (CodeWidgetsSourceListDataModel* self,
                                                       CodeWidgetsSourceListItem* item);
VALA_EXTERN void code_widgets_source_list_set_filter_func (CodeWidgetsSourceList* self,
                                               CodeWidgetsSourceListVisibleFunc visible_func,
                                               gpointer visible_func_target,
                                               gboolean refilter);
static void code_widgets_source_list_data_model_set_filter_func (CodeWidgetsSourceListDataModel* self,
                                                          CodeWidgetsSourceListVisibleFunc visible_func,
                                                          gpointer visible_func_target);
VALA_EXTERN void code_widgets_source_list_refilter (CodeWidgetsSourceList* self);
VALA_EXTERN gboolean code_widgets_source_list_is_item_expanded (CodeWidgetsSourceList* self,
                                                    CodeWidgetsSourceListItem* item);
static GtkTreePath* code_widgets_source_list_data_model_get_item_path (CodeWidgetsSourceListDataModel* self,
                                                                CodeWidgetsSourceListItem* item);
static void _vala_GtkTreePath_free (GtkTreePath* self);
VALA_EXTERN gboolean code_widgets_source_list_start_editing_item (CodeWidgetsSourceList* self,
                                                      CodeWidgetsSourceListItem* item);
static gboolean code_widgets_source_list_tree_start_editing_item (CodeWidgetsSourceListTree* self,
                                                           CodeWidgetsSourceListItem* item);
VALA_EXTERN void code_widgets_source_list_stop_editing (CodeWidgetsSourceList* self);
VALA_EXTERN gboolean code_widgets_source_list_get_editing (CodeWidgetsSourceList* self);
static void code_widgets_source_list_tree_stop_editing (CodeWidgetsSourceListTree* self);
VALA_EXTERN void code_widgets_source_list_enable_drag_source (CodeWidgetsSourceList* self,
                                                  GtkTargetEntry* src_entries,
                                                  gint src_entries_length1);
static void code_widgets_source_list_tree_configure_drag_source (CodeWidgetsSourceListTree* self,
                                                          GtkTargetEntry* src_entries,
                                                          gint src_entries_length1);
VALA_EXTERN void code_widgets_source_list_disable_drag_source (CodeWidgetsSourceList* self);
VALA_EXTERN void code_widgets_source_list_enable_drag_dest (CodeWidgetsSourceList* self,
                                                GtkTargetEntry* dest_entries,
                                                gint dest_entries_length1,
                                                GdkDragAction actions);
static void code_widgets_source_list_tree_configure_drag_dest (CodeWidgetsSourceListTree* self,
                                                        GtkTargetEntry* dest_entries,
                                                        gint dest_entries_length1,
                                                        GdkDragAction actions);
VALA_EXTERN void code_widgets_source_list_disable_drag_dest (CodeWidgetsSourceList* self);
VALA_EXTERN gboolean code_widgets_source_list_scroll_to_item (CodeWidgetsSourceList* self,
                                                  CodeWidgetsSourceListItem* item,
                                                  gboolean expand_parents,
                                                  gboolean use_align,
                                                  gfloat row_align);
VALA_EXTERN CodeWidgetsSourceListExpandableItem* code_widgets_source_list_item_get_parent (CodeWidgetsSourceListItem* self);
VALA_EXTERN void code_widgets_source_list_expandable_item_expand_with_parents (CodeWidgetsSourceListExpandableItem* self);
static gboolean code_widgets_source_list_tree_scroll_to_item (CodeWidgetsSourceListTree* self,
                                                       CodeWidgetsSourceListItem* item,
                                                       gboolean use_align,
                                                       gfloat row_align);
VALA_EXTERN CodeWidgetsSourceListItem* code_widgets_source_list_get_previous_item (CodeWidgetsSourceList* self,
                                                                       CodeWidgetsSourceListItem* reference);
static GtkTreeIter* code_widgets_source_list_data_model_get_item_iter (CodeWidgetsSourceListDataModel* self,
                                                                CodeWidgetsSourceListItem* item);
static CodeWidgetsSourceListItem* code_widgets_source_list_data_model_get_item (CodeWidgetsSourceListDataModel* self,
                                                                         GtkTreeIter* iter);
static void _vala_GtkTreeIter_free (GtkTreeIter* self);
VALA_EXTERN CodeWidgetsSourceListItem* code_widgets_source_list_get_next_item (CodeWidgetsSourceList* self,
                                                                   CodeWidgetsSourceListItem* reference);
VALA_EXTERN CodeWidgetsSourceListItem* code_widgets_source_list_get_first_child (CodeWidgetsSourceList* self,
                                                                     CodeWidgetsSourceListExpandableItem* parent);
static CodeWidgetsSourceListItem* code_widgets_source_list_get_nth_child (CodeWidgetsSourceList* self,
                                                                   CodeWidgetsSourceListExpandableItem* parent,
                                                                   gint index);
VALA_EXTERN CodeWidgetsSourceListItem* code_widgets_source_list_get_last_child (CodeWidgetsSourceList* self,
                                                                    CodeWidgetsSourceListExpandableItem* parent);
VALA_EXTERN guint code_widgets_source_list_get_n_visible_children (CodeWidgetsSourceList* self,
                                                       CodeWidgetsSourceListExpandableItem* parent);
VALA_EXTERN CodeWidgetsSourceListExpandableItem* code_widgets_source_list_get_root (CodeWidgetsSourceList* self);
static CodeWidgetsSourceListExpandableItem* code_widgets_source_list_data_model_get_root (CodeWidgetsSourceListDataModel* self);
static void code_widgets_source_list_data_model_set_root (CodeWidgetsSourceListDataModel* self,
                                                   CodeWidgetsSourceListExpandableItem* value);
VALA_EXTERN CodeWidgetsSourceListItem* code_widgets_source_list_get_selected (CodeWidgetsSourceList* self);
static CodeWidgetsSourceListItem* code_widgets_source_list_tree_get_selected_item (CodeWidgetsSourceListTree* self);
VALA_EXTERN void code_widgets_source_list_set_selected (CodeWidgetsSourceList* self,
                                            CodeWidgetsSourceListItem* value);
static void code_widgets_source_list_tree_set_selected_item (CodeWidgetsSourceListTree* self,
                                                      CodeWidgetsSourceListItem* value);
VALA_EXTERN PangoEllipsizeMode code_widgets_source_list_get_ellipsize_mode (CodeWidgetsSourceList* self);
static PangoEllipsizeMode code_widgets_source_list_tree_get_ellipsize_mode (CodeWidgetsSourceListTree* self);
VALA_EXTERN void code_widgets_source_list_set_ellipsize_mode (CodeWidgetsSourceList* self,
                                                  PangoEllipsizeMode value);
static void code_widgets_source_list_tree_set_ellipsize_mode (CodeWidgetsSourceListTree* self,
                                                       PangoEllipsizeMode value);
static gboolean code_widgets_source_list_tree_get_editing (CodeWidgetsSourceListTree* self);
static void code_widgets_source_list_real_item_selected (CodeWidgetsSourceList* self,
                                                  CodeWidgetsSourceListItem* item);
VALA_EXTERN GtkMenu* code_widgets_source_list_item_get_context_menu (CodeWidgetsSourceListItem* self);
VALA_EXTERN CodeWidgetsSourceListItem* code_widgets_source_list_item_new (const gchar* name);
VALA_EXTERN CodeWidgetsSourceListItem* code_widgets_source_list_item_construct (GType object_type,
                                                                    const gchar* name);
VALA_EXTERN void code_widgets_source_list_item_set_name (CodeWidgetsSourceListItem* self,
                                             const gchar* value);
static GtkMenu* code_widgets_source_list_item_real_get_context_menu (CodeWidgetsSourceListItem* self);
VALA_EXTERN void code_widgets_source_list_item_set_parent (CodeWidgetsSourceListItem* self,
                                               CodeWidgetsSourceListExpandableItem* value);
VALA_EXTERN const gchar* code_widgets_source_list_item_get_name (CodeWidgetsSourceListItem* self);
VALA_EXTERN const gchar* code_widgets_source_list_item_get_tooltip (CodeWidgetsSourceListItem* self);
VALA_EXTERN void code_widgets_source_list_item_set_tooltip (CodeWidgetsSourceListItem* self,
                                                const gchar* value);
VALA_EXTERN const gchar* code_widgets_source_list_item_get_markup (CodeWidgetsSourceListItem* self);
VALA_EXTERN void code_widgets_source_list_item_set_markup (CodeWidgetsSourceListItem* self,
                                               const gchar* value);
VALA_EXTERN const gchar* code_widgets_source_list_item_get_badge (CodeWidgetsSourceListItem* self);
VALA_EXTERN void code_widgets_source_list_item_set_badge (CodeWidgetsSourceListItem* self,
                                              const gchar* value);
VALA_EXTERN gboolean code_widgets_source_list_item_get_editable (CodeWidgetsSourceListItem* self);
VALA_EXTERN void code_widgets_source_list_item_set_editable (CodeWidgetsSourceListItem* self,
                                                 gboolean value);
VALA_EXTERN gboolean code_widgets_source_list_item_get_visible (CodeWidgetsSourceListItem* self);
VALA_EXTERN void code_widgets_source_list_item_set_visible (CodeWidgetsSourceListItem* self,
                                                gboolean value);
VALA_EXTERN gboolean code_widgets_source_list_item_get_selectable (CodeWidgetsSourceListItem* self);
VALA_EXTERN void code_widgets_source_list_item_set_selectable (CodeWidgetsSourceListItem* self,
                                                   gboolean value);
VALA_EXTERN GIcon* code_widgets_source_list_item_get_icon (CodeWidgetsSourceListItem* self);
VALA_EXTERN void code_widgets_source_list_item_set_icon (CodeWidgetsSourceListItem* self,
                                             GIcon* value);
VALA_EXTERN GIcon* code_widgets_source_list_item_get_activatable (CodeWidgetsSourceListItem* self);
VALA_EXTERN void code_widgets_source_list_item_set_activatable (CodeWidgetsSourceListItem* self,
                                                    GIcon* value);
VALA_EXTERN const gchar* code_widgets_source_list_item_get_activatable_tooltip (CodeWidgetsSourceListItem* self);
VALA_EXTERN void code_widgets_source_list_item_set_activatable_tooltip (CodeWidgetsSourceListItem* self,
                                                            const gchar* value);
static void code_widgets_source_list_item_real_edited (CodeWidgetsSourceListItem* self,
                                                const gchar* new_name);
static void code_widgets_source_list_item_real_action_activated (CodeWidgetsSourceListItem* self);
static void code_widgets_source_list_item_real_activated (CodeWidgetsSourceListItem* self);
static void code_widgets_source_list_item_finalize (GObject * obj);
static GType code_widgets_source_list_item_get_type_once (void);
static void _vala_code_widgets_source_list_item_get_property (GObject * object,
                                                       guint property_id,
                                                       GValue * value,
                                                       GParamSpec * pspec);
static void _vala_code_widgets_source_list_item_set_property (GObject * object,
                                                       guint property_id,
                                                       const GValue * value,
                                                       GParamSpec * pspec);
VALA_EXTERN CodeWidgetsSourceListExpandableItem* code_widgets_source_list_expandable_item_new (const gchar* name);
VALA_EXTERN CodeWidgetsSourceListExpandableItem* code_widgets_source_list_expandable_item_construct (GType object_type,
                                                                                         const gchar* name);
VALA_EXTERN gboolean code_widgets_source_list_expandable_item_contains (CodeWidgetsSourceListExpandableItem* self,
                                                            CodeWidgetsSourceListItem* item);
VALA_EXTERN void code_widgets_source_list_expandable_item_add (CodeWidgetsSourceListExpandableItem* self,
                                                   CodeWidgetsSourceListItem* item);
VALA_EXTERN void code_widgets_source_list_expandable_item_remove (CodeWidgetsSourceListExpandableItem* self,
                                                      CodeWidgetsSourceListItem* item);
VALA_EXTERN void code_widgets_source_list_expandable_item_clear (CodeWidgetsSourceListExpandableItem* self);
VALA_EXTERN GeeCollection* code_widgets_source_list_expandable_item_get_children (CodeWidgetsSourceListExpandableItem* self);
VALA_EXTERN void code_widgets_source_list_expandable_item_expand_all (CodeWidgetsSourceListExpandableItem* self,
                                                          gboolean inclusive,
                                                          gboolean recursive);
static void code_widgets_source_list_expandable_item_set_expansion (CodeWidgetsSourceListExpandableItem* item,
                                                             gboolean inclusive,
                                                             gboolean recursive,
                                                             gboolean expanded);
VALA_EXTERN void code_widgets_source_list_expandable_item_collapse_all (CodeWidgetsSourceListExpandableItem* self,
                                                            gboolean inclusive,
                                                            gboolean recursive);
VALA_EXTERN void code_widgets_source_list_expandable_item_set_expanded (CodeWidgetsSourceListExpandableItem* self,
                                                            gboolean value);
VALA_EXTERN void code_widgets_source_list_expandable_item_collapse_with_parents (CodeWidgetsSourceListExpandableItem* self);
VALA_EXTERN gboolean code_widgets_source_list_expandable_item_get_collapsible (CodeWidgetsSourceListExpandableItem* self);
VALA_EXTERN void code_widgets_source_list_expandable_item_set_collapsible (CodeWidgetsSourceListExpandableItem* self,
                                                               gboolean value);
VALA_EXTERN gboolean code_widgets_source_list_expandable_item_get_expanded (CodeWidgetsSourceListExpandableItem* self);
VALA_EXTERN guint code_widgets_source_list_expandable_item_get_n_children (CodeWidgetsSourceListExpandableItem* self);
static void code_widgets_source_list_expandable_item_real_toggled (CodeWidgetsSourceListExpandableItem* self);
static GObject * code_widgets_source_list_expandable_item_constructor (GType type,
                                                                guint n_construct_properties,
                                                                GObjectConstructParam * construct_properties);
static void code_widgets_source_list_expandable_item_finalize (GObject * obj);
static GType code_widgets_source_list_expandable_item_get_type_once (void);
static void _vala_code_widgets_source_list_expandable_item_get_property (GObject * object,
                                                                  guint property_id,
                                                                  GValue * value,
                                                                  GParamSpec * pspec);
static void _vala_code_widgets_source_list_expandable_item_set_property (GObject * object,
                                                                  guint property_id,
                                                                  const GValue * value,
                                                                  GParamSpec * pspec);
static gpointer code_widgets_source_list_data_model_node_wrapper_ref (gpointer instance);
static void code_widgets_source_list_data_model_node_wrapper_unref (gpointer instance);
static GParamSpec* code_widgets_source_list_data_model_param_spec_node_wrapper (const gchar* name,
                                                                         const gchar* nick,
                                                                         const gchar* blurb,
                                                                         GType object_type,
                                                                         GParamFlags flags) G_GNUC_UNUSED ;
static void code_widgets_source_list_data_model_value_set_node_wrapper (GValue* value,
                                                                 gpointer v_object) G_GNUC_UNUSED ;
static void code_widgets_source_list_data_model_value_take_node_wrapper (GValue* value,
                                                                  gpointer v_object) G_GNUC_UNUSED ;
static gpointer code_widgets_source_list_data_model_value_get_node_wrapper (const GValue* value) G_GNUC_UNUSED ;
static GType code_widgets_source_list_data_model_node_wrapper_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (CodeWidgetsSourceListDataModelNodeWrapper, code_widgets_source_list_data_model_node_wrapper_unref)
static gpointer code_widgets_source_list_data_model_item_monitor_ref (gpointer instance);
static void code_widgets_source_list_data_model_item_monitor_unref (gpointer instance);
static GParamSpec* code_widgets_source_list_data_model_param_spec_item_monitor (const gchar* name,
                                                                         const gchar* nick,
                                                                         const gchar* blurb,
                                                                         GType object_type,
                                                                         GParamFlags flags) G_GNUC_UNUSED ;
static void code_widgets_source_list_data_model_value_set_item_monitor (GValue* value,
                                                                 gpointer v_object) G_GNUC_UNUSED ;
static void code_widgets_source_list_data_model_value_take_item_monitor (GValue* value,
                                                                  gpointer v_object) G_GNUC_UNUSED ;
static gpointer code_widgets_source_list_data_model_value_get_item_monitor (const GValue* value) G_GNUC_UNUSED ;
static GType code_widgets_source_list_data_model_item_monitor_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (CodeWidgetsSourceListDataModelItemMonitor, code_widgets_source_list_data_model_item_monitor_unref)
static GType code_widgets_source_list_data_model_column_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
static GType code_widgets_source_list_data_model_column_type (CodeWidgetsSourceListDataModelColumn self);
static void code_widgets_source_list_data_model_update_item (CodeWidgetsSourceListDataModel* self,
                                                      CodeWidgetsSourceListItem* item);
static GtkTreePath* code_widgets_source_list_data_model_node_wrapper_get_path (CodeWidgetsSourceListDataModelNodeWrapper* self);
static GtkTreeIter* code_widgets_source_list_data_model_node_wrapper_get_iter (CodeWidgetsSourceListDataModelNodeWrapper* self);
static void code_widgets_source_list_data_model_add_item (CodeWidgetsSourceListDataModel* self,
                                                   CodeWidgetsSourceListItem* item);
static GtkTreeIter* code_widgets_source_list_data_model_get_item_child_iter (CodeWidgetsSourceListDataModel* self,
                                                                      CodeWidgetsSourceListItem* item);
static GtkTreeIter* _vala_GtkTreeIter_copy (GtkTreeIter* self);
static CodeWidgetsSourceListDataModelNodeWrapper* code_widgets_source_list_data_model_node_wrapper_new (GtkTreeModel* model,
                                                                                                 GtkTreeIter* iter);
static CodeWidgetsSourceListDataModelNodeWrapper* code_widgets_source_list_data_model_node_wrapper_construct (GType object_type,
                                                                                                       GtkTreeModel* model,
                                                                                                       GtkTreeIter* iter);
static void code_widgets_source_list_data_model_add_property_monitor (CodeWidgetsSourceListDataModel* self,
                                                               CodeWidgetsSourceListItem* item);
static void code_widgets_source_list_data_model_push_parent_update (CodeWidgetsSourceListDataModel* self,
                                                             CodeWidgetsSourceListExpandableItem* parent);
static void code_widgets_source_list_data_model_add_children_monitor (CodeWidgetsSourceListDataModel* self,
                                                               CodeWidgetsSourceListExpandableItem* item);
static void code_widgets_source_list_data_model_remove_item (CodeWidgetsSourceListDataModel* self,
                                                      CodeWidgetsSourceListItem* item);
static void code_widgets_source_list_data_model_remove_property_monitor (CodeWidgetsSourceListDataModel* self,
                                                                  CodeWidgetsSourceListItem* item);
static void code_widgets_source_list_data_model_remove_children_monitor (CodeWidgetsSourceListDataModel* self,
                                                                  CodeWidgetsSourceListExpandableItem* item);
static CodeWidgetsSourceListDataModelItemMonitor* code_widgets_source_list_data_model_item_monitor_new (CodeWidgetsSourceListItem* item);
static CodeWidgetsSourceListDataModelItemMonitor* code_widgets_source_list_data_model_item_monitor_construct (GType object_type,
                                                                                                       CodeWidgetsSourceListItem* item);
static void code_widgets_source_list_data_model_on_item_prop_changed (CodeWidgetsSourceListDataModel* self,
                                                               CodeWidgetsSourceListItem* item,
                                                               const gchar* prop_name);
static void _code_widgets_source_list_data_model_on_item_prop_changed_code_widgets_source_list_data_model_item_monitor_changed (CodeWidgetsSourceListDataModelItemMonitor* _sender,
                                                                                                                         CodeWidgetsSourceListItem* _self_,
                                                                                                                         const gchar* prop_name,
                                                                                                                         gpointer self);
static void code_widgets_source_list_data_model_on_item_child_added (CodeWidgetsSourceListDataModel* self,
                                                              CodeWidgetsSourceListItem* item);
static void _code_widgets_source_list_data_model_on_item_child_added_code_widgets_source_list_expandable_item_child_added (CodeWidgetsSourceListExpandableItem* _sender,
                                                                                                                    CodeWidgetsSourceListItem* item,
                                                                                                                    gpointer self);
static void code_widgets_source_list_data_model_on_item_child_removed (CodeWidgetsSourceListDataModel* self,
                                                                CodeWidgetsSourceListItem* item);
static void _code_widgets_source_list_data_model_on_item_child_removed_code_widgets_source_list_expandable_item_child_removed (CodeWidgetsSourceListExpandableItem* _sender,
                                                                                                                        CodeWidgetsSourceListItem* item,
                                                                                                                        gpointer self);
static Block37Data* block37_data_ref (Block37Data* _data37_);
static void block37_data_unref (void * _userdata_);
static gboolean ___lambda4_ (Block37Data* _data37_);
static gboolean ____lambda4__gsource_func (gpointer self);
static CodeWidgetsSourceListItem* code_widgets_source_list_data_model_get_item_from_path (CodeWidgetsSourceListDataModel* self,
                                                                                   GtkTreePath* path);
static GtkTreePath* code_widgets_source_list_data_model_get_item_child_path (CodeWidgetsSourceListDataModel* self,
                                                                      CodeWidgetsSourceListItem* item);
static gboolean code_widgets_source_list_data_model_is_category (CodeWidgetsSourceListDataModel* self,
                                                          CodeWidgetsSourceListItem* item,
                                                          GtkTreeIter* iter,
                                                          GtkTreePath* path);
static gboolean code_widgets_source_list_data_model_is_iter_at_root_level (CodeWidgetsSourceListDataModel* self,
                                                                    GtkTreeIter* iter);
static gboolean code_widgets_source_list_data_model_is_path_at_root_level (CodeWidgetsSourceListDataModel* self,
                                                                    GtkTreePath* path);
static void code_widgets_source_list_data_model_resort (CodeWidgetsSourceListDataModel* self);
static gint code_widgets_source_list_data_model_child_model_sort_func (CodeWidgetsSourceListDataModel* self,
                                                                GtkTreeModel* model,
                                                                GtkTreeIter* a,
                                                                GtkTreeIter* b);
static gboolean code_widgets_source_list_data_model_filter_visible_func (CodeWidgetsSourceListDataModel* self,
                                                                  GtkTreeModel* child_model,
                                                                  GtkTreeIter* iter);
static gboolean code_widgets_source_list_data_model_real_drag_data_received (GtkTreeDragDest* base,
                                                                      GtkTreePath* dest,
                                                                      GtkSelectionData* selection_data);
static gboolean code_widgets_source_list_data_model_child_tree_drag_data_received (CodeWidgetsSourceListDataModel* self,
                                                                            GtkTreePath* dest,
                                                                            GtkTreePath* src_path);
static GtkTreePath* _vala_GtkTreePath_copy (GtkTreePath* self);
static void code_widgets_source_list_data_model_recursive_node_copy (CodeWidgetsSourceListDataModel* self,
                                                              GtkTreeIter* src_iter,
                                                              GtkTreeIter* dest_iter);
static void code_widgets_source_list_data_model_move_item (CodeWidgetsSourceListDataModel* self,
                                                    GtkTreeIter* src_iter,
                                                    GtkTreeIter* dest_iter);
static gboolean code_widgets_source_list_data_model_real_row_drop_possible (GtkTreeDragDest* base,
                                                                     GtkTreePath* dest,
                                                                     GtkSelectionData* selection_data);
static gint* _vala_array_dup1 (gint* self,
                        gssize length);
static gint* _vala_array_dup2 (gint* self,
                        gssize length);
static gboolean code_widgets_source_list_data_model_real_drag_data_get (GtkTreeDragSource* base,
                                                                 GtkTreePath* path,
                                                                 GtkSelectionData* selection_data);
static gboolean code_widgets_source_list_data_model_real_row_draggable (GtkTreeDragSource* base,
                                                                 GtkTreePath* path);
static void _vala_GtkTreeRowReference_free (GtkTreeRowReference* self);
static gboolean code_widgets_source_list_data_model_node_wrapper_get_valid (CodeWidgetsSourceListDataModelNodeWrapper* self);
static void code_widgets_source_list_data_model_node_wrapper_finalize (CodeWidgetsSourceListDataModelNodeWrapper * obj);
static GType code_widgets_source_list_data_model_node_wrapper_get_type_once (void);
static void code_widgets_source_list_data_model_item_monitor_on_notify (CodeWidgetsSourceListDataModelItemMonitor* self,
                                                                 GParamSpec* prop);
static void _code_widgets_source_list_data_model_item_monitor_on_notify_g_object_notify (GObject* _sender,
                                                                                  GParamSpec* pspec,
                                                                                  gpointer self);
static void g_cclosure_user_marshal_VOID__OBJECT_STRING (GClosure * closure,
                                                  GValue * return_value,
                                                  guint n_param_values,
                                                  const GValue * param_values,
                                                  gpointer invocation_hint,
                                                  gpointer marshal_data);
static void code_widgets_source_list_data_model_item_monitor_finalize (CodeWidgetsSourceListDataModelItemMonitor * obj);
static GType code_widgets_source_list_data_model_item_monitor_get_type_once (void);
static GObject * code_widgets_source_list_data_model_constructor (GType type,
                                                           guint n_construct_properties,
                                                           GObjectConstructParam * construct_properties);
static gint _code_widgets_source_list_data_model_child_model_sort_func_gtk_tree_iter_compare_func (GtkTreeModel* model,
                                                                                            GtkTreeIter* a,
                                                                                            GtkTreeIter* b,
                                                                                            gpointer self);
static gboolean _code_widgets_source_list_data_model_filter_visible_func_gtk_tree_model_filter_visible_func (GtkTreeModel* model,
                                                                                                      GtkTreeIter* iter,
                                                                                                      gpointer self);
static void code_widgets_source_list_data_model_finalize (GObject * obj);
static GType code_widgets_source_list_data_model_get_type_once (void);
static void _vala_code_widgets_source_list_data_model_get_property (GObject * object,
                                                             guint property_id,
                                                             GValue * value,
                                                             GParamSpec * pspec);
static void _vala_code_widgets_source_list_data_model_set_property (GObject * object,
                                                             guint property_id,
                                                             const GValue * value,
                                                             GParamSpec * pspec);
static GType code_widgets_source_list_cell_renderer_icon_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (CodeWidgetsSourceListCellRendererIcon, g_object_unref)
static CodeWidgetsSourceListCellRendererIcon* code_widgets_source_list_cell_renderer_icon_new (void);
static CodeWidgetsSourceListCellRendererIcon* code_widgets_source_list_cell_renderer_icon_construct (GType object_type);
static gboolean code_widgets_source_list_cell_renderer_icon_real_activate (GtkCellRenderer* base,
                                                                    GdkEvent* event,
                                                                    GtkWidget* widget,
                                                                    const gchar* path,
                                                                    GdkRectangle* background_area,
                                                                    GdkRectangle* cell_area,
                                                                    GtkCellRendererState flags);
static GObject * code_widgets_source_list_cell_renderer_icon_constructor (GType type,
                                                                   guint n_construct_properties,
                                                                   GObjectConstructParam * construct_properties);
static GType code_widgets_source_list_cell_renderer_icon_get_type_once (void);
static GType code_widgets_source_list_cell_renderer_spacer_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (CodeWidgetsSourceListCellRendererSpacer, g_object_unref)
static GtkSizeRequestMode code_widgets_source_list_cell_renderer_spacer_real_get_request_mode (GtkCellRenderer* base);
static void code_widgets_source_list_cell_renderer_spacer_real_get_preferred_width (GtkCellRenderer* base,
                                                                             GtkWidget* widget,
                                                                             gint* min_size,
                                                                             gint* natural_size);
static void code_widgets_source_list_cell_renderer_spacer_real_get_preferred_height_for_width (GtkCellRenderer* base,
                                                                                        GtkWidget* widget,
                                                                                        gint width,
                                                                                        gint* min_height,
                                                                                        gint* natural_height);
static void code_widgets_source_list_cell_renderer_spacer_real_render (GtkCellRenderer* base,
                                                                cairo_t* context,
                                                                GtkWidget* widget,
                                                                GdkRectangle* bg_area,
                                                                GdkRectangle* cell_area,
                                                                GtkCellRendererState flags);
static CodeWidgetsSourceListCellRendererSpacer* code_widgets_source_list_cell_renderer_spacer_new (void);
static CodeWidgetsSourceListCellRendererSpacer* code_widgets_source_list_cell_renderer_spacer_construct (GType object_type);
static gint code_widgets_source_list_cell_renderer_spacer_get_level (CodeWidgetsSourceListCellRendererSpacer* self);
static void code_widgets_source_list_cell_renderer_spacer_set_level (CodeWidgetsSourceListCellRendererSpacer* self,
                                                              gint value);
static void code_widgets_source_list_cell_renderer_spacer_finalize (GObject * obj);
static GType code_widgets_source_list_cell_renderer_spacer_get_type_once (void);
static void _vala_code_widgets_source_list_cell_renderer_spacer_get_property (GObject * object,
                                                                       guint property_id,
                                                                       GValue * value,
                                                                       GParamSpec * pspec);
static void _vala_code_widgets_source_list_cell_renderer_spacer_set_property (GObject * object,
                                                                       guint property_id,
                                                                       const GValue * value,
                                                                       GParamSpec * pspec);
VALA_EXTERN GType code_widgets_cell_renderer_badge_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (CodeWidgetsCellRendererBadge, g_object_unref)
VALA_EXTERN GType code_widgets_cell_renderer_expander_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (CodeWidgetsCellRendererExpander, g_object_unref)
static void code_widgets_source_list_tree_disable_item_property_monitor (CodeWidgetsSourceListTree* self);
static GType code_widgets_source_list_tree_column_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
static CodeWidgetsSourceListTree* code_widgets_source_list_tree_new (CodeWidgetsSourceListDataModel* data_model);
static CodeWidgetsSourceListTree* code_widgets_source_list_tree_construct (GType object_type,
                                                                    CodeWidgetsSourceListDataModel* data_model);
static gboolean code_widgets_source_list_tree_real_drag_motion (GtkWidget* base,
                                                         GdkDragContext* context,
                                                         gint x,
                                                         gint y,
                                                         guint time);
static CodeWidgetsSourceListDataModel* code_widgets_source_list_tree_get_data_model (CodeWidgetsSourceListTree* self);
static void code_widgets_source_list_tree_real_drag_data_received (GtkWidget* base,
                                                            GdkDragContext* context,
                                                            gint x,
                                                            gint y,
                                                            GtkSelectionData* selection_data,
                                                            guint info,
                                                            guint time);
static GtkTargetEntry* code_widgets_source_list_tree_append_row_target_entry (GtkTargetEntry* orig,
                                                                       gint orig_length1,
                                                                       gint* result_length1);
static gboolean code_widgets_source_list_tree_on_query_tooltip (CodeWidgetsSourceListTree* self,
                                                         gint x,
                                                         gint y,
                                                         gboolean keyboard_tooltip,
                                                         GtkTooltip* tooltip);
static gboolean code_widgets_source_list_tree_over_cell (CodeWidgetsSourceListTree* self,
                                                  GtkTreeViewColumn* col,
                                                  GtkTreePath* path,
                                                  GtkCellRenderer* cell,
                                                  gint x);
static void _vala_array_add4 (GtkTargetEntry* * array,
                       gint* length,
                       gint* size,
                       const GtkTargetEntry* value);
static void _vala_array_add5 (GtkTargetEntry* * array,
                       gint* length,
                       gint* size,
                       const GtkTargetEntry* value);
static void code_widgets_source_list_tree_enable_item_property_monitor (CodeWidgetsSourceListTree* self);
static void code_widgets_source_list_tree_on_model_item_updated (CodeWidgetsSourceListTree* self,
                                                          CodeWidgetsSourceListItem* item);
static void _code_widgets_source_list_tree_on_model_item_updated_code_widgets_source_list_data_model_item_updated (CodeWidgetsSourceListDataModel* _sender,
                                                                                                            CodeWidgetsSourceListItem* item,
                                                                                                            gpointer self);
static void code_widgets_source_list_tree_update_expansion (CodeWidgetsSourceListTree* self,
                                                     CodeWidgetsSourceListExpandableItem* expandable_item);
static void code_widgets_source_list_tree_add_spacer_cell_for_level (CodeWidgetsSourceListTree* self,
                                                              gint level,
                                                              gboolean check_previous);
static void code_widgets_source_list_tree_spacer_cell_data_func (GtkCellLayout* layout,
                                                          GtkCellRenderer* renderer,
                                                          GtkTreeModel* model,
                                                          GtkTreeIter* iter);
static void _code_widgets_source_list_tree_spacer_cell_data_func_gtk_cell_layout_data_func (GtkCellLayout* cell_layout,
                                                                                     GtkCellRenderer* cell,
                                                                                     GtkTreeModel* tree_model,
                                                                                     GtkTreeIter* iter,
                                                                                     gpointer self);
static gboolean code_widgets_source_list_tree_select_func (CodeWidgetsSourceListTree* self,
                                                    GtkTreeSelection* selection,
                                                    GtkTreeModel* model,
                                                    GtkTreePath* path,
                                                    gboolean path_currently_selected);
static GtkTreePath* code_widgets_source_list_tree_get_selected_path (CodeWidgetsSourceListTree* self);
static void __vala_GtkTreePath_free0_ (gpointer var);
static inline void _g_list_free___vala_GtkTreePath_free0_ (GList* self);
static void code_widgets_source_list_tree_set_selected (CodeWidgetsSourceListTree* self,
                                                 CodeWidgetsSourceListItem* item,
                                                 gboolean scroll_to_item);
static void code_widgets_source_list_tree_real_cursor_changed (GtkTreeView* base);
static void code_widgets_source_list_tree_on_editing_started (CodeWidgetsSourceListTree* self,
                                                       GtkCellEditable* editable,
                                                       const gchar* path);
static void code_widgets_source_list_tree_on_editing_done (CodeWidgetsSourceListTree* self);
static void _code_widgets_source_list_tree_on_editing_done_gtk_cell_editable_editing_done (GtkCellEditable* _sender,
                                                                                    gpointer self);
static void code_widgets_source_list_tree_on_editing_canceled (CodeWidgetsSourceListTree* self);
static void code_widgets_source_list_tree_on_activatable_activated (CodeWidgetsSourceListTree* self,
                                                             const gchar* item_path_str);
static CodeWidgetsSourceListItem* code_widgets_source_list_tree_get_item_from_path_string (CodeWidgetsSourceListTree* self,
                                                                                    const gchar* item_path_str);
static gboolean code_widgets_source_list_tree_toggle_expansion (CodeWidgetsSourceListTree* self,
                                                         CodeWidgetsSourceListExpandableItem* item);
static void code_widgets_source_list_tree_real_row_expanded (GtkTreeView* base,
                                                      GtkTreeIter* iter,
                                                      GtkTreePath* path);
static void code_widgets_source_list_tree_real_row_collapsed (GtkTreeView* base,
                                                       GtkTreeIter* iter,
                                                       GtkTreePath* path);
static void code_widgets_source_list_tree_real_row_activated (GtkTreeView* base,
                                                       GtkTreePath* path,
                                                       GtkTreeViewColumn* column);
static gboolean code_widgets_source_list_tree_real_key_release_event (GtkWidget* base,
                                                               GdkEventKey* event);
static gboolean code_widgets_source_list_tree_real_button_release_event (GtkWidget* base,
                                                                  GdkEventButton* event);
static gboolean code_widgets_source_list_tree_real_button_press_event (GtkWidget* base,
                                                                GdkEventButton* event);
static gboolean code_widgets_source_list_tree_popup_context_menu (CodeWidgetsSourceListTree* self,
                                                           CodeWidgetsSourceListItem* item,
                                                           GdkEventButton* event);
static gboolean code_widgets_source_list_tree_over_primary_expander (CodeWidgetsSourceListTree* self,
                                                              GtkTreeViewColumn* col,
                                                              GtkTreePath* path,
                                                              gint x);
static void code_widgets_source_list_tree_expander_cell_data_func (CodeWidgetsSourceListTree* self,
                                                            GtkCellLayout* layout,
                                                            GtkCellRenderer* renderer,
                                                            GtkTreeModel* model,
                                                            GtkTreeIter* iter);
static gint code_widgets_source_list_tree_get_cell_width (CodeWidgetsSourceListTree* self,
                                                   GtkCellRenderer* cell_renderer);
static gboolean code_widgets_source_list_tree_real_popup_menu (GtkWidget* base);
static CodeWidgetsSourceListItem* code_widgets_source_list_tree_get_item_from_model (GtkTreeModel* model,
                                                                              GtkTreeIter* iter);
static void code_widgets_source_list_tree_name_cell_data_func (CodeWidgetsSourceListTree* self,
                                                        GtkCellLayout* layout,
                                                        GtkCellRenderer* renderer,
                                                        GtkTreeModel* model,
                                                        GtkTreeIter* iter);
static void code_widgets_source_list_tree_badge_cell_data_func (CodeWidgetsSourceListTree* self,
                                                         GtkCellLayout* layout,
                                                         GtkCellRenderer* renderer,
                                                         GtkTreeModel* model,
                                                         GtkTreeIter* iter);
VALA_EXTERN void code_widgets_cell_renderer_badge_set_text (CodeWidgetsCellRendererBadge* self,
                                                const gchar* value);
static void code_widgets_source_list_tree_icon_cell_data_func (CodeWidgetsSourceListTree* self,
                                                        GtkCellLayout* layout,
                                                        GtkCellRenderer* renderer,
                                                        GtkTreeModel* model,
                                                        GtkTreeIter* iter);
static void code_widgets_source_list_tree_set_data_model (CodeWidgetsSourceListTree* self,
                                                   CodeWidgetsSourceListDataModel* value);
static GObject * code_widgets_source_list_tree_constructor (GType type,
                                                     guint n_construct_properties,
                                                     GObjectConstructParam * construct_properties);
VALA_EXTERN CodeWidgetsCellRendererExpander* code_widgets_cell_renderer_expander_new (void);
VALA_EXTERN CodeWidgetsCellRendererExpander* code_widgets_cell_renderer_expander_construct (GType object_type);
VALA_EXTERN void code_widgets_cell_renderer_expander_set_is_category_expander (CodeWidgetsCellRendererExpander* self,
                                                                   gboolean value);
static void _code_widgets_source_list_tree_expander_cell_data_func_gtk_cell_layout_data_func (GtkCellLayout* cell_layout,
                                                                                       GtkCellRenderer* cell,
                                                                                       GtkTreeModel* tree_model,
                                                                                       GtkTreeIter* iter,
                                                                                       gpointer self);
static void _code_widgets_source_list_tree_on_activatable_activated_code_widgets_source_list_cell_renderer_icon_activated (CodeWidgetsSourceListCellRendererIcon* _sender,
                                                                                                                    const gchar* path,
                                                                                                                    gpointer self);
static void _code_widgets_source_list_tree_icon_cell_data_func_gtk_cell_layout_data_func (GtkCellLayout* cell_layout,
                                                                                   GtkCellRenderer* cell,
                                                                                   GtkTreeModel* tree_model,
                                                                                   GtkTreeIter* iter,
                                                                                   gpointer self);
VALA_EXTERN CodeWidgetsCellRendererBadge* code_widgets_cell_renderer_badge_new (void);
VALA_EXTERN CodeWidgetsCellRendererBadge* code_widgets_cell_renderer_badge_construct (GType object_type);
static void _code_widgets_source_list_tree_badge_cell_data_func_gtk_cell_layout_data_func (GtkCellLayout* cell_layout,
                                                                                    GtkCellRenderer* cell,
                                                                                    GtkTreeModel* tree_model,
                                                                                    GtkTreeIter* iter,
                                                                                    gpointer self);
static void _code_widgets_source_list_tree_on_editing_started_gtk_cell_renderer_editing_started (GtkCellRenderer* _sender,
                                                                                          GtkCellEditable* editable,
                                                                                          const gchar* path,
                                                                                          gpointer self);
static void _code_widgets_source_list_tree_on_editing_canceled_gtk_cell_renderer_editing_canceled (GtkCellRenderer* _sender,
                                                                                            gpointer self);
static void _code_widgets_source_list_tree_name_cell_data_func_gtk_cell_layout_data_func (GtkCellLayout* cell_layout,
                                                                                   GtkCellRenderer* cell,
                                                                                   GtkTreeModel* tree_model,
                                                                                   GtkTreeIter* iter,
                                                                                   gpointer self);
static gboolean _code_widgets_source_list_tree_select_func_gtk_tree_selection_func (GtkTreeSelection* selection,
                                                                             GtkTreeModel* model,
                                                                             GtkTreePath* path,
                                                                             gboolean path_currently_selected,
                                                                             gpointer self);
static gboolean _code_widgets_source_list_tree_on_query_tooltip_gtk_widget_query_tooltip (GtkWidget* _sender,
                                                                                   gint x,
                                                                                   gint y,
                                                                                   gboolean keyboard_tooltip,
                                                                                   GtkTooltip* tooltip,
                                                                                   gpointer self);
static void code_widgets_source_list_tree_finalize (GObject * obj);
static GType code_widgets_source_list_tree_get_type_once (void);
static void _vala_code_widgets_source_list_tree_get_property (GObject * object,
                                                       guint property_id,
                                                       GValue * value,
                                                       GParamSpec * pspec);
static void _vala_code_widgets_source_list_tree_set_property (GObject * object,
                                                       guint property_id,
                                                       const GValue * value,
                                                       GParamSpec * pspec);
static GObject * code_widgets_source_list_constructor (GType type,
                                                guint n_construct_properties,
                                                GObjectConstructParam * construct_properties);
static void _code_widgets_source_list___lambda5_ (CodeWidgetsSourceList* self,
                                           CodeWidgetsSourceListItem* item);
static void __code_widgets_source_list___lambda5__code_widgets_source_list_tree_item_selected (CodeWidgetsSourceListTree* _sender,
                                                                                        CodeWidgetsSourceListItem* item,
                                                                                        gpointer self);
static void code_widgets_source_list_finalize (GObject * obj);
static GType code_widgets_source_list_get_type_once (void);
static void _vala_code_widgets_source_list_get_property (GObject * object,
                                                  guint property_id,
                                                  GValue * value,
                                                  GParamSpec * pspec);
static void _vala_code_widgets_source_list_set_property (GObject * object,
                                                  guint property_id,
                                                  const GValue * value,
                                                  GParamSpec * pspec);
static inline gpointer _vala_memdup2 (gconstpointer mem,
                        gsize byte_size);

/**
     * Whether this item will allow users to re-arrange its children via DnD.
     *
     * This feature can co-exist with a sort algorithm (implemented
     * by {@link Code.Widgets.SourceListSortable.compare}), but
     * the actual order of the items in the list will always
     * honor that method. The sort function has to be compatible with
     * the kind of DnD reordering the item wants to allow, since the user can
     * only reorder those items for which //compare// returns 0.
     *
     * @return Whether the item's children can be re-arranged by users.
     * @since 0.3
     */
gboolean
code_widgets_source_list_sortable_allow_dnd_sorting (CodeWidgetsSourceListSortable* self)
{
	CodeWidgetsSourceListSortableIface* _iface_;
#line 36 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 36 "../src/Widgets/SourceList/SourceList.vala"
	_iface_ = CODE_WIDGETS_SOURCE_LIST_SORTABLE_GET_INTERFACE (self);
#line 36 "../src/Widgets/SourceList/SourceList.vala"
	if (_iface_->allow_dnd_sorting) {
#line 36 "../src/Widgets/SourceList/SourceList.vala"
		return _iface_->allow_dnd_sorting (self);
#line 1240 "SourceList.c"
	}
#line 36 "../src/Widgets/SourceList/SourceList.vala"
	return FALSE;
#line 1244 "SourceList.c"
}

/**
     * Should return a negative integer, zero, or a positive integer if ''a''
     * sorts //before// ''b'', ''a'' sorts //with// ''b'', or ''a'' sorts
     * //after// ''b'' respectively. If two items compare as equal, their
     * order in the sorted source list is undefined.
     *
     * In order to ensure that the source list behaves as expected, this
     * method must define a partial order on the source list tree; i.e. it
     * must be reflexive, antisymmetric and transitive. Not complying with
     * those requirements could make the program fall into an infinite loop
     * and freeze the user interface.
     *
     * Should return //0// to allow any pair of items to be sortable via DnD.
     *
     * @param a First item.
     * @param b Second item.
     * @return A //negative// integer if //a// sorts before //b//,
     *         //zero// if //a// equals //b//, or a //positive//
     *         integer if //a// sorts after //b//.
     * @since 0.3
     */
gint
code_widgets_source_list_sortable_compare (CodeWidgetsSourceListSortable* self,
                                           CodeWidgetsSourceListItem* a,
                                           CodeWidgetsSourceListItem* b)
{
	CodeWidgetsSourceListSortableIface* _iface_;
#line 59 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 59 "../src/Widgets/SourceList/SourceList.vala"
	_iface_ = CODE_WIDGETS_SOURCE_LIST_SORTABLE_GET_INTERFACE (self);
#line 59 "../src/Widgets/SourceList/SourceList.vala"
	if (_iface_->compare) {
#line 59 "../src/Widgets/SourceList/SourceList.vala"
		return _iface_->compare (self, a, b);
#line 1282 "SourceList.c"
	}
#line 59 "../src/Widgets/SourceList/SourceList.vala"
	return -1;
#line 1286 "SourceList.c"
}

static void
code_widgets_source_list_sortable_default_init (CodeWidgetsSourceListSortableIface * iface,
                                                gpointer iface_data)
{
	/**
	     * Emitted after a user has re-ordered an item via DnD.
	     *
	     * @param moved The item that was moved to a different position by the user.
	     * @since 0.3
	     */
#line 14 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_sortable_signals[CODE_WIDGETS_SOURCE_LIST_SORTABLE_USER_MOVED_ITEM_SIGNAL] = g_signal_new ("user-moved-item", CODE_WIDGETS_TYPE_SOURCE_LIST_SORTABLE, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, CODE_WIDGETS_SOURCE_LIST_TYPE_ITEM);
#line 1301 "SourceList.c"
}

/**
 * An interface for sorting items.
 *
 * @since 0.3
 */
static GType
code_widgets_source_list_sortable_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (CodeWidgetsSourceListSortableIface), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) code_widgets_source_list_sortable_default_init, (GClassFinalizeFunc) NULL, NULL, 0, 0, (GInstanceInitFunc) NULL, NULL };
	GType code_widgets_source_list_sortable_type_id;
	code_widgets_source_list_sortable_type_id = g_type_register_static (G_TYPE_INTERFACE, "CodeWidgetsSourceListSortable", &g_define_type_info, 0);
	g_type_interface_add_prerequisite (code_widgets_source_list_sortable_type_id, CODE_WIDGETS_SOURCE_LIST_TYPE_EXPANDABLE_ITEM);
	return code_widgets_source_list_sortable_type_id;
}

GType
code_widgets_source_list_sortable_get_type (void)
{
	static volatile gsize code_widgets_source_list_sortable_type_id__once = 0;
	if (g_once_init_enter (&code_widgets_source_list_sortable_type_id__once)) {
		GType code_widgets_source_list_sortable_type_id;
		code_widgets_source_list_sortable_type_id = code_widgets_source_list_sortable_get_type_once ();
		g_once_init_leave (&code_widgets_source_list_sortable_type_id__once, code_widgets_source_list_sortable_type_id);
	}
	return code_widgets_source_list_sortable_type_id__once;
}

/**
     * Determines whether this item can be dragged outside the source list widget.
     *
     * Even if this method returns //false//, the item could still be dragged around
     * within the source list if its parent allows DnD reordering. This only happens
     * when the parent implements {@link Code.Widgets.SourceListSortable}.
     *
     * @return //true// if the item can be dragged; //false// otherwise.
     * @since 0.3
     * @see Code.Widgets.SourceListSortable
     */
gboolean
code_widgets_source_list_drag_source_draggable (CodeWidgetsSourceListDragSource* self)
{
	CodeWidgetsSourceListDragSourceIface* _iface_;
#line 79 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 79 "../src/Widgets/SourceList/SourceList.vala"
	_iface_ = CODE_WIDGETS_SOURCE_LIST_DRAG_SOURCE_GET_INTERFACE (self);
#line 79 "../src/Widgets/SourceList/SourceList.vala"
	if (_iface_->draggable) {
#line 79 "../src/Widgets/SourceList/SourceList.vala"
		return _iface_->draggable (self);
#line 1354 "SourceList.c"
	}
#line 79 "../src/Widgets/SourceList/SourceList.vala"
	return FALSE;
#line 1358 "SourceList.c"
}

/**
     * This method is called when the drop site requests the data which is dragged.
     *
     * It is the responsibility of this method to fill //selection_data// with the
     * data in the format which is indicated by {@link Gtk.SelectionData.get_target}.
     *
     * @param selection_data {@link Gtk.SelectionData} containing source data.
     * @since 0.3
     * @see Gtk.SelectionData.set
     * @see Gtk.SelectionData.set_uris
     * @see Gtk.SelectionData.set_text
     */
void
code_widgets_source_list_drag_source_prepare_selection_data (CodeWidgetsSourceListDragSource* self,
                                                             GtkSelectionData* selection_data)
{
	CodeWidgetsSourceListDragSourceIface* _iface_;
#line 93 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 93 "../src/Widgets/SourceList/SourceList.vala"
	_iface_ = CODE_WIDGETS_SOURCE_LIST_DRAG_SOURCE_GET_INTERFACE (self);
#line 93 "../src/Widgets/SourceList/SourceList.vala"
	if (_iface_->prepare_selection_data) {
#line 93 "../src/Widgets/SourceList/SourceList.vala"
		_iface_->prepare_selection_data (self, selection_data);
#line 1386 "SourceList.c"
	}
}

static void
code_widgets_source_list_drag_source_default_init (CodeWidgetsSourceListDragSourceIface * iface,
                                                   gpointer iface_data)
{
}

/**
 * An interface for dragging items out of the source list widget.
 *
 * @since 0.3
 */
static GType
code_widgets_source_list_drag_source_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (CodeWidgetsSourceListDragSourceIface), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) code_widgets_source_list_drag_source_default_init, (GClassFinalizeFunc) NULL, NULL, 0, 0, (GInstanceInitFunc) NULL, NULL };
	GType code_widgets_source_list_drag_source_type_id;
	code_widgets_source_list_drag_source_type_id = g_type_register_static (G_TYPE_INTERFACE, "CodeWidgetsSourceListDragSource", &g_define_type_info, 0);
	g_type_interface_add_prerequisite (code_widgets_source_list_drag_source_type_id, CODE_WIDGETS_SOURCE_LIST_TYPE_ITEM);
	return code_widgets_source_list_drag_source_type_id;
}

GType
code_widgets_source_list_drag_source_get_type (void)
{
	static volatile gsize code_widgets_source_list_drag_source_type_id__once = 0;
	if (g_once_init_enter (&code_widgets_source_list_drag_source_type_id__once)) {
		GType code_widgets_source_list_drag_source_type_id;
		code_widgets_source_list_drag_source_type_id = code_widgets_source_list_drag_source_get_type_once ();
		g_once_init_leave (&code_widgets_source_list_drag_source_type_id__once, code_widgets_source_list_drag_source_type_id);
	}
	return code_widgets_source_list_drag_source_type_id__once;
}

/**
     * Determines whether //data// can be dropped into this item.
     *
     * @param context The drag context.
     * @param data {@link Gtk.SelectionData} containing source data.
     * @return //true// if the drop is possible; //false// otherwise.
     * @since 0.3
     */
gboolean
code_widgets_source_list_drag_dest_data_drop_possible (CodeWidgetsSourceListDragDest* self,
                                                       GdkDragContext* context,
                                                       GtkSelectionData* data)
{
	CodeWidgetsSourceListDragDestIface* _iface_;
#line 110 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 110 "../src/Widgets/SourceList/SourceList.vala"
	_iface_ = CODE_WIDGETS_SOURCE_LIST_DRAG_DEST_GET_INTERFACE (self);
#line 110 "../src/Widgets/SourceList/SourceList.vala"
	if (_iface_->data_drop_possible) {
#line 110 "../src/Widgets/SourceList/SourceList.vala"
		return _iface_->data_drop_possible (self, context, data);
#line 1445 "SourceList.c"
	}
#line 110 "../src/Widgets/SourceList/SourceList.vala"
	return FALSE;
#line 1449 "SourceList.c"
}

/**
     * If a data drop is deemed possible, then this method is called
     * when the data is actually dropped into this item. Any actions
     * consequence of the data received should be handled here.
     *
     * @param context The drag context.
     * @param data {@link Gtk.SelectionData} containing source data.
     * @return The action taken, or //0// to indicate that the dropped data was not accepted.
     * @since 0.3
     */
GdkDragAction
code_widgets_source_list_drag_dest_data_received (CodeWidgetsSourceListDragDest* self,
                                                  GdkDragContext* context,
                                                  GtkSelectionData* data)
{
	CodeWidgetsSourceListDragDestIface* _iface_;
#line 122 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, 0U);
#line 122 "../src/Widgets/SourceList/SourceList.vala"
	_iface_ = CODE_WIDGETS_SOURCE_LIST_DRAG_DEST_GET_INTERFACE (self);
#line 122 "../src/Widgets/SourceList/SourceList.vala"
	if (_iface_->data_received) {
#line 122 "../src/Widgets/SourceList/SourceList.vala"
		return _iface_->data_received (self, context, data);
#line 1476 "SourceList.c"
	}
#line 122 "../src/Widgets/SourceList/SourceList.vala"
	return 0U;
#line 1480 "SourceList.c"
}

static void
code_widgets_source_list_drag_dest_default_init (CodeWidgetsSourceListDragDestIface * iface,
                                                 gpointer iface_data)
{
}

/**
 * An interface for receiving data from other widgets via drag-and-drop.
 *
 * @since 0.3
 */
static GType
code_widgets_source_list_drag_dest_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (CodeWidgetsSourceListDragDestIface), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) code_widgets_source_list_drag_dest_default_init, (GClassFinalizeFunc) NULL, NULL, 0, 0, (GInstanceInitFunc) NULL, NULL };
	GType code_widgets_source_list_drag_dest_type_id;
	code_widgets_source_list_drag_dest_type_id = g_type_register_static (G_TYPE_INTERFACE, "CodeWidgetsSourceListDragDest", &g_define_type_info, 0);
	g_type_interface_add_prerequisite (code_widgets_source_list_drag_dest_type_id, CODE_WIDGETS_SOURCE_LIST_TYPE_ITEM);
	return code_widgets_source_list_drag_dest_type_id;
}

GType
code_widgets_source_list_drag_dest_get_type (void)
{
	static volatile gsize code_widgets_source_list_drag_dest_type_id__once = 0;
	if (g_once_init_enter (&code_widgets_source_list_drag_dest_type_id__once)) {
		GType code_widgets_source_list_drag_dest_type_id;
		code_widgets_source_list_drag_dest_type_id = code_widgets_source_list_drag_dest_get_type_once ();
		g_once_init_leave (&code_widgets_source_list_drag_dest_type_id__once, code_widgets_source_list_drag_dest_type_id);
	}
	return code_widgets_source_list_drag_dest_type_id__once;
}

static inline gpointer
code_widgets_source_list_get_instance_private (CodeWidgetsSourceList* self)
{
	return G_STRUCT_MEMBER_P (self, CodeWidgetsSourceList_private_offset);
}

/**
     * Creates a new {@link Code.Widgets.SourceList}.
     *
     * @return A new {@link Code.Widgets.SourceList}.
     * @since 0.2
     */
CodeWidgetsSourceList*
code_widgets_source_list_construct (GType object_type,
                                    CodeWidgetsSourceListExpandableItem* root)
{
	CodeWidgetsSourceList * self = NULL;
#line 2658 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (root != NULL, NULL);
#line 2658 "../src/Widgets/SourceList/SourceList.vala"
	self = (CodeWidgetsSourceList*) g_object_new (object_type, NULL);
#line 2659 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_set_root (self, root);
#line 2658 "../src/Widgets/SourceList/SourceList.vala"
	return self;
#line 1541 "SourceList.c"
}

CodeWidgetsSourceList*
code_widgets_source_list_new (CodeWidgetsSourceListExpandableItem* root)
{
#line 2658 "../src/Widgets/SourceList/SourceList.vala"
	return code_widgets_source_list_construct (CODE_WIDGETS_TYPE_SOURCE_LIST, root);
#line 1549 "SourceList.c"
}

/**
     * Checks whether //item// is part of the source list.
     *
     * @param item The item to query.
     * @return //true// if the item belongs to the source list; //false// otherwise.
     * @since 0.2
     */
gboolean
code_widgets_source_list_has_item (CodeWidgetsSourceList* self,
                                   CodeWidgetsSourceListItem* item)
{
	CodeWidgetsSourceListDataModel* _tmp0_;
	gboolean result;
#line 2679 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 2679 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (item != NULL, FALSE);
#line 2680 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->data_model;
#line 2680 "../src/Widgets/SourceList/SourceList.vala"
	result = code_widgets_source_list_data_model_has_item (_tmp0_, item);
#line 2680 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 1575 "SourceList.c"
}

/**
     * Sets the method used for filtering out items.
     *
     * @param visible_func The method to use for filtering items.
     * @param refilter Whether to call {@link Code.Widgets.SourceList.refilter} using the new function.
     * @see Code.Widgets.SourceList.VisibleFunc
     * @see Code.Widgets.SourceList.refilter
     * @since 0.2
     */
void
code_widgets_source_list_set_filter_func (CodeWidgetsSourceList* self,
                                          CodeWidgetsSourceListVisibleFunc visible_func,
                                          gpointer visible_func_target,
                                          gboolean refilter)
{
	CodeWidgetsSourceListDataModel* _tmp0_;
#line 2692 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 2693 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->data_model;
#line 2693 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_data_model_set_filter_func (_tmp0_, visible_func, visible_func_target);
#line 2694 "../src/Widgets/SourceList/SourceList.vala"
	if (refilter) {
#line 2695 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_refilter (self);
#line 1604 "SourceList.c"
	}
}

/**
     * Applies the filter method set by {@link Code.Widgets.SourceList.set_filter_func}
     * to all the items that are part of the current tree.
     *
     * @see Code.Widgets.SourceList.VisibleFunc
     * @see Code.Widgets.SourceList.set_filter_func
     * @since 0.2
     */
void
code_widgets_source_list_refilter (CodeWidgetsSourceList* self)
{
	CodeWidgetsSourceListDataModel* _tmp0_;
#line 2706 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 2707 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->data_model;
#line 2707 "../src/Widgets/SourceList/SourceList.vala"
	gtk_tree_model_filter_refilter ((GtkTreeModelFilter*) _tmp0_);
#line 1626 "SourceList.c"
}

/**
     * Queries the actual expansion state of //item//.
     *
     * @see Code.Widgets.SourceList.ExpandableItem.expanded
     * @return Whether //item// is expanded or not.
     * @since 0.2
     */
static void
_vala_GtkTreePath_free (GtkTreePath* self)
{
#line 2719 "../src/Widgets/SourceList/SourceList.vala"
	g_boxed_free (gtk_tree_path_get_type (), self);
#line 1641 "SourceList.c"
}

gboolean
code_widgets_source_list_is_item_expanded (CodeWidgetsSourceList* self,
                                           CodeWidgetsSourceListItem* item)
{
	GtkTreePath* path = NULL;
	CodeWidgetsSourceListDataModel* _tmp0_;
	GtkTreePath* _tmp1_;
	gboolean _tmp2_ = FALSE;
	GtkTreePath* _tmp3_;
	gboolean result;
#line 2717 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 2717 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (item != NULL, FALSE);
#line 2717 "../src/Widgets/SourceList/SourceList.vala"
	_vala_return_val_if_fail (code_widgets_source_list_has_item (self, item), "has_item (item)", FALSE);
#line 2718 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->data_model;
#line 2718 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = code_widgets_source_list_data_model_get_item_path (_tmp0_, item);
#line 2718 "../src/Widgets/SourceList/SourceList.vala"
	path = _tmp1_;
#line 2719 "../src/Widgets/SourceList/SourceList.vala"
	_tmp3_ = path;
#line 2719 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp3_ != NULL) {
#line 1670 "SourceList.c"
		CodeWidgetsSourceListTree* _tmp4_;
		GtkTreePath* _tmp5_;
#line 2719 "../src/Widgets/SourceList/SourceList.vala"
		_tmp4_ = self->priv->tree;
#line 2719 "../src/Widgets/SourceList/SourceList.vala"
		_tmp5_ = path;
#line 2719 "../src/Widgets/SourceList/SourceList.vala"
		_tmp2_ = gtk_tree_view_row_expanded ((GtkTreeView*) _tmp4_, _tmp5_);
#line 1679 "SourceList.c"
	} else {
#line 2719 "../src/Widgets/SourceList/SourceList.vala"
		_tmp2_ = FALSE;
#line 1683 "SourceList.c"
	}
#line 2719 "../src/Widgets/SourceList/SourceList.vala"
	result = _tmp2_;
#line 2719 "../src/Widgets/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (path);
#line 2719 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 1691 "SourceList.c"
}

/**
     * If //item// is editable, this activates the editor; otherwise, it does nothing.
     * If an item was already being edited, this will fail.
     *
     * @param item Item to edit.
     * @see Code.Widgets.SourceList.Item.editable
     * @see Code.Widgets.SourceList.editing
     * @see Code.Widgets.SourceList.stop_editing
     * @return true if the editing started successfully; false otherwise.
     * @since 0.2
     */
gboolean
code_widgets_source_list_start_editing_item (CodeWidgetsSourceList* self,
                                             CodeWidgetsSourceListItem* item)
{
	CodeWidgetsSourceListTree* _tmp0_;
	gboolean result;
#line 2733 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 2733 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (item != NULL, FALSE);
#line 2733 "../src/Widgets/SourceList/SourceList.vala"
	_vala_return_val_if_fail (code_widgets_source_list_has_item (self, item), "has_item (item)", FALSE);
#line 2734 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->tree;
#line 2734 "../src/Widgets/SourceList/SourceList.vala"
	result = code_widgets_source_list_tree_start_editing_item (_tmp0_, item);
#line 2734 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 1723 "SourceList.c"
}

/**
     * Cancels any editing operation going on.
     *
     * @see Code.Widgets.SourceList.editing
     * @see Code.Widgets.SourceList.start_editing_item
     * @since 0.2
     */
void
code_widgets_source_list_stop_editing (CodeWidgetsSourceList* self)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
#line 2744 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 2745 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = code_widgets_source_list_get_editing (self);
#line 2745 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = _tmp0_;
#line 2745 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp1_) {
#line 1746 "SourceList.c"
		CodeWidgetsSourceListTree* _tmp2_;
#line 2746 "../src/Widgets/SourceList/SourceList.vala"
		_tmp2_ = self->priv->tree;
#line 2746 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_tree_stop_editing (_tmp2_);
#line 1752 "SourceList.c"
	}
}

/**
     * Turns Source List into a //drag source//.
     *
     * This enables items that implement {@link Code.Widgets.SourceListDragSource}
     * to be dragged outside the Source List and drop data into external widgets.
     *
     * @param src_entries an array of {@link Gtk.TargetEntry}s indicating the targets
     * that the drag will support.
     * @see Code.Widgets.SourceListDragSource
     * @see Code.Widgets.SourceList.disable_drag_source
     * @since 0.3
     */
void
code_widgets_source_list_enable_drag_source (CodeWidgetsSourceList* self,
                                             GtkTargetEntry* src_entries,
                                             gint src_entries_length1)
{
	CodeWidgetsSourceListTree* _tmp0_;
#line 2761 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 2762 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->tree;
#line 2762 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_tree_configure_drag_source (_tmp0_, src_entries, (gint) src_entries_length1);
#line 1780 "SourceList.c"
}

/**
     * Undoes the effect of {@link Code.Widgets.SourceList.enable_drag_source}
     *
     * @see Code.Widgets.SourceList.enable_drag_source
     * @since 0.3
     */
void
code_widgets_source_list_disable_drag_source (CodeWidgetsSourceList* self)
{
	CodeWidgetsSourceListTree* _tmp0_;
#line 2771 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 2772 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->tree;
#line 2772 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_tree_configure_drag_source (_tmp0_, NULL, (gint) 0);
#line 1799 "SourceList.c"
}

/**
     * Turns Source List into a //drop destination//.
     *
     * This enables items that implement {@link Code.Widgets.SourceListDragDest}
     * to receive data from external widgets via drag-and-drop.
     *
     * @param dest_entries an array of {@link Gtk.TargetEntry}s indicating the drop
     * types that Source List items will accept.
     * @param actions a bitmask of possible actions for a drop onto Source List items.
     * @see Code.Widgets.SourceListDragDest
     * @see Code.Widgets.SourceList.disable_drag_dest
     * @since 0.3
     */
void
code_widgets_source_list_enable_drag_dest (CodeWidgetsSourceList* self,
                                           GtkTargetEntry* dest_entries,
                                           gint dest_entries_length1,
                                           GdkDragAction actions)
{
	CodeWidgetsSourceListTree* _tmp0_;
#line 2788 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 2789 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->tree;
#line 2789 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_tree_configure_drag_dest (_tmp0_, dest_entries, (gint) dest_entries_length1, actions);
#line 1828 "SourceList.c"
}

/**
     * Undoes the effect of {@link Code.Widgets.SourceList.enable_drag_dest}
     *
     * @see Code.Widgets.SourceList.enable_drag_dest
     * @since 0.3
     */
void
code_widgets_source_list_disable_drag_dest (CodeWidgetsSourceList* self)
{
	CodeWidgetsSourceListTree* _tmp0_;
#line 2798 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 2799 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->tree;
#line 2799 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_tree_configure_drag_dest (_tmp0_, NULL, (gint) 0, 0);
#line 1847 "SourceList.c"
}

/**
     * Scrolls the source list tree to make //item// visible.
     *
     * {@link Code.Widgets.SourceList.ExpandableItem.expand_with_parents} is called
     * for the item's parent if //expand_parents// is //true//, to make sure it's not
     * hidden behind a collapsed row.
     *
     * If use_align is //false//, then the row_align argument is ignored, and the tree
     * does the minimum amount of work to scroll the item onto the screen. This means that
     * the item will be scrolled to the edge closest to its current position. If the item
     * is currently visible on the screen, nothing is done.
     *
     * @param item Item to scroll to.
     * @param expand_parents Whether to recursively expand item's parent in case they are collapsed.
     * @param use_align Whether to use the //row_align// argument.
     * @param row_align The vertical alignment of //item//. 0.0 means top, 0.5 center, and 1.0 bottom.
     * @return //true// if successful; //false// otherwise.
     * @since 0.2
     */
gboolean
code_widgets_source_list_scroll_to_item (CodeWidgetsSourceList* self,
                                         CodeWidgetsSourceListItem* item,
                                         gboolean expand_parents,
                                         gboolean use_align,
                                         gfloat row_align)
{
	gboolean _tmp0_ = FALSE;
	CodeWidgetsSourceListTree* _tmp5_;
	gboolean result;
#line 2821 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 2821 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (item != NULL, FALSE);
#line 2821 "../src/Widgets/SourceList/SourceList.vala"
	_vala_return_val_if_fail (code_widgets_source_list_has_item (self, item), "has_item (item)", FALSE);
#line 2827 "../src/Widgets/SourceList/SourceList.vala"
	if (expand_parents) {
#line 1887 "SourceList.c"
		CodeWidgetsSourceListExpandableItem* _tmp1_;
		CodeWidgetsSourceListExpandableItem* _tmp2_;
#line 2827 "../src/Widgets/SourceList/SourceList.vala"
		_tmp1_ = code_widgets_source_list_item_get_parent (item);
#line 2827 "../src/Widgets/SourceList/SourceList.vala"
		_tmp2_ = _tmp1_;
#line 2827 "../src/Widgets/SourceList/SourceList.vala"
		_tmp0_ = _tmp2_ != NULL;
#line 1896 "SourceList.c"
	} else {
#line 2827 "../src/Widgets/SourceList/SourceList.vala"
		_tmp0_ = FALSE;
#line 1900 "SourceList.c"
	}
#line 2827 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp0_) {
#line 1904 "SourceList.c"
		CodeWidgetsSourceListExpandableItem* _tmp3_;
		CodeWidgetsSourceListExpandableItem* _tmp4_;
#line 2828 "../src/Widgets/SourceList/SourceList.vala"
		_tmp3_ = code_widgets_source_list_item_get_parent (item);
#line 2828 "../src/Widgets/SourceList/SourceList.vala"
		_tmp4_ = _tmp3_;
#line 2828 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_expandable_item_expand_with_parents (_tmp4_);
#line 1913 "SourceList.c"
	}
#line 2830 "../src/Widgets/SourceList/SourceList.vala"
	_tmp5_ = self->priv->tree;
#line 2830 "../src/Widgets/SourceList/SourceList.vala"
	result = code_widgets_source_list_tree_scroll_to_item (_tmp5_, item, use_align, row_align);
#line 2830 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 1921 "SourceList.c"
}

/**
     * Gets the previous item with respect to //reference//.
     *
     * @param reference Item to use as reference.
     * @return The item that appears before //reference//, or //null// if there's none.
     * @since 0.2
     */
static void
_vala_GtkTreeIter_free (GtkTreeIter* self)
{
#line 2846 "../src/Widgets/SourceList/SourceList.vala"
	g_boxed_free (gtk_tree_iter_get_type (), self);
#line 1936 "SourceList.c"
}

CodeWidgetsSourceListItem*
code_widgets_source_list_get_previous_item (CodeWidgetsSourceList* self,
                                            CodeWidgetsSourceListItem* reference)
{
	GtkTreeIter* iter = NULL;
	CodeWidgetsSourceListDataModel* _tmp0_;
	GtkTreeIter* _tmp1_;
	GtkTreeIter* _tmp2_;
	CodeWidgetsSourceListItem* result;
#line 2840 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 2840 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (reference != NULL, NULL);
#line 2840 "../src/Widgets/SourceList/SourceList.vala"
	_vala_return_val_if_fail (code_widgets_source_list_has_item (self, reference), "has_item (reference)", NULL);
#line 2842 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->data_model;
#line 2842 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = code_widgets_source_list_data_model_get_item_iter (_tmp0_, reference);
#line 2842 "../src/Widgets/SourceList/SourceList.vala"
	iter = _tmp1_;
#line 2843 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = iter;
#line 2843 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp2_ != NULL) {
#line 1964 "SourceList.c"
		GtkTreeIter new_iter = {0};
		GtkTreeIter* _tmp3_;
		CodeWidgetsSourceListDataModel* _tmp4_;
		gboolean _tmp5_;
#line 2844 "../src/Widgets/SourceList/SourceList.vala"
		_tmp3_ = iter;
#line 2844 "../src/Widgets/SourceList/SourceList.vala"
		new_iter = *_tmp3_;
#line 2845 "../src/Widgets/SourceList/SourceList.vala"
		_tmp4_ = self->priv->data_model;
#line 2845 "../src/Widgets/SourceList/SourceList.vala"
		_tmp5_ = gtk_tree_model_iter_previous ((GtkTreeModel*) _tmp4_, &new_iter);
#line 2845 "../src/Widgets/SourceList/SourceList.vala"
		if (_tmp5_) {
#line 1979 "SourceList.c"
			CodeWidgetsSourceListDataModel* _tmp6_;
			GtkTreeIter _tmp7_;
			CodeWidgetsSourceListItem* _tmp8_;
#line 2846 "../src/Widgets/SourceList/SourceList.vala"
			_tmp6_ = self->priv->data_model;
#line 2846 "../src/Widgets/SourceList/SourceList.vala"
			_tmp7_ = new_iter;
#line 2846 "../src/Widgets/SourceList/SourceList.vala"
			_tmp8_ = code_widgets_source_list_data_model_get_item (_tmp6_, &_tmp7_);
#line 2846 "../src/Widgets/SourceList/SourceList.vala"
			result = _tmp8_;
#line 2846 "../src/Widgets/SourceList/SourceList.vala"
			__vala_GtkTreeIter_free0 (iter);
#line 2846 "../src/Widgets/SourceList/SourceList.vala"
			return result;
#line 1995 "SourceList.c"
		}
	}
#line 2849 "../src/Widgets/SourceList/SourceList.vala"
	result = NULL;
#line 2849 "../src/Widgets/SourceList/SourceList.vala"
	__vala_GtkTreeIter_free0 (iter);
#line 2849 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 2004 "SourceList.c"
}

/**
     * Gets the next item with respect to //reference//.
     *
     * @param reference Item to use as reference.
     * @return The item that appears after //reference//, or //null// if there's none.
     * @since 0.2
     */
CodeWidgetsSourceListItem*
code_widgets_source_list_get_next_item (CodeWidgetsSourceList* self,
                                        CodeWidgetsSourceListItem* reference)
{
	GtkTreeIter* iter = NULL;
	CodeWidgetsSourceListDataModel* _tmp0_;
	GtkTreeIter* _tmp1_;
	GtkTreeIter* _tmp2_;
	CodeWidgetsSourceListItem* result;
#line 2859 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 2859 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (reference != NULL, NULL);
#line 2859 "../src/Widgets/SourceList/SourceList.vala"
	_vala_return_val_if_fail (code_widgets_source_list_has_item (self, reference), "has_item (reference)", NULL);
#line 2861 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->data_model;
#line 2861 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = code_widgets_source_list_data_model_get_item_iter (_tmp0_, reference);
#line 2861 "../src/Widgets/SourceList/SourceList.vala"
	iter = _tmp1_;
#line 2862 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = iter;
#line 2862 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp2_ != NULL) {
#line 2039 "SourceList.c"
		GtkTreeIter new_iter = {0};
		GtkTreeIter* _tmp3_;
		CodeWidgetsSourceListDataModel* _tmp4_;
		gboolean _tmp5_;
#line 2863 "../src/Widgets/SourceList/SourceList.vala"
		_tmp3_ = iter;
#line 2863 "../src/Widgets/SourceList/SourceList.vala"
		new_iter = *_tmp3_;
#line 2864 "../src/Widgets/SourceList/SourceList.vala"
		_tmp4_ = self->priv->data_model;
#line 2864 "../src/Widgets/SourceList/SourceList.vala"
		_tmp5_ = gtk_tree_model_iter_next ((GtkTreeModel*) _tmp4_, &new_iter);
#line 2864 "../src/Widgets/SourceList/SourceList.vala"
		if (_tmp5_) {
#line 2054 "SourceList.c"
			CodeWidgetsSourceListDataModel* _tmp6_;
			GtkTreeIter _tmp7_;
			CodeWidgetsSourceListItem* _tmp8_;
#line 2865 "../src/Widgets/SourceList/SourceList.vala"
			_tmp6_ = self->priv->data_model;
#line 2865 "../src/Widgets/SourceList/SourceList.vala"
			_tmp7_ = new_iter;
#line 2865 "../src/Widgets/SourceList/SourceList.vala"
			_tmp8_ = code_widgets_source_list_data_model_get_item (_tmp6_, &_tmp7_);
#line 2865 "../src/Widgets/SourceList/SourceList.vala"
			result = _tmp8_;
#line 2865 "../src/Widgets/SourceList/SourceList.vala"
			__vala_GtkTreeIter_free0 (iter);
#line 2865 "../src/Widgets/SourceList/SourceList.vala"
			return result;
#line 2070 "SourceList.c"
		}
	}
#line 2868 "../src/Widgets/SourceList/SourceList.vala"
	result = NULL;
#line 2868 "../src/Widgets/SourceList/SourceList.vala"
	__vala_GtkTreeIter_free0 (iter);
#line 2868 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 2079 "SourceList.c"
}

/**
     * Gets the first visible child of an expandable item.
     *
     * @param parent Parent of the child to look up.
     * @return The first visible child of //parent//, or null if it was not found.
     * @since 0.2
     */
CodeWidgetsSourceListItem*
code_widgets_source_list_get_first_child (CodeWidgetsSourceList* self,
                                          CodeWidgetsSourceListExpandableItem* parent)
{
	CodeWidgetsSourceListItem* _tmp0_;
	CodeWidgetsSourceListItem* result;
#line 2878 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 2878 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (parent != NULL, NULL);
#line 2879 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = code_widgets_source_list_get_nth_child (self, parent, 0);
#line 2879 "../src/Widgets/SourceList/SourceList.vala"
	result = _tmp0_;
#line 2879 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 2105 "SourceList.c"
}

/**
     * Gets the last visible child of an expandable item.
     *
     * @param parent Parent of the child to look up.
     * @return The last visible child of //parent//, or null if it was not found.
     * @since 0.2
     */
CodeWidgetsSourceListItem*
code_widgets_source_list_get_last_child (CodeWidgetsSourceList* self,
                                         CodeWidgetsSourceListExpandableItem* parent)
{
	CodeWidgetsSourceListItem* _tmp0_;
	CodeWidgetsSourceListItem* result;
#line 2889 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 2889 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (parent != NULL, NULL);
#line 2890 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = code_widgets_source_list_get_nth_child (self, parent, ((gint) code_widgets_source_list_get_n_visible_children (self, parent)) - 1);
#line 2890 "../src/Widgets/SourceList/SourceList.vala"
	result = _tmp0_;
#line 2890 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 2131 "SourceList.c"
}

/**
     * Gets the number of visible children of an expandable item.
     *
     * @param parent Item to query.
     * @return Number of visible children of //parent//.
     * @since 0.2
     */
guint
code_widgets_source_list_get_n_visible_children (CodeWidgetsSourceList* self,
                                                 CodeWidgetsSourceListExpandableItem* parent)
{
	GtkTreeIter* parent_iter = NULL;
	CodeWidgetsSourceListDataModel* _tmp0_;
	GtkTreeIter* _tmp1_;
	CodeWidgetsSourceListDataModel* _tmp2_;
	guint result;
#line 2900 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, 0U);
#line 2900 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (parent != NULL, 0U);
#line 2902 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->data_model;
#line 2902 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = code_widgets_source_list_data_model_get_item_iter (_tmp0_, (CodeWidgetsSourceListItem*) parent);
#line 2902 "../src/Widgets/SourceList/SourceList.vala"
	parent_iter = _tmp1_;
#line 2903 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = self->priv->data_model;
#line 2903 "../src/Widgets/SourceList/SourceList.vala"
	result = (guint) gtk_tree_model_iter_n_children ((GtkTreeModel*) _tmp2_, parent_iter);
#line 2903 "../src/Widgets/SourceList/SourceList.vala"
	__vala_GtkTreeIter_free0 (parent_iter);
#line 2903 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 2168 "SourceList.c"
}

static CodeWidgetsSourceListItem*
code_widgets_source_list_get_nth_child (CodeWidgetsSourceList* self,
                                        CodeWidgetsSourceListExpandableItem* parent,
                                        gint index)
{
	GtkTreeIter* parent_iter = NULL;
	CodeWidgetsSourceListDataModel* _tmp0_;
	GtkTreeIter* _tmp1_;
	GtkTreeIter child_iter = {0};
	CodeWidgetsSourceListDataModel* _tmp2_;
	GtkTreeIter* _tmp3_;
	GtkTreeIter _tmp4_ = {0};
	gboolean _tmp5_;
	CodeWidgetsSourceListItem* result;
#line 2906 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 2906 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (parent != NULL, NULL);
#line 2907 "../src/Widgets/SourceList/SourceList.vala"
	if (index < 0) {
#line 2908 "../src/Widgets/SourceList/SourceList.vala"
		result = NULL;
#line 2908 "../src/Widgets/SourceList/SourceList.vala"
		return result;
#line 2195 "SourceList.c"
	}
#line 2911 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->data_model;
#line 2911 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = code_widgets_source_list_data_model_get_item_iter (_tmp0_, (CodeWidgetsSourceListItem*) parent);
#line 2911 "../src/Widgets/SourceList/SourceList.vala"
	parent_iter = _tmp1_;
#line 2914 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = self->priv->data_model;
#line 2914 "../src/Widgets/SourceList/SourceList.vala"
	_tmp3_ = parent_iter;
#line 2914 "../src/Widgets/SourceList/SourceList.vala"
	_tmp5_ = gtk_tree_model_iter_nth_child ((GtkTreeModel*) _tmp2_, &_tmp4_, _tmp3_, index);
#line 2914 "../src/Widgets/SourceList/SourceList.vala"
	child_iter = _tmp4_;
#line 2914 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp5_) {
#line 2213 "SourceList.c"
		CodeWidgetsSourceListDataModel* _tmp6_;
		GtkTreeIter _tmp7_;
		CodeWidgetsSourceListItem* _tmp8_;
#line 2915 "../src/Widgets/SourceList/SourceList.vala"
		_tmp6_ = self->priv->data_model;
#line 2915 "../src/Widgets/SourceList/SourceList.vala"
		_tmp7_ = child_iter;
#line 2915 "../src/Widgets/SourceList/SourceList.vala"
		_tmp8_ = code_widgets_source_list_data_model_get_item (_tmp6_, &_tmp7_);
#line 2915 "../src/Widgets/SourceList/SourceList.vala"
		result = _tmp8_;
#line 2915 "../src/Widgets/SourceList/SourceList.vala"
		__vala_GtkTreeIter_free0 (parent_iter);
#line 2915 "../src/Widgets/SourceList/SourceList.vala"
		return result;
#line 2229 "SourceList.c"
	}
#line 2917 "../src/Widgets/SourceList/SourceList.vala"
	result = NULL;
#line 2917 "../src/Widgets/SourceList/SourceList.vala"
	__vala_GtkTreeIter_free0 (parent_iter);
#line 2917 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 2237 "SourceList.c"
}

CodeWidgetsSourceListExpandableItem*
code_widgets_source_list_get_root (CodeWidgetsSourceList* self)
{
	CodeWidgetsSourceListExpandableItem* result;
	CodeWidgetsSourceListDataModel* _tmp0_;
	CodeWidgetsSourceListExpandableItem* _tmp1_;
	CodeWidgetsSourceListExpandableItem* _tmp2_;
#line 2607 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 2607 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->data_model;
#line 2607 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = code_widgets_source_list_data_model_get_root (_tmp0_);
#line 2607 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = _tmp1_;
#line 2607 "../src/Widgets/SourceList/SourceList.vala"
	result = _tmp2_;
#line 2607 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 2259 "SourceList.c"
}

void
code_widgets_source_list_set_root (CodeWidgetsSourceList* self,
                                   CodeWidgetsSourceListExpandableItem* value)
{
	CodeWidgetsSourceListDataModel* _tmp0_;
#line 2608 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 2608 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->data_model;
#line 2608 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_data_model_set_root (_tmp0_, value);
#line 2608 "../src/Widgets/SourceList/SourceList.vala"
	g_object_notify_by_pspec ((GObject *) self, code_widgets_source_list_properties[CODE_WIDGETS_SOURCE_LIST_ROOT_PROPERTY]);
#line 2275 "SourceList.c"
}

CodeWidgetsSourceListItem*
code_widgets_source_list_get_selected (CodeWidgetsSourceList* self)
{
	CodeWidgetsSourceListItem* result;
	CodeWidgetsSourceListTree* _tmp0_;
	CodeWidgetsSourceListItem* _tmp1_;
	CodeWidgetsSourceListItem* _tmp2_;
#line 2621 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 2621 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->tree;
#line 2621 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = code_widgets_source_list_tree_get_selected_item (_tmp0_);
#line 2621 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = _tmp1_;
#line 2621 "../src/Widgets/SourceList/SourceList.vala"
	result = _tmp2_;
#line 2621 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 2297 "SourceList.c"
}

void
code_widgets_source_list_set_selected (CodeWidgetsSourceList* self,
                                       CodeWidgetsSourceListItem* value)
{
	gboolean _tmp0_ = FALSE;
	CodeWidgetsSourceListTree* _tmp5_;
#line 2622 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 2623 "../src/Widgets/SourceList/SourceList.vala"
	if (value != NULL) {
#line 2310 "SourceList.c"
		CodeWidgetsSourceListExpandableItem* _tmp1_;
		CodeWidgetsSourceListExpandableItem* _tmp2_;
#line 2623 "../src/Widgets/SourceList/SourceList.vala"
		_tmp1_ = code_widgets_source_list_item_get_parent (value);
#line 2623 "../src/Widgets/SourceList/SourceList.vala"
		_tmp2_ = _tmp1_;
#line 2623 "../src/Widgets/SourceList/SourceList.vala"
		_tmp0_ = _tmp2_ != NULL;
#line 2319 "SourceList.c"
	} else {
#line 2623 "../src/Widgets/SourceList/SourceList.vala"
		_tmp0_ = FALSE;
#line 2323 "SourceList.c"
	}
#line 2623 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp0_) {
#line 2327 "SourceList.c"
		CodeWidgetsSourceListExpandableItem* _tmp3_;
		CodeWidgetsSourceListExpandableItem* _tmp4_;
#line 2624 "../src/Widgets/SourceList/SourceList.vala"
		_tmp3_ = code_widgets_source_list_item_get_parent (value);
#line 2624 "../src/Widgets/SourceList/SourceList.vala"
		_tmp4_ = _tmp3_;
#line 2624 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_expandable_item_expand_with_parents (_tmp4_);
#line 2336 "SourceList.c"
	}
#line 2625 "../src/Widgets/SourceList/SourceList.vala"
	_tmp5_ = self->priv->tree;
#line 2625 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_tree_set_selected_item (_tmp5_, value);
#line 2622 "../src/Widgets/SourceList/SourceList.vala"
	g_object_notify_by_pspec ((GObject *) self, code_widgets_source_list_properties[CODE_WIDGETS_SOURCE_LIST_SELECTED_PROPERTY]);
#line 2344 "SourceList.c"
}

PangoEllipsizeMode
code_widgets_source_list_get_ellipsize_mode (CodeWidgetsSourceList* self)
{
	PangoEllipsizeMode result;
	CodeWidgetsSourceListTree* _tmp0_;
	PangoEllipsizeMode _tmp1_;
	PangoEllipsizeMode _tmp2_;
#line 2635 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 2635 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->tree;
#line 2635 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = code_widgets_source_list_tree_get_ellipsize_mode (_tmp0_);
#line 2635 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = _tmp1_;
#line 2635 "../src/Widgets/SourceList/SourceList.vala"
	result = _tmp2_;
#line 2635 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 2366 "SourceList.c"
}

void
code_widgets_source_list_set_ellipsize_mode (CodeWidgetsSourceList* self,
                                             PangoEllipsizeMode value)
{
	CodeWidgetsSourceListTree* _tmp0_;
#line 2636 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 2636 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->tree;
#line 2636 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_tree_set_ellipsize_mode (_tmp0_, value);
#line 2636 "../src/Widgets/SourceList/SourceList.vala"
	g_object_notify_by_pspec ((GObject *) self, code_widgets_source_list_properties[CODE_WIDGETS_SOURCE_LIST_ELLIPSIZE_MODE_PROPERTY]);
#line 2382 "SourceList.c"
}

gboolean
code_widgets_source_list_get_editing (CodeWidgetsSourceList* self)
{
	gboolean result;
	CodeWidgetsSourceListTree* _tmp0_;
	gboolean _tmp1_;
	gboolean _tmp2_;
#line 2646 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 2646 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->tree;
#line 2646 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = code_widgets_source_list_tree_get_editing (_tmp0_);
#line 2646 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = _tmp1_;
#line 2646 "../src/Widgets/SourceList/SourceList.vala"
	result = _tmp2_;
#line 2646 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 2404 "SourceList.c"
}

static void
code_widgets_source_list_real_item_selected (CodeWidgetsSourceList* self,
                                             CodeWidgetsSourceListItem* item)
{
}

static inline gpointer
code_widgets_source_list_item_get_instance_private (CodeWidgetsSourceListItem* self)
{
	return G_STRUCT_MEMBER_P (self, CodeWidgetsSourceListItem_private_offset);
}

/**
         * Creates a new {@link Code.Widgets.SourceList.Item}.
         *
         * @param name Name of the item.
         * @return (transfer full) A new {@link Code.Widgets.SourceList.Item}.
         * @since 0.2
         */
CodeWidgetsSourceListItem*
code_widgets_source_list_item_construct (GType object_type,
                                         const gchar* name)
{
	CodeWidgetsSourceListItem * self = NULL;
#line 414 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (name != NULL, NULL);
#line 414 "../src/Widgets/SourceList/SourceList.vala"
	self = (CodeWidgetsSourceListItem*) g_object_new (object_type, NULL);
#line 415 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_item_set_name (self, name);
#line 414 "../src/Widgets/SourceList/SourceList.vala"
	return self;
#line 2439 "SourceList.c"
}

CodeWidgetsSourceListItem*
code_widgets_source_list_item_new (const gchar* name)
{
#line 414 "../src/Widgets/SourceList/SourceList.vala"
	return code_widgets_source_list_item_construct (CODE_WIDGETS_SOURCE_LIST_TYPE_ITEM, name);
#line 2447 "SourceList.c"
}

/**
         * Invoked when the item is secondary-clicked or when the usual menu keys are pressed.
         *
         * Note that since Granite 5.0, right clicking on an item no longer selects/activates it, so
         * any context menu items should be actioned on the item instance rather than the selected item
         * in the SourceList
         *
         * @return A {@link Gtk.Menu} or //null// if nothing should be displayed.
         * @since 0.2
         */
static GtkMenu*
code_widgets_source_list_item_real_get_context_menu (CodeWidgetsSourceListItem* self)
{
	GtkMenu* result;
#line 429 "../src/Widgets/SourceList/SourceList.vala"
	result = NULL;
#line 429 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 2468 "SourceList.c"
}

GtkMenu*
code_widgets_source_list_item_get_context_menu (CodeWidgetsSourceListItem* self)
{
	CodeWidgetsSourceListItemClass* _klass_;
#line 428 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 428 "../src/Widgets/SourceList/SourceList.vala"
	_klass_ = CODE_WIDGETS_SOURCE_LIST_ITEM_GET_CLASS (self);
#line 428 "../src/Widgets/SourceList/SourceList.vala"
	if (_klass_->get_context_menu) {
#line 428 "../src/Widgets/SourceList/SourceList.vala"
		return _klass_->get_context_menu (self);
#line 2483 "SourceList.c"
	}
#line 428 "../src/Widgets/SourceList/SourceList.vala"
	return NULL;
#line 2487 "SourceList.c"
}

CodeWidgetsSourceListExpandableItem*
code_widgets_source_list_item_get_parent (CodeWidgetsSourceListItem* self)
{
	CodeWidgetsSourceListExpandableItem* result;
	CodeWidgetsSourceListExpandableItem* _tmp0_;
#line 306 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 306 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->_parent;
#line 306 "../src/Widgets/SourceList/SourceList.vala"
	result = _tmp0_;
#line 306 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 2503 "SourceList.c"
}

static gpointer
_g_object_ref0 (gpointer self)
{
#line 306 "../src/Widgets/SourceList/SourceList.vala"
	return self ? g_object_ref (self) : NULL;
#line 2511 "SourceList.c"
}

void
code_widgets_source_list_item_set_parent (CodeWidgetsSourceListItem* self,
                                          CodeWidgetsSourceListExpandableItem* value)
{
	CodeWidgetsSourceListExpandableItem* old_value;
#line 306 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 306 "../src/Widgets/SourceList/SourceList.vala"
	old_value = code_widgets_source_list_item_get_parent (self);
#line 306 "../src/Widgets/SourceList/SourceList.vala"
	if (old_value != value) {
#line 2525 "SourceList.c"
		CodeWidgetsSourceListExpandableItem* _tmp0_;
#line 306 "../src/Widgets/SourceList/SourceList.vala"
		_tmp0_ = _g_object_ref0 (value);
#line 306 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (self->priv->_parent);
#line 306 "../src/Widgets/SourceList/SourceList.vala"
		self->priv->_parent = _tmp0_;
#line 306 "../src/Widgets/SourceList/SourceList.vala"
		g_object_notify_by_pspec ((GObject *) self, code_widgets_source_list_item_properties[CODE_WIDGETS_SOURCE_LIST_ITEM_PARENT_PROPERTY]);
#line 2535 "SourceList.c"
	}
}

const gchar*
code_widgets_source_list_item_get_name (CodeWidgetsSourceListItem* self)
{
	const gchar* result;
	const gchar* _tmp0_;
#line 313 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 313 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->_name;
#line 313 "../src/Widgets/SourceList/SourceList.vala"
	result = _tmp0_;
#line 313 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 2552 "SourceList.c"
}

void
code_widgets_source_list_item_set_name (CodeWidgetsSourceListItem* self,
                                        const gchar* value)
{
	gchar* old_value;
#line 313 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 313 "../src/Widgets/SourceList/SourceList.vala"
	old_value = code_widgets_source_list_item_get_name (self);
#line 313 "../src/Widgets/SourceList/SourceList.vala"
	if (g_strcmp0 (value, old_value) != 0) {
#line 2566 "SourceList.c"
		gchar* _tmp0_;
#line 313 "../src/Widgets/SourceList/SourceList.vala"
		_tmp0_ = g_strdup (value);
#line 313 "../src/Widgets/SourceList/SourceList.vala"
		_g_free0 (self->priv->_name);
#line 313 "../src/Widgets/SourceList/SourceList.vala"
		self->priv->_name = _tmp0_;
#line 313 "../src/Widgets/SourceList/SourceList.vala"
		g_object_notify_by_pspec ((GObject *) self, code_widgets_source_list_item_properties[CODE_WIDGETS_SOURCE_LIST_ITEM_NAME_PROPERTY]);
#line 2576 "SourceList.c"
	}
}

const gchar*
code_widgets_source_list_item_get_tooltip (CodeWidgetsSourceListItem* self)
{
	const gchar* result;
	const gchar* _tmp0_;
#line 321 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 321 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->_tooltip;
#line 321 "../src/Widgets/SourceList/SourceList.vala"
	result = _tmp0_;
#line 321 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 2593 "SourceList.c"
}

void
code_widgets_source_list_item_set_tooltip (CodeWidgetsSourceListItem* self,
                                           const gchar* value)
{
	gchar* old_value;
#line 321 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 321 "../src/Widgets/SourceList/SourceList.vala"
	old_value = code_widgets_source_list_item_get_tooltip (self);
#line 321 "../src/Widgets/SourceList/SourceList.vala"
	if (g_strcmp0 (value, old_value) != 0) {
#line 2607 "SourceList.c"
		gchar* _tmp0_;
#line 321 "../src/Widgets/SourceList/SourceList.vala"
		_tmp0_ = g_strdup (value);
#line 321 "../src/Widgets/SourceList/SourceList.vala"
		_g_free0 (self->priv->_tooltip);
#line 321 "../src/Widgets/SourceList/SourceList.vala"
		self->priv->_tooltip = _tmp0_;
#line 321 "../src/Widgets/SourceList/SourceList.vala"
		g_object_notify_by_pspec ((GObject *) self, code_widgets_source_list_item_properties[CODE_WIDGETS_SOURCE_LIST_ITEM_TOOLTIP_PROPERTY]);
#line 2617 "SourceList.c"
	}
}

const gchar*
code_widgets_source_list_item_get_markup (CodeWidgetsSourceListItem* self)
{
	const gchar* result;
	const gchar* _tmp0_;
#line 334 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 334 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->_markup;
#line 334 "../src/Widgets/SourceList/SourceList.vala"
	result = _tmp0_;
#line 334 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 2634 "SourceList.c"
}

void
code_widgets_source_list_item_set_markup (CodeWidgetsSourceListItem* self,
                                          const gchar* value)
{
	gchar* old_value;
#line 334 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 334 "../src/Widgets/SourceList/SourceList.vala"
	old_value = code_widgets_source_list_item_get_markup (self);
#line 334 "../src/Widgets/SourceList/SourceList.vala"
	if (g_strcmp0 (value, old_value) != 0) {
#line 2648 "SourceList.c"
		gchar* _tmp0_;
#line 334 "../src/Widgets/SourceList/SourceList.vala"
		_tmp0_ = g_strdup (value);
#line 334 "../src/Widgets/SourceList/SourceList.vala"
		_g_free0 (self->priv->_markup);
#line 334 "../src/Widgets/SourceList/SourceList.vala"
		self->priv->_markup = _tmp0_;
#line 334 "../src/Widgets/SourceList/SourceList.vala"
		g_object_notify_by_pspec ((GObject *) self, code_widgets_source_list_item_properties[CODE_WIDGETS_SOURCE_LIST_ITEM_MARKUP_PROPERTY]);
#line 2658 "SourceList.c"
	}
}

const gchar*
code_widgets_source_list_item_get_badge (CodeWidgetsSourceListItem* self)
{
	const gchar* result;
	const gchar* _tmp0_;
#line 344 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 344 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->_badge;
#line 344 "../src/Widgets/SourceList/SourceList.vala"
	result = _tmp0_;
#line 344 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 2675 "SourceList.c"
}

void
code_widgets_source_list_item_set_badge (CodeWidgetsSourceListItem* self,
                                         const gchar* value)
{
	gchar* old_value;
#line 344 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 344 "../src/Widgets/SourceList/SourceList.vala"
	old_value = code_widgets_source_list_item_get_badge (self);
#line 344 "../src/Widgets/SourceList/SourceList.vala"
	if (g_strcmp0 (value, old_value) != 0) {
#line 2689 "SourceList.c"
		gchar* _tmp0_;
#line 344 "../src/Widgets/SourceList/SourceList.vala"
		_tmp0_ = g_strdup (value);
#line 344 "../src/Widgets/SourceList/SourceList.vala"
		_g_free0 (self->priv->_badge);
#line 344 "../src/Widgets/SourceList/SourceList.vala"
		self->priv->_badge = _tmp0_;
#line 344 "../src/Widgets/SourceList/SourceList.vala"
		g_object_notify_by_pspec ((GObject *) self, code_widgets_source_list_item_properties[CODE_WIDGETS_SOURCE_LIST_ITEM_BADGE_PROPERTY]);
#line 2699 "SourceList.c"
	}
}

gboolean
code_widgets_source_list_item_get_editable (CodeWidgetsSourceListItem* self)
{
	gboolean result;
#line 358 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 358 "../src/Widgets/SourceList/SourceList.vala"
	result = self->priv->_editable;
#line 358 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 2713 "SourceList.c"
}

void
code_widgets_source_list_item_set_editable (CodeWidgetsSourceListItem* self,
                                            gboolean value)
{
	gboolean old_value;
#line 358 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 358 "../src/Widgets/SourceList/SourceList.vala"
	old_value = code_widgets_source_list_item_get_editable (self);
#line 358 "../src/Widgets/SourceList/SourceList.vala"
	if (old_value != value) {
#line 358 "../src/Widgets/SourceList/SourceList.vala"
		self->priv->_editable = value;
#line 358 "../src/Widgets/SourceList/SourceList.vala"
		g_object_notify_by_pspec ((GObject *) self, code_widgets_source_list_item_properties[CODE_WIDGETS_SOURCE_LIST_ITEM_EDITABLE_PROPERTY]);
#line 2731 "SourceList.c"
	}
}

gboolean
code_widgets_source_list_item_get_visible (CodeWidgetsSourceListItem* self)
{
	gboolean result;
#line 365 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 365 "../src/Widgets/SourceList/SourceList.vala"
	result = self->priv->_visible;
#line 365 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 2745 "SourceList.c"
}

void
code_widgets_source_list_item_set_visible (CodeWidgetsSourceListItem* self,
                                           gboolean value)
{
	gboolean old_value;
#line 365 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 365 "../src/Widgets/SourceList/SourceList.vala"
	old_value = code_widgets_source_list_item_get_visible (self);
#line 365 "../src/Widgets/SourceList/SourceList.vala"
	if (old_value != value) {
#line 365 "../src/Widgets/SourceList/SourceList.vala"
		self->priv->_visible = value;
#line 365 "../src/Widgets/SourceList/SourceList.vala"
		g_object_notify_by_pspec ((GObject *) self, code_widgets_source_list_item_properties[CODE_WIDGETS_SOURCE_LIST_ITEM_VISIBLE_PROPERTY]);
#line 2763 "SourceList.c"
	}
}

gboolean
code_widgets_source_list_item_get_selectable (CodeWidgetsSourceListItem* self)
{
	gboolean result;
#line 378 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 378 "../src/Widgets/SourceList/SourceList.vala"
	result = self->priv->_selectable;
#line 378 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 2777 "SourceList.c"
}

void
code_widgets_source_list_item_set_selectable (CodeWidgetsSourceListItem* self,
                                              gboolean value)
{
	gboolean old_value;
#line 378 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 378 "../src/Widgets/SourceList/SourceList.vala"
	old_value = code_widgets_source_list_item_get_selectable (self);
#line 378 "../src/Widgets/SourceList/SourceList.vala"
	if (old_value != value) {
#line 378 "../src/Widgets/SourceList/SourceList.vala"
		self->priv->_selectable = value;
#line 378 "../src/Widgets/SourceList/SourceList.vala"
		g_object_notify_by_pspec ((GObject *) self, code_widgets_source_list_item_properties[CODE_WIDGETS_SOURCE_LIST_ITEM_SELECTABLE_PROPERTY]);
#line 2795 "SourceList.c"
	}
}

GIcon*
code_widgets_source_list_item_get_icon (CodeWidgetsSourceListItem* self)
{
	GIcon* result;
	GIcon* _tmp0_;
#line 388 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 388 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->_icon;
#line 388 "../src/Widgets/SourceList/SourceList.vala"
	result = _tmp0_;
#line 388 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 2812 "SourceList.c"
}

void
code_widgets_source_list_item_set_icon (CodeWidgetsSourceListItem* self,
                                        GIcon* value)
{
	GIcon* old_value;
#line 388 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 388 "../src/Widgets/SourceList/SourceList.vala"
	old_value = code_widgets_source_list_item_get_icon (self);
#line 388 "../src/Widgets/SourceList/SourceList.vala"
	if (old_value != value) {
#line 2826 "SourceList.c"
		GIcon* _tmp0_;
#line 388 "../src/Widgets/SourceList/SourceList.vala"
		_tmp0_ = _g_object_ref0 (value);
#line 388 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (self->priv->_icon);
#line 388 "../src/Widgets/SourceList/SourceList.vala"
		self->priv->_icon = _tmp0_;
#line 388 "../src/Widgets/SourceList/SourceList.vala"
		g_object_notify_by_pspec ((GObject *) self, code_widgets_source_list_item_properties[CODE_WIDGETS_SOURCE_LIST_ITEM_ICON_PROPERTY]);
#line 2836 "SourceList.c"
	}
}

GIcon*
code_widgets_source_list_item_get_activatable (CodeWidgetsSourceListItem* self)
{
	GIcon* result;
	GIcon* _tmp0_;
#line 398 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 398 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->_activatable;
#line 398 "../src/Widgets/SourceList/SourceList.vala"
	result = _tmp0_;
#line 398 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 2853 "SourceList.c"
}

void
code_widgets_source_list_item_set_activatable (CodeWidgetsSourceListItem* self,
                                               GIcon* value)
{
	GIcon* old_value;
#line 398 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 398 "../src/Widgets/SourceList/SourceList.vala"
	old_value = code_widgets_source_list_item_get_activatable (self);
#line 398 "../src/Widgets/SourceList/SourceList.vala"
	if (old_value != value) {
#line 2867 "SourceList.c"
		GIcon* _tmp0_;
#line 398 "../src/Widgets/SourceList/SourceList.vala"
		_tmp0_ = _g_object_ref0 (value);
#line 398 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (self->priv->_activatable);
#line 398 "../src/Widgets/SourceList/SourceList.vala"
		self->priv->_activatable = _tmp0_;
#line 398 "../src/Widgets/SourceList/SourceList.vala"
		g_object_notify_by_pspec ((GObject *) self, code_widgets_source_list_item_properties[CODE_WIDGETS_SOURCE_LIST_ITEM_ACTIVATABLE_PROPERTY]);
#line 2877 "SourceList.c"
	}
}

const gchar*
code_widgets_source_list_item_get_activatable_tooltip (CodeWidgetsSourceListItem* self)
{
	const gchar* result;
	const gchar* _tmp0_;
#line 405 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 405 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->_activatable_tooltip;
#line 405 "../src/Widgets/SourceList/SourceList.vala"
	result = _tmp0_;
#line 405 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 2894 "SourceList.c"
}

void
code_widgets_source_list_item_set_activatable_tooltip (CodeWidgetsSourceListItem* self,
                                                       const gchar* value)
{
	gchar* old_value;
#line 405 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 405 "../src/Widgets/SourceList/SourceList.vala"
	old_value = code_widgets_source_list_item_get_activatable_tooltip (self);
#line 405 "../src/Widgets/SourceList/SourceList.vala"
	if (g_strcmp0 (value, old_value) != 0) {
#line 2908 "SourceList.c"
		gchar* _tmp0_;
#line 405 "../src/Widgets/SourceList/SourceList.vala"
		_tmp0_ = g_strdup (value);
#line 405 "../src/Widgets/SourceList/SourceList.vala"
		_g_free0 (self->priv->_activatable_tooltip);
#line 405 "../src/Widgets/SourceList/SourceList.vala"
		self->priv->_activatable_tooltip = _tmp0_;
#line 405 "../src/Widgets/SourceList/SourceList.vala"
		g_object_notify_by_pspec ((GObject *) self, code_widgets_source_list_item_properties[CODE_WIDGETS_SOURCE_LIST_ITEM_ACTIVATABLE_TOOLTIP_PROPERTY]);
#line 2918 "SourceList.c"
	}
}

static gchar*
string_strip (const gchar* self)
{
	gchar* _result_ = NULL;
	gchar* _tmp0_;
	gchar* result;
#line 1413 "glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, NULL);
#line 1414 "glib-2.0.vapi"
	_tmp0_ = g_strdup (self);
#line 1414 "glib-2.0.vapi"
	_result_ = _tmp0_;
#line 1415 "glib-2.0.vapi"
	g_strstrip (_result_);
#line 1416 "glib-2.0.vapi"
	result = _result_;
#line 1416 "glib-2.0.vapi"
	return result;
#line 2940 "SourceList.c"
}

static void
code_widgets_source_list_item_real_edited (CodeWidgetsSourceListItem* self,
                                           const gchar* new_name)
{
	gboolean _tmp0_ = FALSE;
	gboolean _tmp1_;
#line 278 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (new_name != NULL);
#line 279 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = self->priv->_editable;
#line 279 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp1_) {
#line 2955 "SourceList.c"
		gchar* _tmp2_;
		gchar* _tmp3_;
#line 279 "../src/Widgets/SourceList/SourceList.vala"
		_tmp2_ = string_strip (new_name);
#line 279 "../src/Widgets/SourceList/SourceList.vala"
		_tmp3_ = _tmp2_;
#line 279 "../src/Widgets/SourceList/SourceList.vala"
		_tmp0_ = g_strcmp0 (_tmp3_, "") != 0;
#line 279 "../src/Widgets/SourceList/SourceList.vala"
		_g_free0 (_tmp3_);
#line 2966 "SourceList.c"
	} else {
#line 279 "../src/Widgets/SourceList/SourceList.vala"
		_tmp0_ = FALSE;
#line 2970 "SourceList.c"
	}
#line 279 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp0_) {
#line 280 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_item_set_name (self, new_name);
#line 2976 "SourceList.c"
	}
}

static void
code_widgets_source_list_item_real_action_activated (CodeWidgetsSourceListItem* self)
{
}

static void
code_widgets_source_list_item_real_activated (CodeWidgetsSourceListItem* self)
{
}

static void
code_widgets_source_list_item_class_init (CodeWidgetsSourceListItemClass * klass,
                                          gpointer klass_data)
{
#line 267 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_item_parent_class = g_type_class_peek_parent (klass);
#line 267 "../src/Widgets/SourceList/SourceList.vala"
	g_type_class_adjust_private_offset (klass, &CodeWidgetsSourceListItem_private_offset);
#line 267 "../src/Widgets/SourceList/SourceList.vala"
	((CodeWidgetsSourceListItemClass *) klass)->get_context_menu = (GtkMenu* (*) (CodeWidgetsSourceListItem*)) code_widgets_source_list_item_real_get_context_menu;
#line 267 "../src/Widgets/SourceList/SourceList.vala"
	((CodeWidgetsSourceListItemClass *) klass)->edited = code_widgets_source_list_item_real_edited;
#line 267 "../src/Widgets/SourceList/SourceList.vala"
	((CodeWidgetsSourceListItemClass *) klass)->action_activated = code_widgets_source_list_item_real_action_activated;
#line 267 "../src/Widgets/SourceList/SourceList.vala"
	((CodeWidgetsSourceListItemClass *) klass)->activated = code_widgets_source_list_item_real_activated;
#line 267 "../src/Widgets/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_code_widgets_source_list_item_get_property;
#line 267 "../src/Widgets/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_code_widgets_source_list_item_set_property;
#line 267 "../src/Widgets/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->finalize = code_widgets_source_list_item_finalize;
#line 3012 "SourceList.c"
	/**
	         * Parent {@link Code.Widgets.SourceList.ExpandableItem} of the item.
	         * ''Must not'' be modified.
	         *
	         * @since 0.2
	         */
#line 267 "../src/Widgets/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), CODE_WIDGETS_SOURCE_LIST_ITEM_PARENT_PROPERTY, code_widgets_source_list_item_properties[CODE_WIDGETS_SOURCE_LIST_ITEM_PARENT_PROPERTY] = g_param_spec_object ("parent", "parent", "parent", CODE_WIDGETS_SOURCE_LIST_TYPE_EXPANDABLE_ITEM, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 3021 "SourceList.c"
	/**
	         * The item's name. Primary and most important information.
	         *
	         * @since 0.2
	         */
#line 267 "../src/Widgets/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), CODE_WIDGETS_SOURCE_LIST_ITEM_NAME_PROPERTY, code_widgets_source_list_item_properties[CODE_WIDGETS_SOURCE_LIST_ITEM_NAME_PROPERTY] = g_param_spec_string ("name", "name", "name", NULL, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 3029 "SourceList.c"
	/**
	         * The item's tooltip. If set to null (default), the tooltip for the item will be the
	         * contents of the {@link Code.Widgets.SourceList.Item.name} property.
	         *
	         * @since 5.3
	         */
#line 267 "../src/Widgets/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), CODE_WIDGETS_SOURCE_LIST_ITEM_TOOLTIP_PROPERTY, code_widgets_source_list_item_properties[CODE_WIDGETS_SOURCE_LIST_ITEM_TOOLTIP_PROPERTY] = g_param_spec_string ("tooltip", "tooltip", "tooltip", NULL, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 3038 "SourceList.c"
	/**
	         * Markup to be used instead of {@link Code.Widgets.SourceList.ExpandableItem.name}
	         * This would mean that &, <, etc have to be escaped in the text, but basic formatting
	         * can be done on the item with HTML style tags.
	         *
	         * Note: Only the {@link Code.Widgets.SourceList.ExpandableItem.name} property
	         * is modified for editable items. So this property will be need to updated and
	         * reformatted with editable items.
	         *
	         * @since 5.0
	         */
#line 267 "../src/Widgets/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), CODE_WIDGETS_SOURCE_LIST_ITEM_MARKUP_PROPERTY, code_widgets_source_list_item_properties[CODE_WIDGETS_SOURCE_LIST_ITEM_MARKUP_PROPERTY] = g_param_spec_string ("markup", "markup", "markup", NULL, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 3052 "SourceList.c"
	/**
	         * A badge shown next to the item's name.
	         *
	         * It can be used for displaying the number of unread messages in the "Inbox" item,
	         * for instance.
	         *
	         * @since 0.2
	         */
#line 267 "../src/Widgets/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), CODE_WIDGETS_SOURCE_LIST_ITEM_BADGE_PROPERTY, code_widgets_source_list_item_properties[CODE_WIDGETS_SOURCE_LIST_ITEM_BADGE_PROPERTY] = g_param_spec_string ("badge", "badge", "badge", NULL, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 3063 "SourceList.c"
	/**
	         * Whether the item's name can be edited from within the source list.
	         *
	         * When this property is set to //true//, users can edit the item by pressing
	         * the F2 key, or by double-clicking its name.
	         *
	         * ''This property only works for selectable items''.
	         *
	         * @see Code.Widgets.SourceList.Item.selectable
	         * @see Code.Widgets.SourceList.start_editing_item
	         * @since 0.2
	         */
#line 267 "../src/Widgets/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), CODE_WIDGETS_SOURCE_LIST_ITEM_EDITABLE_PROPERTY, code_widgets_source_list_item_properties[CODE_WIDGETS_SOURCE_LIST_ITEM_EDITABLE_PROPERTY] = g_param_spec_boolean ("editable", "editable", "editable", FALSE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 3078 "SourceList.c"
	/**
	         * Whether the item should appear in the source list's tree or not.
	         *
	         * @since 0.2
	         */
#line 267 "../src/Widgets/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), CODE_WIDGETS_SOURCE_LIST_ITEM_VISIBLE_PROPERTY, code_widgets_source_list_item_properties[CODE_WIDGETS_SOURCE_LIST_ITEM_VISIBLE_PROPERTY] = g_param_spec_boolean ("visible", "visible", "visible", TRUE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 3086 "SourceList.c"
	/**
	         * Whether the item can be selected or not.
	         *
	         * Setting this property to true doesn't guarantee that the item will actually be
	         * selectable, since there are other external factors to take into account, like the
	         * item's {@link Code.Widgets.SourceList.Item.visible} property; whether the item is
	         * a category; the parent item is collapsed, etc.
	         *
	         * @see Code.Widgets.SourceList.Item.visible
	         * @since 0.2
	         */
#line 267 "../src/Widgets/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), CODE_WIDGETS_SOURCE_LIST_ITEM_SELECTABLE_PROPERTY, code_widgets_source_list_item_properties[CODE_WIDGETS_SOURCE_LIST_ITEM_SELECTABLE_PROPERTY] = g_param_spec_boolean ("selectable", "selectable", "selectable", TRUE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 3100 "SourceList.c"
	/**
	         * Primary icon.
	         *
	         * This property should be used to give the user an idea of what the item represents
	         * (i.e. content type.)
	         *
	         * @since 0.2
	         */
#line 267 "../src/Widgets/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), CODE_WIDGETS_SOURCE_LIST_ITEM_ICON_PROPERTY, code_widgets_source_list_item_properties[CODE_WIDGETS_SOURCE_LIST_ITEM_ICON_PROPERTY] = g_param_spec_object ("icon", "icon", "icon", g_icon_get_type (), G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 3111 "SourceList.c"
	/**
	         * An activatable icon that works like a button.
	         *
	         * It can be used for e.g. showing an //"eject"// icon on a device's item.
	         *
	         * @see Code.Widgets.SourceList.Item.action_activated
	         * @since 0.2
	         */
#line 267 "../src/Widgets/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), CODE_WIDGETS_SOURCE_LIST_ITEM_ACTIVATABLE_PROPERTY, code_widgets_source_list_item_properties[CODE_WIDGETS_SOURCE_LIST_ITEM_ACTIVATABLE_PROPERTY] = g_param_spec_object ("activatable", "activatable", "activatable", g_icon_get_type (), G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 3122 "SourceList.c"
	/**
	         * The tooltip for the activatable icon.
	         *
	         * @since 5.0
	         */
#line 267 "../src/Widgets/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), CODE_WIDGETS_SOURCE_LIST_ITEM_ACTIVATABLE_TOOLTIP_PROPERTY, code_widgets_source_list_item_properties[CODE_WIDGETS_SOURCE_LIST_ITEM_ACTIVATABLE_TOOLTIP_PROPERTY] = g_param_spec_string ("activatable-tooltip", "activatable-tooltip", "activatable-tooltip", NULL, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 3130 "SourceList.c"
	/**
	         * Emitted when the user has finished editing the item's name.
	         *
	         * By default, if the name doesn't consist of white space, it is automatically assigned
	         * to the {@link Code.Widgets.SourceList.Item.name} property. The default behavior can
	         * be changed by overriding this signal.
	         * @param new_name The item's new name (result of editing.)
	         * @since 0.2
	         */
#line 267 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_item_signals[CODE_WIDGETS_SOURCE_LIST_ITEM_EDITED_SIGNAL] = g_signal_new ("edited", CODE_WIDGETS_SOURCE_LIST_TYPE_ITEM, G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (CodeWidgetsSourceListItemClass, edited), NULL, NULL, g_cclosure_marshal_VOID__STRING, G_TYPE_NONE, 1, G_TYPE_STRING);
#line 3142 "SourceList.c"
	/**
	         * The {@link Code.Widgets.SourceList.Item.activatable} icon was activated.
	         *
	         * @see Code.Widgets.SourceList.Item.activatable
	         * @since 0.2
	         */
#line 267 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_item_signals[CODE_WIDGETS_SOURCE_LIST_ITEM_ACTION_ACTIVATED_SIGNAL] = g_signal_new ("action-activated", CODE_WIDGETS_SOURCE_LIST_TYPE_ITEM, G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (CodeWidgetsSourceListItemClass, action_activated), NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
#line 3151 "SourceList.c"
	/**
	         * Emitted when the item is double-clicked or when it is selected and one of the keys:
	         * Space, Shift+Space, Return or Enter is pressed. This signal is //also// for
	         * editable items.
	         *
	         * @since 0.2
	         */
#line 267 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_item_signals[CODE_WIDGETS_SOURCE_LIST_ITEM_ACTIVATED_SIGNAL] = g_signal_new ("activated", CODE_WIDGETS_SOURCE_LIST_TYPE_ITEM, G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (CodeWidgetsSourceListItemClass, activated), NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
#line 3161 "SourceList.c"
}

static void
code_widgets_source_list_item_instance_init (CodeWidgetsSourceListItem * self,
                                             gpointer klass)
{
	gchar* _tmp0_;
	gchar* _tmp1_;
	gchar* _tmp2_;
#line 267 "../src/Widgets/SourceList/SourceList.vala"
	self->priv = code_widgets_source_list_item_get_instance_private (self);
#line 313 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = g_strdup ("");
#line 313 "../src/Widgets/SourceList/SourceList.vala"
	self->priv->_name = _tmp0_;
#line 321 "../src/Widgets/SourceList/SourceList.vala"
	self->priv->_tooltip = NULL;
#line 334 "../src/Widgets/SourceList/SourceList.vala"
	self->priv->_markup = NULL;
#line 344 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = g_strdup ("");
#line 344 "../src/Widgets/SourceList/SourceList.vala"
	self->priv->_badge = _tmp1_;
#line 358 "../src/Widgets/SourceList/SourceList.vala"
	self->priv->_editable = FALSE;
#line 365 "../src/Widgets/SourceList/SourceList.vala"
	self->priv->_visible = TRUE;
#line 378 "../src/Widgets/SourceList/SourceList.vala"
	self->priv->_selectable = TRUE;
#line 405 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = g_strdup ("");
#line 405 "../src/Widgets/SourceList/SourceList.vala"
	self->priv->_activatable_tooltip = _tmp2_;
#line 3195 "SourceList.c"
}

static void
code_widgets_source_list_item_finalize (GObject * obj)
{
	CodeWidgetsSourceListItem * self;
#line 267 "../src/Widgets/SourceList/SourceList.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, CODE_WIDGETS_SOURCE_LIST_TYPE_ITEM, CodeWidgetsSourceListItem);
#line 306 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->_parent);
#line 313 "../src/Widgets/SourceList/SourceList.vala"
	_g_free0 (self->priv->_name);
#line 321 "../src/Widgets/SourceList/SourceList.vala"
	_g_free0 (self->priv->_tooltip);
#line 334 "../src/Widgets/SourceList/SourceList.vala"
	_g_free0 (self->priv->_markup);
#line 344 "../src/Widgets/SourceList/SourceList.vala"
	_g_free0 (self->priv->_badge);
#line 388 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->_icon);
#line 398 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->_activatable);
#line 405 "../src/Widgets/SourceList/SourceList.vala"
	_g_free0 (self->priv->_activatable_tooltip);
#line 267 "../src/Widgets/SourceList/SourceList.vala"
	G_OBJECT_CLASS (code_widgets_source_list_item_parent_class)->finalize (obj);
#line 3222 "SourceList.c"
}

/**
     * A source list entry.
     *
     * Any change made to any of its properties will be ''automatically'' reflected
     * by the {@link Code.Widgets.SourceList} widget.
     *
     * @since 0.2
     */
static GType
code_widgets_source_list_item_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (CodeWidgetsSourceListItemClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) code_widgets_source_list_item_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (CodeWidgetsSourceListItem), 0, (GInstanceInitFunc) code_widgets_source_list_item_instance_init, NULL };
	GType code_widgets_source_list_item_type_id;
	code_widgets_source_list_item_type_id = g_type_register_static (G_TYPE_OBJECT, "CodeWidgetsSourceListItem", &g_define_type_info, 0);
	CodeWidgetsSourceListItem_private_offset = g_type_add_instance_private (code_widgets_source_list_item_type_id, sizeof (CodeWidgetsSourceListItemPrivate));
	return code_widgets_source_list_item_type_id;
}

GType
code_widgets_source_list_item_get_type (void)
{
	static volatile gsize code_widgets_source_list_item_type_id__once = 0;
	if (g_once_init_enter (&code_widgets_source_list_item_type_id__once)) {
		GType code_widgets_source_list_item_type_id;
		code_widgets_source_list_item_type_id = code_widgets_source_list_item_get_type_once ();
		g_once_init_leave (&code_widgets_source_list_item_type_id__once, code_widgets_source_list_item_type_id);
	}
	return code_widgets_source_list_item_type_id__once;
}

static void
_vala_code_widgets_source_list_item_get_property (GObject * object,
                                                  guint property_id,
                                                  GValue * value,
                                                  GParamSpec * pspec)
{
	CodeWidgetsSourceListItem * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, CODE_WIDGETS_SOURCE_LIST_TYPE_ITEM, CodeWidgetsSourceListItem);
#line 267 "../src/Widgets/SourceList/SourceList.vala"
	switch (property_id) {
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_ITEM_PARENT_PROPERTY:
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		g_value_set_object (value, code_widgets_source_list_item_get_parent (self));
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_ITEM_NAME_PROPERTY:
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		g_value_set_string (value, code_widgets_source_list_item_get_name (self));
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_ITEM_TOOLTIP_PROPERTY:
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		g_value_set_string (value, code_widgets_source_list_item_get_tooltip (self));
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_ITEM_MARKUP_PROPERTY:
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		g_value_set_string (value, code_widgets_source_list_item_get_markup (self));
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_ITEM_BADGE_PROPERTY:
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		g_value_set_string (value, code_widgets_source_list_item_get_badge (self));
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_ITEM_EDITABLE_PROPERTY:
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		g_value_set_boolean (value, code_widgets_source_list_item_get_editable (self));
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_ITEM_VISIBLE_PROPERTY:
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		g_value_set_boolean (value, code_widgets_source_list_item_get_visible (self));
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_ITEM_SELECTABLE_PROPERTY:
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		g_value_set_boolean (value, code_widgets_source_list_item_get_selectable (self));
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_ITEM_ICON_PROPERTY:
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		g_value_set_object (value, code_widgets_source_list_item_get_icon (self));
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_ITEM_ACTIVATABLE_PROPERTY:
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		g_value_set_object (value, code_widgets_source_list_item_get_activatable (self));
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_ITEM_ACTIVATABLE_TOOLTIP_PROPERTY:
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		g_value_set_string (value, code_widgets_source_list_item_get_activatable_tooltip (self));
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 3331 "SourceList.c"
		default:
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 3337 "SourceList.c"
	}
}

static void
_vala_code_widgets_source_list_item_set_property (GObject * object,
                                                  guint property_id,
                                                  const GValue * value,
                                                  GParamSpec * pspec)
{
	CodeWidgetsSourceListItem * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, CODE_WIDGETS_SOURCE_LIST_TYPE_ITEM, CodeWidgetsSourceListItem);
#line 267 "../src/Widgets/SourceList/SourceList.vala"
	switch (property_id) {
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_ITEM_PARENT_PROPERTY:
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_item_set_parent (self, g_value_get_object (value));
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_ITEM_NAME_PROPERTY:
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_item_set_name (self, g_value_get_string (value));
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_ITEM_TOOLTIP_PROPERTY:
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_item_set_tooltip (self, g_value_get_string (value));
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_ITEM_MARKUP_PROPERTY:
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_item_set_markup (self, g_value_get_string (value));
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_ITEM_BADGE_PROPERTY:
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_item_set_badge (self, g_value_get_string (value));
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_ITEM_EDITABLE_PROPERTY:
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_item_set_editable (self, g_value_get_boolean (value));
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_ITEM_VISIBLE_PROPERTY:
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_item_set_visible (self, g_value_get_boolean (value));
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_ITEM_SELECTABLE_PROPERTY:
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_item_set_selectable (self, g_value_get_boolean (value));
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_ITEM_ICON_PROPERTY:
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_item_set_icon (self, g_value_get_object (value));
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_ITEM_ACTIVATABLE_PROPERTY:
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_item_set_activatable (self, g_value_get_object (value));
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_ITEM_ACTIVATABLE_TOOLTIP_PROPERTY:
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_item_set_activatable_tooltip (self, g_value_get_string (value));
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 3417 "SourceList.c"
		default:
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 267 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 3423 "SourceList.c"
	}
}

static inline gpointer
code_widgets_source_list_expandable_item_get_instance_private (CodeWidgetsSourceListExpandableItem* self)
{
	return G_STRUCT_MEMBER_P (self, CodeWidgetsSourceListExpandableItem_private_offset);
}

/**
         * Creates a new {@link Code.Widgets.SourceList.ExpandableItem}
         *
         * @param name Title of the item.
         * @return (transfer full) A new {@link Code.Widgets.SourceList.ExpandableItem}.
         * @since 0.2
         */
CodeWidgetsSourceListExpandableItem*
code_widgets_source_list_expandable_item_construct (GType object_type,
                                                    const gchar* name)
{
	CodeWidgetsSourceListExpandableItem * self = NULL;
#line 560 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (name != NULL, NULL);
#line 561 "../src/Widgets/SourceList/SourceList.vala"
	self = (CodeWidgetsSourceListExpandableItem*) code_widgets_source_list_item_construct (object_type, name);
#line 560 "../src/Widgets/SourceList/SourceList.vala"
	return self;
#line 3451 "SourceList.c"
}

CodeWidgetsSourceListExpandableItem*
code_widgets_source_list_expandable_item_new (const gchar* name)
{
#line 560 "../src/Widgets/SourceList/SourceList.vala"
	return code_widgets_source_list_expandable_item_construct (CODE_WIDGETS_SOURCE_LIST_TYPE_EXPANDABLE_ITEM, name);
#line 3459 "SourceList.c"
}

/**
         * Checks whether the item contains the specified child.
         *
         * This method only considers the item's immediate children.
         *
         * @param item Item to search.
         * @return Whether the item was found or not.
         * @since 0.2
         */
gboolean
code_widgets_source_list_expandable_item_contains (CodeWidgetsSourceListExpandableItem* self,
                                                   CodeWidgetsSourceListItem* item)
{
	GeeCollection* _tmp0_;
	gboolean result;
#line 577 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 577 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (item != NULL, FALSE);
#line 578 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->children_list;
#line 578 "../src/Widgets/SourceList/SourceList.vala"
	result = gee_collection_contains (_tmp0_, item);
#line 578 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 3487 "SourceList.c"
}

/**
         * Adds an item.
         *
         * {@link Code.Widgets.SourceList.ExpandableItem.child_added} is fired after the item is added.
         *
         * While adding a child item, //the item it's being added to will set itself as the parent//.
         * Please note that items are required to have their //parent// property set to //null// before
         * being added, so make sure the item is removed from its previous parent before attempting
         * to add it to another item. For instance:
         * {{{
         * if (item.parent != null)
         *     item.parent.remove (item); // this will set item's parent to null
         * new_parent.add (item);
         * }}}
         *
         * @param item The item to add. Its parent __must__ be //null//.
         * @see Code.Widgets.SourceList.ExpandableItem.child_added
         * @see Code.Widgets.SourceList.ExpandableItem.remove
         * @since 0.2
         */
void
code_widgets_source_list_expandable_item_add (CodeWidgetsSourceListExpandableItem* self,
                                              CodeWidgetsSourceListItem* item)
{
	CodeWidgetsSourceListExpandableItem* _tmp0_;
	CodeWidgetsSourceListExpandableItem* _tmp1_;
	GeeCollection* _tmp2_;
#line 601 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 601 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (item != NULL);
#line 601 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = code_widgets_source_list_item_get_parent (item);
#line 601 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = _tmp0_;
#line 601 "../src/Widgets/SourceList/SourceList.vala"
	_vala_return_if_fail (_tmp1_ == NULL, "item.parent == null");
#line 602 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_item_set_parent (item, self);
#line 603 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = self->priv->children_list;
#line 603 "../src/Widgets/SourceList/SourceList.vala"
	gee_collection_add (_tmp2_, item);
#line 604 "../src/Widgets/SourceList/SourceList.vala"
	g_signal_emit (self, code_widgets_source_list_expandable_item_signals[CODE_WIDGETS_SOURCE_LIST_EXPANDABLE_ITEM_CHILD_ADDED_SIGNAL], 0, item);
#line 3535 "SourceList.c"
}

/**
         * Removes an item.
         *
         * The {@link Code.Widgets.SourceList.ExpandableItem.child_removed} signal is fired
         * //after removing the item//. Finally (i.e. after all the handlers have been invoked),
         * the item's {@link Code.Widgets.SourceList.Item.parent} property is set to //null//.
         * This has the advantage of letting signal handlers know the parent from which //item//
         * is being removed.
         *
         * @param item The item to remove. This will fail if item has a different parent.
         * @see Code.Widgets.SourceList.ExpandableItem.child_removed
         * @see Code.Widgets.SourceList.ExpandableItem.clear
         * @since 0.2
         */
void
code_widgets_source_list_expandable_item_remove (CodeWidgetsSourceListExpandableItem* self,
                                                 CodeWidgetsSourceListItem* item)
{
	CodeWidgetsSourceListExpandableItem* _tmp0_;
	CodeWidgetsSourceListExpandableItem* _tmp1_;
	GeeCollection* _tmp2_;
#line 621 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 621 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (item != NULL);
#line 621 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = code_widgets_source_list_item_get_parent (item);
#line 621 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = _tmp0_;
#line 621 "../src/Widgets/SourceList/SourceList.vala"
	_vala_return_if_fail (_tmp1_ == self, "item.parent == this");
#line 622 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = self->priv->children_list;
#line 622 "../src/Widgets/SourceList/SourceList.vala"
	gee_collection_remove (_tmp2_, item);
#line 623 "../src/Widgets/SourceList/SourceList.vala"
	g_signal_emit (self, code_widgets_source_list_expandable_item_signals[CODE_WIDGETS_SOURCE_LIST_EXPANDABLE_ITEM_CHILD_REMOVED_SIGNAL], 0, item);
#line 624 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_item_set_parent (item, NULL);
#line 3577 "SourceList.c"
}

/**
         * Removes all the items contained by the item. It works similarly to
         * {@link Code.Widgets.SourceList.ExpandableItem.remove}.
         *
         * @see Code.Widgets.SourceList.ExpandableItem.remove
         * @see Code.Widgets.SourceList.ExpandableItem.child_removed
         * @since 0.2
         */
void
code_widgets_source_list_expandable_item_clear (CodeWidgetsSourceListExpandableItem* self)
{
#line 635 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 3593 "SourceList.c"
	{
		GeeIterator* _item_it = NULL;
		GeeCollection* _tmp0_;
		GeeCollection* _tmp1_;
		GeeCollection* _tmp2_;
		GeeIterator* _tmp3_;
		GeeIterator* _tmp4_;
#line 636 "../src/Widgets/SourceList/SourceList.vala"
		_tmp0_ = code_widgets_source_list_expandable_item_get_children (self);
#line 636 "../src/Widgets/SourceList/SourceList.vala"
		_tmp1_ = _tmp0_;
#line 636 "../src/Widgets/SourceList/SourceList.vala"
		_tmp2_ = _tmp1_;
#line 636 "../src/Widgets/SourceList/SourceList.vala"
		_tmp3_ = gee_iterable_iterator ((GeeIterable*) _tmp2_);
#line 636 "../src/Widgets/SourceList/SourceList.vala"
		_tmp4_ = _tmp3_;
#line 636 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (_tmp2_);
#line 636 "../src/Widgets/SourceList/SourceList.vala"
		_item_it = _tmp4_;
#line 636 "../src/Widgets/SourceList/SourceList.vala"
		while (TRUE) {
#line 3617 "SourceList.c"
			GeeIterator* _tmp5_;
			CodeWidgetsSourceListItem* item = NULL;
			GeeIterator* _tmp6_;
			gpointer _tmp7_;
			CodeWidgetsSourceListItem* _tmp8_;
#line 636 "../src/Widgets/SourceList/SourceList.vala"
			_tmp5_ = _item_it;
#line 636 "../src/Widgets/SourceList/SourceList.vala"
			if (!gee_iterator_next (_tmp5_)) {
#line 636 "../src/Widgets/SourceList/SourceList.vala"
				break;
#line 3629 "SourceList.c"
			}
#line 636 "../src/Widgets/SourceList/SourceList.vala"
			_tmp6_ = _item_it;
#line 636 "../src/Widgets/SourceList/SourceList.vala"
			_tmp7_ = gee_iterator_get (_tmp6_);
#line 636 "../src/Widgets/SourceList/SourceList.vala"
			item = (CodeWidgetsSourceListItem*) _tmp7_;
#line 637 "../src/Widgets/SourceList/SourceList.vala"
			_tmp8_ = item;
#line 637 "../src/Widgets/SourceList/SourceList.vala"
			code_widgets_source_list_expandable_item_remove (self, _tmp8_);
#line 636 "../src/Widgets/SourceList/SourceList.vala"
			_g_object_unref0 (item);
#line 3643 "SourceList.c"
		}
#line 636 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (_item_it);
#line 3647 "SourceList.c"
	}
}

/**
         * Expands the item and/or its children.
         *
         * @param inclusive Whether to also expand this item (true), or only its children (false).
         * @param recursive Whether to recursively expand all the children (true), or only
         * immediate children (false).
         * @see Code.Widgets.SourceList.ExpandableItem.expanded
         * @since 0.2
         */
void
code_widgets_source_list_expandable_item_expand_all (CodeWidgetsSourceListExpandableItem* self,
                                                     gboolean inclusive,
                                                     gboolean recursive)
{
#line 649 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 650 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_expandable_item_set_expansion (self, inclusive, recursive, TRUE);
#line 3669 "SourceList.c"
}

/**
         * Collapses the item and/or its children.
         *
         * @param inclusive Whether to also collapse this item (true), or only its children (false).
         * @param recursive Whether to recursively collapse all the children (true), or only
         * immediate children (false).
         * @see Code.Widgets.SourceList.ExpandableItem.expanded
         * @since 0.2
         */
void
code_widgets_source_list_expandable_item_collapse_all (CodeWidgetsSourceListExpandableItem* self,
                                                       gboolean inclusive,
                                                       gboolean recursive)
{
#line 662 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 663 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_expandable_item_set_expansion (self, inclusive, recursive, FALSE);
#line 3690 "SourceList.c"
}

static void
code_widgets_source_list_expandable_item_set_expansion (CodeWidgetsSourceListExpandableItem* item,
                                                        gboolean inclusive,
                                                        gboolean recursive,
                                                        gboolean expanded)
{
#line 666 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (item != NULL);
#line 667 "../src/Widgets/SourceList/SourceList.vala"
	if (inclusive) {
#line 668 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_expandable_item_set_expanded (item, expanded);
#line 3705 "SourceList.c"
	}
	{
		GeeIterator* _child_item_it = NULL;
		GeeCollection* _tmp0_;
		GeeCollection* _tmp1_;
		GeeCollection* _tmp2_;
		GeeIterator* _tmp3_;
		GeeIterator* _tmp4_;
#line 670 "../src/Widgets/SourceList/SourceList.vala"
		_tmp0_ = code_widgets_source_list_expandable_item_get_children (item);
#line 670 "../src/Widgets/SourceList/SourceList.vala"
		_tmp1_ = _tmp0_;
#line 670 "../src/Widgets/SourceList/SourceList.vala"
		_tmp2_ = _tmp1_;
#line 670 "../src/Widgets/SourceList/SourceList.vala"
		_tmp3_ = gee_iterable_iterator ((GeeIterable*) _tmp2_);
#line 670 "../src/Widgets/SourceList/SourceList.vala"
		_tmp4_ = _tmp3_;
#line 670 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (_tmp2_);
#line 670 "../src/Widgets/SourceList/SourceList.vala"
		_child_item_it = _tmp4_;
#line 670 "../src/Widgets/SourceList/SourceList.vala"
		while (TRUE) {
#line 3730 "SourceList.c"
			GeeIterator* _tmp5_;
			CodeWidgetsSourceListItem* child_item = NULL;
			GeeIterator* _tmp6_;
			gpointer _tmp7_;
			CodeWidgetsSourceListExpandableItem* child_expandable_item = NULL;
			CodeWidgetsSourceListItem* _tmp8_;
			CodeWidgetsSourceListExpandableItem* _tmp9_;
			CodeWidgetsSourceListExpandableItem* _tmp10_;
#line 670 "../src/Widgets/SourceList/SourceList.vala"
			_tmp5_ = _child_item_it;
#line 670 "../src/Widgets/SourceList/SourceList.vala"
			if (!gee_iterator_next (_tmp5_)) {
#line 670 "../src/Widgets/SourceList/SourceList.vala"
				break;
#line 3745 "SourceList.c"
			}
#line 670 "../src/Widgets/SourceList/SourceList.vala"
			_tmp6_ = _child_item_it;
#line 670 "../src/Widgets/SourceList/SourceList.vala"
			_tmp7_ = gee_iterator_get (_tmp6_);
#line 670 "../src/Widgets/SourceList/SourceList.vala"
			child_item = (CodeWidgetsSourceListItem*) _tmp7_;
#line 671 "../src/Widgets/SourceList/SourceList.vala"
			_tmp8_ = child_item;
#line 671 "../src/Widgets/SourceList/SourceList.vala"
			_tmp9_ = _g_object_ref0 (CODE_WIDGETS_SOURCE_LIST_IS_EXPANDABLE_ITEM (_tmp8_) ? ((CodeWidgetsSourceListExpandableItem*) _tmp8_) : NULL);
#line 671 "../src/Widgets/SourceList/SourceList.vala"
			child_expandable_item = _tmp9_;
#line 672 "../src/Widgets/SourceList/SourceList.vala"
			_tmp10_ = child_expandable_item;
#line 672 "../src/Widgets/SourceList/SourceList.vala"
			if (_tmp10_ != NULL) {
#line 673 "../src/Widgets/SourceList/SourceList.vala"
				if (recursive) {
#line 3765 "SourceList.c"
					CodeWidgetsSourceListExpandableItem* _tmp11_;
#line 674 "../src/Widgets/SourceList/SourceList.vala"
					_tmp11_ = child_expandable_item;
#line 674 "../src/Widgets/SourceList/SourceList.vala"
					code_widgets_source_list_expandable_item_set_expansion (_tmp11_, TRUE, TRUE, expanded);
#line 3771 "SourceList.c"
				} else {
					CodeWidgetsSourceListExpandableItem* _tmp12_;
#line 676 "../src/Widgets/SourceList/SourceList.vala"
					_tmp12_ = child_expandable_item;
#line 676 "../src/Widgets/SourceList/SourceList.vala"
					code_widgets_source_list_expandable_item_set_expanded (_tmp12_, expanded);
#line 3778 "SourceList.c"
				}
			}
#line 670 "../src/Widgets/SourceList/SourceList.vala"
			_g_object_unref0 (child_expandable_item);
#line 670 "../src/Widgets/SourceList/SourceList.vala"
			_g_object_unref0 (child_item);
#line 3785 "SourceList.c"
		}
#line 670 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (_child_item_it);
#line 3789 "SourceList.c"
	}
}

/**
         * Recursively expands the item along with its parent(s).
         *
         * @see Code.Widgets.SourceList.ExpandableItem.expanded
         * @since 0.2
         */
void
code_widgets_source_list_expandable_item_expand_with_parents (CodeWidgetsSourceListExpandableItem* self)
{
	CodeWidgetsSourceListExpandableItem* _tmp0_;
	CodeWidgetsSourceListExpandableItem* _tmp1_;
#line 687 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 692 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = code_widgets_source_list_item_get_parent ((CodeWidgetsSourceListItem*) self);
#line 692 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = _tmp0_;
#line 692 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp1_ != NULL) {
#line 3812 "SourceList.c"
		CodeWidgetsSourceListExpandableItem* _tmp2_;
		CodeWidgetsSourceListExpandableItem* _tmp3_;
#line 693 "../src/Widgets/SourceList/SourceList.vala"
		_tmp2_ = code_widgets_source_list_item_get_parent ((CodeWidgetsSourceListItem*) self);
#line 693 "../src/Widgets/SourceList/SourceList.vala"
		_tmp3_ = _tmp2_;
#line 693 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_expandable_item_expand_with_parents (_tmp3_);
#line 3821 "SourceList.c"
	}
#line 694 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_expandable_item_set_expanded (self, TRUE);
#line 3825 "SourceList.c"
}

/**
         * Recursively collapses the item along with its parent(s).
         *
         * @see Code.Widgets.SourceList.ExpandableItem.expanded
         * @since 0.2
         */
void
code_widgets_source_list_expandable_item_collapse_with_parents (CodeWidgetsSourceListExpandableItem* self)
{
	CodeWidgetsSourceListExpandableItem* _tmp0_;
	CodeWidgetsSourceListExpandableItem* _tmp1_;
#line 703 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 704 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = code_widgets_source_list_item_get_parent ((CodeWidgetsSourceListItem*) self);
#line 704 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = _tmp0_;
#line 704 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp1_ != NULL) {
#line 3847 "SourceList.c"
		CodeWidgetsSourceListExpandableItem* _tmp2_;
		CodeWidgetsSourceListExpandableItem* _tmp3_;
#line 705 "../src/Widgets/SourceList/SourceList.vala"
		_tmp2_ = code_widgets_source_list_item_get_parent ((CodeWidgetsSourceListItem*) self);
#line 705 "../src/Widgets/SourceList/SourceList.vala"
		_tmp3_ = _tmp2_;
#line 705 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_expandable_item_collapse_with_parents (_tmp3_);
#line 3856 "SourceList.c"
	}
#line 706 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_expandable_item_set_expanded (self, FALSE);
#line 3860 "SourceList.c"
}

gboolean
code_widgets_source_list_expandable_item_get_collapsible (CodeWidgetsSourceListExpandableItem* self)
{
	gboolean result;
#line 490 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 490 "../src/Widgets/SourceList/SourceList.vala"
	result = self->priv->_collapsible;
#line 490 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 3873 "SourceList.c"
}

void
code_widgets_source_list_expandable_item_set_collapsible (CodeWidgetsSourceListExpandableItem* self,
                                                          gboolean value)
{
	gboolean old_value;
#line 490 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 490 "../src/Widgets/SourceList/SourceList.vala"
	old_value = code_widgets_source_list_expandable_item_get_collapsible (self);
#line 490 "../src/Widgets/SourceList/SourceList.vala"
	if (old_value != value) {
#line 490 "../src/Widgets/SourceList/SourceList.vala"
		self->priv->_collapsible = value;
#line 490 "../src/Widgets/SourceList/SourceList.vala"
		g_object_notify_by_pspec ((GObject *) self, code_widgets_source_list_expandable_item_properties[CODE_WIDGETS_SOURCE_LIST_EXPANDABLE_ITEM_COLLAPSIBLE_PROPERTY]);
#line 3891 "SourceList.c"
	}
}

gboolean
code_widgets_source_list_expandable_item_get_expanded (CodeWidgetsSourceListExpandableItem* self)
{
	gboolean result;
	gboolean _tmp0_ = FALSE;
#line 514 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 514 "../src/Widgets/SourceList/SourceList.vala"
	if (self->priv->_expanded) {
#line 514 "../src/Widgets/SourceList/SourceList.vala"
		_tmp0_ = TRUE;
#line 3906 "SourceList.c"
	} else {
		gboolean _tmp1_;
#line 514 "../src/Widgets/SourceList/SourceList.vala"
		_tmp1_ = self->priv->_collapsible;
#line 514 "../src/Widgets/SourceList/SourceList.vala"
		_tmp0_ = !_tmp1_;
#line 3913 "SourceList.c"
	}
#line 514 "../src/Widgets/SourceList/SourceList.vala"
	result = _tmp0_;
#line 514 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 3919 "SourceList.c"
}

void
code_widgets_source_list_expandable_item_set_expanded (CodeWidgetsSourceListExpandableItem* self,
                                                       gboolean value)
{
#line 515 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 516 "../src/Widgets/SourceList/SourceList.vala"
	if (value != self->priv->_expanded) {
#line 517 "../src/Widgets/SourceList/SourceList.vala"
		self->priv->_expanded = value;
#line 518 "../src/Widgets/SourceList/SourceList.vala"
		g_signal_emit (self, code_widgets_source_list_expandable_item_signals[CODE_WIDGETS_SOURCE_LIST_EXPANDABLE_ITEM_TOGGLED_SIGNAL], 0);
#line 3934 "SourceList.c"
	}
#line 515 "../src/Widgets/SourceList/SourceList.vala"
	g_object_notify_by_pspec ((GObject *) self, code_widgets_source_list_expandable_item_properties[CODE_WIDGETS_SOURCE_LIST_EXPANDABLE_ITEM_EXPANDED_PROPERTY]);
#line 3938 "SourceList.c"
}

guint
code_widgets_source_list_expandable_item_get_n_children (CodeWidgetsSourceListExpandableItem* self)
{
	guint result;
	GeeCollection* _tmp0_;
	gint _tmp1_;
	gint _tmp2_;
#line 529 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, 0U);
#line 529 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->children_list;
#line 529 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = gee_collection_get_size (_tmp0_);
#line 529 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = _tmp1_;
#line 529 "../src/Widgets/SourceList/SourceList.vala"
	result = (guint) _tmp2_;
#line 529 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 3960 "SourceList.c"
}

GeeCollection*
code_widgets_source_list_expandable_item_get_children (CodeWidgetsSourceListExpandableItem* self)
{
	GeeCollection* result;
	GeeArrayList* children_list_copy = NULL;
	GeeArrayList* _tmp0_;
	GeeCollection* _tmp1_;
#line 542 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 545 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = gee_array_list_new (CODE_WIDGETS_SOURCE_LIST_TYPE_ITEM, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref, NULL, NULL, NULL);
#line 545 "../src/Widgets/SourceList/SourceList.vala"
	children_list_copy = _tmp0_;
#line 546 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = self->priv->children_list;
#line 546 "../src/Widgets/SourceList/SourceList.vala"
	gee_array_list_add_all (children_list_copy, _tmp1_);
#line 547 "../src/Widgets/SourceList/SourceList.vala"
	result = (GeeCollection*) children_list_copy;
#line 547 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 3984 "SourceList.c"
}

static void
code_widgets_source_list_expandable_item_real_toggled (CodeWidgetsSourceListExpandableItem* self)
{
}

static GObject *
code_widgets_source_list_expandable_item_constructor (GType type,
                                                      guint n_construct_properties,
                                                      GObjectConstructParam * construct_properties)
{
	GObject * obj;
	GObjectClass * parent_class;
	CodeWidgetsSourceListExpandableItem * self;
#line 564 "../src/Widgets/SourceList/SourceList.vala"
	parent_class = G_OBJECT_CLASS (code_widgets_source_list_expandable_item_parent_class);
#line 564 "../src/Widgets/SourceList/SourceList.vala"
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
#line 564 "../src/Widgets/SourceList/SourceList.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, CODE_WIDGETS_SOURCE_LIST_TYPE_EXPANDABLE_ITEM, CodeWidgetsSourceListExpandableItem);
#line 565 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_item_set_editable ((CodeWidgetsSourceListItem*) self, FALSE);
#line 564 "../src/Widgets/SourceList/SourceList.vala"
	return obj;
#line 4010 "SourceList.c"
}

static void
code_widgets_source_list_expandable_item_class_init (CodeWidgetsSourceListExpandableItemClass * klass,
                                                     gpointer klass_data)
{
#line 453 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_expandable_item_parent_class = g_type_class_peek_parent (klass);
#line 453 "../src/Widgets/SourceList/SourceList.vala"
	g_type_class_adjust_private_offset (klass, &CodeWidgetsSourceListExpandableItem_private_offset);
#line 453 "../src/Widgets/SourceList/SourceList.vala"
	((CodeWidgetsSourceListExpandableItemClass *) klass)->toggled = code_widgets_source_list_expandable_item_real_toggled;
#line 453 "../src/Widgets/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_code_widgets_source_list_expandable_item_get_property;
#line 453 "../src/Widgets/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_code_widgets_source_list_expandable_item_set_property;
#line 453 "../src/Widgets/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->constructor = code_widgets_source_list_expandable_item_constructor;
#line 453 "../src/Widgets/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->finalize = code_widgets_source_list_expandable_item_finalize;
#line 4031 "SourceList.c"
	/**
	         * Whether the item is collapsible or not.
	         *
	         * When set to //false//, the item is //always// expanded and the expander is
	         * not shown. Please note that this will also affect the value returned by the
	         * {@link Code.Widgets.SourceList.ExpandableItem.expanded} property.
	         *
	         * @see Code.Widgets.SourceList.ExpandableItem.expanded
	         * @since 0.2
	         */
#line 453 "../src/Widgets/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), CODE_WIDGETS_SOURCE_LIST_EXPANDABLE_ITEM_COLLAPSIBLE_PROPERTY, code_widgets_source_list_expandable_item_properties[CODE_WIDGETS_SOURCE_LIST_EXPANDABLE_ITEM_COLLAPSIBLE_PROPERTY] = g_param_spec_boolean ("collapsible", "collapsible", "collapsible", TRUE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 453 "../src/Widgets/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), CODE_WIDGETS_SOURCE_LIST_EXPANDABLE_ITEM_EXPANDED_PROPERTY, code_widgets_source_list_expandable_item_properties[CODE_WIDGETS_SOURCE_LIST_EXPANDABLE_ITEM_EXPANDED_PROPERTY] = g_param_spec_boolean ("expanded", "expanded", "expanded", FALSE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 4046 "SourceList.c"
	/**
	         * Number of children contained by the item.
	         *
	         * @since 0.2
	         */
#line 453 "../src/Widgets/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), CODE_WIDGETS_SOURCE_LIST_EXPANDABLE_ITEM_N_CHILDREN_PROPERTY, code_widgets_source_list_expandable_item_properties[CODE_WIDGETS_SOURCE_LIST_EXPANDABLE_ITEM_N_CHILDREN_PROPERTY] = g_param_spec_uint ("n-children", "n-children", "n-children", 0, G_MAXUINT, 0U, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
#line 4054 "SourceList.c"
	/**
	         * The item's children.
	         *
	         * This returns a newly-created list containing the children.
	         * It's safe to iterate it while removing items with
	         * {@link Code.Widgets.SourceList.ExpandableItem.remove}
	         *
	         * @since 0.2
	         */
#line 453 "../src/Widgets/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), CODE_WIDGETS_SOURCE_LIST_EXPANDABLE_ITEM_CHILDREN_PROPERTY, code_widgets_source_list_expandable_item_properties[CODE_WIDGETS_SOURCE_LIST_EXPANDABLE_ITEM_CHILDREN_PROPERTY] = g_param_spec_object ("children", "children", "children", GEE_TYPE_COLLECTION, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
#line 4066 "SourceList.c"
	/**
	         * Emitted when an item is added.
	         *
	         * @param item Item added.
	         * @see Code.Widgets.SourceList.ExpandableItem.add
	         * @since 0.2
	         */
#line 453 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_expandable_item_signals[CODE_WIDGETS_SOURCE_LIST_EXPANDABLE_ITEM_CHILD_ADDED_SIGNAL] = g_signal_new ("child-added", CODE_WIDGETS_SOURCE_LIST_TYPE_EXPANDABLE_ITEM, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, CODE_WIDGETS_SOURCE_LIST_TYPE_ITEM);
#line 4076 "SourceList.c"
	/**
	         * Emitted when an item is removed.
	         *
	         * @param item Item removed.
	         * @see Code.Widgets.SourceList.ExpandableItem.remove
	         * @since 0.2
	         */
#line 453 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_expandable_item_signals[CODE_WIDGETS_SOURCE_LIST_EXPANDABLE_ITEM_CHILD_REMOVED_SIGNAL] = g_signal_new ("child-removed", CODE_WIDGETS_SOURCE_LIST_TYPE_EXPANDABLE_ITEM, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, CODE_WIDGETS_SOURCE_LIST_TYPE_ITEM);
#line 4086 "SourceList.c"
	/**
	         * Emitted when the item is expanded or collapsed.
	         *
	         * @since 0.2
	         */
#line 453 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_expandable_item_signals[CODE_WIDGETS_SOURCE_LIST_EXPANDABLE_ITEM_TOGGLED_SIGNAL] = g_signal_new ("toggled", CODE_WIDGETS_SOURCE_LIST_TYPE_EXPANDABLE_ITEM, G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (CodeWidgetsSourceListExpandableItemClass, toggled), NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
#line 4094 "SourceList.c"
}

static void
code_widgets_source_list_expandable_item_instance_init (CodeWidgetsSourceListExpandableItem * self,
                                                        gpointer klass)
{
	GeeArrayList* _tmp0_;
#line 453 "../src/Widgets/SourceList/SourceList.vala"
	self->priv = code_widgets_source_list_expandable_item_get_instance_private (self);
#line 490 "../src/Widgets/SourceList/SourceList.vala"
	self->priv->_collapsible = TRUE;
#line 512 "../src/Widgets/SourceList/SourceList.vala"
	self->priv->_expanded = FALSE;
#line 551 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = gee_array_list_new (CODE_WIDGETS_SOURCE_LIST_TYPE_ITEM, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref, NULL, NULL, NULL);
#line 551 "../src/Widgets/SourceList/SourceList.vala"
	self->priv->children_list = (GeeCollection*) _tmp0_;
#line 4112 "SourceList.c"
}

static void
code_widgets_source_list_expandable_item_finalize (GObject * obj)
{
	CodeWidgetsSourceListExpandableItem * self;
#line 453 "../src/Widgets/SourceList/SourceList.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, CODE_WIDGETS_SOURCE_LIST_TYPE_EXPANDABLE_ITEM, CodeWidgetsSourceListExpandableItem);
#line 551 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->children_list);
#line 453 "../src/Widgets/SourceList/SourceList.vala"
	G_OBJECT_CLASS (code_widgets_source_list_expandable_item_parent_class)->finalize (obj);
#line 4125 "SourceList.c"
}

/**
     * An item that can contain more items.
     *
     * It supports all the properties inherited from {@link Code.Widgets.SourceList.Item},
     * and behaves like a normal item, except when it is located at the root level; in that case,
     * the following properties are ignored by the widget:
     *
     * * {@link Code.Widgets.SourceList.Item.selectable}
     * * {@link Code.Widgets.SourceList.Item.editable}
     * * {@link Code.Widgets.SourceList.Item.icon}
     * * {@link Code.Widgets.SourceList.Item.activatable}
     * * {@link Code.Widgets.SourceList.Item.badge}
     *
     * Root-level expandable items (i.e. Main Categories) are ''not'' displayed when they contain
     * zero visible children.
     *
     * @since 0.2
     */
static GType
code_widgets_source_list_expandable_item_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (CodeWidgetsSourceListExpandableItemClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) code_widgets_source_list_expandable_item_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (CodeWidgetsSourceListExpandableItem), 0, (GInstanceInitFunc) code_widgets_source_list_expandable_item_instance_init, NULL };
	GType code_widgets_source_list_expandable_item_type_id;
	code_widgets_source_list_expandable_item_type_id = g_type_register_static (CODE_WIDGETS_SOURCE_LIST_TYPE_ITEM, "CodeWidgetsSourceListExpandableItem", &g_define_type_info, 0);
	CodeWidgetsSourceListExpandableItem_private_offset = g_type_add_instance_private (code_widgets_source_list_expandable_item_type_id, sizeof (CodeWidgetsSourceListExpandableItemPrivate));
	return code_widgets_source_list_expandable_item_type_id;
}

GType
code_widgets_source_list_expandable_item_get_type (void)
{
	static volatile gsize code_widgets_source_list_expandable_item_type_id__once = 0;
	if (g_once_init_enter (&code_widgets_source_list_expandable_item_type_id__once)) {
		GType code_widgets_source_list_expandable_item_type_id;
		code_widgets_source_list_expandable_item_type_id = code_widgets_source_list_expandable_item_get_type_once ();
		g_once_init_leave (&code_widgets_source_list_expandable_item_type_id__once, code_widgets_source_list_expandable_item_type_id);
	}
	return code_widgets_source_list_expandable_item_type_id__once;
}

static void
_vala_code_widgets_source_list_expandable_item_get_property (GObject * object,
                                                             guint property_id,
                                                             GValue * value,
                                                             GParamSpec * pspec)
{
	CodeWidgetsSourceListExpandableItem * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, CODE_WIDGETS_SOURCE_LIST_TYPE_EXPANDABLE_ITEM, CodeWidgetsSourceListExpandableItem);
#line 453 "../src/Widgets/SourceList/SourceList.vala"
	switch (property_id) {
#line 453 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_EXPANDABLE_ITEM_COLLAPSIBLE_PROPERTY:
#line 453 "../src/Widgets/SourceList/SourceList.vala"
		g_value_set_boolean (value, code_widgets_source_list_expandable_item_get_collapsible (self));
#line 453 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 453 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_EXPANDABLE_ITEM_EXPANDED_PROPERTY:
#line 453 "../src/Widgets/SourceList/SourceList.vala"
		g_value_set_boolean (value, code_widgets_source_list_expandable_item_get_expanded (self));
#line 453 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 453 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_EXPANDABLE_ITEM_N_CHILDREN_PROPERTY:
#line 453 "../src/Widgets/SourceList/SourceList.vala"
		g_value_set_uint (value, code_widgets_source_list_expandable_item_get_n_children (self));
#line 453 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 453 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_EXPANDABLE_ITEM_CHILDREN_PROPERTY:
#line 453 "../src/Widgets/SourceList/SourceList.vala"
		g_value_take_object (value, code_widgets_source_list_expandable_item_get_children (self));
#line 453 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 4202 "SourceList.c"
		default:
#line 453 "../src/Widgets/SourceList/SourceList.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 453 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 4208 "SourceList.c"
	}
}

static void
_vala_code_widgets_source_list_expandable_item_set_property (GObject * object,
                                                             guint property_id,
                                                             const GValue * value,
                                                             GParamSpec * pspec)
{
	CodeWidgetsSourceListExpandableItem * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, CODE_WIDGETS_SOURCE_LIST_TYPE_EXPANDABLE_ITEM, CodeWidgetsSourceListExpandableItem);
#line 453 "../src/Widgets/SourceList/SourceList.vala"
	switch (property_id) {
#line 453 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_EXPANDABLE_ITEM_COLLAPSIBLE_PROPERTY:
#line 453 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_expandable_item_set_collapsible (self, g_value_get_boolean (value));
#line 453 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 453 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_EXPANDABLE_ITEM_EXPANDED_PROPERTY:
#line 453 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_expandable_item_set_expanded (self, g_value_get_boolean (value));
#line 453 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 4234 "SourceList.c"
		default:
#line 453 "../src/Widgets/SourceList/SourceList.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 453 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 4240 "SourceList.c"
	}
}

static inline gpointer
code_widgets_source_list_data_model_get_instance_private (CodeWidgetsSourceListDataModel* self)
{
	return G_STRUCT_MEMBER_P (self, CodeWidgetsSourceListDataModel_private_offset);
}

static GType
code_widgets_source_list_data_model_column_type (CodeWidgetsSourceListDataModelColumn self)
{
	GType result;
#line 802 "../src/Widgets/SourceList/SourceList.vala"
	switch (self) {
#line 802 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_COLUMN_ITEM:
#line 4258 "SourceList.c"
		{
#line 804 "../src/Widgets/SourceList/SourceList.vala"
			result = CODE_WIDGETS_SOURCE_LIST_TYPE_ITEM;
#line 804 "../src/Widgets/SourceList/SourceList.vala"
			return result;
#line 4264 "SourceList.c"
		}
		default:
		{
#line 807 "../src/Widgets/SourceList/SourceList.vala"
			g_assert_not_reached ();
#line 4270 "SourceList.c"
		}
	}
}

static GType
code_widgets_source_list_data_model_column_get_type_once (void)
{
	static const GEnumValue values[] = {{CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_COLUMN_ITEM, "CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_COLUMN_ITEM", "item"}, {CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_COLUMN_N_COLUMNS, "CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_COLUMN_N_COLUMNS", "n-columns"}, {0, NULL, NULL}};
	GType code_widgets_source_list_data_model_column_type_id;
	code_widgets_source_list_data_model_column_type_id = g_enum_register_static ("CodeWidgetsSourceListDataModelColumn", values);
	return code_widgets_source_list_data_model_column_type_id;
}

static GType
code_widgets_source_list_data_model_column_get_type (void)
{
	static volatile gsize code_widgets_source_list_data_model_column_type_id__once = 0;
	if (g_once_init_enter (&code_widgets_source_list_data_model_column_type_id__once)) {
		GType code_widgets_source_list_data_model_column_type_id;
		code_widgets_source_list_data_model_column_type_id = code_widgets_source_list_data_model_column_get_type_once ();
		g_once_init_leave (&code_widgets_source_list_data_model_column_type_id__once, code_widgets_source_list_data_model_column_type_id);
	}
	return code_widgets_source_list_data_model_column_type_id__once;
}

static CodeWidgetsSourceListDataModel*
code_widgets_source_list_data_model_construct (GType object_type)
{
	CodeWidgetsSourceListDataModel * self = NULL;
#line 854 "../src/Widgets/SourceList/SourceList.vala"
	self = (CodeWidgetsSourceListDataModel*) g_object_new (object_type, NULL);
#line 854 "../src/Widgets/SourceList/SourceList.vala"
	return self;
#line 4304 "SourceList.c"
}

static CodeWidgetsSourceListDataModel*
code_widgets_source_list_data_model_new (void)
{
#line 854 "../src/Widgets/SourceList/SourceList.vala"
	return code_widgets_source_list_data_model_construct (CODE_WIDGETS_SOURCE_LIST_TYPE_DATA_MODEL);
#line 4312 "SourceList.c"
}

static gboolean
code_widgets_source_list_data_model_has_item (CodeWidgetsSourceListDataModel* self,
                                              CodeWidgetsSourceListItem* item)
{
	GeeHashMap* _tmp0_;
	gboolean result;
#line 869 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 869 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (item != NULL, FALSE);
#line 870 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->items;
#line 870 "../src/Widgets/SourceList/SourceList.vala"
	result = gee_abstract_map_has_key ((GeeAbstractMap*) _tmp0_, item);
#line 870 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 4331 "SourceList.c"
}

static void
code_widgets_source_list_data_model_update_item (CodeWidgetsSourceListDataModel* self,
                                                 CodeWidgetsSourceListItem* item)
{
	CodeWidgetsSourceListExpandableItem* _tmp0_;
	CodeWidgetsSourceListExpandableItem* _tmp1_;
	CodeWidgetsSourceListDataModelNodeWrapper* node_reference = NULL;
	GeeHashMap* _tmp2_;
	gpointer _tmp3_;
	CodeWidgetsSourceListDataModelNodeWrapper* _tmp4_;
#line 873 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 873 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (item != NULL);
#line 873 "../src/Widgets/SourceList/SourceList.vala"
	_vala_return_if_fail (code_widgets_source_list_data_model_has_item (self, item), "has_item (item)");
#line 874 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = code_widgets_source_list_data_model_get_root (self);
#line 874 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = _tmp0_;
#line 874 "../src/Widgets/SourceList/SourceList.vala"
	_vala_assert (_tmp1_ != NULL, "root != null");
#line 879 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = self->priv->items;
#line 879 "../src/Widgets/SourceList/SourceList.vala"
	_tmp3_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp2_, item);
#line 879 "../src/Widgets/SourceList/SourceList.vala"
	node_reference = (CodeWidgetsSourceListDataModelNodeWrapper*) _tmp3_;
#line 880 "../src/Widgets/SourceList/SourceList.vala"
	_tmp4_ = node_reference;
#line 880 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp4_ != NULL) {
#line 4366 "SourceList.c"
		GtkTreePath* path = NULL;
		CodeWidgetsSourceListDataModelNodeWrapper* _tmp5_;
		GtkTreePath* _tmp6_;
		GtkTreePath* _tmp7_;
		GtkTreeIter* iter = NULL;
		CodeWidgetsSourceListDataModelNodeWrapper* _tmp8_;
		GtkTreeIter* _tmp9_;
		GtkTreeIter* _tmp10_;
		gboolean _tmp11_ = FALSE;
		GtkTreePath* _tmp12_;
#line 881 "../src/Widgets/SourceList/SourceList.vala"
		_tmp5_ = node_reference;
#line 881 "../src/Widgets/SourceList/SourceList.vala"
		_tmp6_ = code_widgets_source_list_data_model_node_wrapper_get_path (_tmp5_);
#line 881 "../src/Widgets/SourceList/SourceList.vala"
		_tmp7_ = _tmp6_;
#line 881 "../src/Widgets/SourceList/SourceList.vala"
		path = _tmp7_;
#line 882 "../src/Widgets/SourceList/SourceList.vala"
		_tmp8_ = node_reference;
#line 882 "../src/Widgets/SourceList/SourceList.vala"
		_tmp9_ = code_widgets_source_list_data_model_node_wrapper_get_iter (_tmp8_);
#line 882 "../src/Widgets/SourceList/SourceList.vala"
		_tmp10_ = _tmp9_;
#line 882 "../src/Widgets/SourceList/SourceList.vala"
		iter = _tmp10_;
#line 883 "../src/Widgets/SourceList/SourceList.vala"
		_tmp12_ = path;
#line 883 "../src/Widgets/SourceList/SourceList.vala"
		if (_tmp12_ != NULL) {
#line 4397 "SourceList.c"
			GtkTreeIter* _tmp13_;
#line 883 "../src/Widgets/SourceList/SourceList.vala"
			_tmp13_ = iter;
#line 883 "../src/Widgets/SourceList/SourceList.vala"
			_tmp11_ = _tmp13_ != NULL;
#line 4403 "SourceList.c"
		} else {
#line 883 "../src/Widgets/SourceList/SourceList.vala"
			_tmp11_ = FALSE;
#line 4407 "SourceList.c"
		}
#line 883 "../src/Widgets/SourceList/SourceList.vala"
		if (_tmp11_) {
#line 4411 "SourceList.c"
			GtkTreeStore* _tmp14_;
			GtkTreePath* _tmp15_;
			GtkTreeIter* _tmp16_;
			GtkTreeIter _tmp17_;
#line 884 "../src/Widgets/SourceList/SourceList.vala"
			_tmp14_ = self->priv->child_tree;
#line 884 "../src/Widgets/SourceList/SourceList.vala"
			_tmp15_ = path;
#line 884 "../src/Widgets/SourceList/SourceList.vala"
			_tmp16_ = iter;
#line 884 "../src/Widgets/SourceList/SourceList.vala"
			_tmp17_ = *_tmp16_;
#line 884 "../src/Widgets/SourceList/SourceList.vala"
			gtk_tree_model_row_changed ((GtkTreeModel*) _tmp14_, _tmp15_, &_tmp17_);
#line 885 "../src/Widgets/SourceList/SourceList.vala"
			g_signal_emit (self, code_widgets_source_list_data_model_signals[CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_ITEM_UPDATED_SIGNAL], 0, item);
#line 4428 "SourceList.c"
		}
#line 880 "../src/Widgets/SourceList/SourceList.vala"
		__vala_GtkTreeIter_free0 (iter);
#line 880 "../src/Widgets/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (path);
#line 4434 "SourceList.c"
	}
#line 873 "../src/Widgets/SourceList/SourceList.vala"
	_code_widgets_source_list_data_model_node_wrapper_unref0 (node_reference);
#line 4438 "SourceList.c"
}

static GtkTreeIter*
_vala_GtkTreeIter_copy (GtkTreeIter* self)
{
#line 909 "../src/Widgets/SourceList/SourceList.vala"
	return g_boxed_copy (gtk_tree_iter_get_type (), self);
#line 4446 "SourceList.c"
}

static gpointer
__vala_GtkTreeIter_copy0 (gpointer self)
{
#line 909 "../src/Widgets/SourceList/SourceList.vala"
	return self ? _vala_GtkTreeIter_copy (self) : NULL;
#line 4454 "SourceList.c"
}

static void
code_widgets_source_list_data_model_add_item (CodeWidgetsSourceListDataModel* self,
                                              CodeWidgetsSourceListItem* item)
{
	CodeWidgetsSourceListExpandableItem* _tmp0_;
	CodeWidgetsSourceListExpandableItem* _tmp1_;
	GtkTreeIter* parent_child_iter = NULL;
	GtkTreeIter* child_iter = NULL;
	CodeWidgetsSourceListExpandableItem* parent = NULL;
	CodeWidgetsSourceListExpandableItem* _tmp2_;
	CodeWidgetsSourceListExpandableItem* _tmp3_;
	CodeWidgetsSourceListExpandableItem* _tmp4_;
	gboolean _tmp5_ = FALSE;
	CodeWidgetsSourceListExpandableItem* _tmp6_;
	GtkTreeStore* _tmp15_;
	GtkTreeIter* _tmp16_;
	GtkTreeIter _tmp17_ = {0};
	GtkTreeIter _tmp18_;
	GtkTreeIter* _tmp19_;
	GtkTreeStore* _tmp20_;
	GtkTreeIter* _tmp21_;
	GtkTreeIter _tmp22_;
	GeeHashMap* _tmp23_;
	GtkTreeStore* _tmp24_;
	GtkTreeIter* _tmp25_;
	GtkTreeIter _tmp26_;
	CodeWidgetsSourceListDataModelNodeWrapper* _tmp27_;
	CodeWidgetsSourceListDataModelNodeWrapper* _tmp28_;
	CodeWidgetsSourceListExpandableItem* _tmp29_;
	CodeWidgetsSourceListExpandableItem* expandable = NULL;
	CodeWidgetsSourceListExpandableItem* _tmp30_;
	CodeWidgetsSourceListExpandableItem* _tmp31_;
#line 890 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 890 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (item != NULL);
#line 890 "../src/Widgets/SourceList/SourceList.vala"
	_vala_return_if_fail (!code_widgets_source_list_data_model_has_item (self, item), "!has_item (item)");
#line 891 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = code_widgets_source_list_data_model_get_root (self);
#line 891 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = _tmp0_;
#line 891 "../src/Widgets/SourceList/SourceList.vala"
	_vala_assert (_tmp1_ != NULL, "root != null");
#line 894 "../src/Widgets/SourceList/SourceList.vala"
	parent_child_iter = NULL;
#line 895 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = code_widgets_source_list_item_get_parent (item);
#line 895 "../src/Widgets/SourceList/SourceList.vala"
	_tmp3_ = _tmp2_;
#line 895 "../src/Widgets/SourceList/SourceList.vala"
	_tmp4_ = _g_object_ref0 (_tmp3_);
#line 895 "../src/Widgets/SourceList/SourceList.vala"
	parent = _tmp4_;
#line 897 "../src/Widgets/SourceList/SourceList.vala"
	_tmp6_ = parent;
#line 897 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp6_ != NULL) {
#line 4515 "SourceList.c"
		CodeWidgetsSourceListExpandableItem* _tmp7_;
		CodeWidgetsSourceListExpandableItem* _tmp8_;
		CodeWidgetsSourceListExpandableItem* _tmp9_;
#line 897 "../src/Widgets/SourceList/SourceList.vala"
		_tmp7_ = parent;
#line 897 "../src/Widgets/SourceList/SourceList.vala"
		_tmp8_ = code_widgets_source_list_data_model_get_root (self);
#line 897 "../src/Widgets/SourceList/SourceList.vala"
		_tmp9_ = _tmp8_;
#line 897 "../src/Widgets/SourceList/SourceList.vala"
		_tmp5_ = _tmp7_ != _tmp9_;
#line 4527 "SourceList.c"
	} else {
#line 897 "../src/Widgets/SourceList/SourceList.vala"
		_tmp5_ = FALSE;
#line 4531 "SourceList.c"
	}
#line 897 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp5_) {
#line 4535 "SourceList.c"
		CodeWidgetsSourceListExpandableItem* _tmp10_;
		CodeWidgetsSourceListExpandableItem* _tmp12_;
		GtkTreeIter* _tmp13_;
		GtkTreeIter* _tmp14_;
#line 899 "../src/Widgets/SourceList/SourceList.vala"
		_tmp10_ = parent;
#line 899 "../src/Widgets/SourceList/SourceList.vala"
		if (!code_widgets_source_list_data_model_has_item (self, (CodeWidgetsSourceListItem*) _tmp10_)) {
#line 4544 "SourceList.c"
			CodeWidgetsSourceListExpandableItem* _tmp11_;
#line 900 "../src/Widgets/SourceList/SourceList.vala"
			_tmp11_ = parent;
#line 900 "../src/Widgets/SourceList/SourceList.vala"
			code_widgets_source_list_data_model_add_item (self, (CodeWidgetsSourceListItem*) _tmp11_);
#line 4550 "SourceList.c"
		}
#line 903 "../src/Widgets/SourceList/SourceList.vala"
		_tmp12_ = parent;
#line 903 "../src/Widgets/SourceList/SourceList.vala"
		_tmp13_ = code_widgets_source_list_data_model_get_item_child_iter (self, (CodeWidgetsSourceListItem*) _tmp12_);
#line 903 "../src/Widgets/SourceList/SourceList.vala"
		__vala_GtkTreeIter_free0 (parent_child_iter);
#line 903 "../src/Widgets/SourceList/SourceList.vala"
		parent_child_iter = _tmp13_;
#line 906 "../src/Widgets/SourceList/SourceList.vala"
		_tmp14_ = parent_child_iter;
#line 906 "../src/Widgets/SourceList/SourceList.vala"
		_vala_assert (_tmp14_ != NULL, "parent_child_iter != null");
#line 4564 "SourceList.c"
	}
#line 909 "../src/Widgets/SourceList/SourceList.vala"
	_tmp15_ = self->priv->child_tree;
#line 909 "../src/Widgets/SourceList/SourceList.vala"
	_tmp16_ = parent_child_iter;
#line 909 "../src/Widgets/SourceList/SourceList.vala"
	gtk_tree_store_append (_tmp15_, &_tmp17_, _tmp16_);
#line 909 "../src/Widgets/SourceList/SourceList.vala"
	__vala_GtkTreeIter_free0 (child_iter);
#line 909 "../src/Widgets/SourceList/SourceList.vala"
	_tmp18_ = _tmp17_;
#line 909 "../src/Widgets/SourceList/SourceList.vala"
	_tmp19_ = __vala_GtkTreeIter_copy0 (&_tmp18_);
#line 909 "../src/Widgets/SourceList/SourceList.vala"
	child_iter = _tmp19_;
#line 910 "../src/Widgets/SourceList/SourceList.vala"
	_tmp20_ = self->priv->child_tree;
#line 910 "../src/Widgets/SourceList/SourceList.vala"
	_tmp21_ = child_iter;
#line 910 "../src/Widgets/SourceList/SourceList.vala"
	_tmp22_ = *_tmp21_;
#line 910 "../src/Widgets/SourceList/SourceList.vala"
	gtk_tree_store_set (_tmp20_, &_tmp22_, CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_COLUMN_ITEM, item, -1, -1);
#line 912 "../src/Widgets/SourceList/SourceList.vala"
	_tmp23_ = self->priv->items;
#line 912 "../src/Widgets/SourceList/SourceList.vala"
	_tmp24_ = self->priv->child_tree;
#line 912 "../src/Widgets/SourceList/SourceList.vala"
	_tmp25_ = child_iter;
#line 912 "../src/Widgets/SourceList/SourceList.vala"
	_tmp26_ = *_tmp25_;
#line 912 "../src/Widgets/SourceList/SourceList.vala"
	_tmp27_ = code_widgets_source_list_data_model_node_wrapper_new ((GtkTreeModel*) _tmp24_, &_tmp26_);
#line 912 "../src/Widgets/SourceList/SourceList.vala"
	_tmp28_ = _tmp27_;
#line 912 "../src/Widgets/SourceList/SourceList.vala"
	gee_abstract_map_set ((GeeAbstractMap*) _tmp23_, item, _tmp28_);
#line 912 "../src/Widgets/SourceList/SourceList.vala"
	_code_widgets_source_list_data_model_node_wrapper_unref0 (_tmp28_);
#line 916 "../src/Widgets/SourceList/SourceList.vala"
	g_signal_emit (self, code_widgets_source_list_data_model_signals[CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_ITEM_UPDATED_SIGNAL], 0, item);
#line 918 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_data_model_add_property_monitor (self, item);
#line 920 "../src/Widgets/SourceList/SourceList.vala"
	_tmp29_ = parent;
#line 920 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_data_model_push_parent_update (self, _tmp29_);
#line 923 "../src/Widgets/SourceList/SourceList.vala"
	_tmp30_ = _g_object_ref0 (CODE_WIDGETS_SOURCE_LIST_IS_EXPANDABLE_ITEM (item) ? ((CodeWidgetsSourceListExpandableItem*) item) : NULL);
#line 923 "../src/Widgets/SourceList/SourceList.vala"
	expandable = _tmp30_;
#line 924 "../src/Widgets/SourceList/SourceList.vala"
	_tmp31_ = expandable;
#line 924 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp31_ != NULL) {
#line 4620 "SourceList.c"
		CodeWidgetsSourceListExpandableItem* _tmp42_;
		{
			GeeIterator* _child_item_it = NULL;
			CodeWidgetsSourceListExpandableItem* _tmp32_;
			GeeCollection* _tmp33_;
			GeeCollection* _tmp34_;
			GeeCollection* _tmp35_;
			GeeIterator* _tmp36_;
			GeeIterator* _tmp37_;
#line 925 "../src/Widgets/SourceList/SourceList.vala"
			_tmp32_ = expandable;
#line 925 "../src/Widgets/SourceList/SourceList.vala"
			_tmp33_ = code_widgets_source_list_expandable_item_get_children (_tmp32_);
#line 925 "../src/Widgets/SourceList/SourceList.vala"
			_tmp34_ = _tmp33_;
#line 925 "../src/Widgets/SourceList/SourceList.vala"
			_tmp35_ = _tmp34_;
#line 925 "../src/Widgets/SourceList/SourceList.vala"
			_tmp36_ = gee_iterable_iterator ((GeeIterable*) _tmp35_);
#line 925 "../src/Widgets/SourceList/SourceList.vala"
			_tmp37_ = _tmp36_;
#line 925 "../src/Widgets/SourceList/SourceList.vala"
			_g_object_unref0 (_tmp35_);
#line 925 "../src/Widgets/SourceList/SourceList.vala"
			_child_item_it = _tmp37_;
#line 925 "../src/Widgets/SourceList/SourceList.vala"
			while (TRUE) {
#line 4648 "SourceList.c"
				GeeIterator* _tmp38_;
				CodeWidgetsSourceListItem* child_item = NULL;
				GeeIterator* _tmp39_;
				gpointer _tmp40_;
				CodeWidgetsSourceListItem* _tmp41_;
#line 925 "../src/Widgets/SourceList/SourceList.vala"
				_tmp38_ = _child_item_it;
#line 925 "../src/Widgets/SourceList/SourceList.vala"
				if (!gee_iterator_next (_tmp38_)) {
#line 925 "../src/Widgets/SourceList/SourceList.vala"
					break;
#line 4660 "SourceList.c"
				}
#line 925 "../src/Widgets/SourceList/SourceList.vala"
				_tmp39_ = _child_item_it;
#line 925 "../src/Widgets/SourceList/SourceList.vala"
				_tmp40_ = gee_iterator_get (_tmp39_);
#line 925 "../src/Widgets/SourceList/SourceList.vala"
				child_item = (CodeWidgetsSourceListItem*) _tmp40_;
#line 926 "../src/Widgets/SourceList/SourceList.vala"
				_tmp41_ = child_item;
#line 926 "../src/Widgets/SourceList/SourceList.vala"
				code_widgets_source_list_data_model_add_item (self, _tmp41_);
#line 925 "../src/Widgets/SourceList/SourceList.vala"
				_g_object_unref0 (child_item);
#line 4674 "SourceList.c"
			}
#line 925 "../src/Widgets/SourceList/SourceList.vala"
			_g_object_unref0 (_child_item_it);
#line 4678 "SourceList.c"
		}
#line 929 "../src/Widgets/SourceList/SourceList.vala"
		_tmp42_ = expandable;
#line 929 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_data_model_add_children_monitor (self, _tmp42_);
#line 4684 "SourceList.c"
	}
#line 890 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (expandable);
#line 890 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (parent);
#line 890 "../src/Widgets/SourceList/SourceList.vala"
	__vala_GtkTreeIter_free0 (child_iter);
#line 890 "../src/Widgets/SourceList/SourceList.vala"
	__vala_GtkTreeIter_free0 (parent_child_iter);
#line 4694 "SourceList.c"
}

static void
code_widgets_source_list_data_model_remove_item (CodeWidgetsSourceListDataModel* self,
                                                 CodeWidgetsSourceListItem* item)
{
	CodeWidgetsSourceListExpandableItem* _tmp0_;
	CodeWidgetsSourceListExpandableItem* _tmp1_;
	GtkTreeIter* child_iter = NULL;
	GtkTreeIter* _tmp2_;
	GeeHashMap* _tmp3_;
	GtkTreeIter* _tmp4_;
	CodeWidgetsSourceListExpandableItem* _tmp6_;
	CodeWidgetsSourceListExpandableItem* _tmp7_;
	CodeWidgetsSourceListExpandableItem* expandable = NULL;
	CodeWidgetsSourceListExpandableItem* _tmp8_;
	CodeWidgetsSourceListExpandableItem* _tmp9_;
#line 933 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 933 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (item != NULL);
#line 933 "../src/Widgets/SourceList/SourceList.vala"
	_vala_return_if_fail (code_widgets_source_list_data_model_has_item (self, item), "has_item (item)");
#line 934 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = code_widgets_source_list_data_model_get_root (self);
#line 934 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = _tmp0_;
#line 934 "../src/Widgets/SourceList/SourceList.vala"
	_vala_assert (_tmp1_ != NULL, "root != null");
#line 936 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_data_model_remove_property_monitor (self, item);
#line 941 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = code_widgets_source_list_data_model_get_item_child_iter (self, item);
#line 941 "../src/Widgets/SourceList/SourceList.vala"
	child_iter = _tmp2_;
#line 947 "../src/Widgets/SourceList/SourceList.vala"
	_tmp3_ = self->priv->items;
#line 947 "../src/Widgets/SourceList/SourceList.vala"
	gee_abstract_map_unset ((GeeAbstractMap*) _tmp3_, item, NULL);
#line 949 "../src/Widgets/SourceList/SourceList.vala"
	_tmp4_ = child_iter;
#line 949 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp4_ != NULL) {
#line 4738 "SourceList.c"
		GtkTreeStore* _tmp5_;
#line 950 "../src/Widgets/SourceList/SourceList.vala"
		_tmp5_ = self->priv->child_tree;
#line 950 "../src/Widgets/SourceList/SourceList.vala"
		gtk_tree_store_remove (_tmp5_, child_iter);
#line 4744 "SourceList.c"
	}
#line 952 "../src/Widgets/SourceList/SourceList.vala"
	_tmp6_ = code_widgets_source_list_item_get_parent (item);
#line 952 "../src/Widgets/SourceList/SourceList.vala"
	_tmp7_ = _tmp6_;
#line 952 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_data_model_push_parent_update (self, _tmp7_);
#line 955 "../src/Widgets/SourceList/SourceList.vala"
	_tmp8_ = _g_object_ref0 (CODE_WIDGETS_SOURCE_LIST_IS_EXPANDABLE_ITEM (item) ? ((CodeWidgetsSourceListExpandableItem*) item) : NULL);
#line 955 "../src/Widgets/SourceList/SourceList.vala"
	expandable = _tmp8_;
#line 956 "../src/Widgets/SourceList/SourceList.vala"
	_tmp9_ = expandable;
#line 956 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp9_ != NULL) {
#line 4760 "SourceList.c"
		CodeWidgetsSourceListExpandableItem* _tmp10_;
#line 958 "../src/Widgets/SourceList/SourceList.vala"
		_tmp10_ = expandable;
#line 958 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_data_model_remove_children_monitor (self, _tmp10_);
#line 4766 "SourceList.c"
		{
			GeeIterator* _child_item_it = NULL;
			CodeWidgetsSourceListExpandableItem* _tmp11_;
			GeeCollection* _tmp12_;
			GeeCollection* _tmp13_;
			GeeCollection* _tmp14_;
			GeeIterator* _tmp15_;
			GeeIterator* _tmp16_;
#line 960 "../src/Widgets/SourceList/SourceList.vala"
			_tmp11_ = expandable;
#line 960 "../src/Widgets/SourceList/SourceList.vala"
			_tmp12_ = code_widgets_source_list_expandable_item_get_children (_tmp11_);
#line 960 "../src/Widgets/SourceList/SourceList.vala"
			_tmp13_ = _tmp12_;
#line 960 "../src/Widgets/SourceList/SourceList.vala"
			_tmp14_ = _tmp13_;
#line 960 "../src/Widgets/SourceList/SourceList.vala"
			_tmp15_ = gee_iterable_iterator ((GeeIterable*) _tmp14_);
#line 960 "../src/Widgets/SourceList/SourceList.vala"
			_tmp16_ = _tmp15_;
#line 960 "../src/Widgets/SourceList/SourceList.vala"
			_g_object_unref0 (_tmp14_);
#line 960 "../src/Widgets/SourceList/SourceList.vala"
			_child_item_it = _tmp16_;
#line 960 "../src/Widgets/SourceList/SourceList.vala"
			while (TRUE) {
#line 4793 "SourceList.c"
				GeeIterator* _tmp17_;
				CodeWidgetsSourceListItem* child_item = NULL;
				GeeIterator* _tmp18_;
				gpointer _tmp19_;
				CodeWidgetsSourceListItem* _tmp20_;
#line 960 "../src/Widgets/SourceList/SourceList.vala"
				_tmp17_ = _child_item_it;
#line 960 "../src/Widgets/SourceList/SourceList.vala"
				if (!gee_iterator_next (_tmp17_)) {
#line 960 "../src/Widgets/SourceList/SourceList.vala"
					break;
#line 4805 "SourceList.c"
				}
#line 960 "../src/Widgets/SourceList/SourceList.vala"
				_tmp18_ = _child_item_it;
#line 960 "../src/Widgets/SourceList/SourceList.vala"
				_tmp19_ = gee_iterator_get (_tmp18_);
#line 960 "../src/Widgets/SourceList/SourceList.vala"
				child_item = (CodeWidgetsSourceListItem*) _tmp19_;
#line 961 "../src/Widgets/SourceList/SourceList.vala"
				_tmp20_ = child_item;
#line 961 "../src/Widgets/SourceList/SourceList.vala"
				code_widgets_source_list_data_model_remove_item (self, _tmp20_);
#line 960 "../src/Widgets/SourceList/SourceList.vala"
				_g_object_unref0 (child_item);
#line 4819 "SourceList.c"
			}
#line 960 "../src/Widgets/SourceList/SourceList.vala"
			_g_object_unref0 (_child_item_it);
#line 4823 "SourceList.c"
		}
	}
#line 933 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (expandable);
#line 933 "../src/Widgets/SourceList/SourceList.vala"
	__vala_GtkTreeIter_free0 (child_iter);
#line 4830 "SourceList.c"
}

static void
_code_widgets_source_list_data_model_on_item_prop_changed_code_widgets_source_list_data_model_item_monitor_changed (CodeWidgetsSourceListDataModelItemMonitor* _sender,
                                                                                                                    CodeWidgetsSourceListItem* _self_,
                                                                                                                    const gchar* prop_name,
                                                                                                                    gpointer self)
{
#line 968 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_data_model_on_item_prop_changed ((CodeWidgetsSourceListDataModel*) self, _self_, prop_name);
#line 4841 "SourceList.c"
}

static void
code_widgets_source_list_data_model_add_property_monitor (CodeWidgetsSourceListDataModel* self,
                                                          CodeWidgetsSourceListItem* item)
{
	CodeWidgetsSourceListDataModelItemMonitor* wrapper = NULL;
	CodeWidgetsSourceListDataModelItemMonitor* _tmp0_;
	GeeHashMap* _tmp1_;
#line 965 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 965 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (item != NULL);
#line 966 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = code_widgets_source_list_data_model_item_monitor_new (item);
#line 966 "../src/Widgets/SourceList/SourceList.vala"
	wrapper = _tmp0_;
#line 967 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = self->priv->monitors;
#line 967 "../src/Widgets/SourceList/SourceList.vala"
	gee_abstract_map_set ((GeeAbstractMap*) _tmp1_, item, wrapper);
#line 968 "../src/Widgets/SourceList/SourceList.vala"
	g_signal_connect_object (wrapper, "changed", (GCallback) _code_widgets_source_list_data_model_on_item_prop_changed_code_widgets_source_list_data_model_item_monitor_changed, self, 0);
#line 965 "../src/Widgets/SourceList/SourceList.vala"
	_code_widgets_source_list_data_model_item_monitor_unref0 (wrapper);
#line 4867 "SourceList.c"
}

static void
code_widgets_source_list_data_model_remove_property_monitor (CodeWidgetsSourceListDataModel* self,
                                                             CodeWidgetsSourceListItem* item)
{
	CodeWidgetsSourceListDataModelItemMonitor* wrapper = NULL;
	GeeHashMap* _tmp0_;
	gpointer _tmp1_;
	CodeWidgetsSourceListDataModelItemMonitor* _tmp2_;
	GeeHashMap* _tmp5_;
#line 971 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 971 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (item != NULL);
#line 972 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->monitors;
#line 972 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp0_, item);
#line 972 "../src/Widgets/SourceList/SourceList.vala"
	wrapper = (CodeWidgetsSourceListDataModelItemMonitor*) _tmp1_;
#line 973 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = wrapper;
#line 973 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp2_ != NULL) {
#line 4893 "SourceList.c"
		CodeWidgetsSourceListDataModelItemMonitor* _tmp3_;
		guint _tmp4_;
#line 974 "../src/Widgets/SourceList/SourceList.vala"
		_tmp3_ = wrapper;
#line 974 "../src/Widgets/SourceList/SourceList.vala"
		g_signal_parse_name ("changed", CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_TYPE_ITEM_MONITOR, &_tmp4_, NULL, FALSE);
#line 974 "../src/Widgets/SourceList/SourceList.vala"
		g_signal_handlers_disconnect_matched (_tmp3_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp4_, 0, NULL, (GCallback) _code_widgets_source_list_data_model_on_item_prop_changed_code_widgets_source_list_data_model_item_monitor_changed, self);
#line 4902 "SourceList.c"
	}
#line 975 "../src/Widgets/SourceList/SourceList.vala"
	_tmp5_ = self->priv->monitors;
#line 975 "../src/Widgets/SourceList/SourceList.vala"
	gee_abstract_map_unset ((GeeAbstractMap*) _tmp5_, item, NULL);
#line 971 "../src/Widgets/SourceList/SourceList.vala"
	_code_widgets_source_list_data_model_item_monitor_unref0 (wrapper);
#line 4910 "SourceList.c"
}

static void
_code_widgets_source_list_data_model_on_item_child_added_code_widgets_source_list_expandable_item_child_added (CodeWidgetsSourceListExpandableItem* _sender,
                                                                                                               CodeWidgetsSourceListItem* item,
                                                                                                               gpointer self)
{
#line 979 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_data_model_on_item_child_added ((CodeWidgetsSourceListDataModel*) self, item);
#line 4920 "SourceList.c"
}

static void
_code_widgets_source_list_data_model_on_item_child_removed_code_widgets_source_list_expandable_item_child_removed (CodeWidgetsSourceListExpandableItem* _sender,
                                                                                                                   CodeWidgetsSourceListItem* item,
                                                                                                                   gpointer self)
{
#line 980 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_data_model_on_item_child_removed ((CodeWidgetsSourceListDataModel*) self, item);
#line 4930 "SourceList.c"
}

static void
code_widgets_source_list_data_model_add_children_monitor (CodeWidgetsSourceListDataModel* self,
                                                          CodeWidgetsSourceListExpandableItem* item)
{
#line 978 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 978 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (item != NULL);
#line 979 "../src/Widgets/SourceList/SourceList.vala"
	g_signal_connect_object (item, "child-added", (GCallback) _code_widgets_source_list_data_model_on_item_child_added_code_widgets_source_list_expandable_item_child_added, self, G_CONNECT_AFTER);
#line 980 "../src/Widgets/SourceList/SourceList.vala"
	g_signal_connect_object (item, "child-removed", (GCallback) _code_widgets_source_list_data_model_on_item_child_removed_code_widgets_source_list_expandable_item_child_removed, self, G_CONNECT_AFTER);
#line 4945 "SourceList.c"
}

static void
code_widgets_source_list_data_model_remove_children_monitor (CodeWidgetsSourceListDataModel* self,
                                                             CodeWidgetsSourceListExpandableItem* item)
{
	guint _tmp0_;
	guint _tmp1_;
#line 983 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 983 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (item != NULL);
#line 984 "../src/Widgets/SourceList/SourceList.vala"
	g_signal_parse_name ("child-added", CODE_WIDGETS_SOURCE_LIST_TYPE_EXPANDABLE_ITEM, &_tmp0_, NULL, FALSE);
#line 984 "../src/Widgets/SourceList/SourceList.vala"
	g_signal_handlers_disconnect_matched (item, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp0_, 0, NULL, (GCallback) _code_widgets_source_list_data_model_on_item_child_added_code_widgets_source_list_expandable_item_child_added, self);
#line 985 "../src/Widgets/SourceList/SourceList.vala"
	g_signal_parse_name ("child-removed", CODE_WIDGETS_SOURCE_LIST_TYPE_EXPANDABLE_ITEM, &_tmp1_, NULL, FALSE);
#line 985 "../src/Widgets/SourceList/SourceList.vala"
	g_signal_handlers_disconnect_matched (item, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp1_, 0, NULL, (GCallback) _code_widgets_source_list_data_model_on_item_child_removed_code_widgets_source_list_expandable_item_child_removed, self);
#line 4966 "SourceList.c"
}

static void
code_widgets_source_list_data_model_on_item_child_added (CodeWidgetsSourceListDataModel* self,
                                                         CodeWidgetsSourceListItem* item)
{
#line 988 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 988 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (item != NULL);
#line 989 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_data_model_add_item (self, item);
#line 4979 "SourceList.c"
}

static void
code_widgets_source_list_data_model_on_item_child_removed (CodeWidgetsSourceListDataModel* self,
                                                           CodeWidgetsSourceListItem* item)
{
#line 992 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 992 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (item != NULL);
#line 993 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_data_model_remove_item (self, item);
#line 4992 "SourceList.c"
}

static void
code_widgets_source_list_data_model_on_item_prop_changed (CodeWidgetsSourceListDataModel* self,
                                                          CodeWidgetsSourceListItem* item,
                                                          const gchar* prop_name)
{
#line 996 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 996 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (item != NULL);
#line 996 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (prop_name != NULL);
#line 997 "../src/Widgets/SourceList/SourceList.vala"
	if (g_strcmp0 (prop_name, "parent") != 0) {
#line 998 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_data_model_update_item (self, item);
#line 5010 "SourceList.c"
	}
}

/**
         * Pushes a call to update_item() if //parent// is not //null//.
         *
         * This is needed because the visibility of categories depends on their n_children property,
         * and also because item expansion should be updated after adding or removing items.
         * If many updates are pushed, and the item has still not been updated, only one is processed.
         * This guarantees efficiency as updating a category item could trigger expensive actions.
         */
static Block37Data*
block37_data_ref (Block37Data* _data37_)
{
#line 1009 "../src/Widgets/SourceList/SourceList.vala"
	g_atomic_int_inc (&_data37_->_ref_count_);
#line 1009 "../src/Widgets/SourceList/SourceList.vala"
	return _data37_;
#line 5029 "SourceList.c"
}

static void
block37_data_unref (void * _userdata_)
{
	Block37Data* _data37_;
	_data37_ = (Block37Data*) _userdata_;
#line 1009 "../src/Widgets/SourceList/SourceList.vala"
	if (g_atomic_int_dec_and_test (&_data37_->_ref_count_)) {
#line 5039 "SourceList.c"
		CodeWidgetsSourceListDataModel* self;
#line 1009 "../src/Widgets/SourceList/SourceList.vala"
		self = _data37_->self;
#line 1009 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (_data37_->parent);
#line 1009 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (self);
#line 1009 "../src/Widgets/SourceList/SourceList.vala"
		g_slice_free (Block37Data, _data37_);
#line 5049 "SourceList.c"
	}
}

static gboolean
___lambda4_ (Block37Data* _data37_)
{
	CodeWidgetsSourceListDataModel* self;
	gboolean result;
#line 1026 "../src/Widgets/SourceList/SourceList.vala"
	self = _data37_->self;
#line 1027 "../src/Widgets/SourceList/SourceList.vala"
	if (_data37_->parent != NULL) {
#line 1028 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_data_model_update_item (self, (CodeWidgetsSourceListItem*) _data37_->parent);
#line 1031 "../src/Widgets/SourceList/SourceList.vala"
		g_object_set_data_full ((GObject*) _data37_->parent, CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_ITEM_PARENT_NEEDS_UPDATE, (gpointer) ((gintptr) FALSE), NULL);
#line 5066 "SourceList.c"
	}
#line 1034 "../src/Widgets/SourceList/SourceList.vala"
	result = FALSE;
#line 1034 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 5072 "SourceList.c"
}

static gboolean
____lambda4__gsource_func (gpointer self)
{
	gboolean result;
	result = ___lambda4_ (self);
#line 1026 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 5082 "SourceList.c"
}

static void
code_widgets_source_list_data_model_push_parent_update (CodeWidgetsSourceListDataModel* self,
                                                        CodeWidgetsSourceListExpandableItem* parent)
{
	Block37Data* _data37_;
	CodeWidgetsSourceListExpandableItem* _tmp0_;
	gboolean needs_update = FALSE;
	gconstpointer _tmp1_;
	GtkTreePath* path = NULL;
	GtkTreePath* _tmp2_;
	GtkTreePath* _tmp3_;
#line 1009 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 1009 "../src/Widgets/SourceList/SourceList.vala"
	_data37_ = g_slice_new0 (Block37Data);
#line 1009 "../src/Widgets/SourceList/SourceList.vala"
	_data37_->_ref_count_ = 1;
#line 1009 "../src/Widgets/SourceList/SourceList.vala"
	_data37_->self = g_object_ref (self);
#line 1009 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = _g_object_ref0 (parent);
#line 1009 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (_data37_->parent);
#line 1009 "../src/Widgets/SourceList/SourceList.vala"
	_data37_->parent = _tmp0_;
#line 1010 "../src/Widgets/SourceList/SourceList.vala"
	if (_data37_->parent == NULL) {
#line 1011 "../src/Widgets/SourceList/SourceList.vala"
		block37_data_unref (_data37_);
#line 1011 "../src/Widgets/SourceList/SourceList.vala"
		_data37_ = NULL;
#line 1011 "../src/Widgets/SourceList/SourceList.vala"
		return;
#line 5118 "SourceList.c"
	}
#line 1013 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = g_object_get_data ((GObject*) _data37_->parent, CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_ITEM_PARENT_NEEDS_UPDATE);
#line 1013 "../src/Widgets/SourceList/SourceList.vala"
	needs_update = (gboolean) ((gintptr) _tmp1_);
#line 1017 "../src/Widgets/SourceList/SourceList.vala"
	if (needs_update) {
#line 1018 "../src/Widgets/SourceList/SourceList.vala"
		block37_data_unref (_data37_);
#line 1018 "../src/Widgets/SourceList/SourceList.vala"
		_data37_ = NULL;
#line 1018 "../src/Widgets/SourceList/SourceList.vala"
		return;
#line 5132 "SourceList.c"
	}
#line 1020 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = code_widgets_source_list_data_model_get_item_path (self, (CodeWidgetsSourceListItem*) _data37_->parent);
#line 1020 "../src/Widgets/SourceList/SourceList.vala"
	path = _tmp2_;
#line 1022 "../src/Widgets/SourceList/SourceList.vala"
	_tmp3_ = path;
#line 1022 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp3_ != NULL) {
#line 1024 "../src/Widgets/SourceList/SourceList.vala"
		g_object_set_data_full ((GObject*) _data37_->parent, CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_ITEM_PARENT_NEEDS_UPDATE, (gpointer) ((gintptr) TRUE), NULL);
#line 1026 "../src/Widgets/SourceList/SourceList.vala"
		g_idle_add_full (G_PRIORITY_DEFAULT_IDLE, ____lambda4__gsource_func, block37_data_ref (_data37_), block37_data_unref);
#line 5146 "SourceList.c"
	}
#line 1009 "../src/Widgets/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (path);
#line 1009 "../src/Widgets/SourceList/SourceList.vala"
	block37_data_unref (_data37_);
#line 1009 "../src/Widgets/SourceList/SourceList.vala"
	_data37_ = NULL;
#line 5154 "SourceList.c"
}

/**
         * Returns the Item pointed by iter, or null if the iter doesn't refer to a valid item.
         */
static CodeWidgetsSourceListItem*
code_widgets_source_list_data_model_get_item (CodeWidgetsSourceListDataModel* self,
                                              GtkTreeIter* iter)
{
	CodeWidgetsSourceListItem* item = NULL;
	GtkTreeIter _tmp0_;
	CodeWidgetsSourceListItem* _tmp1_ = NULL;
	CodeWidgetsSourceListItem* result;
#line 1042 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 1042 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (iter != NULL, NULL);
#line 1044 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = *iter;
#line 1044 "../src/Widgets/SourceList/SourceList.vala"
	gtk_tree_model_get ((GtkTreeModel*) self, &_tmp0_, CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_COLUMN_ITEM, &_tmp1_, -1, -1);
#line 1044 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (item);
#line 1044 "../src/Widgets/SourceList/SourceList.vala"
	item = _tmp1_;
#line 1045 "../src/Widgets/SourceList/SourceList.vala"
	result = item;
#line 1045 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 5184 "SourceList.c"
}

/**
         * Returns the Item pointed by path, or null if the path doesn't refer to a valid item.
         */
static CodeWidgetsSourceListItem*
code_widgets_source_list_data_model_get_item_from_path (CodeWidgetsSourceListDataModel* self,
                                                        GtkTreePath* path)
{
	GtkTreeIter iter = {0};
	GtkTreeIter _tmp0_ = {0};
	gboolean _tmp1_;
	CodeWidgetsSourceListItem* result;
#line 1051 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 1051 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (path != NULL, NULL);
#line 1053 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = gtk_tree_model_get_iter ((GtkTreeModel*) self, &_tmp0_, path);
#line 1053 "../src/Widgets/SourceList/SourceList.vala"
	iter = _tmp0_;
#line 1053 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp1_) {
#line 5208 "SourceList.c"
		GtkTreeIter _tmp2_;
		CodeWidgetsSourceListItem* _tmp3_;
#line 1054 "../src/Widgets/SourceList/SourceList.vala"
		_tmp2_ = iter;
#line 1054 "../src/Widgets/SourceList/SourceList.vala"
		_tmp3_ = code_widgets_source_list_data_model_get_item (self, &_tmp2_);
#line 1054 "../src/Widgets/SourceList/SourceList.vala"
		result = _tmp3_;
#line 1054 "../src/Widgets/SourceList/SourceList.vala"
		return result;
#line 5219 "SourceList.c"
	}
#line 1056 "../src/Widgets/SourceList/SourceList.vala"
	result = NULL;
#line 1056 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 5225 "SourceList.c"
}

/**
         * Returns a newly-created path pointing to the item, or null in case a valid path
         * is not found.
         */
static GtkTreePath*
code_widgets_source_list_data_model_get_item_path (CodeWidgetsSourceListDataModel* self,
                                                   CodeWidgetsSourceListItem* item)
{
	GtkTreePath* path = NULL;
	GtkTreePath* child_path = NULL;
	GtkTreePath* _tmp0_;
	GtkTreePath* _tmp1_;
	GtkTreePath* result;
#line 1063 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 1063 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (item != NULL, NULL);
#line 1064 "../src/Widgets/SourceList/SourceList.vala"
	path = NULL;
#line 1064 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = code_widgets_source_list_data_model_get_item_child_path (self, item);
#line 1064 "../src/Widgets/SourceList/SourceList.vala"
	child_path = _tmp0_;
#line 1067 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = child_path;
#line 1067 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp1_ != NULL) {
#line 5255 "SourceList.c"
		GtkTreePath* _tmp2_;
		GtkTreePath* _tmp3_;
#line 1068 "../src/Widgets/SourceList/SourceList.vala"
		_tmp2_ = child_path;
#line 1068 "../src/Widgets/SourceList/SourceList.vala"
		_tmp3_ = gtk_tree_model_filter_convert_child_path_to_path ((GtkTreeModelFilter*) self, _tmp2_);
#line 1068 "../src/Widgets/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (path);
#line 1068 "../src/Widgets/SourceList/SourceList.vala"
		path = _tmp3_;
#line 5266 "SourceList.c"
	}
#line 1070 "../src/Widgets/SourceList/SourceList.vala"
	result = path;
#line 1070 "../src/Widgets/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (child_path);
#line 1070 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 5274 "SourceList.c"
}

/**
         * Returns a newly-created iterator pointing to the item, or null in case a valid iter
         * was not found.
         */
static GtkTreeIter*
code_widgets_source_list_data_model_get_item_iter (CodeWidgetsSourceListDataModel* self,
                                                   CodeWidgetsSourceListItem* item)
{
	GtkTreeIter* child_iter = NULL;
	GtkTreeIter* _tmp0_;
	GtkTreeIter* _tmp1_;
	GtkTreeIter* result;
#line 1077 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 1077 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (item != NULL, NULL);
#line 1078 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = code_widgets_source_list_data_model_get_item_child_iter (self, item);
#line 1078 "../src/Widgets/SourceList/SourceList.vala"
	child_iter = _tmp0_;
#line 1080 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = child_iter;
#line 1080 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp1_ != NULL) {
#line 5301 "SourceList.c"
		GtkTreeIter iter = {0};
		GtkTreeIter* _tmp2_;
		GtkTreeIter _tmp3_ = {0};
		GtkTreeIter _tmp4_;
		gboolean _tmp5_;
#line 1082 "../src/Widgets/SourceList/SourceList.vala"
		_tmp2_ = child_iter;
#line 1082 "../src/Widgets/SourceList/SourceList.vala"
		_tmp4_ = *_tmp2_;
#line 1082 "../src/Widgets/SourceList/SourceList.vala"
		_tmp5_ = gtk_tree_model_filter_convert_child_iter_to_iter ((GtkTreeModelFilter*) self, &_tmp3_, &_tmp4_);
#line 1082 "../src/Widgets/SourceList/SourceList.vala"
		iter = _tmp3_;
#line 1082 "../src/Widgets/SourceList/SourceList.vala"
		if (_tmp5_) {
#line 5317 "SourceList.c"
			GtkTreeIter* _tmp6_;
#line 1083 "../src/Widgets/SourceList/SourceList.vala"
			_tmp6_ = __vala_GtkTreeIter_copy0 (&iter);
#line 1083 "../src/Widgets/SourceList/SourceList.vala"
			result = _tmp6_;
#line 1083 "../src/Widgets/SourceList/SourceList.vala"
			__vala_GtkTreeIter_free0 (child_iter);
#line 1083 "../src/Widgets/SourceList/SourceList.vala"
			return result;
#line 5327 "SourceList.c"
		}
	}
#line 1086 "../src/Widgets/SourceList/SourceList.vala"
	result = NULL;
#line 1086 "../src/Widgets/SourceList/SourceList.vala"
	__vala_GtkTreeIter_free0 (child_iter);
#line 1086 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 5336 "SourceList.c"
}

/**
         * External "extra" filter method.
         */
static void
code_widgets_source_list_data_model_set_filter_func (CodeWidgetsSourceListDataModel* self,
                                                     CodeWidgetsSourceListVisibleFunc visible_func,
                                                     gpointer visible_func_target)
{
#line 1092 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 1093 "../src/Widgets/SourceList/SourceList.vala"
	self->priv->filter_func = visible_func;
#line 1093 "../src/Widgets/SourceList/SourceList.vala"
	self->priv->filter_func_target = visible_func_target;
#line 5353 "SourceList.c"
}

/**
         * Checks whether an item is a category (i.e. a root-level expandable item).
         * The caller must pass an iter or path pointing to the item, but not both
         * (one of them must be null.)
         *
         * TODO: instead of checking the position of the iter or path, we should simply
         * check whether the item's parent is the root item and whether the item is
         * expandable. We don't do so right now because vala still allows client code
         * to access the Item.parent property, even though its setter is defined as internal.
         */
static gboolean
code_widgets_source_list_data_model_is_category (CodeWidgetsSourceListDataModel* self,
                                                 CodeWidgetsSourceListItem* item,
                                                 GtkTreeIter* iter,
                                                 GtkTreePath* path)
{
	gboolean is_category = FALSE;
	gboolean result;
#line 1106 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 1106 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (item != NULL, FALSE);
#line 1107 "../src/Widgets/SourceList/SourceList.vala"
	is_category = FALSE;
#line 1109 "../src/Widgets/SourceList/SourceList.vala"
	if (CODE_WIDGETS_SOURCE_LIST_IS_EXPANDABLE_ITEM (item)) {
#line 1110 "../src/Widgets/SourceList/SourceList.vala"
		if (iter != NULL) {
#line 5384 "SourceList.c"
			GtkTreeIter _tmp0_;
#line 1111 "../src/Widgets/SourceList/SourceList.vala"
			_vala_assert (path == NULL, "path == null");
#line 1112 "../src/Widgets/SourceList/SourceList.vala"
			_tmp0_ = *iter;
#line 1112 "../src/Widgets/SourceList/SourceList.vala"
			is_category = code_widgets_source_list_data_model_is_iter_at_root_level (self, &_tmp0_);
#line 5392 "SourceList.c"
		} else {
#line 1114 "../src/Widgets/SourceList/SourceList.vala"
			_vala_assert (iter == NULL, "iter == null");
#line 1115 "../src/Widgets/SourceList/SourceList.vala"
			is_category = code_widgets_source_list_data_model_is_path_at_root_level (self, path);
#line 5398 "SourceList.c"
		}
	}
#line 1118 "../src/Widgets/SourceList/SourceList.vala"
	result = is_category;
#line 1118 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 5405 "SourceList.c"
}

static gboolean
code_widgets_source_list_data_model_is_iter_at_root_level (CodeWidgetsSourceListDataModel* self,
                                                           GtkTreeIter* iter)
{
	GtkTreeIter _tmp0_;
	GtkTreePath* _tmp1_;
	GtkTreePath* _tmp2_;
	gboolean _tmp3_;
	gboolean result;
#line 1121 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 1121 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (iter != NULL, FALSE);
#line 1122 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = *iter;
#line 1122 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = gtk_tree_model_get_path ((GtkTreeModel*) self, &_tmp0_);
#line 1122 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = _tmp1_;
#line 1122 "../src/Widgets/SourceList/SourceList.vala"
	_tmp3_ = code_widgets_source_list_data_model_is_path_at_root_level (self, _tmp2_);
#line 1122 "../src/Widgets/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (_tmp2_);
#line 1122 "../src/Widgets/SourceList/SourceList.vala"
	result = _tmp3_;
#line 1122 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 5435 "SourceList.c"
}

static gboolean
code_widgets_source_list_data_model_is_path_at_root_level (CodeWidgetsSourceListDataModel* self,
                                                           GtkTreePath* path)
{
	gboolean result;
#line 1125 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 1125 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (path != NULL, FALSE);
#line 1126 "../src/Widgets/SourceList/SourceList.vala"
	result = gtk_tree_path_get_depth (path) == 1;
#line 1126 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 5451 "SourceList.c"
}

static void
code_widgets_source_list_data_model_resort (CodeWidgetsSourceListDataModel* self)
{
	GtkTreeStore* _tmp0_;
	GtkTreeStore* _tmp1_;
#line 1129 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 1130 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->child_tree;
#line 1130 "../src/Widgets/SourceList/SourceList.vala"
	gtk_tree_sortable_set_sort_column_id ((GtkTreeSortable*) _tmp0_, GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID, GTK_SORT_ASCENDING);
#line 1131 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = self->priv->child_tree;
#line 1131 "../src/Widgets/SourceList/SourceList.vala"
	gtk_tree_sortable_set_sort_column_id ((GtkTreeSortable*) _tmp1_, GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID, GTK_SORT_ASCENDING);
#line 5469 "SourceList.c"
}

static gint
code_widgets_source_list_data_model_child_model_sort_func (CodeWidgetsSourceListDataModel* self,
                                                           GtkTreeModel* model,
                                                           GtkTreeIter* a,
                                                           GtkTreeIter* b)
{
	gint order = 0;
	CodeWidgetsSourceListItem* item_a = NULL;
	CodeWidgetsSourceListItem* item_b = NULL;
	GtkTreeStore* _tmp0_;
	GtkTreeIter _tmp1_;
	CodeWidgetsSourceListItem* _tmp2_ = NULL;
	GtkTreeStore* _tmp3_;
	GtkTreeIter _tmp4_;
	CodeWidgetsSourceListItem* _tmp5_ = NULL;
	CodeWidgetsSourceListItem* _tmp6_;
	CodeWidgetsSourceListExpandableItem* _tmp7_;
	CodeWidgetsSourceListExpandableItem* _tmp8_;
	CodeWidgetsSourceListItem* _tmp9_;
	CodeWidgetsSourceListExpandableItem* _tmp10_;
	CodeWidgetsSourceListExpandableItem* _tmp11_;
	CodeWidgetsSourceListSortable* parent = NULL;
	CodeWidgetsSourceListItem* _tmp12_;
	CodeWidgetsSourceListExpandableItem* _tmp13_;
	CodeWidgetsSourceListExpandableItem* _tmp14_;
	CodeWidgetsSourceListSortable* _tmp15_;
	CodeWidgetsSourceListSortable* _tmp16_;
	gint result;
#line 1134 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 1134 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (model != NULL, 0);
#line 1134 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (a != NULL, 0);
#line 1134 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (b != NULL, 0);
#line 1135 "../src/Widgets/SourceList/SourceList.vala"
	order = 0;
#line 1138 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->child_tree;
#line 1138 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = *a;
#line 1138 "../src/Widgets/SourceList/SourceList.vala"
	gtk_tree_model_get ((GtkTreeModel*) _tmp0_, &_tmp1_, CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_COLUMN_ITEM, &_tmp2_, -1, -1);
#line 1138 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (item_a);
#line 1138 "../src/Widgets/SourceList/SourceList.vala"
	item_a = _tmp2_;
#line 1139 "../src/Widgets/SourceList/SourceList.vala"
	_tmp3_ = self->priv->child_tree;
#line 1139 "../src/Widgets/SourceList/SourceList.vala"
	_tmp4_ = *b;
#line 1139 "../src/Widgets/SourceList/SourceList.vala"
	gtk_tree_model_get ((GtkTreeModel*) _tmp3_, &_tmp4_, CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_COLUMN_ITEM, &_tmp5_, -1, -1);
#line 1139 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (item_b);
#line 1139 "../src/Widgets/SourceList/SourceList.vala"
	item_b = _tmp5_;
#line 1142 "../src/Widgets/SourceList/SourceList.vala"
	_tmp6_ = item_a;
#line 1142 "../src/Widgets/SourceList/SourceList.vala"
	_tmp7_ = code_widgets_source_list_item_get_parent (_tmp6_);
#line 1142 "../src/Widgets/SourceList/SourceList.vala"
	_tmp8_ = _tmp7_;
#line 1142 "../src/Widgets/SourceList/SourceList.vala"
	_tmp9_ = item_b;
#line 1142 "../src/Widgets/SourceList/SourceList.vala"
	_tmp10_ = code_widgets_source_list_item_get_parent (_tmp9_);
#line 1142 "../src/Widgets/SourceList/SourceList.vala"
	_tmp11_ = _tmp10_;
#line 1142 "../src/Widgets/SourceList/SourceList.vala"
	_vala_assert (_tmp8_ == _tmp11_, "item_a.parent == item_b.parent");
#line 1144 "../src/Widgets/SourceList/SourceList.vala"
	_tmp12_ = item_a;
#line 1144 "../src/Widgets/SourceList/SourceList.vala"
	_tmp13_ = code_widgets_source_list_item_get_parent (_tmp12_);
#line 1144 "../src/Widgets/SourceList/SourceList.vala"
	_tmp14_ = _tmp13_;
#line 1144 "../src/Widgets/SourceList/SourceList.vala"
	_tmp15_ = _g_object_ref0 (CODE_WIDGETS_IS_SOURCE_LIST_SORTABLE (_tmp14_) ? ((CodeWidgetsSourceListSortable*) _tmp14_) : NULL);
#line 1144 "../src/Widgets/SourceList/SourceList.vala"
	parent = _tmp15_;
#line 1145 "../src/Widgets/SourceList/SourceList.vala"
	_tmp16_ = parent;
#line 1145 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp16_ != NULL) {
#line 5558 "SourceList.c"
		CodeWidgetsSourceListSortable* _tmp17_;
		CodeWidgetsSourceListItem* _tmp18_;
		CodeWidgetsSourceListItem* _tmp19_;
#line 1146 "../src/Widgets/SourceList/SourceList.vala"
		_tmp17_ = parent;
#line 1146 "../src/Widgets/SourceList/SourceList.vala"
		_tmp18_ = item_a;
#line 1146 "../src/Widgets/SourceList/SourceList.vala"
		_tmp19_ = item_b;
#line 1146 "../src/Widgets/SourceList/SourceList.vala"
		order = code_widgets_source_list_sortable_compare (_tmp17_, _tmp18_, _tmp19_);
#line 5570 "SourceList.c"
	}
#line 1148 "../src/Widgets/SourceList/SourceList.vala"
	result = order;
#line 1148 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (parent);
#line 1148 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (item_b);
#line 1148 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (item_a);
#line 1148 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 5582 "SourceList.c"
}

static GtkTreeIter*
code_widgets_source_list_data_model_get_item_child_iter (CodeWidgetsSourceListDataModel* self,
                                                         CodeWidgetsSourceListItem* item)
{
	GtkTreeIter* child_iter = NULL;
	CodeWidgetsSourceListDataModelNodeWrapper* child_node_wrapper = NULL;
	GeeHashMap* _tmp0_;
	gpointer _tmp1_;
	CodeWidgetsSourceListDataModelNodeWrapper* _tmp2_;
	GtkTreeIter* result;
#line 1151 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 1151 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (item != NULL, NULL);
#line 1152 "../src/Widgets/SourceList/SourceList.vala"
	child_iter = NULL;
#line 1154 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->items;
#line 1154 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp0_, item);
#line 1154 "../src/Widgets/SourceList/SourceList.vala"
	child_node_wrapper = (CodeWidgetsSourceListDataModelNodeWrapper*) _tmp1_;
#line 1155 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = child_node_wrapper;
#line 1155 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp2_ != NULL) {
#line 5611 "SourceList.c"
		CodeWidgetsSourceListDataModelNodeWrapper* _tmp3_;
		GtkTreeIter* _tmp4_;
		GtkTreeIter* _tmp5_;
#line 1156 "../src/Widgets/SourceList/SourceList.vala"
		_tmp3_ = child_node_wrapper;
#line 1156 "../src/Widgets/SourceList/SourceList.vala"
		_tmp4_ = code_widgets_source_list_data_model_node_wrapper_get_iter (_tmp3_);
#line 1156 "../src/Widgets/SourceList/SourceList.vala"
		_tmp5_ = _tmp4_;
#line 1156 "../src/Widgets/SourceList/SourceList.vala"
		__vala_GtkTreeIter_free0 (child_iter);
#line 1156 "../src/Widgets/SourceList/SourceList.vala"
		child_iter = _tmp5_;
#line 5625 "SourceList.c"
	}
#line 1158 "../src/Widgets/SourceList/SourceList.vala"
	result = child_iter;
#line 1158 "../src/Widgets/SourceList/SourceList.vala"
	_code_widgets_source_list_data_model_node_wrapper_unref0 (child_node_wrapper);
#line 1158 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 5633 "SourceList.c"
}

static GtkTreePath*
code_widgets_source_list_data_model_get_item_child_path (CodeWidgetsSourceListDataModel* self,
                                                         CodeWidgetsSourceListItem* item)
{
	GtkTreePath* child_path = NULL;
	CodeWidgetsSourceListDataModelNodeWrapper* child_node_wrapper = NULL;
	GeeHashMap* _tmp0_;
	gpointer _tmp1_;
	CodeWidgetsSourceListDataModelNodeWrapper* _tmp2_;
	GtkTreePath* result;
#line 1161 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 1161 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (item != NULL, NULL);
#line 1162 "../src/Widgets/SourceList/SourceList.vala"
	child_path = NULL;
#line 1164 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->items;
#line 1164 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp0_, item);
#line 1164 "../src/Widgets/SourceList/SourceList.vala"
	child_node_wrapper = (CodeWidgetsSourceListDataModelNodeWrapper*) _tmp1_;
#line 1165 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = child_node_wrapper;
#line 1165 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp2_ != NULL) {
#line 5662 "SourceList.c"
		CodeWidgetsSourceListDataModelNodeWrapper* _tmp3_;
		GtkTreePath* _tmp4_;
		GtkTreePath* _tmp5_;
#line 1166 "../src/Widgets/SourceList/SourceList.vala"
		_tmp3_ = child_node_wrapper;
#line 1166 "../src/Widgets/SourceList/SourceList.vala"
		_tmp4_ = code_widgets_source_list_data_model_node_wrapper_get_path (_tmp3_);
#line 1166 "../src/Widgets/SourceList/SourceList.vala"
		_tmp5_ = _tmp4_;
#line 1166 "../src/Widgets/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (child_path);
#line 1166 "../src/Widgets/SourceList/SourceList.vala"
		child_path = _tmp5_;
#line 5676 "SourceList.c"
	}
#line 1168 "../src/Widgets/SourceList/SourceList.vala"
	result = child_path;
#line 1168 "../src/Widgets/SourceList/SourceList.vala"
	_code_widgets_source_list_data_model_node_wrapper_unref0 (child_node_wrapper);
#line 1168 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 5684 "SourceList.c"
}

/**
         * Filters the child-tree items based on their "visible" property.
         */
static gboolean
code_widgets_source_list_data_model_filter_visible_func (CodeWidgetsSourceListDataModel* self,
                                                         GtkTreeModel* child_model,
                                                         GtkTreeIter* iter)
{
	gboolean item_visible = FALSE;
	CodeWidgetsSourceListItem* item = NULL;
	GtkTreeStore* _tmp0_;
	GtkTreeIter _tmp1_;
	CodeWidgetsSourceListItem* _tmp2_ = NULL;
	CodeWidgetsSourceListItem* _tmp3_;
	CodeWidgetsSourceListVisibleFunc _tmp27_;
	gpointer _tmp27__target;
	gboolean result;
#line 1174 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 1174 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (child_model != NULL, FALSE);
#line 1174 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (iter != NULL, FALSE);
#line 1175 "../src/Widgets/SourceList/SourceList.vala"
	item_visible = FALSE;
#line 1178 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->child_tree;
#line 1178 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = *iter;
#line 1178 "../src/Widgets/SourceList/SourceList.vala"
	gtk_tree_model_get ((GtkTreeModel*) _tmp0_, &_tmp1_, CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_COLUMN_ITEM, &_tmp2_, -1, -1);
#line 1178 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (item);
#line 1178 "../src/Widgets/SourceList/SourceList.vala"
	item = _tmp2_;
#line 1180 "../src/Widgets/SourceList/SourceList.vala"
	_tmp3_ = item;
#line 1180 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp3_ != NULL) {
#line 5726 "SourceList.c"
		CodeWidgetsSourceListItem* _tmp4_;
		gboolean _tmp5_;
		gboolean _tmp6_;
		CodeWidgetsSourceListExpandableItem* expandable = NULL;
		CodeWidgetsSourceListItem* _tmp7_;
		CodeWidgetsSourceListExpandableItem* _tmp8_;
		gboolean _tmp9_ = FALSE;
		CodeWidgetsSourceListExpandableItem* _tmp10_;
#line 1181 "../src/Widgets/SourceList/SourceList.vala"
		_tmp4_ = item;
#line 1181 "../src/Widgets/SourceList/SourceList.vala"
		_tmp5_ = code_widgets_source_list_item_get_visible (_tmp4_);
#line 1181 "../src/Widgets/SourceList/SourceList.vala"
		_tmp6_ = _tmp5_;
#line 1181 "../src/Widgets/SourceList/SourceList.vala"
		item_visible = _tmp6_;
#line 1185 "../src/Widgets/SourceList/SourceList.vala"
		_tmp7_ = item;
#line 1185 "../src/Widgets/SourceList/SourceList.vala"
		_tmp8_ = _g_object_ref0 (CODE_WIDGETS_SOURCE_LIST_IS_EXPANDABLE_ITEM (_tmp7_) ? ((CodeWidgetsSourceListExpandableItem*) _tmp7_) : NULL);
#line 1185 "../src/Widgets/SourceList/SourceList.vala"
		expandable = _tmp8_;
#line 1186 "../src/Widgets/SourceList/SourceList.vala"
		_tmp10_ = expandable;
#line 1186 "../src/Widgets/SourceList/SourceList.vala"
		if (_tmp10_ != NULL) {
#line 5753 "SourceList.c"
			GtkTreeStore* _tmp11_;
			GtkTreeIter _tmp12_;
#line 1186 "../src/Widgets/SourceList/SourceList.vala"
			_tmp11_ = self->priv->child_tree;
#line 1186 "../src/Widgets/SourceList/SourceList.vala"
			_tmp12_ = *iter;
#line 1186 "../src/Widgets/SourceList/SourceList.vala"
			_tmp9_ = gtk_tree_store_iter_depth (_tmp11_, &_tmp12_) == 0;
#line 5762 "SourceList.c"
		} else {
#line 1186 "../src/Widgets/SourceList/SourceList.vala"
			_tmp9_ = FALSE;
#line 5766 "SourceList.c"
		}
#line 1186 "../src/Widgets/SourceList/SourceList.vala"
		if (_tmp9_) {
#line 5770 "SourceList.c"
			guint n_visible_children = 0U;
			gboolean _tmp26_ = FALSE;
#line 1187 "../src/Widgets/SourceList/SourceList.vala"
			n_visible_children = (guint) 0;
#line 5775 "SourceList.c"
			{
				GeeIterator* _child_item_it = NULL;
				CodeWidgetsSourceListExpandableItem* _tmp13_;
				GeeCollection* _tmp14_;
				GeeCollection* _tmp15_;
				GeeCollection* _tmp16_;
				GeeIterator* _tmp17_;
				GeeIterator* _tmp18_;
#line 1188 "../src/Widgets/SourceList/SourceList.vala"
				_tmp13_ = expandable;
#line 1188 "../src/Widgets/SourceList/SourceList.vala"
				_tmp14_ = code_widgets_source_list_expandable_item_get_children (_tmp13_);
#line 1188 "../src/Widgets/SourceList/SourceList.vala"
				_tmp15_ = _tmp14_;
#line 1188 "../src/Widgets/SourceList/SourceList.vala"
				_tmp16_ = _tmp15_;
#line 1188 "../src/Widgets/SourceList/SourceList.vala"
				_tmp17_ = gee_iterable_iterator ((GeeIterable*) _tmp16_);
#line 1188 "../src/Widgets/SourceList/SourceList.vala"
				_tmp18_ = _tmp17_;
#line 1188 "../src/Widgets/SourceList/SourceList.vala"
				_g_object_unref0 (_tmp16_);
#line 1188 "../src/Widgets/SourceList/SourceList.vala"
				_child_item_it = _tmp18_;
#line 1188 "../src/Widgets/SourceList/SourceList.vala"
				while (TRUE) {
#line 5802 "SourceList.c"
					GeeIterator* _tmp19_;
					CodeWidgetsSourceListItem* child_item = NULL;
					GeeIterator* _tmp20_;
					gpointer _tmp21_;
					CodeWidgetsSourceListItem* _tmp22_;
					gboolean _tmp23_;
					gboolean _tmp24_;
#line 1188 "../src/Widgets/SourceList/SourceList.vala"
					_tmp19_ = _child_item_it;
#line 1188 "../src/Widgets/SourceList/SourceList.vala"
					if (!gee_iterator_next (_tmp19_)) {
#line 1188 "../src/Widgets/SourceList/SourceList.vala"
						break;
#line 5816 "SourceList.c"
					}
#line 1188 "../src/Widgets/SourceList/SourceList.vala"
					_tmp20_ = _child_item_it;
#line 1188 "../src/Widgets/SourceList/SourceList.vala"
					_tmp21_ = gee_iterator_get (_tmp20_);
#line 1188 "../src/Widgets/SourceList/SourceList.vala"
					child_item = (CodeWidgetsSourceListItem*) _tmp21_;
#line 1189 "../src/Widgets/SourceList/SourceList.vala"
					_tmp22_ = child_item;
#line 1189 "../src/Widgets/SourceList/SourceList.vala"
					_tmp23_ = code_widgets_source_list_item_get_visible (_tmp22_);
#line 1189 "../src/Widgets/SourceList/SourceList.vala"
					_tmp24_ = _tmp23_;
#line 1189 "../src/Widgets/SourceList/SourceList.vala"
					if (_tmp24_) {
#line 5832 "SourceList.c"
						guint _tmp25_;
#line 1190 "../src/Widgets/SourceList/SourceList.vala"
						_tmp25_ = n_visible_children;
#line 1190 "../src/Widgets/SourceList/SourceList.vala"
						n_visible_children = _tmp25_ + 1;
#line 5838 "SourceList.c"
					}
#line 1188 "../src/Widgets/SourceList/SourceList.vala"
					_g_object_unref0 (child_item);
#line 5842 "SourceList.c"
				}
#line 1188 "../src/Widgets/SourceList/SourceList.vala"
				_g_object_unref0 (_child_item_it);
#line 5846 "SourceList.c"
			}
#line 1192 "../src/Widgets/SourceList/SourceList.vala"
			if (item_visible) {
#line 1192 "../src/Widgets/SourceList/SourceList.vala"
				_tmp26_ = n_visible_children > ((guint) 0);
#line 5852 "SourceList.c"
			} else {
#line 1192 "../src/Widgets/SourceList/SourceList.vala"
				_tmp26_ = FALSE;
#line 5856 "SourceList.c"
			}
#line 1192 "../src/Widgets/SourceList/SourceList.vala"
			item_visible = _tmp26_;
#line 5860 "SourceList.c"
		}
#line 1180 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (expandable);
#line 5864 "SourceList.c"
	}
#line 1196 "../src/Widgets/SourceList/SourceList.vala"
	_tmp27_ = self->priv->filter_func;
#line 1196 "../src/Widgets/SourceList/SourceList.vala"
	_tmp27__target = self->priv->filter_func_target;
#line 1196 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp27_ != NULL) {
#line 5872 "SourceList.c"
		gboolean _tmp28_ = FALSE;
#line 1197 "../src/Widgets/SourceList/SourceList.vala"
		if (item_visible) {
#line 5876 "SourceList.c"
			CodeWidgetsSourceListVisibleFunc _tmp29_;
			gpointer _tmp29__target;
			CodeWidgetsSourceListItem* _tmp30_;
#line 1197 "../src/Widgets/SourceList/SourceList.vala"
			_tmp29_ = self->priv->filter_func;
#line 1197 "../src/Widgets/SourceList/SourceList.vala"
			_tmp29__target = self->priv->filter_func_target;
#line 1197 "../src/Widgets/SourceList/SourceList.vala"
			_tmp30_ = item;
#line 1197 "../src/Widgets/SourceList/SourceList.vala"
			_tmp28_ = _tmp29_ (_tmp30_, _tmp29__target);
#line 5888 "SourceList.c"
		} else {
#line 1197 "../src/Widgets/SourceList/SourceList.vala"
			_tmp28_ = FALSE;
#line 5892 "SourceList.c"
		}
#line 1197 "../src/Widgets/SourceList/SourceList.vala"
		item_visible = _tmp28_;
#line 5896 "SourceList.c"
	}
#line 1199 "../src/Widgets/SourceList/SourceList.vala"
	result = item_visible;
#line 1199 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (item);
#line 1199 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 5904 "SourceList.c"
}

/**
         * TreeDragDest implementation
         */
static gboolean
code_widgets_source_list_data_model_real_drag_data_received (GtkTreeDragDest* base,
                                                             GtkTreePath* dest,
                                                             GtkSelectionData* selection_data)
{
	CodeWidgetsSourceListDataModel * self;
	GtkTreeModel* model = NULL;
	GtkTreePath* src_path = NULL;
	gboolean _tmp0_ = FALSE;
	GtkTreeModel* _tmp1_ = NULL;
	GtkTreePath* _tmp2_ = NULL;
	gboolean _tmp3_;
	GtkTreeModel* _tmp4_;
	gboolean result;
#line 1206 "../src/Widgets/SourceList/SourceList.vala"
	self = (CodeWidgetsSourceListDataModel*) base;
#line 1206 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (dest != NULL, FALSE);
#line 1206 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (selection_data != NULL, FALSE);
#line 1214 "../src/Widgets/SourceList/SourceList.vala"
	_tmp3_ = gtk_tree_get_row_drag_data (selection_data, &_tmp1_, &_tmp2_);
#line 1214 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (model);
#line 1214 "../src/Widgets/SourceList/SourceList.vala"
	_tmp4_ = _g_object_ref0 (_tmp1_);
#line 1214 "../src/Widgets/SourceList/SourceList.vala"
	model = _tmp4_;
#line 1214 "../src/Widgets/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (src_path);
#line 1214 "../src/Widgets/SourceList/SourceList.vala"
	src_path = _tmp2_;
#line 1214 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp3_) {
#line 5944 "SourceList.c"
		GtkTreeModel* _tmp5_;
		GtkTreeStore* _tmp6_;
#line 1214 "../src/Widgets/SourceList/SourceList.vala"
		_tmp5_ = model;
#line 1214 "../src/Widgets/SourceList/SourceList.vala"
		_tmp6_ = self->priv->child_tree;
#line 1214 "../src/Widgets/SourceList/SourceList.vala"
		_tmp0_ = _tmp5_ == G_TYPE_CHECK_INSTANCE_CAST (_tmp6_, gtk_tree_model_get_type (), GtkTreeModel);
#line 5953 "SourceList.c"
	} else {
#line 1214 "../src/Widgets/SourceList/SourceList.vala"
		_tmp0_ = FALSE;
#line 5957 "SourceList.c"
	}
#line 1214 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp0_) {
#line 5961 "SourceList.c"
		GtkTreePath* child_dest = NULL;
		GtkTreePath* _tmp7_;
		GtkTreePath* _tmp8_;
#line 1216 "../src/Widgets/SourceList/SourceList.vala"
		_tmp7_ = gtk_tree_model_filter_convert_path_to_child_path ((GtkTreeModelFilter*) self, dest);
#line 1216 "../src/Widgets/SourceList/SourceList.vala"
		child_dest = _tmp7_;
#line 1218 "../src/Widgets/SourceList/SourceList.vala"
		_tmp8_ = child_dest;
#line 1218 "../src/Widgets/SourceList/SourceList.vala"
		if (_tmp8_ != NULL) {
#line 5973 "SourceList.c"
			GtkTreePath* _tmp9_;
			GtkTreePath* _tmp10_;
#line 1220 "../src/Widgets/SourceList/SourceList.vala"
			_tmp9_ = child_dest;
#line 1220 "../src/Widgets/SourceList/SourceList.vala"
			_tmp10_ = src_path;
#line 1220 "../src/Widgets/SourceList/SourceList.vala"
			if (code_widgets_source_list_data_model_child_tree_drag_data_received (self, _tmp9_, _tmp10_)) {
#line 1221 "../src/Widgets/SourceList/SourceList.vala"
				result = TRUE;
#line 1221 "../src/Widgets/SourceList/SourceList.vala"
				__vala_GtkTreePath_free0 (child_dest);
#line 1221 "../src/Widgets/SourceList/SourceList.vala"
				__vala_GtkTreePath_free0 (src_path);
#line 1221 "../src/Widgets/SourceList/SourceList.vala"
				_g_object_unref0 (model);
#line 1221 "../src/Widgets/SourceList/SourceList.vala"
				return result;
#line 5992 "SourceList.c"
			}
		}
#line 1214 "../src/Widgets/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (child_dest);
#line 5997 "SourceList.c"
	}
#line 1226 "../src/Widgets/SourceList/SourceList.vala"
	result = FALSE;
#line 1226 "../src/Widgets/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (src_path);
#line 1226 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (model);
#line 1226 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 6007 "SourceList.c"
}

static GtkTreePath*
_vala_GtkTreePath_copy (GtkTreePath* self)
{
#line 1236 "../src/Widgets/SourceList/SourceList.vala"
	return g_boxed_copy (gtk_tree_path_get_type (), self);
#line 6015 "SourceList.c"
}

static gpointer
__vala_GtkTreePath_copy0 (gpointer self)
{
#line 1236 "../src/Widgets/SourceList/SourceList.vala"
	return self ? _vala_GtkTreePath_copy (self) : NULL;
#line 6023 "SourceList.c"
}

static gboolean
code_widgets_source_list_data_model_child_tree_drag_data_received (CodeWidgetsSourceListDataModel* self,
                                                                   GtkTreePath* dest,
                                                                   GtkTreePath* src_path)
{
	gboolean retval = FALSE;
	GtkTreeIter src_iter = {0};
	GtkTreeIter dest_iter = {0};
	GtkTreeStore* _tmp0_;
	GtkTreeIter _tmp1_ = {0};
	gboolean _tmp2_;
	GtkTreePath* prev = NULL;
	GtkTreePath* _tmp3_;
	GtkTreePath* _tmp4_;
	gboolean result;
#line 1229 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 1229 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (dest != NULL, FALSE);
#line 1229 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (src_path != NULL, FALSE);
#line 1230 "../src/Widgets/SourceList/SourceList.vala"
	retval = FALSE;
#line 1233 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->child_tree;
#line 1233 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = gtk_tree_model_get_iter ((GtkTreeModel*) _tmp0_, &_tmp1_, src_path);
#line 1233 "../src/Widgets/SourceList/SourceList.vala"
	src_iter = _tmp1_;
#line 1233 "../src/Widgets/SourceList/SourceList.vala"
	if (!_tmp2_) {
#line 1234 "../src/Widgets/SourceList/SourceList.vala"
		result = FALSE;
#line 1234 "../src/Widgets/SourceList/SourceList.vala"
		return result;
#line 6061 "SourceList.c"
	}
#line 1236 "../src/Widgets/SourceList/SourceList.vala"
	_tmp3_ = __vala_GtkTreePath_copy0 (dest);
#line 1236 "../src/Widgets/SourceList/SourceList.vala"
	prev = _tmp3_;
#line 1239 "../src/Widgets/SourceList/SourceList.vala"
	_tmp4_ = prev;
#line 1239 "../src/Widgets/SourceList/SourceList.vala"
	if (!gtk_tree_path_prev (_tmp4_)) {
#line 6071 "SourceList.c"
		GtkTreePath* parent = NULL;
		GtkTreePath* _tmp5_;
		GtkTreeIter* dest_parent = NULL;
		gboolean _tmp6_ = FALSE;
		GtkTreePath* _tmp7_;
		GtkTreeStore* _tmp14_;
		GtkTreeIter* _tmp15_;
		GtkTreeIter _tmp16_ = {0};
#line 1243 "../src/Widgets/SourceList/SourceList.vala"
		_tmp5_ = __vala_GtkTreePath_copy0 (dest);
#line 1243 "../src/Widgets/SourceList/SourceList.vala"
		parent = _tmp5_;
#line 1244 "../src/Widgets/SourceList/SourceList.vala"
		dest_parent = NULL;
#line 1246 "../src/Widgets/SourceList/SourceList.vala"
		_tmp7_ = parent;
#line 1246 "../src/Widgets/SourceList/SourceList.vala"
		if (gtk_tree_path_up (_tmp7_)) {
#line 6090 "SourceList.c"
			GtkTreePath* _tmp8_;
#line 1246 "../src/Widgets/SourceList/SourceList.vala"
			_tmp8_ = parent;
#line 1246 "../src/Widgets/SourceList/SourceList.vala"
			_tmp6_ = gtk_tree_path_get_depth (_tmp8_) > 0;
#line 6096 "SourceList.c"
		} else {
#line 1246 "../src/Widgets/SourceList/SourceList.vala"
			_tmp6_ = FALSE;
#line 6100 "SourceList.c"
		}
#line 1246 "../src/Widgets/SourceList/SourceList.vala"
		if (_tmp6_) {
#line 6104 "SourceList.c"
			GtkTreeStore* _tmp9_;
			GtkTreePath* _tmp10_;
			GtkTreeIter _tmp11_ = {0};
			GtkTreeIter _tmp12_;
			GtkTreeIter* _tmp13_;
#line 1247 "../src/Widgets/SourceList/SourceList.vala"
			_tmp9_ = self->priv->child_tree;
#line 1247 "../src/Widgets/SourceList/SourceList.vala"
			_tmp10_ = parent;
#line 1247 "../src/Widgets/SourceList/SourceList.vala"
			gtk_tree_model_get_iter ((GtkTreeModel*) _tmp9_, &_tmp11_, _tmp10_);
#line 1247 "../src/Widgets/SourceList/SourceList.vala"
			__vala_GtkTreeIter_free0 (dest_parent);
#line 1247 "../src/Widgets/SourceList/SourceList.vala"
			_tmp12_ = _tmp11_;
#line 1247 "../src/Widgets/SourceList/SourceList.vala"
			_tmp13_ = __vala_GtkTreeIter_copy0 (&_tmp12_);
#line 1247 "../src/Widgets/SourceList/SourceList.vala"
			dest_parent = _tmp13_;
#line 6124 "SourceList.c"
		}
#line 1249 "../src/Widgets/SourceList/SourceList.vala"
		_tmp14_ = self->priv->child_tree;
#line 1249 "../src/Widgets/SourceList/SourceList.vala"
		_tmp15_ = dest_parent;
#line 1249 "../src/Widgets/SourceList/SourceList.vala"
		gtk_tree_store_prepend (_tmp14_, &_tmp16_, _tmp15_);
#line 1249 "../src/Widgets/SourceList/SourceList.vala"
		dest_iter = _tmp16_;
#line 1250 "../src/Widgets/SourceList/SourceList.vala"
		retval = TRUE;
#line 1239 "../src/Widgets/SourceList/SourceList.vala"
		__vala_GtkTreeIter_free0 (dest_parent);
#line 1239 "../src/Widgets/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (parent);
#line 6140 "SourceList.c"
	} else {
		GtkTreeStore* _tmp17_;
		GtkTreePath* _tmp18_;
		GtkTreeIter _tmp19_ = {0};
		gboolean _tmp20_;
#line 1251 "../src/Widgets/SourceList/SourceList.vala"
		_tmp17_ = self->priv->child_tree;
#line 1251 "../src/Widgets/SourceList/SourceList.vala"
		_tmp18_ = prev;
#line 1251 "../src/Widgets/SourceList/SourceList.vala"
		_tmp20_ = gtk_tree_model_get_iter ((GtkTreeModel*) _tmp17_, &_tmp19_, _tmp18_);
#line 1251 "../src/Widgets/SourceList/SourceList.vala"
		dest_iter = _tmp19_;
#line 1251 "../src/Widgets/SourceList/SourceList.vala"
		if (_tmp20_) {
#line 6156 "SourceList.c"
			GtkTreeIter tmp_iter = {0};
			GtkTreeIter _tmp21_;
			GtkTreeStore* _tmp22_;
			GtkTreeIter _tmp23_;
			GtkTreeIter _tmp24_ = {0};
#line 1252 "../src/Widgets/SourceList/SourceList.vala"
			_tmp21_ = dest_iter;
#line 1252 "../src/Widgets/SourceList/SourceList.vala"
			tmp_iter = _tmp21_;
#line 1253 "../src/Widgets/SourceList/SourceList.vala"
			_tmp22_ = self->priv->child_tree;
#line 1253 "../src/Widgets/SourceList/SourceList.vala"
			_tmp23_ = tmp_iter;
#line 1253 "../src/Widgets/SourceList/SourceList.vala"
			gtk_tree_store_insert_after (_tmp22_, &_tmp24_, NULL, &_tmp23_);
#line 1253 "../src/Widgets/SourceList/SourceList.vala"
			dest_iter = _tmp24_;
#line 1254 "../src/Widgets/SourceList/SourceList.vala"
			retval = TRUE;
#line 6176 "SourceList.c"
		}
	}
#line 1259 "../src/Widgets/SourceList/SourceList.vala"
	if (retval) {
#line 6181 "SourceList.c"
		GtkTreeIter _tmp25_;
		GtkTreeIter _tmp26_;
		CodeWidgetsSourceListItem* item = NULL;
		GtkTreeStore* _tmp27_;
		GtkTreeIter _tmp28_;
		CodeWidgetsSourceListItem* _tmp29_ = NULL;
		CodeWidgetsSourceListItem* _tmp30_;
		CodeWidgetsSourceListItem* _tmp31_;
		CodeWidgetsSourceListSortable* parent = NULL;
		CodeWidgetsSourceListItem* _tmp32_;
		CodeWidgetsSourceListExpandableItem* _tmp33_;
		CodeWidgetsSourceListExpandableItem* _tmp34_;
		CodeWidgetsSourceListSortable* _tmp35_;
		CodeWidgetsSourceListSortable* _tmp36_;
		CodeWidgetsSourceListSortable* _tmp37_;
		CodeWidgetsSourceListItem* _tmp38_;
#line 1260 "../src/Widgets/SourceList/SourceList.vala"
		_tmp25_ = src_iter;
#line 1260 "../src/Widgets/SourceList/SourceList.vala"
		_tmp26_ = dest_iter;
#line 1260 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_data_model_recursive_node_copy (self, &_tmp25_, &_tmp26_);
#line 1264 "../src/Widgets/SourceList/SourceList.vala"
		_tmp27_ = self->priv->child_tree;
#line 1264 "../src/Widgets/SourceList/SourceList.vala"
		_tmp28_ = src_iter;
#line 1264 "../src/Widgets/SourceList/SourceList.vala"
		gtk_tree_model_get ((GtkTreeModel*) _tmp27_, &_tmp28_, CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_COLUMN_ITEM, &_tmp29_, -1, -1);
#line 1264 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (item);
#line 1264 "../src/Widgets/SourceList/SourceList.vala"
		item = _tmp29_;
#line 1265 "../src/Widgets/SourceList/SourceList.vala"
		_tmp30_ = item;
#line 1265 "../src/Widgets/SourceList/SourceList.vala"
		g_return_val_if_fail (_tmp30_ != NULL, retval);
#line 1274 "../src/Widgets/SourceList/SourceList.vala"
		_tmp31_ = item;
#line 1274 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_data_model_update_item (self, _tmp31_);
#line 1276 "../src/Widgets/SourceList/SourceList.vala"
		_tmp32_ = item;
#line 1276 "../src/Widgets/SourceList/SourceList.vala"
		_tmp33_ = code_widgets_source_list_item_get_parent (_tmp32_);
#line 1276 "../src/Widgets/SourceList/SourceList.vala"
		_tmp34_ = _tmp33_;
#line 1276 "../src/Widgets/SourceList/SourceList.vala"
		_tmp35_ = _g_object_ref0 (CODE_WIDGETS_IS_SOURCE_LIST_SORTABLE (_tmp34_) ? ((CodeWidgetsSourceListSortable*) _tmp34_) : NULL);
#line 1276 "../src/Widgets/SourceList/SourceList.vala"
		parent = _tmp35_;
#line 1277 "../src/Widgets/SourceList/SourceList.vala"
		_tmp36_ = parent;
#line 1277 "../src/Widgets/SourceList/SourceList.vala"
		g_return_val_if_fail (_tmp36_ != NULL, retval);
#line 1279 "../src/Widgets/SourceList/SourceList.vala"
		_tmp37_ = parent;
#line 1279 "../src/Widgets/SourceList/SourceList.vala"
		_tmp38_ = item;
#line 1279 "../src/Widgets/SourceList/SourceList.vala"
		g_signal_emit (_tmp37_, code_widgets_source_list_sortable_signals[CODE_WIDGETS_SOURCE_LIST_SORTABLE_USER_MOVED_ITEM_SIGNAL], 0, _tmp38_);
#line 1259 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (parent);
#line 1259 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (item);
#line 6246 "SourceList.c"
	}
#line 1282 "../src/Widgets/SourceList/SourceList.vala"
	result = retval;
#line 1282 "../src/Widgets/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (prev);
#line 1282 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 6254 "SourceList.c"
}

static void
code_widgets_source_list_data_model_recursive_node_copy (CodeWidgetsSourceListDataModel* self,
                                                         GtkTreeIter* src_iter,
                                                         GtkTreeIter* dest_iter)
{
	GtkTreeIter _tmp0_;
	GtkTreeIter _tmp1_;
	GtkTreeIter child = {0};
	GtkTreeStore* _tmp2_;
	GtkTreeIter _tmp3_;
	GtkTreeIter _tmp4_ = {0};
	gboolean _tmp5_;
#line 1285 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 1285 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (src_iter != NULL);
#line 1285 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (dest_iter != NULL);
#line 1286 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = *src_iter;
#line 1286 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = *dest_iter;
#line 1286 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_data_model_move_item (self, &_tmp0_, &_tmp1_);
#line 1289 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = self->priv->child_tree;
#line 1289 "../src/Widgets/SourceList/SourceList.vala"
	_tmp3_ = *src_iter;
#line 1289 "../src/Widgets/SourceList/SourceList.vala"
	_tmp5_ = gtk_tree_model_iter_children ((GtkTreeModel*) _tmp2_, &_tmp4_, &_tmp3_);
#line 1289 "../src/Widgets/SourceList/SourceList.vala"
	child = _tmp4_;
#line 1289 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp5_) {
#line 6291 "SourceList.c"
		{
			gboolean _tmp6_ = FALSE;
#line 1292 "../src/Widgets/SourceList/SourceList.vala"
			_tmp6_ = TRUE;
#line 1292 "../src/Widgets/SourceList/SourceList.vala"
			while (TRUE) {
#line 6298 "SourceList.c"
				GtkTreeIter copy = {0};
				GtkTreeStore* _tmp9_;
				GtkTreeIter _tmp10_;
				GtkTreeIter _tmp11_ = {0};
				GtkTreeIter _tmp12_;
				GtkTreeIter _tmp13_;
#line 1292 "../src/Widgets/SourceList/SourceList.vala"
				if (!_tmp6_) {
#line 6307 "SourceList.c"
					GtkTreeStore* _tmp7_;
					gboolean _tmp8_;
#line 1296 "../src/Widgets/SourceList/SourceList.vala"
					_tmp7_ = self->priv->child_tree;
#line 1296 "../src/Widgets/SourceList/SourceList.vala"
					_tmp8_ = gtk_tree_model_iter_next ((GtkTreeModel*) _tmp7_, &child);
#line 1296 "../src/Widgets/SourceList/SourceList.vala"
					if (!_tmp8_) {
#line 1296 "../src/Widgets/SourceList/SourceList.vala"
						break;
#line 6318 "SourceList.c"
					}
				}
#line 1292 "../src/Widgets/SourceList/SourceList.vala"
				_tmp6_ = FALSE;
#line 1294 "../src/Widgets/SourceList/SourceList.vala"
				_tmp9_ = self->priv->child_tree;
#line 1294 "../src/Widgets/SourceList/SourceList.vala"
				_tmp10_ = *dest_iter;
#line 1294 "../src/Widgets/SourceList/SourceList.vala"
				gtk_tree_store_append (_tmp9_, &_tmp11_, &_tmp10_);
#line 1294 "../src/Widgets/SourceList/SourceList.vala"
				copy = _tmp11_;
#line 1295 "../src/Widgets/SourceList/SourceList.vala"
				_tmp12_ = child;
#line 1295 "../src/Widgets/SourceList/SourceList.vala"
				_tmp13_ = copy;
#line 1295 "../src/Widgets/SourceList/SourceList.vala"
				code_widgets_source_list_data_model_recursive_node_copy (self, &_tmp12_, &_tmp13_);
#line 6337 "SourceList.c"
			}
		}
	}
}

static void
code_widgets_source_list_data_model_move_item (CodeWidgetsSourceListDataModel* self,
                                               GtkTreeIter* src_iter,
                                               GtkTreeIter* dest_iter)
{
	CodeWidgetsSourceListItem* item = NULL;
	GtkTreeStore* _tmp0_;
	GtkTreeIter _tmp1_;
	CodeWidgetsSourceListItem* _tmp2_ = NULL;
	GtkTreeStore* _tmp3_;
	GtkTreeIter _tmp4_;
	GeeHashMap* _tmp5_;
	GtkTreeStore* _tmp6_;
	GtkTreeIter _tmp7_;
	CodeWidgetsSourceListDataModelNodeWrapper* _tmp8_;
	CodeWidgetsSourceListDataModelNodeWrapper* _tmp9_;
#line 1300 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 1300 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (src_iter != NULL);
#line 1300 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (dest_iter != NULL);
#line 1302 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->child_tree;
#line 1302 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = *src_iter;
#line 1302 "../src/Widgets/SourceList/SourceList.vala"
	gtk_tree_model_get ((GtkTreeModel*) _tmp0_, &_tmp1_, CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_COLUMN_ITEM, &_tmp2_, -1, -1);
#line 1302 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (item);
#line 1302 "../src/Widgets/SourceList/SourceList.vala"
	item = _tmp2_;
#line 1303 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (item != NULL);
#line 1306 "../src/Widgets/SourceList/SourceList.vala"
	_tmp3_ = self->priv->child_tree;
#line 1306 "../src/Widgets/SourceList/SourceList.vala"
	_tmp4_ = *dest_iter;
#line 1306 "../src/Widgets/SourceList/SourceList.vala"
	gtk_tree_store_set (_tmp3_, &_tmp4_, CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_COLUMN_ITEM, item, -1, -1);
#line 1307 "../src/Widgets/SourceList/SourceList.vala"
	_tmp5_ = self->priv->items;
#line 1307 "../src/Widgets/SourceList/SourceList.vala"
	_tmp6_ = self->priv->child_tree;
#line 1307 "../src/Widgets/SourceList/SourceList.vala"
	_tmp7_ = *dest_iter;
#line 1307 "../src/Widgets/SourceList/SourceList.vala"
	_tmp8_ = code_widgets_source_list_data_model_node_wrapper_new ((GtkTreeModel*) _tmp6_, &_tmp7_);
#line 1307 "../src/Widgets/SourceList/SourceList.vala"
	_tmp9_ = _tmp8_;
#line 1307 "../src/Widgets/SourceList/SourceList.vala"
	gee_abstract_map_set ((GeeAbstractMap*) _tmp5_, item, _tmp9_);
#line 1307 "../src/Widgets/SourceList/SourceList.vala"
	_code_widgets_source_list_data_model_node_wrapper_unref0 (_tmp9_);
#line 1300 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (item);
#line 6399 "SourceList.c"
}

static gint*
_vala_array_dup1 (gint* self,
                  gssize length)
{
#line 1354 "../src/Widgets/SourceList/SourceList.vala"
	if (length > 0) {
#line 1354 "../src/Widgets/SourceList/SourceList.vala"
		return _vala_memdup2 (self, length * sizeof (gint));
#line 6410 "SourceList.c"
	}
#line 1354 "../src/Widgets/SourceList/SourceList.vala"
	return NULL;
#line 6414 "SourceList.c"
}

static gint*
_vala_array_dup2 (gint* self,
                  gssize length)
{
#line 1355 "../src/Widgets/SourceList/SourceList.vala"
	if (length > 0) {
#line 1355 "../src/Widgets/SourceList/SourceList.vala"
		return _vala_memdup2 (self, length * sizeof (gint));
#line 6425 "SourceList.c"
	}
#line 1355 "../src/Widgets/SourceList/SourceList.vala"
	return NULL;
#line 6429 "SourceList.c"
}

static gboolean
code_widgets_source_list_data_model_real_row_drop_possible (GtkTreeDragDest* base,
                                                            GtkTreePath* dest,
                                                            GtkSelectionData* selection_data)
{
	CodeWidgetsSourceListDataModel * self;
	GtkTreeModel* model = NULL;
	GtkTreePath* src_path = NULL;
	gboolean _tmp0_ = FALSE;
	GtkTreeModel* _tmp1_ = NULL;
	GtkTreePath* _tmp2_ = NULL;
	gboolean _tmp3_;
	GtkTreeModel* _tmp4_;
	GtkTreePath* child_dest = NULL;
	GtkTreePath* _tmp7_;
	gboolean _tmp8_ = FALSE;
	GtkTreePath* _tmp9_;
	gint src_depth = 0;
	GtkTreePath* _tmp12_;
	gint dest_depth = 0;
	GtkTreePath* _tmp13_;
	CodeWidgetsSourceListItem* parent = NULL;
	CodeWidgetsSourceListSortable* sortable = NULL;
	CodeWidgetsSourceListItem* _tmp35_;
	CodeWidgetsSourceListSortable* _tmp36_;
	gboolean _tmp37_ = FALSE;
	CodeWidgetsSourceListSortable* _tmp38_;
	CodeWidgetsSourceListItem* dest_item = NULL;
	CodeWidgetsSourceListItem* _tmp40_;
	CodeWidgetsSourceListItem* _tmp41_;
	CodeWidgetsSourceListItem* source_item = NULL;
	GtkTreePath* filter_src_path = NULL;
	GtkTreePath* _tmp42_;
	GtkTreePath* _tmp43_;
	GtkTreePath* _tmp44_;
	CodeWidgetsSourceListItem* _tmp47_;
	CodeWidgetsSourceListSortable* _tmp48_;
	CodeWidgetsSourceListItem* _tmp49_;
	CodeWidgetsSourceListItem* _tmp50_;
	gboolean result;
#line 1310 "../src/Widgets/SourceList/SourceList.vala"
	self = (CodeWidgetsSourceListDataModel*) base;
#line 1310 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (dest != NULL, FALSE);
#line 1310 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (selection_data != NULL, FALSE);
#line 1317 "../src/Widgets/SourceList/SourceList.vala"
	_tmp3_ = gtk_tree_get_row_drag_data (selection_data, &_tmp1_, &_tmp2_);
#line 1317 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (model);
#line 1317 "../src/Widgets/SourceList/SourceList.vala"
	_tmp4_ = _g_object_ref0 (_tmp1_);
#line 1317 "../src/Widgets/SourceList/SourceList.vala"
	model = _tmp4_;
#line 1317 "../src/Widgets/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (src_path);
#line 1317 "../src/Widgets/SourceList/SourceList.vala"
	src_path = _tmp2_;
#line 1317 "../src/Widgets/SourceList/SourceList.vala"
	if (!_tmp3_) {
#line 1317 "../src/Widgets/SourceList/SourceList.vala"
		_tmp0_ = TRUE;
#line 6494 "SourceList.c"
	} else {
		GtkTreeModel* _tmp5_;
		GtkTreeStore* _tmp6_;
#line 1317 "../src/Widgets/SourceList/SourceList.vala"
		_tmp5_ = model;
#line 1317 "../src/Widgets/SourceList/SourceList.vala"
		_tmp6_ = self->priv->child_tree;
#line 1317 "../src/Widgets/SourceList/SourceList.vala"
		_tmp0_ = _tmp5_ != G_TYPE_CHECK_INSTANCE_CAST (_tmp6_, gtk_tree_model_get_type (), GtkTreeModel);
#line 6504 "SourceList.c"
	}
#line 1317 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp0_) {
#line 1318 "../src/Widgets/SourceList/SourceList.vala"
		result = FALSE;
#line 1318 "../src/Widgets/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (src_path);
#line 1318 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (model);
#line 1318 "../src/Widgets/SourceList/SourceList.vala"
		return result;
#line 6516 "SourceList.c"
	}
#line 1321 "../src/Widgets/SourceList/SourceList.vala"
	_tmp7_ = gtk_tree_model_filter_convert_path_to_child_path ((GtkTreeModelFilter*) self, dest);
#line 1321 "../src/Widgets/SourceList/SourceList.vala"
	child_dest = _tmp7_;
#line 1324 "../src/Widgets/SourceList/SourceList.vala"
	_tmp9_ = child_dest;
#line 1324 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp9_ == NULL) {
#line 1324 "../src/Widgets/SourceList/SourceList.vala"
		_tmp8_ = TRUE;
#line 6528 "SourceList.c"
	} else {
		GtkTreePath* _tmp10_;
		GtkTreePath* _tmp11_;
#line 1324 "../src/Widgets/SourceList/SourceList.vala"
		_tmp10_ = src_path;
#line 1324 "../src/Widgets/SourceList/SourceList.vala"
		_tmp11_ = child_dest;
#line 1324 "../src/Widgets/SourceList/SourceList.vala"
		_tmp8_ = gtk_tree_path_compare (_tmp10_, _tmp11_) == 0;
#line 6538 "SourceList.c"
	}
#line 1324 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp8_) {
#line 1325 "../src/Widgets/SourceList/SourceList.vala"
		result = FALSE;
#line 1325 "../src/Widgets/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (child_dest);
#line 1325 "../src/Widgets/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (src_path);
#line 1325 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (model);
#line 1325 "../src/Widgets/SourceList/SourceList.vala"
		return result;
#line 6552 "SourceList.c"
	}
#line 1329 "../src/Widgets/SourceList/SourceList.vala"
	_tmp12_ = src_path;
#line 1329 "../src/Widgets/SourceList/SourceList.vala"
	src_depth = gtk_tree_path_get_depth (_tmp12_);
#line 1330 "../src/Widgets/SourceList/SourceList.vala"
	_tmp13_ = child_dest;
#line 1330 "../src/Widgets/SourceList/SourceList.vala"
	dest_depth = gtk_tree_path_get_depth (_tmp13_);
#line 1332 "../src/Widgets/SourceList/SourceList.vala"
	if (src_depth != dest_depth) {
#line 1333 "../src/Widgets/SourceList/SourceList.vala"
		result = FALSE;
#line 1333 "../src/Widgets/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (child_dest);
#line 1333 "../src/Widgets/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (src_path);
#line 1333 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (model);
#line 1333 "../src/Widgets/SourceList/SourceList.vala"
		return result;
#line 6574 "SourceList.c"
	}
#line 1336 "../src/Widgets/SourceList/SourceList.vala"
	if (src_depth < 1) {
#line 1337 "../src/Widgets/SourceList/SourceList.vala"
		result = FALSE;
#line 1337 "../src/Widgets/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (child_dest);
#line 1337 "../src/Widgets/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (src_path);
#line 1337 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (model);
#line 1337 "../src/Widgets/SourceList/SourceList.vala"
		return result;
#line 6588 "SourceList.c"
	}
#line 1339 "../src/Widgets/SourceList/SourceList.vala"
	parent = NULL;
#line 1344 "../src/Widgets/SourceList/SourceList.vala"
	if (src_depth == 1) {
#line 6594 "SourceList.c"
		CodeWidgetsSourceListExpandableItem* _tmp14_;
		CodeWidgetsSourceListExpandableItem* _tmp15_;
		CodeWidgetsSourceListItem* _tmp16_;
#line 1345 "../src/Widgets/SourceList/SourceList.vala"
		_tmp14_ = code_widgets_source_list_data_model_get_root (self);
#line 1345 "../src/Widgets/SourceList/SourceList.vala"
		_tmp15_ = _tmp14_;
#line 1345 "../src/Widgets/SourceList/SourceList.vala"
		_tmp16_ = _g_object_ref0 ((CodeWidgetsSourceListItem*) _tmp15_);
#line 1345 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (parent);
#line 1345 "../src/Widgets/SourceList/SourceList.vala"
		parent = _tmp16_;
#line 6608 "SourceList.c"
	} else {
		gint* src_indices = NULL;
		GtkTreePath* _tmp17_;
		gint _tmp18_ = 0;
		gint* _tmp19_;
		gint* _tmp20_;
		gint _tmp20__length1;
		gint src_indices_length1;
		gint _src_indices_size_;
		gint* dest_indices = NULL;
		GtkTreePath* _tmp21_;
		gint _tmp22_ = 0;
		gint* _tmp23_;
		gint* _tmp24_;
		gint _tmp24__length1;
		gint dest_indices_length1;
		gint _dest_indices_size_;
		gint src_parent_index = 0;
		gint* _tmp25_;
		gint _tmp25__length1;
		gint _tmp26_;
		gint dest_parent_index = 0;
		gint* _tmp27_;
		gint _tmp27__length1;
		gint _tmp28_;
		GtkTreePath* dest_parent = NULL;
		GtkTreePath* _tmp29_;
		gboolean _tmp30_ = FALSE;
		GtkTreePath* _tmp31_;
		GtkTreePath* _tmp33_;
		CodeWidgetsSourceListItem* _tmp34_;
#line 1348 "../src/Widgets/SourceList/SourceList.vala"
		_vala_assert (dest_depth > 1, "dest_depth > 1");
#line 1354 "../src/Widgets/SourceList/SourceList.vala"
		_tmp17_ = src_path;
#line 1354 "../src/Widgets/SourceList/SourceList.vala"
		_tmp19_ = gtk_tree_path_get_indices_with_depth (_tmp17_, &_tmp18_);
#line 1354 "../src/Widgets/SourceList/SourceList.vala"
		_tmp20_ = (_tmp19_ != NULL) ? _vala_array_dup1 (_tmp19_, _tmp18_) : _tmp19_;
#line 1354 "../src/Widgets/SourceList/SourceList.vala"
		_tmp20__length1 = _tmp18_;
#line 1354 "../src/Widgets/SourceList/SourceList.vala"
		src_indices = _tmp20_;
#line 1354 "../src/Widgets/SourceList/SourceList.vala"
		src_indices_length1 = _tmp20__length1;
#line 1354 "../src/Widgets/SourceList/SourceList.vala"
		_src_indices_size_ = src_indices_length1;
#line 1355 "../src/Widgets/SourceList/SourceList.vala"
		_tmp21_ = child_dest;
#line 1355 "../src/Widgets/SourceList/SourceList.vala"
		_tmp23_ = gtk_tree_path_get_indices_with_depth (_tmp21_, &_tmp22_);
#line 1355 "../src/Widgets/SourceList/SourceList.vala"
		_tmp24_ = (_tmp23_ != NULL) ? _vala_array_dup2 (_tmp23_, _tmp22_) : _tmp23_;
#line 1355 "../src/Widgets/SourceList/SourceList.vala"
		_tmp24__length1 = _tmp22_;
#line 1355 "../src/Widgets/SourceList/SourceList.vala"
		dest_indices = _tmp24_;
#line 1355 "../src/Widgets/SourceList/SourceList.vala"
		dest_indices_length1 = _tmp24__length1;
#line 1355 "../src/Widgets/SourceList/SourceList.vala"
		_dest_indices_size_ = dest_indices_length1;
#line 1358 "../src/Widgets/SourceList/SourceList.vala"
		_tmp25_ = src_indices;
#line 1358 "../src/Widgets/SourceList/SourceList.vala"
		_tmp25__length1 = src_indices_length1;
#line 1358 "../src/Widgets/SourceList/SourceList.vala"
		_tmp26_ = _tmp25_[src_depth - 2];
#line 1358 "../src/Widgets/SourceList/SourceList.vala"
		src_parent_index = _tmp26_;
#line 1359 "../src/Widgets/SourceList/SourceList.vala"
		_tmp27_ = dest_indices;
#line 1359 "../src/Widgets/SourceList/SourceList.vala"
		_tmp27__length1 = dest_indices_length1;
#line 1359 "../src/Widgets/SourceList/SourceList.vala"
		_tmp28_ = _tmp27_[dest_depth - 2];
#line 1359 "../src/Widgets/SourceList/SourceList.vala"
		dest_parent_index = _tmp28_;
#line 1361 "../src/Widgets/SourceList/SourceList.vala"
		if (src_parent_index != dest_parent_index) {
#line 1362 "../src/Widgets/SourceList/SourceList.vala"
			result = FALSE;
#line 1362 "../src/Widgets/SourceList/SourceList.vala"
			dest_indices = (g_free (dest_indices), NULL);
#line 1362 "../src/Widgets/SourceList/SourceList.vala"
			src_indices = (g_free (src_indices), NULL);
#line 1362 "../src/Widgets/SourceList/SourceList.vala"
			_g_object_unref0 (parent);
#line 1362 "../src/Widgets/SourceList/SourceList.vala"
			__vala_GtkTreePath_free0 (child_dest);
#line 1362 "../src/Widgets/SourceList/SourceList.vala"
			__vala_GtkTreePath_free0 (src_path);
#line 1362 "../src/Widgets/SourceList/SourceList.vala"
			_g_object_unref0 (model);
#line 1362 "../src/Widgets/SourceList/SourceList.vala"
			return result;
#line 6704 "SourceList.c"
		}
#line 1365 "../src/Widgets/SourceList/SourceList.vala"
		_tmp29_ = __vala_GtkTreePath_copy0 (dest);
#line 1365 "../src/Widgets/SourceList/SourceList.vala"
		dest_parent = _tmp29_;
#line 1367 "../src/Widgets/SourceList/SourceList.vala"
		_tmp31_ = dest_parent;
#line 1367 "../src/Widgets/SourceList/SourceList.vala"
		if (!gtk_tree_path_up (_tmp31_)) {
#line 1367 "../src/Widgets/SourceList/SourceList.vala"
			_tmp30_ = TRUE;
#line 6716 "SourceList.c"
		} else {
			GtkTreePath* _tmp32_;
#line 1367 "../src/Widgets/SourceList/SourceList.vala"
			_tmp32_ = dest_parent;
#line 1367 "../src/Widgets/SourceList/SourceList.vala"
			_tmp30_ = gtk_tree_path_get_depth (_tmp32_) < 1;
#line 6723 "SourceList.c"
		}
#line 1367 "../src/Widgets/SourceList/SourceList.vala"
		if (_tmp30_) {
#line 1368 "../src/Widgets/SourceList/SourceList.vala"
			result = FALSE;
#line 1368 "../src/Widgets/SourceList/SourceList.vala"
			__vala_GtkTreePath_free0 (dest_parent);
#line 1368 "../src/Widgets/SourceList/SourceList.vala"
			dest_indices = (g_free (dest_indices), NULL);
#line 1368 "../src/Widgets/SourceList/SourceList.vala"
			src_indices = (g_free (src_indices), NULL);
#line 1368 "../src/Widgets/SourceList/SourceList.vala"
			_g_object_unref0 (parent);
#line 1368 "../src/Widgets/SourceList/SourceList.vala"
			__vala_GtkTreePath_free0 (child_dest);
#line 1368 "../src/Widgets/SourceList/SourceList.vala"
			__vala_GtkTreePath_free0 (src_path);
#line 1368 "../src/Widgets/SourceList/SourceList.vala"
			_g_object_unref0 (model);
#line 1368 "../src/Widgets/SourceList/SourceList.vala"
			return result;
#line 6745 "SourceList.c"
		}
#line 1370 "../src/Widgets/SourceList/SourceList.vala"
		_tmp33_ = dest_parent;
#line 1370 "../src/Widgets/SourceList/SourceList.vala"
		_tmp34_ = code_widgets_source_list_data_model_get_item_from_path (self, _tmp33_);
#line 1370 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (parent);
#line 1370 "../src/Widgets/SourceList/SourceList.vala"
		parent = _tmp34_;
#line 1344 "../src/Widgets/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (dest_parent);
#line 1344 "../src/Widgets/SourceList/SourceList.vala"
		dest_indices = (g_free (dest_indices), NULL);
#line 1344 "../src/Widgets/SourceList/SourceList.vala"
		src_indices = (g_free (src_indices), NULL);
#line 6761 "SourceList.c"
	}
#line 1373 "../src/Widgets/SourceList/SourceList.vala"
	_tmp35_ = parent;
#line 1373 "../src/Widgets/SourceList/SourceList.vala"
	_tmp36_ = _g_object_ref0 (CODE_WIDGETS_IS_SOURCE_LIST_SORTABLE (_tmp35_) ? ((CodeWidgetsSourceListSortable*) _tmp35_) : NULL);
#line 1373 "../src/Widgets/SourceList/SourceList.vala"
	sortable = _tmp36_;
#line 1375 "../src/Widgets/SourceList/SourceList.vala"
	_tmp38_ = sortable;
#line 1375 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp38_ == NULL) {
#line 1375 "../src/Widgets/SourceList/SourceList.vala"
		_tmp37_ = TRUE;
#line 6775 "SourceList.c"
	} else {
		CodeWidgetsSourceListSortable* _tmp39_;
#line 1375 "../src/Widgets/SourceList/SourceList.vala"
		_tmp39_ = sortable;
#line 1375 "../src/Widgets/SourceList/SourceList.vala"
		_tmp37_ = !code_widgets_source_list_sortable_allow_dnd_sorting (_tmp39_);
#line 6782 "SourceList.c"
	}
#line 1375 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp37_) {
#line 1376 "../src/Widgets/SourceList/SourceList.vala"
		result = FALSE;
#line 1376 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (sortable);
#line 1376 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (parent);
#line 1376 "../src/Widgets/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (child_dest);
#line 1376 "../src/Widgets/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (src_path);
#line 1376 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (model);
#line 1376 "../src/Widgets/SourceList/SourceList.vala"
		return result;
#line 6800 "SourceList.c"
	}
#line 1378 "../src/Widgets/SourceList/SourceList.vala"
	_tmp40_ = code_widgets_source_list_data_model_get_item_from_path (self, dest);
#line 1378 "../src/Widgets/SourceList/SourceList.vala"
	dest_item = _tmp40_;
#line 1380 "../src/Widgets/SourceList/SourceList.vala"
	_tmp41_ = dest_item;
#line 1380 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp41_ == NULL) {
#line 1381 "../src/Widgets/SourceList/SourceList.vala"
		result = TRUE;
#line 1381 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (dest_item);
#line 1381 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (sortable);
#line 1381 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (parent);
#line 1381 "../src/Widgets/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (child_dest);
#line 1381 "../src/Widgets/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (src_path);
#line 1381 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (model);
#line 1381 "../src/Widgets/SourceList/SourceList.vala"
		return result;
#line 6826 "SourceList.c"
	}
#line 1383 "../src/Widgets/SourceList/SourceList.vala"
	source_item = NULL;
#line 1384 "../src/Widgets/SourceList/SourceList.vala"
	_tmp42_ = src_path;
#line 1384 "../src/Widgets/SourceList/SourceList.vala"
	_tmp43_ = gtk_tree_model_filter_convert_child_path_to_path ((GtkTreeModelFilter*) self, _tmp42_);
#line 1384 "../src/Widgets/SourceList/SourceList.vala"
	filter_src_path = _tmp43_;
#line 1386 "../src/Widgets/SourceList/SourceList.vala"
	_tmp44_ = filter_src_path;
#line 1386 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp44_ != NULL) {
#line 6840 "SourceList.c"
		GtkTreePath* _tmp45_;
		CodeWidgetsSourceListItem* _tmp46_;
#line 1387 "../src/Widgets/SourceList/SourceList.vala"
		_tmp45_ = filter_src_path;
#line 1387 "../src/Widgets/SourceList/SourceList.vala"
		_tmp46_ = code_widgets_source_list_data_model_get_item_from_path (self, _tmp45_);
#line 1387 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (source_item);
#line 1387 "../src/Widgets/SourceList/SourceList.vala"
		source_item = _tmp46_;
#line 6851 "SourceList.c"
	}
#line 1389 "../src/Widgets/SourceList/SourceList.vala"
	_tmp47_ = source_item;
#line 1389 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp47_ == NULL) {
#line 1390 "../src/Widgets/SourceList/SourceList.vala"
		result = FALSE;
#line 1390 "../src/Widgets/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (filter_src_path);
#line 1390 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (source_item);
#line 1390 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (dest_item);
#line 1390 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (sortable);
#line 1390 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (parent);
#line 1390 "../src/Widgets/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (child_dest);
#line 1390 "../src/Widgets/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (src_path);
#line 1390 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (model);
#line 1390 "../src/Widgets/SourceList/SourceList.vala"
		return result;
#line 6877 "SourceList.c"
	}
#line 1398 "../src/Widgets/SourceList/SourceList.vala"
	_tmp48_ = sortable;
#line 1398 "../src/Widgets/SourceList/SourceList.vala"
	_tmp49_ = dest_item;
#line 1398 "../src/Widgets/SourceList/SourceList.vala"
	_tmp50_ = source_item;
#line 1398 "../src/Widgets/SourceList/SourceList.vala"
	if (code_widgets_source_list_sortable_compare (_tmp48_, _tmp49_, _tmp50_) >= 0) {
#line 6887 "SourceList.c"
		CodeWidgetsSourceListItem* dest_item_prev = NULL;
		CodeWidgetsSourceListItem* _tmp51_;
		gboolean _tmp52_ = FALSE;
		gboolean _tmp53_ = FALSE;
		CodeWidgetsSourceListItem* _tmp54_;
#line 1399 "../src/Widgets/SourceList/SourceList.vala"
		if (!gtk_tree_path_prev (dest)) {
#line 1400 "../src/Widgets/SourceList/SourceList.vala"
			result = TRUE;
#line 1400 "../src/Widgets/SourceList/SourceList.vala"
			__vala_GtkTreePath_free0 (filter_src_path);
#line 1400 "../src/Widgets/SourceList/SourceList.vala"
			_g_object_unref0 (source_item);
#line 1400 "../src/Widgets/SourceList/SourceList.vala"
			_g_object_unref0 (dest_item);
#line 1400 "../src/Widgets/SourceList/SourceList.vala"
			_g_object_unref0 (sortable);
#line 1400 "../src/Widgets/SourceList/SourceList.vala"
			_g_object_unref0 (parent);
#line 1400 "../src/Widgets/SourceList/SourceList.vala"
			__vala_GtkTreePath_free0 (child_dest);
#line 1400 "../src/Widgets/SourceList/SourceList.vala"
			__vala_GtkTreePath_free0 (src_path);
#line 1400 "../src/Widgets/SourceList/SourceList.vala"
			_g_object_unref0 (model);
#line 1400 "../src/Widgets/SourceList/SourceList.vala"
			return result;
#line 6915 "SourceList.c"
		}
#line 1404 "../src/Widgets/SourceList/SourceList.vala"
		_tmp51_ = code_widgets_source_list_data_model_get_item_from_path (self, dest);
#line 1404 "../src/Widgets/SourceList/SourceList.vala"
		dest_item_prev = _tmp51_;
#line 1406 "../src/Widgets/SourceList/SourceList.vala"
		_tmp54_ = dest_item_prev;
#line 1406 "../src/Widgets/SourceList/SourceList.vala"
		if (_tmp54_ != NULL) {
#line 6925 "SourceList.c"
			CodeWidgetsSourceListItem* _tmp55_;
			CodeWidgetsSourceListItem* _tmp56_;
#line 1407 "../src/Widgets/SourceList/SourceList.vala"
			_tmp55_ = dest_item_prev;
#line 1407 "../src/Widgets/SourceList/SourceList.vala"
			_tmp56_ = source_item;
#line 1407 "../src/Widgets/SourceList/SourceList.vala"
			_tmp53_ = _tmp55_ != _tmp56_;
#line 6934 "SourceList.c"
		} else {
#line 1406 "../src/Widgets/SourceList/SourceList.vala"
			_tmp53_ = FALSE;
#line 6938 "SourceList.c"
		}
#line 1406 "../src/Widgets/SourceList/SourceList.vala"
		if (_tmp53_) {
#line 6942 "SourceList.c"
			CodeWidgetsSourceListSortable* _tmp57_;
			CodeWidgetsSourceListItem* _tmp58_;
			CodeWidgetsSourceListItem* _tmp59_;
#line 1408 "../src/Widgets/SourceList/SourceList.vala"
			_tmp57_ = sortable;
#line 1408 "../src/Widgets/SourceList/SourceList.vala"
			_tmp58_ = dest_item_prev;
#line 1408 "../src/Widgets/SourceList/SourceList.vala"
			_tmp59_ = source_item;
#line 1408 "../src/Widgets/SourceList/SourceList.vala"
			_tmp52_ = code_widgets_source_list_sortable_compare (_tmp57_, _tmp58_, _tmp59_) <= 0;
#line 6954 "SourceList.c"
		} else {
#line 1406 "../src/Widgets/SourceList/SourceList.vala"
			_tmp52_ = FALSE;
#line 6958 "SourceList.c"
		}
#line 1406 "../src/Widgets/SourceList/SourceList.vala"
		result = _tmp52_;
#line 1406 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (dest_item_prev);
#line 1406 "../src/Widgets/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (filter_src_path);
#line 1406 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (source_item);
#line 1406 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (dest_item);
#line 1406 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (sortable);
#line 1406 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (parent);
#line 1406 "../src/Widgets/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (child_dest);
#line 1406 "../src/Widgets/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (src_path);
#line 1406 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (model);
#line 1406 "../src/Widgets/SourceList/SourceList.vala"
		return result;
#line 6982 "SourceList.c"
	}
#line 1411 "../src/Widgets/SourceList/SourceList.vala"
	result = FALSE;
#line 1411 "../src/Widgets/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (filter_src_path);
#line 1411 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (source_item);
#line 1411 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (dest_item);
#line 1411 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (sortable);
#line 1411 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (parent);
#line 1411 "../src/Widgets/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (child_dest);
#line 1411 "../src/Widgets/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (src_path);
#line 1411 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (model);
#line 1411 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 7004 "SourceList.c"
}

/**
         * Override default implementation of TreeDragSource
         *
         * drag_data_delete is not overriden because the default implementation
         * does exactly what we need.
         */
static gboolean
code_widgets_source_list_data_model_real_drag_data_get (GtkTreeDragSource* base,
                                                        GtkTreePath* path,
                                                        GtkSelectionData* selection_data)
{
	CodeWidgetsSourceListDataModel * self;
	CodeWidgetsSourceListDragSource* drag_source_item = NULL;
	CodeWidgetsSourceListItem* _tmp0_;
	CodeWidgetsSourceListDragSource* _tmp1_;
	gboolean _tmp2_ = FALSE;
	CodeWidgetsSourceListDragSource* _tmp3_;
	gboolean result;
#line 1421 "../src/Widgets/SourceList/SourceList.vala"
	self = (CodeWidgetsSourceListDataModel*) base;
#line 1421 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (path != NULL, FALSE);
#line 1421 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (selection_data != NULL, FALSE);
#line 1424 "../src/Widgets/SourceList/SourceList.vala"
	if (gtk_selection_data_get_target (selection_data) == gdk_atom_intern_static_string ("GTK_TREE_MODEL_ROW")) {
#line 1425 "../src/Widgets/SourceList/SourceList.vala"
		result = code_widgets_source_list_data_model_gtk_tree_drag_source_parent_iface->drag_data_get ((GtkTreeDragSource*) G_TYPE_CHECK_INSTANCE_CAST (self, gtk_tree_model_filter_get_type (), GtkTreeModelFilter), path, selection_data);
#line 1425 "../src/Widgets/SourceList/SourceList.vala"
		return result;
#line 7037 "SourceList.c"
	}
#line 1428 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = code_widgets_source_list_data_model_get_item_from_path (self, path);
#line 1428 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = CODE_WIDGETS_IS_SOURCE_LIST_DRAG_SOURCE (_tmp0_) ? ((CodeWidgetsSourceListDragSource*) _tmp0_) : NULL;
#line 1428 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp1_ == NULL) {
#line 1428 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (_tmp0_);
#line 7047 "SourceList.c"
	}
#line 1428 "../src/Widgets/SourceList/SourceList.vala"
	drag_source_item = _tmp1_;
#line 1429 "../src/Widgets/SourceList/SourceList.vala"
	_tmp3_ = drag_source_item;
#line 1429 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp3_ != NULL) {
#line 7055 "SourceList.c"
		CodeWidgetsSourceListDragSource* _tmp4_;
#line 1429 "../src/Widgets/SourceList/SourceList.vala"
		_tmp4_ = drag_source_item;
#line 1429 "../src/Widgets/SourceList/SourceList.vala"
		_tmp2_ = code_widgets_source_list_drag_source_draggable (_tmp4_);
#line 7061 "SourceList.c"
	} else {
#line 1429 "../src/Widgets/SourceList/SourceList.vala"
		_tmp2_ = FALSE;
#line 7065 "SourceList.c"
	}
#line 1429 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp2_) {
#line 7069 "SourceList.c"
		CodeWidgetsSourceListDragSource* _tmp5_;
#line 1430 "../src/Widgets/SourceList/SourceList.vala"
		_tmp5_ = drag_source_item;
#line 1430 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_drag_source_prepare_selection_data (_tmp5_, selection_data);
#line 1431 "../src/Widgets/SourceList/SourceList.vala"
		result = TRUE;
#line 1431 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (drag_source_item);
#line 1431 "../src/Widgets/SourceList/SourceList.vala"
		return result;
#line 7081 "SourceList.c"
	}
#line 1434 "../src/Widgets/SourceList/SourceList.vala"
	result = FALSE;
#line 1434 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (drag_source_item);
#line 1434 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 7089 "SourceList.c"
}

static gboolean
code_widgets_source_list_data_model_real_row_draggable (GtkTreeDragSource* base,
                                                        GtkTreePath* path)
{
	CodeWidgetsSourceListDataModel * self;
	CodeWidgetsSourceListItem* item = NULL;
	CodeWidgetsSourceListItem* _tmp0_;
	CodeWidgetsSourceListItem* _tmp1_;
	gboolean result;
#line 1437 "../src/Widgets/SourceList/SourceList.vala"
	self = (CodeWidgetsSourceListDataModel*) base;
#line 1437 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (path != NULL, FALSE);
#line 1438 "../src/Widgets/SourceList/SourceList.vala"
	if (!code_widgets_source_list_data_model_gtk_tree_drag_source_parent_iface->row_draggable ((GtkTreeDragSource*) G_TYPE_CHECK_INSTANCE_CAST (self, gtk_tree_model_filter_get_type (), GtkTreeModelFilter), path)) {
#line 1439 "../src/Widgets/SourceList/SourceList.vala"
		result = FALSE;
#line 1439 "../src/Widgets/SourceList/SourceList.vala"
		return result;
#line 7111 "SourceList.c"
	}
#line 1441 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = code_widgets_source_list_data_model_get_item_from_path (self, path);
#line 1441 "../src/Widgets/SourceList/SourceList.vala"
	item = _tmp0_;
#line 1443 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = item;
#line 1443 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp1_ != NULL) {
#line 7121 "SourceList.c"
		CodeWidgetsSourceListSortable* sortable_item = NULL;
		CodeWidgetsSourceListItem* _tmp2_;
		CodeWidgetsSourceListExpandableItem* _tmp3_;
		CodeWidgetsSourceListExpandableItem* _tmp4_;
		CodeWidgetsSourceListSortable* _tmp5_;
		gboolean _tmp6_ = FALSE;
		CodeWidgetsSourceListSortable* _tmp7_;
		CodeWidgetsSourceListDragSource* drag_source_item = NULL;
		CodeWidgetsSourceListItem* _tmp9_;
		CodeWidgetsSourceListDragSource* _tmp10_;
		gboolean _tmp11_ = FALSE;
		CodeWidgetsSourceListDragSource* _tmp12_;
#line 1445 "../src/Widgets/SourceList/SourceList.vala"
		_tmp2_ = item;
#line 1445 "../src/Widgets/SourceList/SourceList.vala"
		_tmp3_ = code_widgets_source_list_item_get_parent (_tmp2_);
#line 1445 "../src/Widgets/SourceList/SourceList.vala"
		_tmp4_ = _tmp3_;
#line 1445 "../src/Widgets/SourceList/SourceList.vala"
		_tmp5_ = _g_object_ref0 (CODE_WIDGETS_IS_SOURCE_LIST_SORTABLE (_tmp4_) ? ((CodeWidgetsSourceListSortable*) _tmp4_) : NULL);
#line 1445 "../src/Widgets/SourceList/SourceList.vala"
		sortable_item = _tmp5_;
#line 1447 "../src/Widgets/SourceList/SourceList.vala"
		_tmp7_ = sortable_item;
#line 1447 "../src/Widgets/SourceList/SourceList.vala"
		if (_tmp7_ != NULL) {
#line 7148 "SourceList.c"
			CodeWidgetsSourceListSortable* _tmp8_;
#line 1447 "../src/Widgets/SourceList/SourceList.vala"
			_tmp8_ = sortable_item;
#line 1447 "../src/Widgets/SourceList/SourceList.vala"
			_tmp6_ = code_widgets_source_list_sortable_allow_dnd_sorting (_tmp8_);
#line 7154 "SourceList.c"
		} else {
#line 1447 "../src/Widgets/SourceList/SourceList.vala"
			_tmp6_ = FALSE;
#line 7158 "SourceList.c"
		}
#line 1447 "../src/Widgets/SourceList/SourceList.vala"
		if (_tmp6_) {
#line 1448 "../src/Widgets/SourceList/SourceList.vala"
			result = TRUE;
#line 1448 "../src/Widgets/SourceList/SourceList.vala"
			_g_object_unref0 (sortable_item);
#line 1448 "../src/Widgets/SourceList/SourceList.vala"
			_g_object_unref0 (item);
#line 1448 "../src/Widgets/SourceList/SourceList.vala"
			return result;
#line 7170 "SourceList.c"
		}
#line 1452 "../src/Widgets/SourceList/SourceList.vala"
		_tmp9_ = item;
#line 1452 "../src/Widgets/SourceList/SourceList.vala"
		_tmp10_ = _g_object_ref0 (CODE_WIDGETS_IS_SOURCE_LIST_DRAG_SOURCE (_tmp9_) ? ((CodeWidgetsSourceListDragSource*) _tmp9_) : NULL);
#line 1452 "../src/Widgets/SourceList/SourceList.vala"
		drag_source_item = _tmp10_;
#line 1454 "../src/Widgets/SourceList/SourceList.vala"
		_tmp12_ = drag_source_item;
#line 1454 "../src/Widgets/SourceList/SourceList.vala"
		if (_tmp12_ != NULL) {
#line 7182 "SourceList.c"
			CodeWidgetsSourceListDragSource* _tmp13_;
#line 1454 "../src/Widgets/SourceList/SourceList.vala"
			_tmp13_ = drag_source_item;
#line 1454 "../src/Widgets/SourceList/SourceList.vala"
			_tmp11_ = code_widgets_source_list_drag_source_draggable (_tmp13_);
#line 7188 "SourceList.c"
		} else {
#line 1454 "../src/Widgets/SourceList/SourceList.vala"
			_tmp11_ = FALSE;
#line 7192 "SourceList.c"
		}
#line 1454 "../src/Widgets/SourceList/SourceList.vala"
		if (_tmp11_) {
#line 1455 "../src/Widgets/SourceList/SourceList.vala"
			result = TRUE;
#line 1455 "../src/Widgets/SourceList/SourceList.vala"
			_g_object_unref0 (drag_source_item);
#line 1455 "../src/Widgets/SourceList/SourceList.vala"
			_g_object_unref0 (sortable_item);
#line 1455 "../src/Widgets/SourceList/SourceList.vala"
			_g_object_unref0 (item);
#line 1455 "../src/Widgets/SourceList/SourceList.vala"
			return result;
#line 7206 "SourceList.c"
		}
#line 1443 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (drag_source_item);
#line 1443 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (sortable_item);
#line 7212 "SourceList.c"
	}
#line 1458 "../src/Widgets/SourceList/SourceList.vala"
	result = FALSE;
#line 1458 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (item);
#line 1458 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 7220 "SourceList.c"
}

static CodeWidgetsSourceListExpandableItem*
code_widgets_source_list_data_model_get_root (CodeWidgetsSourceListDataModel* self)
{
	CodeWidgetsSourceListExpandableItem* result;
	CodeWidgetsSourceListExpandableItem* _tmp0_;
#line 828 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 828 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->_root;
#line 828 "../src/Widgets/SourceList/SourceList.vala"
	result = _tmp0_;
#line 828 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 7236 "SourceList.c"
}

static void
code_widgets_source_list_data_model_set_root (CodeWidgetsSourceListDataModel* self,
                                              CodeWidgetsSourceListExpandableItem* value)
{
	CodeWidgetsSourceListExpandableItem* _tmp0_;
	CodeWidgetsSourceListExpandableItem* _tmp12_;
	CodeWidgetsSourceListExpandableItem* _tmp13_;
#line 829 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 830 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->_root;
#line 830 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp0_ != NULL) {
#line 7252 "SourceList.c"
		CodeWidgetsSourceListExpandableItem* _tmp1_;
#line 831 "../src/Widgets/SourceList/SourceList.vala"
		_tmp1_ = self->priv->_root;
#line 831 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_data_model_remove_children_monitor (self, _tmp1_);
#line 7258 "SourceList.c"
		{
			GeeIterator* _item_it = NULL;
			CodeWidgetsSourceListExpandableItem* _tmp2_;
			GeeCollection* _tmp3_;
			GeeCollection* _tmp4_;
			GeeCollection* _tmp5_;
			GeeIterator* _tmp6_;
			GeeIterator* _tmp7_;
#line 832 "../src/Widgets/SourceList/SourceList.vala"
			_tmp2_ = self->priv->_root;
#line 832 "../src/Widgets/SourceList/SourceList.vala"
			_tmp3_ = code_widgets_source_list_expandable_item_get_children (_tmp2_);
#line 832 "../src/Widgets/SourceList/SourceList.vala"
			_tmp4_ = _tmp3_;
#line 832 "../src/Widgets/SourceList/SourceList.vala"
			_tmp5_ = _tmp4_;
#line 832 "../src/Widgets/SourceList/SourceList.vala"
			_tmp6_ = gee_iterable_iterator ((GeeIterable*) _tmp5_);
#line 832 "../src/Widgets/SourceList/SourceList.vala"
			_tmp7_ = _tmp6_;
#line 832 "../src/Widgets/SourceList/SourceList.vala"
			_g_object_unref0 (_tmp5_);
#line 832 "../src/Widgets/SourceList/SourceList.vala"
			_item_it = _tmp7_;
#line 832 "../src/Widgets/SourceList/SourceList.vala"
			while (TRUE) {
#line 7285 "SourceList.c"
				GeeIterator* _tmp8_;
				CodeWidgetsSourceListItem* item = NULL;
				GeeIterator* _tmp9_;
				gpointer _tmp10_;
				CodeWidgetsSourceListItem* _tmp11_;
#line 832 "../src/Widgets/SourceList/SourceList.vala"
				_tmp8_ = _item_it;
#line 832 "../src/Widgets/SourceList/SourceList.vala"
				if (!gee_iterator_next (_tmp8_)) {
#line 832 "../src/Widgets/SourceList/SourceList.vala"
					break;
#line 7297 "SourceList.c"
				}
#line 832 "../src/Widgets/SourceList/SourceList.vala"
				_tmp9_ = _item_it;
#line 832 "../src/Widgets/SourceList/SourceList.vala"
				_tmp10_ = gee_iterator_get (_tmp9_);
#line 832 "../src/Widgets/SourceList/SourceList.vala"
				item = (CodeWidgetsSourceListItem*) _tmp10_;
#line 833 "../src/Widgets/SourceList/SourceList.vala"
				_tmp11_ = item;
#line 833 "../src/Widgets/SourceList/SourceList.vala"
				code_widgets_source_list_data_model_remove_item (self, _tmp11_);
#line 832 "../src/Widgets/SourceList/SourceList.vala"
				_g_object_unref0 (item);
#line 7311 "SourceList.c"
			}
#line 832 "../src/Widgets/SourceList/SourceList.vala"
			_g_object_unref0 (_item_it);
#line 7315 "SourceList.c"
		}
	}
#line 836 "../src/Widgets/SourceList/SourceList.vala"
	_tmp12_ = _g_object_ref0 (value);
#line 836 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->_root);
#line 836 "../src/Widgets/SourceList/SourceList.vala"
	self->priv->_root = _tmp12_;
#line 838 "../src/Widgets/SourceList/SourceList.vala"
	_tmp13_ = self->priv->_root;
#line 838 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_data_model_add_children_monitor (self, _tmp13_);
#line 7328 "SourceList.c"
	{
		GeeIterator* _item_it = NULL;
		CodeWidgetsSourceListExpandableItem* _tmp14_;
		GeeCollection* _tmp15_;
		GeeCollection* _tmp16_;
		GeeCollection* _tmp17_;
		GeeIterator* _tmp18_;
		GeeIterator* _tmp19_;
#line 839 "../src/Widgets/SourceList/SourceList.vala"
		_tmp14_ = self->priv->_root;
#line 839 "../src/Widgets/SourceList/SourceList.vala"
		_tmp15_ = code_widgets_source_list_expandable_item_get_children (_tmp14_);
#line 839 "../src/Widgets/SourceList/SourceList.vala"
		_tmp16_ = _tmp15_;
#line 839 "../src/Widgets/SourceList/SourceList.vala"
		_tmp17_ = _tmp16_;
#line 839 "../src/Widgets/SourceList/SourceList.vala"
		_tmp18_ = gee_iterable_iterator ((GeeIterable*) _tmp17_);
#line 839 "../src/Widgets/SourceList/SourceList.vala"
		_tmp19_ = _tmp18_;
#line 839 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (_tmp17_);
#line 839 "../src/Widgets/SourceList/SourceList.vala"
		_item_it = _tmp19_;
#line 839 "../src/Widgets/SourceList/SourceList.vala"
		while (TRUE) {
#line 7355 "SourceList.c"
			GeeIterator* _tmp20_;
			CodeWidgetsSourceListItem* item = NULL;
			GeeIterator* _tmp21_;
			gpointer _tmp22_;
			CodeWidgetsSourceListItem* _tmp23_;
#line 839 "../src/Widgets/SourceList/SourceList.vala"
			_tmp20_ = _item_it;
#line 839 "../src/Widgets/SourceList/SourceList.vala"
			if (!gee_iterator_next (_tmp20_)) {
#line 839 "../src/Widgets/SourceList/SourceList.vala"
				break;
#line 7367 "SourceList.c"
			}
#line 839 "../src/Widgets/SourceList/SourceList.vala"
			_tmp21_ = _item_it;
#line 839 "../src/Widgets/SourceList/SourceList.vala"
			_tmp22_ = gee_iterator_get (_tmp21_);
#line 839 "../src/Widgets/SourceList/SourceList.vala"
			item = (CodeWidgetsSourceListItem*) _tmp22_;
#line 840 "../src/Widgets/SourceList/SourceList.vala"
			_tmp23_ = item;
#line 840 "../src/Widgets/SourceList/SourceList.vala"
			code_widgets_source_list_data_model_add_item (self, _tmp23_);
#line 839 "../src/Widgets/SourceList/SourceList.vala"
			_g_object_unref0 (item);
#line 7381 "SourceList.c"
		}
#line 839 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (_item_it);
#line 7385 "SourceList.c"
	}
#line 829 "../src/Widgets/SourceList/SourceList.vala"
	g_object_notify_by_pspec ((GObject *) self, code_widgets_source_list_data_model_properties[CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_ROOT_PROPERTY]);
#line 7389 "SourceList.c"
}

static inline gpointer
code_widgets_source_list_data_model_node_wrapper_get_instance_private (CodeWidgetsSourceListDataModelNodeWrapper* self)
{
	return G_STRUCT_MEMBER_P (self, CodeWidgetsSourceListDataModelNodeWrapper_private_offset);
}

static void
_vala_GtkTreeRowReference_free (GtkTreeRowReference* self)
{
#line 734 "../src/Widgets/SourceList/SourceList.vala"
	g_boxed_free (gtk_tree_row_reference_get_type (), self);
#line 7403 "SourceList.c"
}

static CodeWidgetsSourceListDataModelNodeWrapper*
code_widgets_source_list_data_model_node_wrapper_construct (GType object_type,
                                                            GtkTreeModel* model,
                                                            GtkTreeIter* iter)
{
	CodeWidgetsSourceListDataModelNodeWrapper* self = NULL;
	GtkTreeIter _tmp0_;
	GtkTreePath* _tmp1_;
	GtkTreePath* _tmp2_;
	GtkTreeRowReference* _tmp3_;
#line 771 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (model != NULL, NULL);
#line 771 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (iter != NULL, NULL);
#line 771 "../src/Widgets/SourceList/SourceList.vala"
	self = (CodeWidgetsSourceListDataModelNodeWrapper*) g_type_create_instance (object_type);
#line 772 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = *iter;
#line 772 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = gtk_tree_model_get_path (model, &_tmp0_);
#line 772 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = _tmp1_;
#line 772 "../src/Widgets/SourceList/SourceList.vala"
	_tmp3_ = gtk_tree_row_reference_new (model, _tmp2_);
#line 772 "../src/Widgets/SourceList/SourceList.vala"
	__vala_GtkTreeRowReference_free0 (self->priv->row_reference);
#line 772 "../src/Widgets/SourceList/SourceList.vala"
	self->priv->row_reference = _tmp3_;
#line 772 "../src/Widgets/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (_tmp2_);
#line 771 "../src/Widgets/SourceList/SourceList.vala"
	return self;
#line 7438 "SourceList.c"
}

static CodeWidgetsSourceListDataModelNodeWrapper*
code_widgets_source_list_data_model_node_wrapper_new (GtkTreeModel* model,
                                                      GtkTreeIter* iter)
{
#line 771 "../src/Widgets/SourceList/SourceList.vala"
	return code_widgets_source_list_data_model_node_wrapper_construct (CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_TYPE_NODE_WRAPPER, model, iter);
#line 7447 "SourceList.c"
}

static GtkTreeIter*
code_widgets_source_list_data_model_node_wrapper_get_iter (CodeWidgetsSourceListDataModelNodeWrapper* self)
{
	GtkTreeIter* result;
	GtkTreeIter* rv = NULL;
	gboolean _tmp0_;
	gboolean _tmp1_;
#line 740 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 741 "../src/Widgets/SourceList/SourceList.vala"
	rv = NULL;
#line 743 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = code_widgets_source_list_data_model_node_wrapper_get_valid (self);
#line 743 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = _tmp0_;
#line 743 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp1_) {
#line 7467 "SourceList.c"
		GtkTreePath* _path = NULL;
		GtkTreePath* _tmp2_;
		GtkTreePath* _tmp3_;
		GtkTreePath* _tmp4_;
#line 744 "../src/Widgets/SourceList/SourceList.vala"
		_tmp2_ = code_widgets_source_list_data_model_node_wrapper_get_path (self);
#line 744 "../src/Widgets/SourceList/SourceList.vala"
		_tmp3_ = _tmp2_;
#line 744 "../src/Widgets/SourceList/SourceList.vala"
		_path = _tmp3_;
#line 745 "../src/Widgets/SourceList/SourceList.vala"
		_tmp4_ = _path;
#line 745 "../src/Widgets/SourceList/SourceList.vala"
		if (_tmp4_ != NULL) {
#line 7482 "SourceList.c"
			GtkTreeIter _iter = {0};
			GtkTreeRowReference* _tmp5_;
			GtkTreeModel* _tmp6_;
			GtkTreePath* _tmp7_;
			GtkTreeIter _tmp8_ = {0};
			gboolean _tmp9_;
#line 747 "../src/Widgets/SourceList/SourceList.vala"
			_tmp5_ = self->priv->row_reference;
#line 747 "../src/Widgets/SourceList/SourceList.vala"
			_tmp6_ = gtk_tree_row_reference_get_model (_tmp5_);
#line 747 "../src/Widgets/SourceList/SourceList.vala"
			_tmp7_ = _path;
#line 747 "../src/Widgets/SourceList/SourceList.vala"
			_tmp9_ = gtk_tree_model_get_iter (_tmp6_, &_tmp8_, _tmp7_);
#line 747 "../src/Widgets/SourceList/SourceList.vala"
			_iter = _tmp8_;
#line 747 "../src/Widgets/SourceList/SourceList.vala"
			if (_tmp9_) {
#line 7501 "SourceList.c"
				GtkTreeIter _tmp10_;
				GtkTreeIter* _tmp11_;
#line 748 "../src/Widgets/SourceList/SourceList.vala"
				_tmp10_ = _iter;
#line 748 "../src/Widgets/SourceList/SourceList.vala"
				_tmp11_ = __vala_GtkTreeIter_copy0 (&_tmp10_);
#line 748 "../src/Widgets/SourceList/SourceList.vala"
				__vala_GtkTreeIter_free0 (rv);
#line 748 "../src/Widgets/SourceList/SourceList.vala"
				rv = _tmp11_;
#line 7512 "SourceList.c"
			}
		}
#line 743 "../src/Widgets/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (_path);
#line 7517 "SourceList.c"
	}
#line 752 "../src/Widgets/SourceList/SourceList.vala"
	result = rv;
#line 752 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 7523 "SourceList.c"
}

static GtkTreePath*
code_widgets_source_list_data_model_node_wrapper_get_path (CodeWidgetsSourceListDataModelNodeWrapper* self)
{
	GtkTreePath* result;
	GtkTreePath* _tmp0_ = NULL;
	gboolean _tmp1_;
	gboolean _tmp2_;
#line 760 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 760 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = code_widgets_source_list_data_model_node_wrapper_get_valid (self);
#line 760 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = _tmp1_;
#line 760 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp2_) {
#line 7541 "SourceList.c"
		GtkTreeRowReference* _tmp3_;
		GtkTreePath* _tmp4_;
#line 760 "../src/Widgets/SourceList/SourceList.vala"
		_tmp3_ = self->priv->row_reference;
#line 760 "../src/Widgets/SourceList/SourceList.vala"
		_tmp4_ = gtk_tree_row_reference_get_path (_tmp3_);
#line 760 "../src/Widgets/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (_tmp0_);
#line 760 "../src/Widgets/SourceList/SourceList.vala"
		_tmp0_ = _tmp4_;
#line 7552 "SourceList.c"
	} else {
#line 760 "../src/Widgets/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (_tmp0_);
#line 760 "../src/Widgets/SourceList/SourceList.vala"
		_tmp0_ = NULL;
#line 7558 "SourceList.c"
	}
#line 760 "../src/Widgets/SourceList/SourceList.vala"
	result = _tmp0_;
#line 760 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 7564 "SourceList.c"
}

static gboolean
code_widgets_source_list_data_model_node_wrapper_get_valid (CodeWidgetsSourceListDataModelNodeWrapper* self)
{
	gboolean result;
	gboolean _tmp0_ = FALSE;
	GtkTreeRowReference* _tmp1_;
#line 768 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 768 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = self->priv->row_reference;
#line 768 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp1_ != NULL) {
#line 7579 "SourceList.c"
		GtkTreeRowReference* _tmp2_;
#line 768 "../src/Widgets/SourceList/SourceList.vala"
		_tmp2_ = self->priv->row_reference;
#line 768 "../src/Widgets/SourceList/SourceList.vala"
		_tmp0_ = gtk_tree_row_reference_valid (_tmp2_);
#line 7585 "SourceList.c"
	} else {
#line 768 "../src/Widgets/SourceList/SourceList.vala"
		_tmp0_ = FALSE;
#line 7589 "SourceList.c"
	}
#line 768 "../src/Widgets/SourceList/SourceList.vala"
	result = _tmp0_;
#line 768 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 7595 "SourceList.c"
}

static void
code_widgets_source_list_data_model_value_node_wrapper_init (GValue* value)
{
#line 729 "../src/Widgets/SourceList/SourceList.vala"
	value->data[0].v_pointer = NULL;
#line 7603 "SourceList.c"
}

static void
code_widgets_source_list_data_model_value_node_wrapper_free_value (GValue* value)
{
#line 729 "../src/Widgets/SourceList/SourceList.vala"
	if (value->data[0].v_pointer) {
#line 729 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_data_model_node_wrapper_unref (value->data[0].v_pointer);
#line 7613 "SourceList.c"
	}
}

static void
code_widgets_source_list_data_model_value_node_wrapper_copy_value (const GValue* src_value,
                                                                   GValue* dest_value)
{
#line 729 "../src/Widgets/SourceList/SourceList.vala"
	if (src_value->data[0].v_pointer) {
#line 729 "../src/Widgets/SourceList/SourceList.vala"
		dest_value->data[0].v_pointer = code_widgets_source_list_data_model_node_wrapper_ref (src_value->data[0].v_pointer);
#line 7625 "SourceList.c"
	} else {
#line 729 "../src/Widgets/SourceList/SourceList.vala"
		dest_value->data[0].v_pointer = NULL;
#line 7629 "SourceList.c"
	}
}

static gpointer
code_widgets_source_list_data_model_value_node_wrapper_peek_pointer (const GValue* value)
{
#line 729 "../src/Widgets/SourceList/SourceList.vala"
	return value->data[0].v_pointer;
#line 7638 "SourceList.c"
}

static gchar*
code_widgets_source_list_data_model_value_node_wrapper_collect_value (GValue* value,
                                                                      guint n_collect_values,
                                                                      GTypeCValue* collect_values,
                                                                      guint collect_flags)
{
#line 729 "../src/Widgets/SourceList/SourceList.vala"
	if (collect_values[0].v_pointer) {
#line 7649 "SourceList.c"
		CodeWidgetsSourceListDataModelNodeWrapper * object;
		object = collect_values[0].v_pointer;
#line 729 "../src/Widgets/SourceList/SourceList.vala"
		if (object->parent_instance.g_class == NULL) {
#line 729 "../src/Widgets/SourceList/SourceList.vala"
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
#line 7656 "SourceList.c"
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
#line 729 "../src/Widgets/SourceList/SourceList.vala"
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
#line 7660 "SourceList.c"
		}
#line 729 "../src/Widgets/SourceList/SourceList.vala"
		value->data[0].v_pointer = code_widgets_source_list_data_model_node_wrapper_ref (object);
#line 7664 "SourceList.c"
	} else {
#line 729 "../src/Widgets/SourceList/SourceList.vala"
		value->data[0].v_pointer = NULL;
#line 7668 "SourceList.c"
	}
#line 729 "../src/Widgets/SourceList/SourceList.vala"
	return NULL;
#line 7672 "SourceList.c"
}

static gchar*
code_widgets_source_list_data_model_value_node_wrapper_lcopy_value (const GValue* value,
                                                                    guint n_collect_values,
                                                                    GTypeCValue* collect_values,
                                                                    guint collect_flags)
{
	CodeWidgetsSourceListDataModelNodeWrapper ** object_p;
	object_p = collect_values[0].v_pointer;
#line 729 "../src/Widgets/SourceList/SourceList.vala"
	if (!object_p) {
#line 729 "../src/Widgets/SourceList/SourceList.vala"
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
#line 7687 "SourceList.c"
	}
#line 729 "../src/Widgets/SourceList/SourceList.vala"
	if (!value->data[0].v_pointer) {
#line 729 "../src/Widgets/SourceList/SourceList.vala"
		*object_p = NULL;
#line 7693 "SourceList.c"
	} else if (collect_flags & G_VALUE_NOCOPY_CONTENTS) {
#line 729 "../src/Widgets/SourceList/SourceList.vala"
		*object_p = value->data[0].v_pointer;
#line 7697 "SourceList.c"
	} else {
#line 729 "../src/Widgets/SourceList/SourceList.vala"
		*object_p = code_widgets_source_list_data_model_node_wrapper_ref (value->data[0].v_pointer);
#line 7701 "SourceList.c"
	}
#line 729 "../src/Widgets/SourceList/SourceList.vala"
	return NULL;
#line 7705 "SourceList.c"
}

static GParamSpec*
code_widgets_source_list_data_model_param_spec_node_wrapper (const gchar* name,
                                                             const gchar* nick,
                                                             const gchar* blurb,
                                                             GType object_type,
                                                             GParamFlags flags)
{
	CodeWidgetsSourceListDataModelParamSpecNodeWrapper* spec;
#line 729 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (g_type_is_a (object_type, CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_TYPE_NODE_WRAPPER), NULL);
#line 729 "../src/Widgets/SourceList/SourceList.vala"
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
#line 729 "../src/Widgets/SourceList/SourceList.vala"
	G_PARAM_SPEC (spec)->value_type = object_type;
#line 729 "../src/Widgets/SourceList/SourceList.vala"
	return G_PARAM_SPEC (spec);
#line 7724 "SourceList.c"
}

static gpointer
code_widgets_source_list_data_model_value_get_node_wrapper (const GValue* value)
{
#line 729 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_TYPE_NODE_WRAPPER), NULL);
#line 729 "../src/Widgets/SourceList/SourceList.vala"
	return value->data[0].v_pointer;
#line 7734 "SourceList.c"
}

static void
code_widgets_source_list_data_model_value_set_node_wrapper (GValue* value,
                                                            gpointer v_object)
{
	CodeWidgetsSourceListDataModelNodeWrapper * old;
#line 729 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_TYPE_NODE_WRAPPER));
#line 729 "../src/Widgets/SourceList/SourceList.vala"
	old = value->data[0].v_pointer;
#line 729 "../src/Widgets/SourceList/SourceList.vala"
	if (v_object) {
#line 729 "../src/Widgets/SourceList/SourceList.vala"
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_TYPE_NODE_WRAPPER));
#line 729 "../src/Widgets/SourceList/SourceList.vala"
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
#line 729 "../src/Widgets/SourceList/SourceList.vala"
		value->data[0].v_pointer = v_object;
#line 729 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_data_model_node_wrapper_ref (value->data[0].v_pointer);
#line 7756 "SourceList.c"
	} else {
#line 729 "../src/Widgets/SourceList/SourceList.vala"
		value->data[0].v_pointer = NULL;
#line 7760 "SourceList.c"
	}
#line 729 "../src/Widgets/SourceList/SourceList.vala"
	if (old) {
#line 729 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_data_model_node_wrapper_unref (old);
#line 7766 "SourceList.c"
	}
}

static void
code_widgets_source_list_data_model_value_take_node_wrapper (GValue* value,
                                                             gpointer v_object)
{
	CodeWidgetsSourceListDataModelNodeWrapper * old;
#line 729 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_TYPE_NODE_WRAPPER));
#line 729 "../src/Widgets/SourceList/SourceList.vala"
	old = value->data[0].v_pointer;
#line 729 "../src/Widgets/SourceList/SourceList.vala"
	if (v_object) {
#line 729 "../src/Widgets/SourceList/SourceList.vala"
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_TYPE_NODE_WRAPPER));
#line 729 "../src/Widgets/SourceList/SourceList.vala"
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
#line 729 "../src/Widgets/SourceList/SourceList.vala"
		value->data[0].v_pointer = v_object;
#line 7787 "SourceList.c"
	} else {
#line 729 "../src/Widgets/SourceList/SourceList.vala"
		value->data[0].v_pointer = NULL;
#line 7791 "SourceList.c"
	}
#line 729 "../src/Widgets/SourceList/SourceList.vala"
	if (old) {
#line 729 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_data_model_node_wrapper_unref (old);
#line 7797 "SourceList.c"
	}
}

static void
code_widgets_source_list_data_model_node_wrapper_class_init (CodeWidgetsSourceListDataModelNodeWrapperClass * klass,
                                                             gpointer klass_data)
{
#line 729 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_data_model_node_wrapper_parent_class = g_type_class_peek_parent (klass);
#line 729 "../src/Widgets/SourceList/SourceList.vala"
	((CodeWidgetsSourceListDataModelNodeWrapperClass *) klass)->finalize = code_widgets_source_list_data_model_node_wrapper_finalize;
#line 729 "../src/Widgets/SourceList/SourceList.vala"
	g_type_class_adjust_private_offset (klass, &CodeWidgetsSourceListDataModelNodeWrapper_private_offset);
#line 7811 "SourceList.c"
}

static void
code_widgets_source_list_data_model_node_wrapper_instance_init (CodeWidgetsSourceListDataModelNodeWrapper * self,
                                                                gpointer klass)
{
#line 729 "../src/Widgets/SourceList/SourceList.vala"
	self->priv = code_widgets_source_list_data_model_node_wrapper_get_instance_private (self);
#line 729 "../src/Widgets/SourceList/SourceList.vala"
	self->ref_count = 1;
#line 7822 "SourceList.c"
}

static void
code_widgets_source_list_data_model_node_wrapper_finalize (CodeWidgetsSourceListDataModelNodeWrapper * obj)
{
	CodeWidgetsSourceListDataModelNodeWrapper * self;
#line 729 "../src/Widgets/SourceList/SourceList.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_TYPE_NODE_WRAPPER, CodeWidgetsSourceListDataModelNodeWrapper);
#line 729 "../src/Widgets/SourceList/SourceList.vala"
	g_signal_handlers_destroy (self);
#line 734 "../src/Widgets/SourceList/SourceList.vala"
	__vala_GtkTreeRowReference_free0 (self->priv->row_reference);
#line 7835 "SourceList.c"
}

/**
         * An object that references a particular row in a model. This class is a wrapper built around
         * Gtk.TreeRowReference, and exists with the purpose of ensuring we never use invalid tree paths
         * or iters in the model, since most of these errors provoke failures due to GTK+ assertions
         * or, even worse, unexpected behavior.
         */
static GType
code_widgets_source_list_data_model_node_wrapper_get_type_once (void)
{
	static const GTypeValueTable g_define_type_value_table = { code_widgets_source_list_data_model_value_node_wrapper_init, code_widgets_source_list_data_model_value_node_wrapper_free_value, code_widgets_source_list_data_model_value_node_wrapper_copy_value, code_widgets_source_list_data_model_value_node_wrapper_peek_pointer, "p", code_widgets_source_list_data_model_value_node_wrapper_collect_value, "p", code_widgets_source_list_data_model_value_node_wrapper_lcopy_value };
	static const GTypeInfo g_define_type_info = { sizeof (CodeWidgetsSourceListDataModelNodeWrapperClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) code_widgets_source_list_data_model_node_wrapper_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (CodeWidgetsSourceListDataModelNodeWrapper), 0, (GInstanceInitFunc) code_widgets_source_list_data_model_node_wrapper_instance_init, &g_define_type_value_table };
	static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
	GType code_widgets_source_list_data_model_node_wrapper_type_id;
	code_widgets_source_list_data_model_node_wrapper_type_id = g_type_register_fundamental (g_type_fundamental_next (), "CodeWidgetsSourceListDataModelNodeWrapper", &g_define_type_info, &g_define_type_fundamental_info, 0);
	CodeWidgetsSourceListDataModelNodeWrapper_private_offset = g_type_add_instance_private (code_widgets_source_list_data_model_node_wrapper_type_id, sizeof (CodeWidgetsSourceListDataModelNodeWrapperPrivate));
	return code_widgets_source_list_data_model_node_wrapper_type_id;
}

static GType
code_widgets_source_list_data_model_node_wrapper_get_type (void)
{
	static volatile gsize code_widgets_source_list_data_model_node_wrapper_type_id__once = 0;
	if (g_once_init_enter (&code_widgets_source_list_data_model_node_wrapper_type_id__once)) {
		GType code_widgets_source_list_data_model_node_wrapper_type_id;
		code_widgets_source_list_data_model_node_wrapper_type_id = code_widgets_source_list_data_model_node_wrapper_get_type_once ();
		g_once_init_leave (&code_widgets_source_list_data_model_node_wrapper_type_id__once, code_widgets_source_list_data_model_node_wrapper_type_id);
	}
	return code_widgets_source_list_data_model_node_wrapper_type_id__once;
}

static gpointer
code_widgets_source_list_data_model_node_wrapper_ref (gpointer instance)
{
	CodeWidgetsSourceListDataModelNodeWrapper * self;
	self = instance;
#line 729 "../src/Widgets/SourceList/SourceList.vala"
	g_atomic_int_inc (&self->ref_count);
#line 729 "../src/Widgets/SourceList/SourceList.vala"
	return instance;
#line 7877 "SourceList.c"
}

static void
code_widgets_source_list_data_model_node_wrapper_unref (gpointer instance)
{
	CodeWidgetsSourceListDataModelNodeWrapper * self;
	self = instance;
#line 729 "../src/Widgets/SourceList/SourceList.vala"
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
#line 729 "../src/Widgets/SourceList/SourceList.vala"
		CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_NODE_WRAPPER_GET_CLASS (self)->finalize (self);
#line 729 "../src/Widgets/SourceList/SourceList.vala"
		g_type_free_instance ((GTypeInstance *) self);
#line 7891 "SourceList.c"
	}
}

static inline gpointer
code_widgets_source_list_data_model_item_monitor_get_instance_private (CodeWidgetsSourceListDataModelItemMonitor* self)
{
	return G_STRUCT_MEMBER_P (self, CodeWidgetsSourceListDataModelItemMonitor_private_offset);
}

static void
_code_widgets_source_list_data_model_item_monitor_on_notify_g_object_notify (GObject* _sender,
                                                                             GParamSpec* pspec,
                                                                             gpointer self)
{
#line 789 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_data_model_item_monitor_on_notify ((CodeWidgetsSourceListDataModelItemMonitor*) self, pspec);
#line 7908 "SourceList.c"
}

static CodeWidgetsSourceListDataModelItemMonitor*
code_widgets_source_list_data_model_item_monitor_construct (GType object_type,
                                                            CodeWidgetsSourceListItem* item)
{
	CodeWidgetsSourceListDataModelItemMonitor* self = NULL;
	CodeWidgetsSourceListItem* _tmp0_;
#line 783 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (item != NULL, NULL);
#line 783 "../src/Widgets/SourceList/SourceList.vala"
	self = (CodeWidgetsSourceListDataModelItemMonitor*) g_type_create_instance (object_type);
#line 784 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = _g_object_ref0 (item);
#line 784 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->item);
#line 784 "../src/Widgets/SourceList/SourceList.vala"
	self->priv->item = _tmp0_;
#line 785 "../src/Widgets/SourceList/SourceList.vala"
	g_signal_connect_after ((GObject*) item, "notify", (GCallback) _code_widgets_source_list_data_model_item_monitor_on_notify_g_object_notify, self);
#line 783 "../src/Widgets/SourceList/SourceList.vala"
	return self;
#line 7931 "SourceList.c"
}

static CodeWidgetsSourceListDataModelItemMonitor*
code_widgets_source_list_data_model_item_monitor_new (CodeWidgetsSourceListItem* item)
{
#line 783 "../src/Widgets/SourceList/SourceList.vala"
	return code_widgets_source_list_data_model_item_monitor_construct (CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_TYPE_ITEM_MONITOR, item);
#line 7939 "SourceList.c"
}

static void
code_widgets_source_list_data_model_item_monitor_on_notify (CodeWidgetsSourceListDataModelItemMonitor* self,
                                                            GParamSpec* prop)
{
	CodeWidgetsSourceListItem* _tmp0_;
	const gchar* _tmp1_;
#line 792 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 792 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (prop != NULL);
#line 793 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->item;
#line 793 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = prop->name;
#line 793 "../src/Widgets/SourceList/SourceList.vala"
	g_signal_emit (self, code_widgets_source_list_data_model_item_monitor_signals[CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_ITEM_MONITOR_CHANGED_SIGNAL], 0, _tmp0_, _tmp1_);
#line 7958 "SourceList.c"
}

static void
g_cclosure_user_marshal_VOID__OBJECT_STRING (GClosure * closure,
                                             GValue * return_value,
                                             guint n_param_values,
                                             const GValue * param_values,
                                             gpointer invocation_hint,
                                             gpointer marshal_data)
{
	typedef void (*GMarshalFunc_VOID__OBJECT_STRING) (gpointer data1, gpointer arg_1, const char* arg_2, gpointer data2);
	register GMarshalFunc_VOID__OBJECT_STRING callback;
	register GCClosure * cc;
	register gpointer data1;
	register gpointer data2;
	cc = (GCClosure *) closure;
#line 779 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (n_param_values == 3);
#line 779 "../src/Widgets/SourceList/SourceList.vala"
	if (G_CCLOSURE_SWAP_DATA (closure)) {
#line 779 "../src/Widgets/SourceList/SourceList.vala"
		data1 = closure->data;
#line 779 "../src/Widgets/SourceList/SourceList.vala"
		data2 = param_values->data[0].v_pointer;
#line 7983 "SourceList.c"
	} else {
#line 779 "../src/Widgets/SourceList/SourceList.vala"
		data1 = param_values->data[0].v_pointer;
#line 779 "../src/Widgets/SourceList/SourceList.vala"
		data2 = closure->data;
#line 7989 "SourceList.c"
	}
#line 779 "../src/Widgets/SourceList/SourceList.vala"
	callback = (GMarshalFunc_VOID__OBJECT_STRING) (marshal_data ? marshal_data : cc->callback);
#line 779 "../src/Widgets/SourceList/SourceList.vala"
	callback (data1, g_value_get_object (param_values + 1), g_value_get_string (param_values + 2), data2);
#line 7995 "SourceList.c"
}

static void
code_widgets_source_list_data_model_value_item_monitor_init (GValue* value)
{
#line 779 "../src/Widgets/SourceList/SourceList.vala"
	value->data[0].v_pointer = NULL;
#line 8003 "SourceList.c"
}

static void
code_widgets_source_list_data_model_value_item_monitor_free_value (GValue* value)
{
#line 779 "../src/Widgets/SourceList/SourceList.vala"
	if (value->data[0].v_pointer) {
#line 779 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_data_model_item_monitor_unref (value->data[0].v_pointer);
#line 8013 "SourceList.c"
	}
}

static void
code_widgets_source_list_data_model_value_item_monitor_copy_value (const GValue* src_value,
                                                                   GValue* dest_value)
{
#line 779 "../src/Widgets/SourceList/SourceList.vala"
	if (src_value->data[0].v_pointer) {
#line 779 "../src/Widgets/SourceList/SourceList.vala"
		dest_value->data[0].v_pointer = code_widgets_source_list_data_model_item_monitor_ref (src_value->data[0].v_pointer);
#line 8025 "SourceList.c"
	} else {
#line 779 "../src/Widgets/SourceList/SourceList.vala"
		dest_value->data[0].v_pointer = NULL;
#line 8029 "SourceList.c"
	}
}

static gpointer
code_widgets_source_list_data_model_value_item_monitor_peek_pointer (const GValue* value)
{
#line 779 "../src/Widgets/SourceList/SourceList.vala"
	return value->data[0].v_pointer;
#line 8038 "SourceList.c"
}

static gchar*
code_widgets_source_list_data_model_value_item_monitor_collect_value (GValue* value,
                                                                      guint n_collect_values,
                                                                      GTypeCValue* collect_values,
                                                                      guint collect_flags)
{
#line 779 "../src/Widgets/SourceList/SourceList.vala"
	if (collect_values[0].v_pointer) {
#line 8049 "SourceList.c"
		CodeWidgetsSourceListDataModelItemMonitor * object;
		object = collect_values[0].v_pointer;
#line 779 "../src/Widgets/SourceList/SourceList.vala"
		if (object->parent_instance.g_class == NULL) {
#line 779 "../src/Widgets/SourceList/SourceList.vala"
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
#line 8056 "SourceList.c"
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
#line 779 "../src/Widgets/SourceList/SourceList.vala"
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
#line 8060 "SourceList.c"
		}
#line 779 "../src/Widgets/SourceList/SourceList.vala"
		value->data[0].v_pointer = code_widgets_source_list_data_model_item_monitor_ref (object);
#line 8064 "SourceList.c"
	} else {
#line 779 "../src/Widgets/SourceList/SourceList.vala"
		value->data[0].v_pointer = NULL;
#line 8068 "SourceList.c"
	}
#line 779 "../src/Widgets/SourceList/SourceList.vala"
	return NULL;
#line 8072 "SourceList.c"
}

static gchar*
code_widgets_source_list_data_model_value_item_monitor_lcopy_value (const GValue* value,
                                                                    guint n_collect_values,
                                                                    GTypeCValue* collect_values,
                                                                    guint collect_flags)
{
	CodeWidgetsSourceListDataModelItemMonitor ** object_p;
	object_p = collect_values[0].v_pointer;
#line 779 "../src/Widgets/SourceList/SourceList.vala"
	if (!object_p) {
#line 779 "../src/Widgets/SourceList/SourceList.vala"
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
#line 8087 "SourceList.c"
	}
#line 779 "../src/Widgets/SourceList/SourceList.vala"
	if (!value->data[0].v_pointer) {
#line 779 "../src/Widgets/SourceList/SourceList.vala"
		*object_p = NULL;
#line 8093 "SourceList.c"
	} else if (collect_flags & G_VALUE_NOCOPY_CONTENTS) {
#line 779 "../src/Widgets/SourceList/SourceList.vala"
		*object_p = value->data[0].v_pointer;
#line 8097 "SourceList.c"
	} else {
#line 779 "../src/Widgets/SourceList/SourceList.vala"
		*object_p = code_widgets_source_list_data_model_item_monitor_ref (value->data[0].v_pointer);
#line 8101 "SourceList.c"
	}
#line 779 "../src/Widgets/SourceList/SourceList.vala"
	return NULL;
#line 8105 "SourceList.c"
}

static GParamSpec*
code_widgets_source_list_data_model_param_spec_item_monitor (const gchar* name,
                                                             const gchar* nick,
                                                             const gchar* blurb,
                                                             GType object_type,
                                                             GParamFlags flags)
{
	CodeWidgetsSourceListDataModelParamSpecItemMonitor* spec;
#line 779 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (g_type_is_a (object_type, CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_TYPE_ITEM_MONITOR), NULL);
#line 779 "../src/Widgets/SourceList/SourceList.vala"
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
#line 779 "../src/Widgets/SourceList/SourceList.vala"
	G_PARAM_SPEC (spec)->value_type = object_type;
#line 779 "../src/Widgets/SourceList/SourceList.vala"
	return G_PARAM_SPEC (spec);
#line 8124 "SourceList.c"
}

static gpointer
code_widgets_source_list_data_model_value_get_item_monitor (const GValue* value)
{
#line 779 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_TYPE_ITEM_MONITOR), NULL);
#line 779 "../src/Widgets/SourceList/SourceList.vala"
	return value->data[0].v_pointer;
#line 8134 "SourceList.c"
}

static void
code_widgets_source_list_data_model_value_set_item_monitor (GValue* value,
                                                            gpointer v_object)
{
	CodeWidgetsSourceListDataModelItemMonitor * old;
#line 779 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_TYPE_ITEM_MONITOR));
#line 779 "../src/Widgets/SourceList/SourceList.vala"
	old = value->data[0].v_pointer;
#line 779 "../src/Widgets/SourceList/SourceList.vala"
	if (v_object) {
#line 779 "../src/Widgets/SourceList/SourceList.vala"
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_TYPE_ITEM_MONITOR));
#line 779 "../src/Widgets/SourceList/SourceList.vala"
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
#line 779 "../src/Widgets/SourceList/SourceList.vala"
		value->data[0].v_pointer = v_object;
#line 779 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_data_model_item_monitor_ref (value->data[0].v_pointer);
#line 8156 "SourceList.c"
	} else {
#line 779 "../src/Widgets/SourceList/SourceList.vala"
		value->data[0].v_pointer = NULL;
#line 8160 "SourceList.c"
	}
#line 779 "../src/Widgets/SourceList/SourceList.vala"
	if (old) {
#line 779 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_data_model_item_monitor_unref (old);
#line 8166 "SourceList.c"
	}
}

static void
code_widgets_source_list_data_model_value_take_item_monitor (GValue* value,
                                                             gpointer v_object)
{
	CodeWidgetsSourceListDataModelItemMonitor * old;
#line 779 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_TYPE_ITEM_MONITOR));
#line 779 "../src/Widgets/SourceList/SourceList.vala"
	old = value->data[0].v_pointer;
#line 779 "../src/Widgets/SourceList/SourceList.vala"
	if (v_object) {
#line 779 "../src/Widgets/SourceList/SourceList.vala"
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_TYPE_ITEM_MONITOR));
#line 779 "../src/Widgets/SourceList/SourceList.vala"
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
#line 779 "../src/Widgets/SourceList/SourceList.vala"
		value->data[0].v_pointer = v_object;
#line 8187 "SourceList.c"
	} else {
#line 779 "../src/Widgets/SourceList/SourceList.vala"
		value->data[0].v_pointer = NULL;
#line 8191 "SourceList.c"
	}
#line 779 "../src/Widgets/SourceList/SourceList.vala"
	if (old) {
#line 779 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_data_model_item_monitor_unref (old);
#line 8197 "SourceList.c"
	}
}

static void
code_widgets_source_list_data_model_item_monitor_class_init (CodeWidgetsSourceListDataModelItemMonitorClass * klass,
                                                             gpointer klass_data)
{
#line 779 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_data_model_item_monitor_parent_class = g_type_class_peek_parent (klass);
#line 779 "../src/Widgets/SourceList/SourceList.vala"
	((CodeWidgetsSourceListDataModelItemMonitorClass *) klass)->finalize = code_widgets_source_list_data_model_item_monitor_finalize;
#line 779 "../src/Widgets/SourceList/SourceList.vala"
	g_type_class_adjust_private_offset (klass, &CodeWidgetsSourceListDataModelItemMonitor_private_offset);
#line 779 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_data_model_item_monitor_signals[CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_ITEM_MONITOR_CHANGED_SIGNAL] = g_signal_new ("changed", CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_TYPE_ITEM_MONITOR, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_user_marshal_VOID__OBJECT_STRING, G_TYPE_NONE, 2, CODE_WIDGETS_SOURCE_LIST_TYPE_ITEM, G_TYPE_STRING);
#line 8213 "SourceList.c"
}

static void
code_widgets_source_list_data_model_item_monitor_instance_init (CodeWidgetsSourceListDataModelItemMonitor * self,
                                                                gpointer klass)
{
#line 779 "../src/Widgets/SourceList/SourceList.vala"
	self->priv = code_widgets_source_list_data_model_item_monitor_get_instance_private (self);
#line 779 "../src/Widgets/SourceList/SourceList.vala"
	self->ref_count = 1;
#line 8224 "SourceList.c"
}

static void
code_widgets_source_list_data_model_item_monitor_finalize (CodeWidgetsSourceListDataModelItemMonitor * obj)
{
	CodeWidgetsSourceListDataModelItemMonitor * self;
	CodeWidgetsSourceListItem* _tmp0_;
	guint _tmp1_;
#line 779 "../src/Widgets/SourceList/SourceList.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_TYPE_ITEM_MONITOR, CodeWidgetsSourceListDataModelItemMonitor);
#line 779 "../src/Widgets/SourceList/SourceList.vala"
	g_signal_handlers_destroy (self);
#line 789 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->item;
#line 789 "../src/Widgets/SourceList/SourceList.vala"
	g_signal_parse_name ("notify", G_TYPE_OBJECT, &_tmp1_, NULL, FALSE);
#line 789 "../src/Widgets/SourceList/SourceList.vala"
	g_signal_handlers_disconnect_matched ((GObject*) _tmp0_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp1_, 0, NULL, (GCallback) _code_widgets_source_list_data_model_item_monitor_on_notify_g_object_notify, self);
#line 781 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->item);
#line 8245 "SourceList.c"
}

/**
         * Helper object used to monitor item property changes.
         */
static GType
code_widgets_source_list_data_model_item_monitor_get_type_once (void)
{
	static const GTypeValueTable g_define_type_value_table = { code_widgets_source_list_data_model_value_item_monitor_init, code_widgets_source_list_data_model_value_item_monitor_free_value, code_widgets_source_list_data_model_value_item_monitor_copy_value, code_widgets_source_list_data_model_value_item_monitor_peek_pointer, "p", code_widgets_source_list_data_model_value_item_monitor_collect_value, "p", code_widgets_source_list_data_model_value_item_monitor_lcopy_value };
	static const GTypeInfo g_define_type_info = { sizeof (CodeWidgetsSourceListDataModelItemMonitorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) code_widgets_source_list_data_model_item_monitor_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (CodeWidgetsSourceListDataModelItemMonitor), 0, (GInstanceInitFunc) code_widgets_source_list_data_model_item_monitor_instance_init, &g_define_type_value_table };
	static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
	GType code_widgets_source_list_data_model_item_monitor_type_id;
	code_widgets_source_list_data_model_item_monitor_type_id = g_type_register_fundamental (g_type_fundamental_next (), "CodeWidgetsSourceListDataModelItemMonitor", &g_define_type_info, &g_define_type_fundamental_info, 0);
	CodeWidgetsSourceListDataModelItemMonitor_private_offset = g_type_add_instance_private (code_widgets_source_list_data_model_item_monitor_type_id, sizeof (CodeWidgetsSourceListDataModelItemMonitorPrivate));
	return code_widgets_source_list_data_model_item_monitor_type_id;
}

static GType
code_widgets_source_list_data_model_item_monitor_get_type (void)
{
	static volatile gsize code_widgets_source_list_data_model_item_monitor_type_id__once = 0;
	if (g_once_init_enter (&code_widgets_source_list_data_model_item_monitor_type_id__once)) {
		GType code_widgets_source_list_data_model_item_monitor_type_id;
		code_widgets_source_list_data_model_item_monitor_type_id = code_widgets_source_list_data_model_item_monitor_get_type_once ();
		g_once_init_leave (&code_widgets_source_list_data_model_item_monitor_type_id__once, code_widgets_source_list_data_model_item_monitor_type_id);
	}
	return code_widgets_source_list_data_model_item_monitor_type_id__once;
}

static gpointer
code_widgets_source_list_data_model_item_monitor_ref (gpointer instance)
{
	CodeWidgetsSourceListDataModelItemMonitor * self;
	self = instance;
#line 779 "../src/Widgets/SourceList/SourceList.vala"
	g_atomic_int_inc (&self->ref_count);
#line 779 "../src/Widgets/SourceList/SourceList.vala"
	return instance;
#line 8284 "SourceList.c"
}

static void
code_widgets_source_list_data_model_item_monitor_unref (gpointer instance)
{
	CodeWidgetsSourceListDataModelItemMonitor * self;
	self = instance;
#line 779 "../src/Widgets/SourceList/SourceList.vala"
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
#line 779 "../src/Widgets/SourceList/SourceList.vala"
		CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_ITEM_MONITOR_GET_CLASS (self)->finalize (self);
#line 779 "../src/Widgets/SourceList/SourceList.vala"
		g_type_free_instance ((GTypeInstance *) self);
#line 8298 "SourceList.c"
	}
}

static gint
_code_widgets_source_list_data_model_child_model_sort_func_gtk_tree_iter_compare_func (GtkTreeModel* model,
                                                                                       GtkTreeIter* a,
                                                                                       GtkTreeIter* b,
                                                                                       gpointer self)
{
	gint result;
	result = code_widgets_source_list_data_model_child_model_sort_func ((CodeWidgetsSourceListDataModel*) self, model, a, b);
#line 863 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 8312 "SourceList.c"
}

static gboolean
_code_widgets_source_list_data_model_filter_visible_func_gtk_tree_model_filter_visible_func (GtkTreeModel* model,
                                                                                             GtkTreeIter* iter,
                                                                                             gpointer self)
{
	gboolean result;
	result = code_widgets_source_list_data_model_filter_visible_func ((CodeWidgetsSourceListDataModel*) self, model, iter);
#line 866 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 8324 "SourceList.c"
}

static GObject *
code_widgets_source_list_data_model_constructor (GType type,
                                                 guint n_construct_properties,
                                                 GObjectConstructParam * construct_properties)
{
	GObject * obj;
	GObjectClass * parent_class;
	CodeWidgetsSourceListDataModel * self;
	GtkTreeStore* _tmp0_;
	GtkTreeStore* _tmp1_;
	GtkTreeStore* _tmp2_;
#line 858 "../src/Widgets/SourceList/SourceList.vala"
	parent_class = G_OBJECT_CLASS (code_widgets_source_list_data_model_parent_class);
#line 858 "../src/Widgets/SourceList/SourceList.vala"
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
#line 858 "../src/Widgets/SourceList/SourceList.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, CODE_WIDGETS_SOURCE_LIST_TYPE_DATA_MODEL, CodeWidgetsSourceListDataModel);
#line 859 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = gtk_tree_store_new ((gint) CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_COLUMN_N_COLUMNS, code_widgets_source_list_data_model_column_type (CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_COLUMN_ITEM));
#line 859 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->child_tree);
#line 859 "../src/Widgets/SourceList/SourceList.vala"
	self->priv->child_tree = _tmp0_;
#line 860 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = self->priv->child_tree;
#line 860 "../src/Widgets/SourceList/SourceList.vala"
	g_object_set ((GtkTreeModelFilter*) self, "child-model", (GtkTreeModel*) _tmp1_, NULL);
#line 861 "../src/Widgets/SourceList/SourceList.vala"
	g_object_set ((GtkTreeModelFilter*) self, "virtual-root", NULL, NULL);
#line 863 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = self->priv->child_tree;
#line 863 "../src/Widgets/SourceList/SourceList.vala"
	gtk_tree_sortable_set_default_sort_func ((GtkTreeSortable*) _tmp2_, _code_widgets_source_list_data_model_child_model_sort_func_gtk_tree_iter_compare_func, g_object_ref (self), g_object_unref);
#line 864 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_data_model_resort (self);
#line 866 "../src/Widgets/SourceList/SourceList.vala"
	gtk_tree_model_filter_set_visible_func ((GtkTreeModelFilter*) self, _code_widgets_source_list_data_model_filter_visible_func_gtk_tree_model_filter_visible_func, g_object_ref (self), g_object_unref);
#line 858 "../src/Widgets/SourceList/SourceList.vala"
	return obj;
#line 8366 "SourceList.c"
}

static void
code_widgets_source_list_data_model_class_init (CodeWidgetsSourceListDataModelClass * klass,
                                                gpointer klass_data)
{
#line 721 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_data_model_parent_class = g_type_class_peek_parent (klass);
#line 721 "../src/Widgets/SourceList/SourceList.vala"
	g_type_class_adjust_private_offset (klass, &CodeWidgetsSourceListDataModel_private_offset);
#line 721 "../src/Widgets/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_code_widgets_source_list_data_model_get_property;
#line 721 "../src/Widgets/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_code_widgets_source_list_data_model_set_property;
#line 721 "../src/Widgets/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->constructor = code_widgets_source_list_data_model_constructor;
#line 721 "../src/Widgets/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->finalize = code_widgets_source_list_data_model_finalize;
#line 8385 "SourceList.c"
	/**
	         * Root item.
	         *
	         * This item is not actually part of the model. It's only used as a proxy
	         * for adding and removing items.
	         */
#line 721 "../src/Widgets/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_ROOT_PROPERTY, code_widgets_source_list_data_model_properties[CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_ROOT_PROPERTY] = g_param_spec_object ("root", "root", "root", CODE_WIDGETS_SOURCE_LIST_TYPE_EXPANDABLE_ITEM, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 721 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_data_model_signals[CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_ITEM_UPDATED_SIGNAL] = g_signal_new ("item-updated", CODE_WIDGETS_SOURCE_LIST_TYPE_DATA_MODEL, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, CODE_WIDGETS_SOURCE_LIST_TYPE_ITEM);
#line 8396 "SourceList.c"
}

static void
code_widgets_source_list_data_model_gtk_tree_drag_source_interface_init (GtkTreeDragSourceIface * iface,
                                                                         gpointer iface_data)
{
#line 721 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_data_model_gtk_tree_drag_source_parent_iface = g_type_interface_peek_parent (iface);
#line 721 "../src/Widgets/SourceList/SourceList.vala"
	iface->drag_data_get = (gboolean (*) (GtkTreeDragSource*, GtkTreePath*, GtkSelectionData*)) code_widgets_source_list_data_model_real_drag_data_get;
#line 721 "../src/Widgets/SourceList/SourceList.vala"
	iface->row_draggable = (gboolean (*) (GtkTreeDragSource*, GtkTreePath*)) code_widgets_source_list_data_model_real_row_draggable;
#line 8409 "SourceList.c"
}

static void
code_widgets_source_list_data_model_gtk_tree_drag_dest_interface_init (GtkTreeDragDestIface * iface,
                                                                       gpointer iface_data)
{
#line 721 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_data_model_gtk_tree_drag_dest_parent_iface = g_type_interface_peek_parent (iface);
#line 721 "../src/Widgets/SourceList/SourceList.vala"
	iface->drag_data_received = (gboolean (*) (GtkTreeDragDest*, GtkTreePath*, GtkSelectionData*)) code_widgets_source_list_data_model_real_drag_data_received;
#line 721 "../src/Widgets/SourceList/SourceList.vala"
	iface->row_drop_possible = (gboolean (*) (GtkTreeDragDest*, GtkTreePath*, GtkSelectionData*)) code_widgets_source_list_data_model_real_row_drop_possible;
#line 8422 "SourceList.c"
}

static void
code_widgets_source_list_data_model_instance_init (CodeWidgetsSourceListDataModel * self,
                                                   gpointer klass)
{
	GeeHashMap* _tmp0_;
	GeeHashMap* _tmp1_;
#line 721 "../src/Widgets/SourceList/SourceList.vala"
	self->priv = code_widgets_source_list_data_model_get_instance_private (self);
#line 847 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = gee_hash_map_new (CODE_WIDGETS_SOURCE_LIST_TYPE_ITEM, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref, CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_TYPE_NODE_WRAPPER, (GBoxedCopyFunc) code_widgets_source_list_data_model_node_wrapper_ref, (GDestroyNotify) code_widgets_source_list_data_model_node_wrapper_unref, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
#line 847 "../src/Widgets/SourceList/SourceList.vala"
	self->priv->items = _tmp0_;
#line 849 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = gee_hash_map_new (CODE_WIDGETS_SOURCE_LIST_TYPE_ITEM, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref, CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_TYPE_ITEM_MONITOR, (GBoxedCopyFunc) code_widgets_source_list_data_model_item_monitor_ref, (GDestroyNotify) code_widgets_source_list_data_model_item_monitor_unref, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
#line 849 "../src/Widgets/SourceList/SourceList.vala"
	self->priv->monitors = _tmp1_;
#line 8441 "SourceList.c"
}

static void
code_widgets_source_list_data_model_finalize (GObject * obj)
{
	CodeWidgetsSourceListDataModel * self;
#line 721 "../src/Widgets/SourceList/SourceList.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, CODE_WIDGETS_SOURCE_LIST_TYPE_DATA_MODEL, CodeWidgetsSourceListDataModel);
#line 819 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->_root);
#line 847 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->items);
#line 849 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->monitors);
#line 851 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->child_tree);
#line 721 "../src/Widgets/SourceList/SourceList.vala"
	G_OBJECT_CLASS (code_widgets_source_list_data_model_parent_class)->finalize (obj);
#line 8460 "SourceList.c"
}

/**
     * The model backing the SourceList tree.
     *
     * It monitors item property changes, and handles children additions and removals. It also controls
     * the visibility of the items based on their "visible" property, and on their number of children,
     * if they happen to be categories. Its main purpose is to provide an easy and practical interface
     * for sorting, adding, removing and updating items, eliminating the need of repeatedly dealing with
     * the Gtk.TreeModel API directly.
     */
static GType
code_widgets_source_list_data_model_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (CodeWidgetsSourceListDataModelClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) code_widgets_source_list_data_model_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (CodeWidgetsSourceListDataModel), 0, (GInstanceInitFunc) code_widgets_source_list_data_model_instance_init, NULL };
	static const GInterfaceInfo gtk_tree_drag_source_info = { (GInterfaceInitFunc) code_widgets_source_list_data_model_gtk_tree_drag_source_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	static const GInterfaceInfo gtk_tree_drag_dest_info = { (GInterfaceInitFunc) code_widgets_source_list_data_model_gtk_tree_drag_dest_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	GType code_widgets_source_list_data_model_type_id;
	code_widgets_source_list_data_model_type_id = g_type_register_static (gtk_tree_model_filter_get_type (), "CodeWidgetsSourceListDataModel", &g_define_type_info, 0);
	g_type_add_interface_static (code_widgets_source_list_data_model_type_id, gtk_tree_drag_source_get_type (), &gtk_tree_drag_source_info);
	g_type_add_interface_static (code_widgets_source_list_data_model_type_id, gtk_tree_drag_dest_get_type (), &gtk_tree_drag_dest_info);
	CodeWidgetsSourceListDataModel_private_offset = g_type_add_instance_private (code_widgets_source_list_data_model_type_id, sizeof (CodeWidgetsSourceListDataModelPrivate));
	return code_widgets_source_list_data_model_type_id;
}

static GType
code_widgets_source_list_data_model_get_type (void)
{
	static volatile gsize code_widgets_source_list_data_model_type_id__once = 0;
	if (g_once_init_enter (&code_widgets_source_list_data_model_type_id__once)) {
		GType code_widgets_source_list_data_model_type_id;
		code_widgets_source_list_data_model_type_id = code_widgets_source_list_data_model_get_type_once ();
		g_once_init_leave (&code_widgets_source_list_data_model_type_id__once, code_widgets_source_list_data_model_type_id);
	}
	return code_widgets_source_list_data_model_type_id__once;
}

static void
_vala_code_widgets_source_list_data_model_get_property (GObject * object,
                                                        guint property_id,
                                                        GValue * value,
                                                        GParamSpec * pspec)
{
	CodeWidgetsSourceListDataModel * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, CODE_WIDGETS_SOURCE_LIST_TYPE_DATA_MODEL, CodeWidgetsSourceListDataModel);
#line 721 "../src/Widgets/SourceList/SourceList.vala"
	switch (property_id) {
#line 721 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_ROOT_PROPERTY:
#line 721 "../src/Widgets/SourceList/SourceList.vala"
		g_value_set_object (value, code_widgets_source_list_data_model_get_root (self));
#line 721 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 8514 "SourceList.c"
		default:
#line 721 "../src/Widgets/SourceList/SourceList.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 721 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 8520 "SourceList.c"
	}
}

static void
_vala_code_widgets_source_list_data_model_set_property (GObject * object,
                                                        guint property_id,
                                                        const GValue * value,
                                                        GParamSpec * pspec)
{
	CodeWidgetsSourceListDataModel * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, CODE_WIDGETS_SOURCE_LIST_TYPE_DATA_MODEL, CodeWidgetsSourceListDataModel);
#line 721 "../src/Widgets/SourceList/SourceList.vala"
	switch (property_id) {
#line 721 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_DATA_MODEL_ROOT_PROPERTY:
#line 721 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_data_model_set_root (self, g_value_get_object (value));
#line 721 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 8540 "SourceList.c"
		default:
#line 721 "../src/Widgets/SourceList/SourceList.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 721 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 8546 "SourceList.c"
	}
}

static CodeWidgetsSourceListCellRendererIcon*
code_widgets_source_list_cell_renderer_icon_construct (GType object_type)
{
	CodeWidgetsSourceListCellRendererIcon * self = NULL;
#line 1472 "../src/Widgets/SourceList/SourceList.vala"
	self = (CodeWidgetsSourceListCellRendererIcon*) g_object_new (object_type, NULL);
#line 1472 "../src/Widgets/SourceList/SourceList.vala"
	return self;
#line 8558 "SourceList.c"
}

static CodeWidgetsSourceListCellRendererIcon*
code_widgets_source_list_cell_renderer_icon_new (void)
{
#line 1472 "../src/Widgets/SourceList/SourceList.vala"
	return code_widgets_source_list_cell_renderer_icon_construct (CODE_WIDGETS_SOURCE_LIST_TYPE_CELL_RENDERER_ICON);
#line 8566 "SourceList.c"
}

static gboolean
code_widgets_source_list_cell_renderer_icon_real_activate (GtkCellRenderer* base,
                                                           GdkEvent* event,
                                                           GtkWidget* widget,
                                                           const gchar* path,
                                                           GdkRectangle* background_area,
                                                           GdkRectangle* cell_area,
                                                           GtkCellRendererState flags)
{
	CodeWidgetsSourceListCellRendererIcon * self;
	gboolean result;
#line 1481 "../src/Widgets/SourceList/SourceList.vala"
	self = (CodeWidgetsSourceListCellRendererIcon*) base;
#line 1481 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (event != NULL, FALSE);
#line 1481 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (widget != NULL, FALSE);
#line 1481 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (path != NULL, FALSE);
#line 1481 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (background_area != NULL, FALSE);
#line 1481 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (cell_area != NULL, FALSE);
#line 1489 "../src/Widgets/SourceList/SourceList.vala"
	g_signal_emit (self, code_widgets_source_list_cell_renderer_icon_signals[CODE_WIDGETS_SOURCE_LIST_CELL_RENDERER_ICON_ACTIVATED_SIGNAL], 0, path);
#line 1490 "../src/Widgets/SourceList/SourceList.vala"
	result = TRUE;
#line 1490 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 8598 "SourceList.c"
}

static GObject *
code_widgets_source_list_cell_renderer_icon_constructor (GType type,
                                                         guint n_construct_properties,
                                                         GObjectConstructParam * construct_properties)
{
	GObject * obj;
	GObjectClass * parent_class;
	CodeWidgetsSourceListCellRendererIcon * self;
#line 1476 "../src/Widgets/SourceList/SourceList.vala"
	parent_class = G_OBJECT_CLASS (code_widgets_source_list_cell_renderer_icon_parent_class);
#line 1476 "../src/Widgets/SourceList/SourceList.vala"
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
#line 1476 "../src/Widgets/SourceList/SourceList.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, CODE_WIDGETS_SOURCE_LIST_TYPE_CELL_RENDERER_ICON, CodeWidgetsSourceListCellRendererIcon);
#line 1477 "../src/Widgets/SourceList/SourceList.vala"
	g_object_set ((GtkCellRenderer*) self, "mode", GTK_CELL_RENDERER_MODE_ACTIVATABLE, NULL);
#line 1478 "../src/Widgets/SourceList/SourceList.vala"
	g_object_set ((GtkCellRendererPixbuf*) self, "stock-size", (guint) CODE_WIDGETS_SOURCE_LIST_CELL_RENDERER_ICON_ICON_SIZE, NULL);
#line 1476 "../src/Widgets/SourceList/SourceList.vala"
	return obj;
#line 8621 "SourceList.c"
}

static void
code_widgets_source_list_cell_renderer_icon_class_init (CodeWidgetsSourceListCellRendererIconClass * klass,
                                                        gpointer klass_data)
{
#line 1467 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_cell_renderer_icon_parent_class = g_type_class_peek_parent (klass);
#line 1467 "../src/Widgets/SourceList/SourceList.vala"
	((GtkCellRendererClass *) klass)->activate = (gboolean (*) (GtkCellRenderer*, GdkEvent*, GtkWidget*, const gchar*, GdkRectangle*, GdkRectangle*, GtkCellRendererState)) code_widgets_source_list_cell_renderer_icon_real_activate;
#line 1467 "../src/Widgets/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->constructor = code_widgets_source_list_cell_renderer_icon_constructor;
#line 1467 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_cell_renderer_icon_signals[CODE_WIDGETS_SOURCE_LIST_CELL_RENDERER_ICON_ACTIVATED_SIGNAL] = g_signal_new ("activated", CODE_WIDGETS_SOURCE_LIST_TYPE_CELL_RENDERER_ICON, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__STRING, G_TYPE_NONE, 1, G_TYPE_STRING);
#line 8636 "SourceList.c"
}

static void
code_widgets_source_list_cell_renderer_icon_instance_init (CodeWidgetsSourceListCellRendererIcon * self,
                                                           gpointer klass)
{
}

/**
     * Class responsible for rendering Item.icon and Item.activatable. It also
     * notifies about clicks through the activated() signal.
     */
static GType
code_widgets_source_list_cell_renderer_icon_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (CodeWidgetsSourceListCellRendererIconClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) code_widgets_source_list_cell_renderer_icon_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (CodeWidgetsSourceListCellRendererIcon), 0, (GInstanceInitFunc) code_widgets_source_list_cell_renderer_icon_instance_init, NULL };
	GType code_widgets_source_list_cell_renderer_icon_type_id;
	code_widgets_source_list_cell_renderer_icon_type_id = g_type_register_static (gtk_cell_renderer_pixbuf_get_type (), "CodeWidgetsSourceListCellRendererIcon", &g_define_type_info, 0);
	return code_widgets_source_list_cell_renderer_icon_type_id;
}

static GType
code_widgets_source_list_cell_renderer_icon_get_type (void)
{
	static volatile gsize code_widgets_source_list_cell_renderer_icon_type_id__once = 0;
	if (g_once_init_enter (&code_widgets_source_list_cell_renderer_icon_type_id__once)) {
		GType code_widgets_source_list_cell_renderer_icon_type_id;
		code_widgets_source_list_cell_renderer_icon_type_id = code_widgets_source_list_cell_renderer_icon_get_type_once ();
		g_once_init_leave (&code_widgets_source_list_cell_renderer_icon_type_id__once, code_widgets_source_list_cell_renderer_icon_type_id);
	}
	return code_widgets_source_list_cell_renderer_icon_type_id__once;
}

static inline gpointer
code_widgets_source_list_cell_renderer_spacer_get_instance_private (CodeWidgetsSourceListCellRendererSpacer* self)
{
	return G_STRUCT_MEMBER_P (self, CodeWidgetsSourceListCellRendererSpacer_private_offset);
}

static GtkSizeRequestMode
code_widgets_source_list_cell_renderer_spacer_real_get_request_mode (GtkCellRenderer* base)
{
	CodeWidgetsSourceListCellRendererSpacer * self;
	GtkSizeRequestMode result;
#line 1505 "../src/Widgets/SourceList/SourceList.vala"
	self = (CodeWidgetsSourceListCellRendererSpacer*) base;
#line 1506 "../src/Widgets/SourceList/SourceList.vala"
	result = GTK_SIZE_REQUEST_HEIGHT_FOR_WIDTH;
#line 1506 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 8687 "SourceList.c"
}

static void
code_widgets_source_list_cell_renderer_spacer_real_get_preferred_width (GtkCellRenderer* base,
                                                                        GtkWidget* widget,
                                                                        gint* min_size,
                                                                        gint* natural_size)
{
	CodeWidgetsSourceListCellRendererSpacer * self;
	gint _vala_min_size = 0;
	gint _vala_natural_size = 0;
	guint _tmp0_;
	guint _tmp1_;
#line 1509 "../src/Widgets/SourceList/SourceList.vala"
	self = (CodeWidgetsSourceListCellRendererSpacer*) base;
#line 1509 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (widget != NULL);
#line 1510 "../src/Widgets/SourceList/SourceList.vala"
	g_object_get ((GtkCellRenderer*) self, "xpad", &_tmp0_, NULL);
#line 1510 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = _tmp0_;
#line 1510 "../src/Widgets/SourceList/SourceList.vala"
	_vala_natural_size = 2 * ((gint) _tmp1_);
#line 1510 "../src/Widgets/SourceList/SourceList.vala"
	_vala_min_size = _vala_natural_size;
#line 1509 "../src/Widgets/SourceList/SourceList.vala"
	if (min_size) {
#line 1509 "../src/Widgets/SourceList/SourceList.vala"
		*min_size = _vala_min_size;
#line 8717 "SourceList.c"
	}
#line 1509 "../src/Widgets/SourceList/SourceList.vala"
	if (natural_size) {
#line 1509 "../src/Widgets/SourceList/SourceList.vala"
		*natural_size = _vala_natural_size;
#line 8723 "SourceList.c"
	}
}

static void
code_widgets_source_list_cell_renderer_spacer_real_get_preferred_height_for_width (GtkCellRenderer* base,
                                                                                   GtkWidget* widget,
                                                                                   gint width,
                                                                                   gint* min_height,
                                                                                   gint* natural_height)
{
	CodeWidgetsSourceListCellRendererSpacer * self;
	gint _vala_min_height = 0;
	gint _vala_natural_height = 0;
	guint _tmp0_;
	guint _tmp1_;
#line 1513 "../src/Widgets/SourceList/SourceList.vala"
	self = (CodeWidgetsSourceListCellRendererSpacer*) base;
#line 1513 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (widget != NULL);
#line 1519 "../src/Widgets/SourceList/SourceList.vala"
	g_object_get ((GtkCellRenderer*) self, "ypad", &_tmp0_, NULL);
#line 1519 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = _tmp0_;
#line 1519 "../src/Widgets/SourceList/SourceList.vala"
	_vala_natural_height = 2 * ((gint) _tmp1_);
#line 1519 "../src/Widgets/SourceList/SourceList.vala"
	_vala_min_height = _vala_natural_height;
#line 1513 "../src/Widgets/SourceList/SourceList.vala"
	if (min_height) {
#line 1513 "../src/Widgets/SourceList/SourceList.vala"
		*min_height = _vala_min_height;
#line 8755 "SourceList.c"
	}
#line 1513 "../src/Widgets/SourceList/SourceList.vala"
	if (natural_height) {
#line 1513 "../src/Widgets/SourceList/SourceList.vala"
		*natural_height = _vala_natural_height;
#line 8761 "SourceList.c"
	}
}

static void
code_widgets_source_list_cell_renderer_spacer_real_render (GtkCellRenderer* base,
                                                           cairo_t* context,
                                                           GtkWidget* widget,
                                                           GdkRectangle* bg_area,
                                                           GdkRectangle* cell_area,
                                                           GtkCellRendererState flags)
{
	CodeWidgetsSourceListCellRendererSpacer * self;
#line 1522 "../src/Widgets/SourceList/SourceList.vala"
	self = (CodeWidgetsSourceListCellRendererSpacer*) base;
#line 1522 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (context != NULL);
#line 1522 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (widget != NULL);
#line 1522 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (bg_area != NULL);
#line 1522 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (cell_area != NULL);
#line 8784 "SourceList.c"
}

static CodeWidgetsSourceListCellRendererSpacer*
code_widgets_source_list_cell_renderer_spacer_construct (GType object_type)
{
	CodeWidgetsSourceListCellRendererSpacer * self = NULL;
#line 1499 "../src/Widgets/SourceList/SourceList.vala"
	self = (CodeWidgetsSourceListCellRendererSpacer*) g_object_new (object_type, NULL);
#line 1499 "../src/Widgets/SourceList/SourceList.vala"
	return self;
#line 8795 "SourceList.c"
}

static CodeWidgetsSourceListCellRendererSpacer*
code_widgets_source_list_cell_renderer_spacer_new (void)
{
#line 1499 "../src/Widgets/SourceList/SourceList.vala"
	return code_widgets_source_list_cell_renderer_spacer_construct (CODE_WIDGETS_SOURCE_LIST_TYPE_CELL_RENDERER_SPACER);
#line 8803 "SourceList.c"
}

static gint
code_widgets_source_list_cell_renderer_spacer_get_level (CodeWidgetsSourceListCellRendererSpacer* self)
{
	gint result;
#line 1503 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 1503 "../src/Widgets/SourceList/SourceList.vala"
	result = self->priv->_level;
#line 1503 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 8816 "SourceList.c"
}

static void
code_widgets_source_list_cell_renderer_spacer_set_level (CodeWidgetsSourceListCellRendererSpacer* self,
                                                         gint value)
{
	gint old_value;
#line 1503 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 1503 "../src/Widgets/SourceList/SourceList.vala"
	old_value = code_widgets_source_list_cell_renderer_spacer_get_level (self);
#line 1503 "../src/Widgets/SourceList/SourceList.vala"
	if (old_value != value) {
#line 1503 "../src/Widgets/SourceList/SourceList.vala"
		self->priv->_level = value;
#line 1503 "../src/Widgets/SourceList/SourceList.vala"
		g_object_notify_by_pspec ((GObject *) self, code_widgets_source_list_cell_renderer_spacer_properties[CODE_WIDGETS_SOURCE_LIST_CELL_RENDERER_SPACER_LEVEL_PROPERTY]);
#line 8834 "SourceList.c"
	}
}

static void
code_widgets_source_list_cell_renderer_spacer_class_init (CodeWidgetsSourceListCellRendererSpacerClass * klass,
                                                          gpointer klass_data)
{
#line 1499 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_cell_renderer_spacer_parent_class = g_type_class_peek_parent (klass);
#line 1499 "../src/Widgets/SourceList/SourceList.vala"
	g_type_class_adjust_private_offset (klass, &CodeWidgetsSourceListCellRendererSpacer_private_offset);
#line 1499 "../src/Widgets/SourceList/SourceList.vala"
	((GtkCellRendererClass *) klass)->get_request_mode = (GtkSizeRequestMode (*) (GtkCellRenderer*)) code_widgets_source_list_cell_renderer_spacer_real_get_request_mode;
#line 1499 "../src/Widgets/SourceList/SourceList.vala"
	((GtkCellRendererClass *) klass)->get_preferred_width = (void (*) (GtkCellRenderer*, GtkWidget*, gint*, gint*)) code_widgets_source_list_cell_renderer_spacer_real_get_preferred_width;
#line 1499 "../src/Widgets/SourceList/SourceList.vala"
	((GtkCellRendererClass *) klass)->get_preferred_height_for_width = (void (*) (GtkCellRenderer*, GtkWidget*, gint, gint*, gint*)) code_widgets_source_list_cell_renderer_spacer_real_get_preferred_height_for_width;
#line 1499 "../src/Widgets/SourceList/SourceList.vala"
	((GtkCellRendererClass *) klass)->render = (void (*) (GtkCellRenderer*, cairo_t*, GtkWidget*, GdkRectangle*, GdkRectangle*, GtkCellRendererState)) code_widgets_source_list_cell_renderer_spacer_real_render;
#line 1499 "../src/Widgets/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_code_widgets_source_list_cell_renderer_spacer_get_property;
#line 1499 "../src/Widgets/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_code_widgets_source_list_cell_renderer_spacer_set_property;
#line 1499 "../src/Widgets/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->finalize = code_widgets_source_list_cell_renderer_spacer_finalize;
#line 8860 "SourceList.c"
	/**
	         * Indentation level represented by this cell renderer
	         */
#line 1499 "../src/Widgets/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), CODE_WIDGETS_SOURCE_LIST_CELL_RENDERER_SPACER_LEVEL_PROPERTY, code_widgets_source_list_cell_renderer_spacer_properties[CODE_WIDGETS_SOURCE_LIST_CELL_RENDERER_SPACER_LEVEL_PROPERTY] = g_param_spec_int ("level", "level", "level", G_MININT, G_MAXINT, -1, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 8866 "SourceList.c"
}

static void
code_widgets_source_list_cell_renderer_spacer_instance_init (CodeWidgetsSourceListCellRendererSpacer * self,
                                                             gpointer klass)
{
#line 1499 "../src/Widgets/SourceList/SourceList.vala"
	self->priv = code_widgets_source_list_cell_renderer_spacer_get_instance_private (self);
#line 1503 "../src/Widgets/SourceList/SourceList.vala"
	self->priv->_level = -1;
#line 8877 "SourceList.c"
}

static void
code_widgets_source_list_cell_renderer_spacer_finalize (GObject * obj)
{
	CodeWidgetsSourceListCellRendererSpacer * self;
#line 1499 "../src/Widgets/SourceList/SourceList.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, CODE_WIDGETS_SOURCE_LIST_TYPE_CELL_RENDERER_SPACER, CodeWidgetsSourceListCellRendererSpacer);
#line 1499 "../src/Widgets/SourceList/SourceList.vala"
	G_OBJECT_CLASS (code_widgets_source_list_cell_renderer_spacer_parent_class)->finalize (obj);
#line 8888 "SourceList.c"
}

/**
     * A cell renderer that only adds space.
     */
static GType
code_widgets_source_list_cell_renderer_spacer_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (CodeWidgetsSourceListCellRendererSpacerClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) code_widgets_source_list_cell_renderer_spacer_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (CodeWidgetsSourceListCellRendererSpacer), 0, (GInstanceInitFunc) code_widgets_source_list_cell_renderer_spacer_instance_init, NULL };
	GType code_widgets_source_list_cell_renderer_spacer_type_id;
	code_widgets_source_list_cell_renderer_spacer_type_id = g_type_register_static (gtk_cell_renderer_get_type (), "CodeWidgetsSourceListCellRendererSpacer", &g_define_type_info, 0);
	CodeWidgetsSourceListCellRendererSpacer_private_offset = g_type_add_instance_private (code_widgets_source_list_cell_renderer_spacer_type_id, sizeof (CodeWidgetsSourceListCellRendererSpacerPrivate));
	return code_widgets_source_list_cell_renderer_spacer_type_id;
}

static GType
code_widgets_source_list_cell_renderer_spacer_get_type (void)
{
	static volatile gsize code_widgets_source_list_cell_renderer_spacer_type_id__once = 0;
	if (g_once_init_enter (&code_widgets_source_list_cell_renderer_spacer_type_id__once)) {
		GType code_widgets_source_list_cell_renderer_spacer_type_id;
		code_widgets_source_list_cell_renderer_spacer_type_id = code_widgets_source_list_cell_renderer_spacer_get_type_once ();
		g_once_init_leave (&code_widgets_source_list_cell_renderer_spacer_type_id__once, code_widgets_source_list_cell_renderer_spacer_type_id);
	}
	return code_widgets_source_list_cell_renderer_spacer_type_id__once;
}

static void
_vala_code_widgets_source_list_cell_renderer_spacer_get_property (GObject * object,
                                                                  guint property_id,
                                                                  GValue * value,
                                                                  GParamSpec * pspec)
{
	CodeWidgetsSourceListCellRendererSpacer * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, CODE_WIDGETS_SOURCE_LIST_TYPE_CELL_RENDERER_SPACER, CodeWidgetsSourceListCellRendererSpacer);
#line 1499 "../src/Widgets/SourceList/SourceList.vala"
	switch (property_id) {
#line 1499 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_CELL_RENDERER_SPACER_LEVEL_PROPERTY:
#line 1499 "../src/Widgets/SourceList/SourceList.vala"
		g_value_set_int (value, code_widgets_source_list_cell_renderer_spacer_get_level (self));
#line 1499 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 8932 "SourceList.c"
		default:
#line 1499 "../src/Widgets/SourceList/SourceList.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 1499 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 8938 "SourceList.c"
	}
}

static void
_vala_code_widgets_source_list_cell_renderer_spacer_set_property (GObject * object,
                                                                  guint property_id,
                                                                  const GValue * value,
                                                                  GParamSpec * pspec)
{
	CodeWidgetsSourceListCellRendererSpacer * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, CODE_WIDGETS_SOURCE_LIST_TYPE_CELL_RENDERER_SPACER, CodeWidgetsSourceListCellRendererSpacer);
#line 1499 "../src/Widgets/SourceList/SourceList.vala"
	switch (property_id) {
#line 1499 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_CELL_RENDERER_SPACER_LEVEL_PROPERTY:
#line 1499 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_cell_renderer_spacer_set_level (self, g_value_get_int (value));
#line 1499 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 8958 "SourceList.c"
		default:
#line 1499 "../src/Widgets/SourceList/SourceList.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 1499 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 8964 "SourceList.c"
	}
}

static inline gpointer
code_widgets_source_list_tree_get_instance_private (CodeWidgetsSourceListTree* self)
{
	return G_STRUCT_MEMBER_P (self, CodeWidgetsSourceListTree_private_offset);
}

static GType
code_widgets_source_list_tree_column_get_type_once (void)
{
	static const GEnumValue values[] = {{CODE_WIDGETS_SOURCE_LIST_TREE_COLUMN_ITEM, "CODE_WIDGETS_SOURCE_LIST_TREE_COLUMN_ITEM", "item"}, {CODE_WIDGETS_SOURCE_LIST_TREE_COLUMN_N_COLS, "CODE_WIDGETS_SOURCE_LIST_TREE_COLUMN_N_COLS", "n-cols"}, {0, NULL, NULL}};
	GType code_widgets_source_list_tree_column_type_id;
	code_widgets_source_list_tree_column_type_id = g_enum_register_static ("CodeWidgetsSourceListTreeColumn", values);
	return code_widgets_source_list_tree_column_type_id;
}

static GType
code_widgets_source_list_tree_column_get_type (void)
{
	static volatile gsize code_widgets_source_list_tree_column_type_id__once = 0;
	if (g_once_init_enter (&code_widgets_source_list_tree_column_type_id__once)) {
		GType code_widgets_source_list_tree_column_type_id;
		code_widgets_source_list_tree_column_type_id = code_widgets_source_list_tree_column_get_type_once ();
		g_once_init_leave (&code_widgets_source_list_tree_column_type_id__once, code_widgets_source_list_tree_column_type_id);
	}
	return code_widgets_source_list_tree_column_type_id__once;
}

static CodeWidgetsSourceListTree*
code_widgets_source_list_tree_construct (GType object_type,
                                         CodeWidgetsSourceListDataModel* data_model)
{
	CodeWidgetsSourceListTree * self = NULL;
#line 1623 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (data_model != NULL, NULL);
#line 1624 "../src/Widgets/SourceList/SourceList.vala"
	self = (CodeWidgetsSourceListTree*) g_object_new (object_type, "data-model", data_model, NULL);
#line 1623 "../src/Widgets/SourceList/SourceList.vala"
	return self;
#line 9006 "SourceList.c"
}

static CodeWidgetsSourceListTree*
code_widgets_source_list_tree_new (CodeWidgetsSourceListDataModel* data_model)
{
#line 1623 "../src/Widgets/SourceList/SourceList.vala"
	return code_widgets_source_list_tree_construct (CODE_WIDGETS_SOURCE_LIST_TYPE_TREE, data_model);
#line 9014 "SourceList.c"
}

static gpointer
_gtk_target_list_ref0 (gpointer self)
{
#line 1772 "../src/Widgets/SourceList/SourceList.vala"
	return self ? gtk_target_list_ref (self) : NULL;
#line 9022 "SourceList.c"
}

static gboolean
code_widgets_source_list_tree_real_drag_motion (GtkWidget* base,
                                                GdkDragContext* context,
                                                gint x,
                                                gint y,
                                                guint time)
{
	CodeWidgetsSourceListTree * self;
	GtkTreePath* suggested_path = NULL;
	GtkTreePath* current_path = NULL;
	GtkTreeViewDropPosition suggested_pos = 0;
	GtkTreeViewDropPosition current_pos = 0;
	GtkTreePath* _tmp0_ = NULL;
	GtkTreeViewDropPosition _tmp1_ = 0;
	gboolean _tmp2_;
	gboolean result;
#line 1730 "../src/Widgets/SourceList/SourceList.vala"
	self = (CodeWidgetsSourceListTree*) base;
#line 1730 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (context != NULL, FALSE);
#line 1732 "../src/Widgets/SourceList/SourceList.vala"
	if (!GTK_WIDGET_CLASS (code_widgets_source_list_tree_parent_class)->drag_motion ((GtkWidget*) G_TYPE_CHECK_INSTANCE_CAST (self, gtk_tree_view_get_type (), GtkTreeView), context, x, y, time)) {
#line 1733 "../src/Widgets/SourceList/SourceList.vala"
		result = FALSE;
#line 1733 "../src/Widgets/SourceList/SourceList.vala"
		return result;
#line 9051 "SourceList.c"
	}
#line 1738 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = gtk_tree_view_get_dest_row_at_pos ((GtkTreeView*) self, x, y, &_tmp0_, &_tmp1_);
#line 1738 "../src/Widgets/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (suggested_path);
#line 1738 "../src/Widgets/SourceList/SourceList.vala"
	suggested_path = _tmp0_;
#line 1738 "../src/Widgets/SourceList/SourceList.vala"
	suggested_pos = _tmp1_;
#line 1738 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp2_) {
#line 9063 "SourceList.c"
		GtkTreePath* _tmp3_ = NULL;
		GtkTreeViewDropPosition _tmp4_ = 0;
		gboolean _tmp5_ = FALSE;
		GtkTreePath* _tmp6_;
#line 1742 "../src/Widgets/SourceList/SourceList.vala"
		gtk_tree_view_get_drag_dest_row ((GtkTreeView*) self, &_tmp3_, &_tmp4_);
#line 1742 "../src/Widgets/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (current_path);
#line 1742 "../src/Widgets/SourceList/SourceList.vala"
		current_path = _tmp3_;
#line 1742 "../src/Widgets/SourceList/SourceList.vala"
		current_pos = _tmp4_;
#line 1744 "../src/Widgets/SourceList/SourceList.vala"
		_tmp6_ = current_path;
#line 1744 "../src/Widgets/SourceList/SourceList.vala"
		if (_tmp6_ != NULL) {
#line 9080 "SourceList.c"
			GtkTreePath* _tmp7_;
			GtkTreePath* _tmp8_;
#line 1744 "../src/Widgets/SourceList/SourceList.vala"
			_tmp7_ = suggested_path;
#line 1744 "../src/Widgets/SourceList/SourceList.vala"
			_tmp8_ = current_path;
#line 1744 "../src/Widgets/SourceList/SourceList.vala"
			_tmp5_ = gtk_tree_path_compare (_tmp7_, _tmp8_) == 0;
#line 9089 "SourceList.c"
		} else {
#line 1744 "../src/Widgets/SourceList/SourceList.vala"
			_tmp5_ = FALSE;
#line 9093 "SourceList.c"
		}
#line 1744 "../src/Widgets/SourceList/SourceList.vala"
		if (_tmp5_) {
#line 9097 "SourceList.c"
			GtkWidget* source_widget = NULL;
			GtkWidget* _tmp9_;
			GtkWidget* _tmp10_;
			gboolean dragging_treemodel_row = FALSE;
			GtkWidget* _tmp11_;
#line 1748 "../src/Widgets/SourceList/SourceList.vala"
			_tmp9_ = gtk_drag_get_source_widget (context);
#line 1748 "../src/Widgets/SourceList/SourceList.vala"
			_tmp10_ = _g_object_ref0 (_tmp9_);
#line 1748 "../src/Widgets/SourceList/SourceList.vala"
			source_widget = _tmp10_;
#line 1749 "../src/Widgets/SourceList/SourceList.vala"
			_tmp11_ = source_widget;
#line 1749 "../src/Widgets/SourceList/SourceList.vala"
			dragging_treemodel_row = _tmp11_ == G_TYPE_CHECK_INSTANCE_CAST (self, gtk_widget_get_type (), GtkWidget);
#line 1751 "../src/Widgets/SourceList/SourceList.vala"
			if (dragging_treemodel_row) {
#line 1754 "../src/Widgets/SourceList/SourceList.vala"
				if (current_pos != GTK_TREE_VIEW_DROP_BEFORE) {
#line 1755 "../src/Widgets/SourceList/SourceList.vala"
					if (current_pos == GTK_TREE_VIEW_DROP_INTO_OR_BEFORE) {
#line 9119 "SourceList.c"
						GtkTreePath* _tmp12_;
#line 1756 "../src/Widgets/SourceList/SourceList.vala"
						_tmp12_ = current_path;
#line 1756 "../src/Widgets/SourceList/SourceList.vala"
						gtk_tree_view_set_drag_dest_row ((GtkTreeView*) self, _tmp12_, GTK_TREE_VIEW_DROP_BEFORE);
#line 9125 "SourceList.c"
					} else {
#line 1758 "../src/Widgets/SourceList/SourceList.vala"
						gtk_tree_view_set_drag_dest_row ((GtkTreeView*) self, NULL, GTK_TREE_VIEW_DROP_AFTER);
#line 9129 "SourceList.c"
					}
				}
			} else {
				CodeWidgetsSourceListDragDest* dest = NULL;
				CodeWidgetsSourceListDataModel* _tmp15_;
				GtkTreePath* _tmp16_;
				CodeWidgetsSourceListItem* _tmp17_;
				CodeWidgetsSourceListDragDest* _tmp18_;
				CodeWidgetsSourceListDragDest* _tmp19_;
#line 1763 "../src/Widgets/SourceList/SourceList.vala"
				if (current_pos == GTK_TREE_VIEW_DROP_BEFORE) {
#line 9141 "SourceList.c"
					GtkTreePath* _tmp13_;
#line 1764 "../src/Widgets/SourceList/SourceList.vala"
					_tmp13_ = current_path;
#line 1764 "../src/Widgets/SourceList/SourceList.vala"
					gtk_tree_view_set_drag_dest_row ((GtkTreeView*) self, _tmp13_, GTK_TREE_VIEW_DROP_INTO_OR_BEFORE);
#line 9147 "SourceList.c"
				} else {
#line 1765 "../src/Widgets/SourceList/SourceList.vala"
					if (current_pos == GTK_TREE_VIEW_DROP_AFTER) {
#line 9151 "SourceList.c"
						GtkTreePath* _tmp14_;
#line 1766 "../src/Widgets/SourceList/SourceList.vala"
						_tmp14_ = current_path;
#line 1766 "../src/Widgets/SourceList/SourceList.vala"
						gtk_tree_view_set_drag_dest_row ((GtkTreeView*) self, _tmp14_, GTK_TREE_VIEW_DROP_INTO_OR_AFTER);
#line 9157 "SourceList.c"
					}
				}
#line 1769 "../src/Widgets/SourceList/SourceList.vala"
				_tmp15_ = self->priv->_data_model;
#line 1769 "../src/Widgets/SourceList/SourceList.vala"
				_tmp16_ = current_path;
#line 1769 "../src/Widgets/SourceList/SourceList.vala"
				_tmp17_ = code_widgets_source_list_data_model_get_item_from_path (_tmp15_, _tmp16_);
#line 1769 "../src/Widgets/SourceList/SourceList.vala"
				_tmp18_ = CODE_WIDGETS_IS_SOURCE_LIST_DRAG_DEST (_tmp17_) ? ((CodeWidgetsSourceListDragDest*) _tmp17_) : NULL;
#line 1769 "../src/Widgets/SourceList/SourceList.vala"
				if (_tmp18_ == NULL) {
#line 1769 "../src/Widgets/SourceList/SourceList.vala"
					_g_object_unref0 (_tmp17_);
#line 9172 "SourceList.c"
				}
#line 1769 "../src/Widgets/SourceList/SourceList.vala"
				dest = _tmp18_;
#line 1771 "../src/Widgets/SourceList/SourceList.vala"
				_tmp19_ = dest;
#line 1771 "../src/Widgets/SourceList/SourceList.vala"
				if (_tmp19_ != NULL) {
#line 9180 "SourceList.c"
					GtkTargetList* target_list = NULL;
					GtkTargetList* _tmp20_;
					GtkTargetList* _tmp21_;
					GdkAtom target = (GdkAtom) 0U;
					GtkTargetList* _tmp22_;
#line 1772 "../src/Widgets/SourceList/SourceList.vala"
					_tmp20_ = gtk_drag_dest_get_target_list ((GtkWidget*) self);
#line 1772 "../src/Widgets/SourceList/SourceList.vala"
					_tmp21_ = _gtk_target_list_ref0 (_tmp20_);
#line 1772 "../src/Widgets/SourceList/SourceList.vala"
					target_list = _tmp21_;
#line 1773 "../src/Widgets/SourceList/SourceList.vala"
					_tmp22_ = target_list;
#line 1773 "../src/Widgets/SourceList/SourceList.vala"
					target = gtk_drag_dest_find_target ((GtkWidget*) self, context, _tmp22_);
#line 1777 "../src/Widgets/SourceList/SourceList.vala"
					g_object_set_data_full ((GObject*) context, "suggested-dnd-action", (gpointer) ((gintptr) gdk_drag_context_get_suggested_action (context)), NULL);
#line 1778 "../src/Widgets/SourceList/SourceList.vala"
					gtk_drag_get_data ((GtkWidget*) self, context, target, (guint32) time);
#line 1771 "../src/Widgets/SourceList/SourceList.vala"
					_gtk_target_list_unref0 (target_list);
#line 9202 "SourceList.c"
				} else {
#line 1781 "../src/Widgets/SourceList/SourceList.vala"
					gtk_tree_view_set_drag_dest_row ((GtkTreeView*) self, NULL, GTK_TREE_VIEW_DROP_BEFORE);
#line 9206 "SourceList.c"
				}
#line 1751 "../src/Widgets/SourceList/SourceList.vala"
				_g_object_unref0 (dest);
#line 9210 "SourceList.c"
			}
#line 1744 "../src/Widgets/SourceList/SourceList.vala"
			_g_object_unref0 (source_widget);
#line 9214 "SourceList.c"
		}
	} else {
#line 1787 "../src/Widgets/SourceList/SourceList.vala"
		gtk_tree_view_set_drag_dest_row ((GtkTreeView*) self, NULL, GTK_TREE_VIEW_DROP_AFTER);
#line 1788 "../src/Widgets/SourceList/SourceList.vala"
		result = FALSE;
#line 1788 "../src/Widgets/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (current_path);
#line 1788 "../src/Widgets/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (suggested_path);
#line 1788 "../src/Widgets/SourceList/SourceList.vala"
		return result;
#line 9227 "SourceList.c"
	}
#line 1791 "../src/Widgets/SourceList/SourceList.vala"
	result = TRUE;
#line 1791 "../src/Widgets/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (current_path);
#line 1791 "../src/Widgets/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (suggested_path);
#line 1791 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 9237 "SourceList.c"
}

static void
code_widgets_source_list_tree_real_drag_data_received (GtkWidget* base,
                                                       GdkDragContext* context,
                                                       gint x,
                                                       gint y,
                                                       GtkSelectionData* selection_data,
                                                       guint info,
                                                       guint time)
{
	CodeWidgetsSourceListTree * self;
	GtkTargetList* target_list = NULL;
	GtkTargetList* _tmp0_;
	GtkTargetList* _tmp1_;
	GdkAtom target = (GdkAtom) 0U;
	GtkTargetList* _tmp2_;
	GtkTreePath* path = NULL;
	GtkTreeViewDropPosition pos = 0;
	gconstpointer _tmp3_;
#line 1794 "../src/Widgets/SourceList/SourceList.vala"
	self = (CodeWidgetsSourceListTree*) base;
#line 1794 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (context != NULL);
#line 1794 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (selection_data != NULL);
#line 1802 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = gtk_drag_dest_get_target_list ((GtkWidget*) self);
#line 1802 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = _gtk_target_list_ref0 (_tmp0_);
#line 1802 "../src/Widgets/SourceList/SourceList.vala"
	target_list = _tmp1_;
#line 1803 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = target_list;
#line 1803 "../src/Widgets/SourceList/SourceList.vala"
	target = gtk_drag_dest_find_target ((GtkWidget*) self, context, _tmp2_);
#line 1805 "../src/Widgets/SourceList/SourceList.vala"
	if (target == gdk_atom_intern_static_string ("GTK_TREE_MODEL_ROW")) {
#line 1806 "../src/Widgets/SourceList/SourceList.vala"
		GTK_WIDGET_CLASS (code_widgets_source_list_tree_parent_class)->drag_data_received ((GtkWidget*) G_TYPE_CHECK_INSTANCE_CAST (self, gtk_tree_view_get_type (), GtkTreeView), context, x, y, selection_data, info, time);
#line 1807 "../src/Widgets/SourceList/SourceList.vala"
		_gtk_target_list_unref0 (target_list);
#line 1807 "../src/Widgets/SourceList/SourceList.vala"
		return;
#line 9282 "SourceList.c"
	}
#line 1813 "../src/Widgets/SourceList/SourceList.vala"
	_tmp3_ = g_object_get_data ((GObject*) context, "suggested-dnd-action");
#line 1813 "../src/Widgets/SourceList/SourceList.vala"
	if (((gint) ((gintptr) _tmp3_)) != 0) {
#line 9288 "SourceList.c"
		GtkTreePath* _tmp4_ = NULL;
		GtkTreeViewDropPosition _tmp5_ = 0;
		GtkTreePath* _tmp6_;
#line 1814 "../src/Widgets/SourceList/SourceList.vala"
		g_object_set_data_full ((GObject*) context, "suggested-dnd-action", (gpointer) ((gintptr) 0), NULL);
#line 1816 "../src/Widgets/SourceList/SourceList.vala"
		gtk_tree_view_get_drag_dest_row ((GtkTreeView*) self, &_tmp4_, &_tmp5_);
#line 1816 "../src/Widgets/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (path);
#line 1816 "../src/Widgets/SourceList/SourceList.vala"
		path = _tmp4_;
#line 1816 "../src/Widgets/SourceList/SourceList.vala"
		pos = _tmp5_;
#line 1818 "../src/Widgets/SourceList/SourceList.vala"
		_tmp6_ = path;
#line 1818 "../src/Widgets/SourceList/SourceList.vala"
		if (_tmp6_ != NULL) {
#line 9306 "SourceList.c"
			CodeWidgetsSourceListDragDest* dest = NULL;
			CodeWidgetsSourceListDataModel* _tmp7_;
			GtkTreePath* _tmp8_;
			CodeWidgetsSourceListItem* _tmp9_;
			CodeWidgetsSourceListDragDest* _tmp10_;
			gboolean _tmp11_ = FALSE;
			CodeWidgetsSourceListDragDest* _tmp12_;
#line 1820 "../src/Widgets/SourceList/SourceList.vala"
			_tmp7_ = self->priv->_data_model;
#line 1820 "../src/Widgets/SourceList/SourceList.vala"
			_tmp8_ = path;
#line 1820 "../src/Widgets/SourceList/SourceList.vala"
			_tmp9_ = code_widgets_source_list_data_model_get_item_from_path (_tmp7_, _tmp8_);
#line 1820 "../src/Widgets/SourceList/SourceList.vala"
			_tmp10_ = CODE_WIDGETS_IS_SOURCE_LIST_DRAG_DEST (_tmp9_) ? ((CodeWidgetsSourceListDragDest*) _tmp9_) : NULL;
#line 1820 "../src/Widgets/SourceList/SourceList.vala"
			if (_tmp10_ == NULL) {
#line 1820 "../src/Widgets/SourceList/SourceList.vala"
				_g_object_unref0 (_tmp9_);
#line 9326 "SourceList.c"
			}
#line 1820 "../src/Widgets/SourceList/SourceList.vala"
			dest = _tmp10_;
#line 1822 "../src/Widgets/SourceList/SourceList.vala"
			_tmp12_ = dest;
#line 1822 "../src/Widgets/SourceList/SourceList.vala"
			if (_tmp12_ == NULL) {
#line 1822 "../src/Widgets/SourceList/SourceList.vala"
				_tmp11_ = TRUE;
#line 9336 "SourceList.c"
			} else {
				CodeWidgetsSourceListDragDest* _tmp13_;
#line 1822 "../src/Widgets/SourceList/SourceList.vala"
				_tmp13_ = dest;
#line 1822 "../src/Widgets/SourceList/SourceList.vala"
				_tmp11_ = !code_widgets_source_list_drag_dest_data_drop_possible (_tmp13_, context, selection_data);
#line 9343 "SourceList.c"
			}
#line 1822 "../src/Widgets/SourceList/SourceList.vala"
			if (_tmp11_) {
#line 1825 "../src/Widgets/SourceList/SourceList.vala"
				gtk_tree_view_set_drag_dest_row ((GtkTreeView*) self, NULL, GTK_TREE_VIEW_DROP_BEFORE);
#line 1826 "../src/Widgets/SourceList/SourceList.vala"
				gdk_drag_status (context, 0, (guint32) time);
#line 1827 "../src/Widgets/SourceList/SourceList.vala"
				_g_object_unref0 (dest);
#line 1827 "../src/Widgets/SourceList/SourceList.vala"
				__vala_GtkTreePath_free0 (path);
#line 1827 "../src/Widgets/SourceList/SourceList.vala"
				_gtk_target_list_unref0 (target_list);
#line 1827 "../src/Widgets/SourceList/SourceList.vala"
				return;
#line 9359 "SourceList.c"
			}
#line 1818 "../src/Widgets/SourceList/SourceList.vala"
			_g_object_unref0 (dest);
#line 9363 "SourceList.c"
		}
#line 1831 "../src/Widgets/SourceList/SourceList.vala"
		gdk_drag_status (context, gdk_drag_context_get_suggested_action (context), (guint32) time);
#line 9367 "SourceList.c"
	} else {
		GtkTreePath* _tmp14_ = NULL;
		GtkTreeViewDropPosition _tmp15_ = 0;
		gboolean _tmp16_;
#line 1833 "../src/Widgets/SourceList/SourceList.vala"
		_tmp16_ = gtk_tree_view_get_dest_row_at_pos ((GtkTreeView*) self, x, y, &_tmp14_, &_tmp15_);
#line 1833 "../src/Widgets/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (path);
#line 1833 "../src/Widgets/SourceList/SourceList.vala"
		path = _tmp14_;
#line 1833 "../src/Widgets/SourceList/SourceList.vala"
		pos = _tmp15_;
#line 1833 "../src/Widgets/SourceList/SourceList.vala"
		if (_tmp16_) {
#line 9382 "SourceList.c"
			CodeWidgetsSourceListDragDest* drag_dest = NULL;
			CodeWidgetsSourceListDataModel* _tmp17_;
			GtkTreePath* _tmp18_;
			CodeWidgetsSourceListItem* _tmp19_;
			CodeWidgetsSourceListDragDest* _tmp20_;
			CodeWidgetsSourceListDragDest* _tmp21_;
#line 1839 "../src/Widgets/SourceList/SourceList.vala"
			_tmp17_ = self->priv->_data_model;
#line 1839 "../src/Widgets/SourceList/SourceList.vala"
			_tmp18_ = path;
#line 1839 "../src/Widgets/SourceList/SourceList.vala"
			_tmp19_ = code_widgets_source_list_data_model_get_item_from_path (_tmp17_, _tmp18_);
#line 1839 "../src/Widgets/SourceList/SourceList.vala"
			_tmp20_ = CODE_WIDGETS_IS_SOURCE_LIST_DRAG_DEST (_tmp19_) ? ((CodeWidgetsSourceListDragDest*) _tmp19_) : NULL;
#line 1839 "../src/Widgets/SourceList/SourceList.vala"
			if (_tmp20_ == NULL) {
#line 1839 "../src/Widgets/SourceList/SourceList.vala"
				_g_object_unref0 (_tmp19_);
#line 9401 "SourceList.c"
			}
#line 1839 "../src/Widgets/SourceList/SourceList.vala"
			drag_dest = _tmp20_;
#line 1841 "../src/Widgets/SourceList/SourceList.vala"
			_tmp21_ = drag_dest;
#line 1841 "../src/Widgets/SourceList/SourceList.vala"
			if (_tmp21_ != NULL) {
#line 9409 "SourceList.c"
				GdkDragAction action = 0U;
				CodeWidgetsSourceListDragDest* _tmp22_;
#line 1842 "../src/Widgets/SourceList/SourceList.vala"
				_tmp22_ = drag_dest;
#line 1842 "../src/Widgets/SourceList/SourceList.vala"
				action = code_widgets_source_list_drag_dest_data_received (_tmp22_, context, selection_data);
#line 1843 "../src/Widgets/SourceList/SourceList.vala"
				gtk_drag_finish (context, action != 0, action == GDK_ACTION_MOVE, (guint32) time);
#line 1844 "../src/Widgets/SourceList/SourceList.vala"
				_g_object_unref0 (drag_dest);
#line 1844 "../src/Widgets/SourceList/SourceList.vala"
				__vala_GtkTreePath_free0 (path);
#line 1844 "../src/Widgets/SourceList/SourceList.vala"
				_gtk_target_list_unref0 (target_list);
#line 1844 "../src/Widgets/SourceList/SourceList.vala"
				return;
#line 9426 "SourceList.c"
			}
#line 1833 "../src/Widgets/SourceList/SourceList.vala"
			_g_object_unref0 (drag_dest);
#line 9430 "SourceList.c"
		}
#line 1849 "../src/Widgets/SourceList/SourceList.vala"
		gtk_drag_finish (context, FALSE, FALSE, (guint32) time);
#line 9434 "SourceList.c"
	}
#line 1794 "../src/Widgets/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (path);
#line 1794 "../src/Widgets/SourceList/SourceList.vala"
	_gtk_target_list_unref0 (target_list);
#line 9440 "SourceList.c"
}

static void
code_widgets_source_list_tree_configure_drag_source (CodeWidgetsSourceListTree* self,
                                                     GtkTargetEntry* src_entries,
                                                     gint src_entries_length1)
{
	GtkTargetEntry* entries = NULL;
	gint _tmp0_ = 0;
	GtkTargetEntry* _tmp1_;
	gint entries_length1;
	gint _entries_size_;
#line 1853 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 1855 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = code_widgets_source_list_tree_append_row_target_entry (src_entries, (gint) src_entries_length1, &_tmp0_);
#line 1855 "../src/Widgets/SourceList/SourceList.vala"
	entries = _tmp1_;
#line 1855 "../src/Widgets/SourceList/SourceList.vala"
	entries_length1 = _tmp0_;
#line 1855 "../src/Widgets/SourceList/SourceList.vala"
	_entries_size_ = entries_length1;
#line 1857 "../src/Widgets/SourceList/SourceList.vala"
	gtk_tree_view_unset_rows_drag_source ((GtkTreeView*) self);
#line 1858 "../src/Widgets/SourceList/SourceList.vala"
	gtk_tree_view_enable_model_drag_source ((GtkTreeView*) self, GDK_BUTTON1_MASK, entries, (gint) entries_length1, GDK_ACTION_MOVE);
#line 1853 "../src/Widgets/SourceList/SourceList.vala"
	entries = (g_free (entries), NULL);
#line 9469 "SourceList.c"
}

static void
code_widgets_source_list_tree_configure_drag_dest (CodeWidgetsSourceListTree* self,
                                                   GtkTargetEntry* dest_entries,
                                                   gint dest_entries_length1,
                                                   GdkDragAction actions)
{
	GtkTargetEntry* entries = NULL;
	gint _tmp0_ = 0;
	GtkTargetEntry* _tmp1_;
	gint entries_length1;
	gint _entries_size_;
#line 1861 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 1863 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = code_widgets_source_list_tree_append_row_target_entry (dest_entries, (gint) dest_entries_length1, &_tmp0_);
#line 1863 "../src/Widgets/SourceList/SourceList.vala"
	entries = _tmp1_;
#line 1863 "../src/Widgets/SourceList/SourceList.vala"
	entries_length1 = _tmp0_;
#line 1863 "../src/Widgets/SourceList/SourceList.vala"
	_entries_size_ = entries_length1;
#line 1865 "../src/Widgets/SourceList/SourceList.vala"
	gtk_tree_view_unset_rows_drag_dest ((GtkTreeView*) self);
#line 1868 "../src/Widgets/SourceList/SourceList.vala"
	gtk_tree_view_enable_model_drag_dest ((GtkTreeView*) self, entries, (gint) entries_length1, GDK_ACTION_MOVE | actions);
#line 1861 "../src/Widgets/SourceList/SourceList.vala"
	entries = (g_free (entries), NULL);
#line 9499 "SourceList.c"
}

static gboolean
code_widgets_source_list_tree_on_query_tooltip (CodeWidgetsSourceListTree* self,
                                                gint x,
                                                gint y,
                                                gboolean keyboard_tooltip,
                                                GtkTooltip* tooltip)
{
	GtkTreePath* path = NULL;
	GtkTreeViewColumn* column = NULL;
	GtkTreeViewColumn* _tmp0_;
	GtkTreeViewColumn* _tmp1_;
	GtkTreePath* _tmp2_ = NULL;
	GtkTreePath* _tmp3_;
	CodeWidgetsSourceListItem* item = NULL;
	CodeWidgetsSourceListDataModel* _tmp4_;
	GtkTreePath* _tmp5_;
	CodeWidgetsSourceListItem* _tmp6_;
	CodeWidgetsSourceListItem* _tmp7_;
	gboolean result;
#line 1871 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 1871 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (tooltip != NULL, FALSE);
#line 1873 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = gtk_tree_view_get_column ((GtkTreeView*) self, (gint) CODE_WIDGETS_SOURCE_LIST_TREE_COLUMN_ITEM);
#line 1873 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 1873 "../src/Widgets/SourceList/SourceList.vala"
	column = _tmp1_;
#line 1875 "../src/Widgets/SourceList/SourceList.vala"
	gtk_tree_view_get_tooltip_context ((GtkTreeView*) self, &x, &y, keyboard_tooltip, NULL, &_tmp2_, NULL);
#line 1875 "../src/Widgets/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (path);
#line 1875 "../src/Widgets/SourceList/SourceList.vala"
	path = _tmp2_;
#line 1876 "../src/Widgets/SourceList/SourceList.vala"
	_tmp3_ = path;
#line 1876 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp3_ == NULL) {
#line 1877 "../src/Widgets/SourceList/SourceList.vala"
		result = FALSE;
#line 1877 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (column);
#line 1877 "../src/Widgets/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (path);
#line 1877 "../src/Widgets/SourceList/SourceList.vala"
		return result;
#line 9549 "SourceList.c"
	}
#line 1880 "../src/Widgets/SourceList/SourceList.vala"
	_tmp4_ = self->priv->_data_model;
#line 1880 "../src/Widgets/SourceList/SourceList.vala"
	_tmp5_ = path;
#line 1880 "../src/Widgets/SourceList/SourceList.vala"
	_tmp6_ = code_widgets_source_list_data_model_get_item_from_path (_tmp4_, _tmp5_);
#line 1880 "../src/Widgets/SourceList/SourceList.vala"
	item = _tmp6_;
#line 1881 "../src/Widgets/SourceList/SourceList.vala"
	_tmp7_ = item;
#line 1881 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp7_ != NULL) {
#line 9563 "SourceList.c"
		gboolean should_show = FALSE;
		GdkRectangle start_cell_area = {0};
		GtkTreePath* _tmp8_;
		GtkTreeViewColumn* _tmp9_;
		GdkRectangle _tmp10_ = {0};
		GtkTreePath* _tmp11_;
		CodeWidgetsSourceListItem* _tmp12_;
		const gchar* _tmp13_;
		const gchar* _tmp14_;
		gboolean _tmp24_ = FALSE;
		GtkTreeViewColumn* _tmp25_;
		GtkTreePath* _tmp26_;
		GtkCellRendererText* _tmp27_;
		GdkRectangle _tmp28_;
#line 1882 "../src/Widgets/SourceList/SourceList.vala"
		should_show = FALSE;
#line 1885 "../src/Widgets/SourceList/SourceList.vala"
		_tmp8_ = path;
#line 1885 "../src/Widgets/SourceList/SourceList.vala"
		_tmp9_ = column;
#line 1885 "../src/Widgets/SourceList/SourceList.vala"
		gtk_tree_view_get_cell_area ((GtkTreeView*) self, _tmp8_, _tmp9_, &_tmp10_);
#line 1885 "../src/Widgets/SourceList/SourceList.vala"
		start_cell_area = _tmp10_;
#line 1887 "../src/Widgets/SourceList/SourceList.vala"
		_tmp11_ = path;
#line 1887 "../src/Widgets/SourceList/SourceList.vala"
		gtk_tree_view_set_tooltip_row ((GtkTreeView*) self, tooltip, _tmp11_);
#line 1889 "../src/Widgets/SourceList/SourceList.vala"
		_tmp12_ = item;
#line 1889 "../src/Widgets/SourceList/SourceList.vala"
		_tmp13_ = code_widgets_source_list_item_get_tooltip (_tmp12_);
#line 1889 "../src/Widgets/SourceList/SourceList.vala"
		_tmp14_ = _tmp13_;
#line 1889 "../src/Widgets/SourceList/SourceList.vala"
		if (_tmp14_ == NULL) {
#line 9600 "SourceList.c"
			CodeWidgetsSourceListItem* _tmp15_;
			const gchar* _tmp16_;
			const gchar* _tmp17_;
#line 1890 "../src/Widgets/SourceList/SourceList.vala"
			_tmp15_ = item;
#line 1890 "../src/Widgets/SourceList/SourceList.vala"
			_tmp16_ = code_widgets_source_list_item_get_name (_tmp15_);
#line 1890 "../src/Widgets/SourceList/SourceList.vala"
			_tmp17_ = _tmp16_;
#line 1890 "../src/Widgets/SourceList/SourceList.vala"
			gtk_tooltip_set_markup (tooltip, _tmp17_);
#line 1891 "../src/Widgets/SourceList/SourceList.vala"
			should_show = TRUE;
#line 9614 "SourceList.c"
		} else {
			CodeWidgetsSourceListItem* _tmp18_;
			const gchar* _tmp19_;
			const gchar* _tmp20_;
#line 1892 "../src/Widgets/SourceList/SourceList.vala"
			_tmp18_ = item;
#line 1892 "../src/Widgets/SourceList/SourceList.vala"
			_tmp19_ = code_widgets_source_list_item_get_tooltip (_tmp18_);
#line 1892 "../src/Widgets/SourceList/SourceList.vala"
			_tmp20_ = _tmp19_;
#line 1892 "../src/Widgets/SourceList/SourceList.vala"
			if (g_strcmp0 (_tmp20_, "") != 0) {
#line 9627 "SourceList.c"
				CodeWidgetsSourceListItem* _tmp21_;
				const gchar* _tmp22_;
				const gchar* _tmp23_;
#line 1893 "../src/Widgets/SourceList/SourceList.vala"
				_tmp21_ = item;
#line 1893 "../src/Widgets/SourceList/SourceList.vala"
				_tmp22_ = code_widgets_source_list_item_get_tooltip (_tmp21_);
#line 1893 "../src/Widgets/SourceList/SourceList.vala"
				_tmp23_ = _tmp22_;
#line 1893 "../src/Widgets/SourceList/SourceList.vala"
				gtk_tooltip_set_markup (tooltip, _tmp23_);
#line 1894 "../src/Widgets/SourceList/SourceList.vala"
				should_show = TRUE;
#line 9641 "SourceList.c"
			}
		}
#line 1897 "../src/Widgets/SourceList/SourceList.vala"
		if (keyboard_tooltip) {
#line 1898 "../src/Widgets/SourceList/SourceList.vala"
			result = should_show;
#line 1898 "../src/Widgets/SourceList/SourceList.vala"
			_g_object_unref0 (item);
#line 1898 "../src/Widgets/SourceList/SourceList.vala"
			_g_object_unref0 (column);
#line 1898 "../src/Widgets/SourceList/SourceList.vala"
			__vala_GtkTreePath_free0 (path);
#line 1898 "../src/Widgets/SourceList/SourceList.vala"
			return result;
#line 9656 "SourceList.c"
		}
#line 1901 "../src/Widgets/SourceList/SourceList.vala"
		_tmp25_ = column;
#line 1901 "../src/Widgets/SourceList/SourceList.vala"
		_tmp26_ = path;
#line 1901 "../src/Widgets/SourceList/SourceList.vala"
		_tmp27_ = self->priv->text_cell;
#line 1901 "../src/Widgets/SourceList/SourceList.vala"
		_tmp28_ = start_cell_area;
#line 1901 "../src/Widgets/SourceList/SourceList.vala"
		if (code_widgets_source_list_tree_over_cell (self, _tmp25_, _tmp26_, (GtkCellRenderer*) _tmp27_, x - _tmp28_.x)) {
#line 1901 "../src/Widgets/SourceList/SourceList.vala"
			_tmp24_ = TRUE;
#line 9670 "SourceList.c"
		} else {
			GtkTreeViewColumn* _tmp29_;
			GtkTreePath* _tmp30_;
			CodeWidgetsSourceListCellRendererIcon* _tmp31_;
			GdkRectangle _tmp32_;
#line 1902 "../src/Widgets/SourceList/SourceList.vala"
			_tmp29_ = column;
#line 1902 "../src/Widgets/SourceList/SourceList.vala"
			_tmp30_ = path;
#line 1902 "../src/Widgets/SourceList/SourceList.vala"
			_tmp31_ = self->priv->icon_cell;
#line 1902 "../src/Widgets/SourceList/SourceList.vala"
			_tmp32_ = start_cell_area;
#line 1902 "../src/Widgets/SourceList/SourceList.vala"
			_tmp24_ = code_widgets_source_list_tree_over_cell (self, _tmp29_, _tmp30_, (GtkCellRenderer*) _tmp31_, x - _tmp32_.x);
#line 9686 "SourceList.c"
		}
#line 1901 "../src/Widgets/SourceList/SourceList.vala"
		if (_tmp24_) {
#line 1904 "../src/Widgets/SourceList/SourceList.vala"
			result = should_show;
#line 1904 "../src/Widgets/SourceList/SourceList.vala"
			_g_object_unref0 (item);
#line 1904 "../src/Widgets/SourceList/SourceList.vala"
			_g_object_unref0 (column);
#line 1904 "../src/Widgets/SourceList/SourceList.vala"
			__vala_GtkTreePath_free0 (path);
#line 1904 "../src/Widgets/SourceList/SourceList.vala"
			return result;
#line 9700 "SourceList.c"
		} else {
			GtkTreeViewColumn* _tmp33_;
			GtkTreePath* _tmp34_;
			CodeWidgetsSourceListCellRendererIcon* _tmp35_;
			GdkRectangle _tmp36_;
#line 1905 "../src/Widgets/SourceList/SourceList.vala"
			_tmp33_ = column;
#line 1905 "../src/Widgets/SourceList/SourceList.vala"
			_tmp34_ = path;
#line 1905 "../src/Widgets/SourceList/SourceList.vala"
			_tmp35_ = self->priv->activatable_cell;
#line 1905 "../src/Widgets/SourceList/SourceList.vala"
			_tmp36_ = start_cell_area;
#line 1905 "../src/Widgets/SourceList/SourceList.vala"
			if (code_widgets_source_list_tree_over_cell (self, _tmp33_, _tmp34_, (GtkCellRenderer*) _tmp35_, x - _tmp36_.x)) {
#line 9716 "SourceList.c"
				CodeWidgetsSourceListItem* _tmp37_;
				const gchar* _tmp38_;
				const gchar* _tmp39_;
#line 1906 "../src/Widgets/SourceList/SourceList.vala"
				_tmp37_ = item;
#line 1906 "../src/Widgets/SourceList/SourceList.vala"
				_tmp38_ = code_widgets_source_list_item_get_activatable_tooltip (_tmp37_);
#line 1906 "../src/Widgets/SourceList/SourceList.vala"
				_tmp39_ = _tmp38_;
#line 1906 "../src/Widgets/SourceList/SourceList.vala"
				if (g_strcmp0 (_tmp39_, "") == 0) {
#line 1907 "../src/Widgets/SourceList/SourceList.vala"
					result = FALSE;
#line 1907 "../src/Widgets/SourceList/SourceList.vala"
					_g_object_unref0 (item);
#line 1907 "../src/Widgets/SourceList/SourceList.vala"
					_g_object_unref0 (column);
#line 1907 "../src/Widgets/SourceList/SourceList.vala"
					__vala_GtkTreePath_free0 (path);
#line 1907 "../src/Widgets/SourceList/SourceList.vala"
					return result;
#line 9738 "SourceList.c"
				} else {
					CodeWidgetsSourceListItem* _tmp40_;
					const gchar* _tmp41_;
					const gchar* _tmp42_;
#line 1909 "../src/Widgets/SourceList/SourceList.vala"
					_tmp40_ = item;
#line 1909 "../src/Widgets/SourceList/SourceList.vala"
					_tmp41_ = code_widgets_source_list_item_get_activatable_tooltip (_tmp40_);
#line 1909 "../src/Widgets/SourceList/SourceList.vala"
					_tmp42_ = _tmp41_;
#line 1909 "../src/Widgets/SourceList/SourceList.vala"
					gtk_tooltip_set_markup (tooltip, _tmp42_);
#line 1910 "../src/Widgets/SourceList/SourceList.vala"
					result = TRUE;
#line 1910 "../src/Widgets/SourceList/SourceList.vala"
					_g_object_unref0 (item);
#line 1910 "../src/Widgets/SourceList/SourceList.vala"
					_g_object_unref0 (column);
#line 1910 "../src/Widgets/SourceList/SourceList.vala"
					__vala_GtkTreePath_free0 (path);
#line 1910 "../src/Widgets/SourceList/SourceList.vala"
					return result;
#line 9761 "SourceList.c"
				}
			}
		}
	}
#line 1915 "../src/Widgets/SourceList/SourceList.vala"
	result = FALSE;
#line 1915 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (item);
#line 1915 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (column);
#line 1915 "../src/Widgets/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (path);
#line 1915 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 9776 "SourceList.c"
}

static void
_vala_array_add4 (GtkTargetEntry* * array,
                  gint* length,
                  gint* size,
                  const GtkTargetEntry* value)
{
#line 1926 "../src/Widgets/SourceList/SourceList.vala"
	if ((*length) == (*size)) {
#line 1926 "../src/Widgets/SourceList/SourceList.vala"
		*size = (*size) ? (2 * (*size)) : 4;
#line 1926 "../src/Widgets/SourceList/SourceList.vala"
		*array = g_renew (GtkTargetEntry, *array, *size);
#line 9791 "SourceList.c"
	}
#line 1926 "../src/Widgets/SourceList/SourceList.vala"
	(*array)[(*length)++] = *value;
#line 9795 "SourceList.c"
}

static void
_vala_array_add5 (GtkTargetEntry* * array,
                  gint* length,
                  gint* size,
                  const GtkTargetEntry* value)
{
#line 1930 "../src/Widgets/SourceList/SourceList.vala"
	if ((*length) == (*size)) {
#line 1930 "../src/Widgets/SourceList/SourceList.vala"
		*size = (*size) ? (2 * (*size)) : 4;
#line 1930 "../src/Widgets/SourceList/SourceList.vala"
		*array = g_renew (GtkTargetEntry, *array, *size);
#line 9810 "SourceList.c"
	}
#line 1930 "../src/Widgets/SourceList/SourceList.vala"
	(*array)[(*length)++] = *value;
#line 9814 "SourceList.c"
}

static GtkTargetEntry*
code_widgets_source_list_tree_append_row_target_entry (GtkTargetEntry* orig,
                                                       gint orig_length1,
                                                       gint* result_length1)
{
#line 1919 "../src/Widgets/SourceList/SourceList.vala"
	static const GtkTargetEntry row_target_entry = {"GTK_TREE_MODEL_ROW", (guint) GTK_TARGET_SAME_WIDGET, (guint) 0};
#line 9824 "SourceList.c"
	GtkTargetEntry* entries = NULL;
	GtkTargetEntry* _tmp0_;
	gint entries_length1;
	gint _entries_size_;
	GtkTargetEntry* _tmp2_;
	gint _tmp2__length1;
	GtkTargetEntry* result;
#line 1925 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = g_new0 (GtkTargetEntry, 0);
#line 1925 "../src/Widgets/SourceList/SourceList.vala"
	entries = _tmp0_;
#line 1925 "../src/Widgets/SourceList/SourceList.vala"
	entries_length1 = 0;
#line 1925 "../src/Widgets/SourceList/SourceList.vala"
	_entries_size_ = entries_length1;
#line 1926 "../src/Widgets/SourceList/SourceList.vala"
	_vala_array_add4 (&entries, &entries_length1, &_entries_size_, &row_target_entry);
#line 1928 "../src/Widgets/SourceList/SourceList.vala"
	if (orig != NULL) {
#line 9844 "SourceList.c"
		{
			GtkTargetEntry* target_entry_collection = NULL;
			gint target_entry_collection_length1 = 0;
			gint _target_entry_collection_size_ = 0;
			gint target_entry_it = 0;
#line 1929 "../src/Widgets/SourceList/SourceList.vala"
			target_entry_collection = orig;
#line 1929 "../src/Widgets/SourceList/SourceList.vala"
			target_entry_collection_length1 = orig_length1;
#line 1929 "../src/Widgets/SourceList/SourceList.vala"
			for (target_entry_it = 0; target_entry_it < target_entry_collection_length1; target_entry_it = target_entry_it + 1) {
#line 9856 "SourceList.c"
				GtkTargetEntry target_entry = {0};
#line 1929 "../src/Widgets/SourceList/SourceList.vala"
				target_entry = target_entry_collection[target_entry_it];
#line 9860 "SourceList.c"
				{
					GtkTargetEntry _tmp1_;
#line 1930 "../src/Widgets/SourceList/SourceList.vala"
					_tmp1_ = target_entry;
#line 1930 "../src/Widgets/SourceList/SourceList.vala"
					_vala_array_add5 (&entries, &entries_length1, &_entries_size_, &_tmp1_);
#line 9867 "SourceList.c"
				}
			}
		}
	}
#line 1933 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = entries;
#line 1933 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2__length1 = entries_length1;
#line 1933 "../src/Widgets/SourceList/SourceList.vala"
	if (result_length1) {
#line 1933 "../src/Widgets/SourceList/SourceList.vala"
		*result_length1 = _tmp2__length1;
#line 9880 "SourceList.c"
	}
#line 1933 "../src/Widgets/SourceList/SourceList.vala"
	result = _tmp2_;
#line 1933 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 9886 "SourceList.c"
}

static void
_code_widgets_source_list_tree_on_model_item_updated_code_widgets_source_list_data_model_item_updated (CodeWidgetsSourceListDataModel* _sender,
                                                                                                       CodeWidgetsSourceListItem* item,
                                                                                                       gpointer self)
{
#line 1937 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_tree_on_model_item_updated ((CodeWidgetsSourceListTree*) self, item);
#line 9896 "SourceList.c"
}

static void
code_widgets_source_list_tree_enable_item_property_monitor (CodeWidgetsSourceListTree* self)
{
	CodeWidgetsSourceListDataModel* _tmp0_;
#line 1936 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 1937 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->_data_model;
#line 1937 "../src/Widgets/SourceList/SourceList.vala"
	g_signal_connect_object (_tmp0_, "item-updated", (GCallback) _code_widgets_source_list_tree_on_model_item_updated_code_widgets_source_list_data_model_item_updated, self, G_CONNECT_AFTER);
#line 9909 "SourceList.c"
}

static void
code_widgets_source_list_tree_disable_item_property_monitor (CodeWidgetsSourceListTree* self)
{
	CodeWidgetsSourceListDataModel* _tmp0_;
	guint _tmp1_;
#line 1940 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 1941 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->_data_model;
#line 1941 "../src/Widgets/SourceList/SourceList.vala"
	g_signal_parse_name ("item-updated", CODE_WIDGETS_SOURCE_LIST_TYPE_DATA_MODEL, &_tmp1_, NULL, FALSE);
#line 1941 "../src/Widgets/SourceList/SourceList.vala"
	g_signal_handlers_disconnect_matched (_tmp0_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp1_, 0, NULL, (GCallback) _code_widgets_source_list_tree_on_model_item_updated_code_widgets_source_list_data_model_item_updated, self);
#line 9925 "SourceList.c"
}

static void
code_widgets_source_list_tree_on_model_item_updated (CodeWidgetsSourceListTree* self,
                                                     CodeWidgetsSourceListItem* item)
{
	CodeWidgetsSourceListExpandableItem* expandable_item = NULL;
	CodeWidgetsSourceListExpandableItem* _tmp0_;
	CodeWidgetsSourceListExpandableItem* _tmp1_;
#line 1944 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 1944 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (item != NULL);
#line 1947 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = _g_object_ref0 (CODE_WIDGETS_SOURCE_LIST_IS_EXPANDABLE_ITEM (item) ? ((CodeWidgetsSourceListExpandableItem*) item) : NULL);
#line 1947 "../src/Widgets/SourceList/SourceList.vala"
	expandable_item = _tmp0_;
#line 1948 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = expandable_item;
#line 1948 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp1_ != NULL) {
#line 9947 "SourceList.c"
		CodeWidgetsSourceListExpandableItem* _tmp2_;
#line 1949 "../src/Widgets/SourceList/SourceList.vala"
		_tmp2_ = expandable_item;
#line 1949 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_tree_update_expansion (self, _tmp2_);
#line 9953 "SourceList.c"
	}
#line 1944 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (expandable_item);
#line 9957 "SourceList.c"
}

static void
_code_widgets_source_list_tree_spacer_cell_data_func_gtk_cell_layout_data_func (GtkCellLayout* cell_layout,
                                                                                GtkCellRenderer* cell,
                                                                                GtkTreeModel* tree_model,
                                                                                GtkTreeIter* iter,
                                                                                gpointer self)
{
#line 1994 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_tree_spacer_cell_data_func (cell_layout, cell, tree_model, iter);
#line 9969 "SourceList.c"
}

static void
code_widgets_source_list_tree_add_spacer_cell_for_level (CodeWidgetsSourceListTree* self,
                                                         gint level,
                                                         gboolean check_previous)
{
	GeeHashMap* _tmp0_;
	GeeHashMap* _tmp2_;
#line 1952 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 1952 "../src/Widgets/SourceList/SourceList.vala"
	_vala_return_if_fail (level > 0, "level > 0");
#line 1956 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->spacer_cells;
#line 1956 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp0_ == NULL) {
#line 9987 "SourceList.c"
		GeeHashMap* _tmp1_;
#line 1957 "../src/Widgets/SourceList/SourceList.vala"
		_tmp1_ = gee_hash_map_new (G_TYPE_INT, NULL, NULL, CODE_WIDGETS_SOURCE_LIST_TYPE_CELL_RENDERER_SPACER, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
#line 1957 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (self->priv->spacer_cells);
#line 1957 "../src/Widgets/SourceList/SourceList.vala"
		self->priv->spacer_cells = _tmp1_;
#line 9995 "SourceList.c"
	}
#line 1959 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = self->priv->spacer_cells;
#line 1959 "../src/Widgets/SourceList/SourceList.vala"
	if (!gee_abstract_map_has_key ((GeeAbstractMap*) _tmp2_, (gpointer) ((gintptr) level))) {
#line 10001 "SourceList.c"
		CodeWidgetsSourceListCellRendererSpacer* spacer_cell = NULL;
		CodeWidgetsSourceListCellRendererSpacer* _tmp3_;
		CodeWidgetsSourceListCellRendererSpacer* _tmp4_;
		GeeHashMap* _tmp5_;
		CodeWidgetsSourceListCellRendererSpacer* _tmp6_;
		guint cell_xpadding = 0U;
		CodeWidgetsSourceListCellRendererSpacer* _tmp9_;
		GtkTreeViewColumn* item_column = NULL;
		GtkTreeViewColumn* _tmp10_;
		GtkTreeViewColumn* _tmp11_;
		GtkTreeViewColumn* _tmp12_;
		CodeWidgetsSourceListCellRendererSpacer* _tmp13_;
		GtkTreeViewColumn* _tmp14_;
		CodeWidgetsSourceListCellRendererSpacer* _tmp15_;
#line 1960 "../src/Widgets/SourceList/SourceList.vala"
		_tmp3_ = code_widgets_source_list_cell_renderer_spacer_new ();
#line 1960 "../src/Widgets/SourceList/SourceList.vala"
		g_object_ref_sink (_tmp3_);
#line 1960 "../src/Widgets/SourceList/SourceList.vala"
		spacer_cell = _tmp3_;
#line 1961 "../src/Widgets/SourceList/SourceList.vala"
		_tmp4_ = spacer_cell;
#line 1961 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_cell_renderer_spacer_set_level (_tmp4_, level);
#line 1962 "../src/Widgets/SourceList/SourceList.vala"
		_tmp5_ = self->priv->spacer_cells;
#line 1962 "../src/Widgets/SourceList/SourceList.vala"
		_tmp6_ = spacer_cell;
#line 1962 "../src/Widgets/SourceList/SourceList.vala"
		gee_abstract_map_set ((GeeAbstractMap*) _tmp5_, (gpointer) ((gintptr) level), _tmp6_);
#line 1972 "../src/Widgets/SourceList/SourceList.vala"
		switch (level) {
#line 1972 "../src/Widgets/SourceList/SourceList.vala"
			case 1:
#line 10036 "SourceList.c"
			{
				gint left_padding = 0;
				gint _tmp7_ = 0;
#line 1975 "../src/Widgets/SourceList/SourceList.vala"
				gtk_widget_style_get ((GtkWidget*) self, CODE_WIDGETS_SOURCE_LIST_TREE_STYLE_PROP_LEFT_PADDING, &_tmp7_, NULL);
#line 1975 "../src/Widgets/SourceList/SourceList.vala"
				left_padding = _tmp7_;
#line 1976 "../src/Widgets/SourceList/SourceList.vala"
				cell_xpadding = (guint) (left_padding / 2);
#line 1977 "../src/Widgets/SourceList/SourceList.vala"
				break;
#line 10048 "SourceList.c"
			}
#line 1972 "../src/Widgets/SourceList/SourceList.vala"
			case 2:
#line 10052 "SourceList.c"
			{
#line 1980 "../src/Widgets/SourceList/SourceList.vala"
				cell_xpadding = (guint) 0;
#line 1981 "../src/Widgets/SourceList/SourceList.vala"
				break;
#line 10058 "SourceList.c"
			}
			default:
			{
				gint level_indentation = 0;
				gint _tmp8_ = 0;
#line 1985 "../src/Widgets/SourceList/SourceList.vala"
				gtk_widget_style_get ((GtkWidget*) self, CODE_WIDGETS_SOURCE_LIST_TREE_STYLE_PROP_LEVEL_INDENTATION, &_tmp8_, NULL);
#line 1985 "../src/Widgets/SourceList/SourceList.vala"
				level_indentation = _tmp8_;
#line 1986 "../src/Widgets/SourceList/SourceList.vala"
				cell_xpadding = (guint) (level_indentation / 2);
#line 1987 "../src/Widgets/SourceList/SourceList.vala"
				break;
#line 10072 "SourceList.c"
			}
		}
#line 1990 "../src/Widgets/SourceList/SourceList.vala"
		_tmp9_ = spacer_cell;
#line 1990 "../src/Widgets/SourceList/SourceList.vala"
		g_object_set ((GtkCellRenderer*) _tmp9_, "xpad", cell_xpadding, NULL);
#line 1992 "../src/Widgets/SourceList/SourceList.vala"
		_tmp10_ = gtk_tree_view_get_column ((GtkTreeView*) self, (gint) CODE_WIDGETS_SOURCE_LIST_TREE_COLUMN_ITEM);
#line 1992 "../src/Widgets/SourceList/SourceList.vala"
		_tmp11_ = _g_object_ref0 (_tmp10_);
#line 1992 "../src/Widgets/SourceList/SourceList.vala"
		item_column = _tmp11_;
#line 1993 "../src/Widgets/SourceList/SourceList.vala"
		_tmp12_ = item_column;
#line 1993 "../src/Widgets/SourceList/SourceList.vala"
		_tmp13_ = spacer_cell;
#line 1993 "../src/Widgets/SourceList/SourceList.vala"
		gtk_cell_layout_pack_start ((GtkCellLayout*) _tmp12_, (GtkCellRenderer*) _tmp13_, FALSE);
#line 1994 "../src/Widgets/SourceList/SourceList.vala"
		_tmp14_ = item_column;
#line 1994 "../src/Widgets/SourceList/SourceList.vala"
		_tmp15_ = spacer_cell;
#line 1994 "../src/Widgets/SourceList/SourceList.vala"
		gtk_cell_layout_set_cell_data_func ((GtkCellLayout*) _tmp14_, (GtkCellRenderer*) _tmp15_, _code_widgets_source_list_tree_spacer_cell_data_func_gtk_cell_layout_data_func, NULL, NULL);
#line 1997 "../src/Widgets/SourceList/SourceList.vala"
		if (check_previous) {
#line 10099 "SourceList.c"
			{
				gint i = 0;
#line 1998 "../src/Widgets/SourceList/SourceList.vala"
				i = level - 1;
#line 10104 "SourceList.c"
				{
					gboolean _tmp16_ = FALSE;
#line 1998 "../src/Widgets/SourceList/SourceList.vala"
					_tmp16_ = TRUE;
#line 1998 "../src/Widgets/SourceList/SourceList.vala"
					while (TRUE) {
#line 1998 "../src/Widgets/SourceList/SourceList.vala"
						if (!_tmp16_) {
#line 10113 "SourceList.c"
							gint _tmp17_;
#line 1998 "../src/Widgets/SourceList/SourceList.vala"
							_tmp17_ = i;
#line 1998 "../src/Widgets/SourceList/SourceList.vala"
							i = _tmp17_ - 1;
#line 10119 "SourceList.c"
						}
#line 1998 "../src/Widgets/SourceList/SourceList.vala"
						_tmp16_ = FALSE;
#line 1998 "../src/Widgets/SourceList/SourceList.vala"
						if (!(i > 0)) {
#line 1998 "../src/Widgets/SourceList/SourceList.vala"
							break;
#line 10127 "SourceList.c"
						}
#line 1999 "../src/Widgets/SourceList/SourceList.vala"
						code_widgets_source_list_tree_add_spacer_cell_for_level (self, i, FALSE);
#line 10131 "SourceList.c"
					}
				}
			}
		}
#line 1959 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (item_column);
#line 1959 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (spacer_cell);
#line 10140 "SourceList.c"
	}
}

/**
         * Evaluates whether the item at the specified path can be selected or not.
         */
static gboolean
code_widgets_source_list_tree_select_func (CodeWidgetsSourceListTree* self,
                                           GtkTreeSelection* selection,
                                           GtkTreeModel* model,
                                           GtkTreePath* path,
                                           gboolean path_currently_selected)
{
	gboolean selectable = FALSE;
	CodeWidgetsSourceListItem* item = NULL;
	CodeWidgetsSourceListDataModel* _tmp0_;
	CodeWidgetsSourceListItem* _tmp1_;
	CodeWidgetsSourceListItem* _tmp2_;
	gboolean result;
#line 2007 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 2007 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (selection != NULL, FALSE);
#line 2007 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (model != NULL, FALSE);
#line 2007 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (path != NULL, FALSE);
#line 2013 "../src/Widgets/SourceList/SourceList.vala"
	selectable = FALSE;
#line 2014 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->_data_model;
#line 2014 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = code_widgets_source_list_data_model_get_item_from_path (_tmp0_, path);
#line 2014 "../src/Widgets/SourceList/SourceList.vala"
	item = _tmp1_;
#line 2016 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = item;
#line 2016 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp2_ != NULL) {
#line 10180 "SourceList.c"
		CodeWidgetsSourceListDataModel* _tmp3_;
		CodeWidgetsSourceListItem* _tmp4_;
#line 2018 "../src/Widgets/SourceList/SourceList.vala"
		_tmp3_ = self->priv->_data_model;
#line 2018 "../src/Widgets/SourceList/SourceList.vala"
		_tmp4_ = item;
#line 2018 "../src/Widgets/SourceList/SourceList.vala"
		if (!code_widgets_source_list_data_model_is_category (_tmp3_, _tmp4_, NULL, path)) {
#line 10189 "SourceList.c"
			CodeWidgetsSourceListItem* _tmp5_;
			gboolean _tmp6_;
			gboolean _tmp7_;
#line 2019 "../src/Widgets/SourceList/SourceList.vala"
			_tmp5_ = item;
#line 2019 "../src/Widgets/SourceList/SourceList.vala"
			_tmp6_ = code_widgets_source_list_item_get_selectable (_tmp5_);
#line 2019 "../src/Widgets/SourceList/SourceList.vala"
			_tmp7_ = _tmp6_;
#line 2019 "../src/Widgets/SourceList/SourceList.vala"
			selectable = _tmp7_;
#line 10201 "SourceList.c"
		}
	}
#line 2022 "../src/Widgets/SourceList/SourceList.vala"
	result = selectable;
#line 2022 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (item);
#line 2022 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 10210 "SourceList.c"
}

static void
__vala_GtkTreePath_free0_ (gpointer var)
{
#line 2029 "../src/Widgets/SourceList/SourceList.vala"
	(var == NULL) ? NULL : (var = (_vala_GtkTreePath_free (var), NULL));
#line 10218 "SourceList.c"
}

static inline void
_g_list_free___vala_GtkTreePath_free0_ (GList* self)
{
#line 2029 "../src/Widgets/SourceList/SourceList.vala"
	g_list_free_full (self, (GDestroyNotify) __vala_GtkTreePath_free0_);
#line 10226 "SourceList.c"
}

static GtkTreePath*
code_widgets_source_list_tree_get_selected_path (CodeWidgetsSourceListTree* self)
{
	GtkTreePath* selected_path = NULL;
	GtkTreeSelection* selection = NULL;
	GtkTreeSelection* _tmp0_;
	GtkTreeSelection* _tmp1_;
	GtkTreeSelection* _tmp2_;
	GtkTreePath* result;
#line 2025 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 2026 "../src/Widgets/SourceList/SourceList.vala"
	selected_path = NULL;
#line 2027 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = gtk_tree_view_get_selection ((GtkTreeView*) self);
#line 2027 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 2027 "../src/Widgets/SourceList/SourceList.vala"
	selection = _tmp1_;
#line 2029 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = selection;
#line 2029 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp2_ != NULL) {
#line 10252 "SourceList.c"
		GtkTreeModel* model = NULL;
		GList* selected_rows = NULL;
		GtkTreeSelection* _tmp3_;
		GtkTreeModel* _tmp4_ = NULL;
		GList* _tmp5_;
		GtkTreeModel* _tmp6_;
		GList* _tmp7_;
#line 2031 "../src/Widgets/SourceList/SourceList.vala"
		_tmp3_ = selection;
#line 2031 "../src/Widgets/SourceList/SourceList.vala"
		_tmp5_ = gtk_tree_selection_get_selected_rows (_tmp3_, &_tmp4_);
#line 2031 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (model);
#line 2031 "../src/Widgets/SourceList/SourceList.vala"
		_tmp6_ = _g_object_ref0 (_tmp4_);
#line 2031 "../src/Widgets/SourceList/SourceList.vala"
		model = _tmp6_;
#line 2031 "../src/Widgets/SourceList/SourceList.vala"
		selected_rows = _tmp5_;
#line 2032 "../src/Widgets/SourceList/SourceList.vala"
		_tmp7_ = selected_rows;
#line 2032 "../src/Widgets/SourceList/SourceList.vala"
		if (g_list_length (_tmp7_) == ((guint) 1)) {
#line 10276 "SourceList.c"
			GList* _tmp8_;
			gconstpointer _tmp9_;
			GtkTreePath* _tmp10_;
#line 2033 "../src/Widgets/SourceList/SourceList.vala"
			_tmp8_ = selected_rows;
#line 2033 "../src/Widgets/SourceList/SourceList.vala"
			_tmp9_ = g_list_nth_data (_tmp8_, (guint) 0);
#line 2033 "../src/Widgets/SourceList/SourceList.vala"
			_tmp10_ = __vala_GtkTreePath_copy0 ((GtkTreePath*) _tmp9_);
#line 2033 "../src/Widgets/SourceList/SourceList.vala"
			__vala_GtkTreePath_free0 (selected_path);
#line 2033 "../src/Widgets/SourceList/SourceList.vala"
			selected_path = _tmp10_;
#line 10290 "SourceList.c"
		}
#line 2029 "../src/Widgets/SourceList/SourceList.vala"
		(selected_rows == NULL) ? NULL : (selected_rows = (_g_list_free___vala_GtkTreePath_free0_ (selected_rows), NULL));
#line 2029 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (model);
#line 10296 "SourceList.c"
	}
#line 2036 "../src/Widgets/SourceList/SourceList.vala"
	result = selected_path;
#line 2036 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (selection);
#line 2036 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 10304 "SourceList.c"
}

static void
code_widgets_source_list_tree_set_selected (CodeWidgetsSourceListTree* self,
                                            CodeWidgetsSourceListItem* item,
                                            gboolean scroll_to_item)
{
#line 2039 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 2040 "../src/Widgets/SourceList/SourceList.vala"
	if (item == NULL) {
#line 10316 "SourceList.c"
		GtkTreeSelection* selection = NULL;
		GtkTreeSelection* _tmp0_;
		GtkTreeSelection* _tmp1_;
		GtkTreeSelection* _tmp2_;
#line 2041 "../src/Widgets/SourceList/SourceList.vala"
		_tmp0_ = gtk_tree_view_get_selection ((GtkTreeView*) self);
#line 2041 "../src/Widgets/SourceList/SourceList.vala"
		_tmp1_ = _g_object_ref0 (_tmp0_);
#line 2041 "../src/Widgets/SourceList/SourceList.vala"
		selection = _tmp1_;
#line 2042 "../src/Widgets/SourceList/SourceList.vala"
		_tmp2_ = selection;
#line 2042 "../src/Widgets/SourceList/SourceList.vala"
		if (_tmp2_ != NULL) {
#line 10331 "SourceList.c"
			GtkTreeSelection* _tmp3_;
#line 2043 "../src/Widgets/SourceList/SourceList.vala"
			_tmp3_ = selection;
#line 2043 "../src/Widgets/SourceList/SourceList.vala"
			gtk_tree_selection_unselect_all (_tmp3_);
#line 10337 "SourceList.c"
		}
#line 2054 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (self->priv->selected);
#line 2054 "../src/Widgets/SourceList/SourceList.vala"
		self->priv->selected = NULL;
#line 2055 "../src/Widgets/SourceList/SourceList.vala"
		g_signal_emit (self, code_widgets_source_list_tree_signals[CODE_WIDGETS_SOURCE_LIST_TREE_ITEM_SELECTED_SIGNAL], 0, NULL);
#line 2040 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (selection);
#line 10347 "SourceList.c"
	} else {
		gboolean _tmp4_;
		gboolean _tmp5_;
#line 2056 "../src/Widgets/SourceList/SourceList.vala"
		_tmp4_ = code_widgets_source_list_item_get_selectable (item);
#line 2056 "../src/Widgets/SourceList/SourceList.vala"
		_tmp5_ = _tmp4_;
#line 2056 "../src/Widgets/SourceList/SourceList.vala"
		if (_tmp5_) {
#line 10357 "SourceList.c"
			GtkTreePath* to_select = NULL;
			CodeWidgetsSourceListDataModel* _tmp6_;
			GtkTreePath* _tmp7_;
			GtkTreePath* _tmp8_;
#line 2057 "../src/Widgets/SourceList/SourceList.vala"
			if (scroll_to_item) {
#line 2058 "../src/Widgets/SourceList/SourceList.vala"
				code_widgets_source_list_tree_scroll_to_item (self, item, FALSE, (gfloat) 0);
#line 10366 "SourceList.c"
			}
#line 2060 "../src/Widgets/SourceList/SourceList.vala"
			_tmp6_ = self->priv->_data_model;
#line 2060 "../src/Widgets/SourceList/SourceList.vala"
			_tmp7_ = code_widgets_source_list_data_model_get_item_path (_tmp6_, item);
#line 2060 "../src/Widgets/SourceList/SourceList.vala"
			to_select = _tmp7_;
#line 2061 "../src/Widgets/SourceList/SourceList.vala"
			_tmp8_ = to_select;
#line 2061 "../src/Widgets/SourceList/SourceList.vala"
			if (_tmp8_ != NULL) {
#line 10378 "SourceList.c"
				GtkTreePath* _tmp9_;
				GtkTreeViewColumn* _tmp10_;
				GtkCellRendererText* _tmp11_;
#line 2062 "../src/Widgets/SourceList/SourceList.vala"
				_tmp9_ = to_select;
#line 2062 "../src/Widgets/SourceList/SourceList.vala"
				_tmp10_ = gtk_tree_view_get_column ((GtkTreeView*) self, (gint) CODE_WIDGETS_SOURCE_LIST_TREE_COLUMN_ITEM);
#line 2062 "../src/Widgets/SourceList/SourceList.vala"
				_tmp11_ = self->priv->text_cell;
#line 2062 "../src/Widgets/SourceList/SourceList.vala"
				gtk_tree_view_set_cursor_on_cell ((GtkTreeView*) self, _tmp9_, _tmp10_, (GtkCellRenderer*) _tmp11_, FALSE);
#line 10390 "SourceList.c"
			}
#line 2056 "../src/Widgets/SourceList/SourceList.vala"
			__vala_GtkTreePath_free0 (to_select);
#line 10394 "SourceList.c"
		}
	}
}

static void
code_widgets_source_list_tree_real_cursor_changed (GtkTreeView* base)
{
	CodeWidgetsSourceListTree * self;
	GtkTreePath* path = NULL;
	GtkTreePath* _tmp0_;
	CodeWidgetsSourceListItem* _tmp1_ = NULL;
	GtkTreePath* _tmp2_;
	CodeWidgetsSourceListItem* new_item = NULL;
	CodeWidgetsSourceListItem* _tmp6_;
	gboolean _tmp7_ = FALSE;
	CodeWidgetsSourceListItem* _tmp8_;
#line 2066 "../src/Widgets/SourceList/SourceList.vala"
	self = (CodeWidgetsSourceListTree*) base;
#line 2067 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = code_widgets_source_list_tree_get_selected_path (self);
#line 2067 "../src/Widgets/SourceList/SourceList.vala"
	path = _tmp0_;
#line 2068 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = path;
#line 2068 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp2_ != NULL) {
#line 10421 "SourceList.c"
		CodeWidgetsSourceListDataModel* _tmp3_;
		GtkTreePath* _tmp4_;
		CodeWidgetsSourceListItem* _tmp5_;
#line 2068 "../src/Widgets/SourceList/SourceList.vala"
		_tmp3_ = self->priv->_data_model;
#line 2068 "../src/Widgets/SourceList/SourceList.vala"
		_tmp4_ = path;
#line 2068 "../src/Widgets/SourceList/SourceList.vala"
		_tmp5_ = code_widgets_source_list_data_model_get_item_from_path (_tmp3_, _tmp4_);
#line 2068 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (_tmp1_);
#line 2068 "../src/Widgets/SourceList/SourceList.vala"
		_tmp1_ = _tmp5_;
#line 10435 "SourceList.c"
	} else {
#line 2068 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (_tmp1_);
#line 2068 "../src/Widgets/SourceList/SourceList.vala"
		_tmp1_ = NULL;
#line 10441 "SourceList.c"
	}
#line 2068 "../src/Widgets/SourceList/SourceList.vala"
	_tmp6_ = _g_object_ref0 (_tmp1_);
#line 2068 "../src/Widgets/SourceList/SourceList.vala"
	new_item = _tmp6_;
#line 2081 "../src/Widgets/SourceList/SourceList.vala"
	_tmp8_ = new_item;
#line 2081 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp8_ != NULL) {
#line 10451 "SourceList.c"
		CodeWidgetsSourceListItem* _tmp9_;
		CodeWidgetsSourceListItem* _tmp10_;
#line 2081 "../src/Widgets/SourceList/SourceList.vala"
		_tmp9_ = new_item;
#line 2081 "../src/Widgets/SourceList/SourceList.vala"
		_tmp10_ = self->priv->selected;
#line 2081 "../src/Widgets/SourceList/SourceList.vala"
		_tmp7_ = _tmp9_ != _tmp10_;
#line 10460 "SourceList.c"
	} else {
#line 2081 "../src/Widgets/SourceList/SourceList.vala"
		_tmp7_ = FALSE;
#line 10464 "SourceList.c"
	}
#line 2081 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp7_) {
#line 10468 "SourceList.c"
		CodeWidgetsSourceListItem* _tmp11_;
		CodeWidgetsSourceListItem* _tmp12_;
		CodeWidgetsSourceListItem* _tmp13_;
#line 2082 "../src/Widgets/SourceList/SourceList.vala"
		_tmp11_ = new_item;
#line 2082 "../src/Widgets/SourceList/SourceList.vala"
		_tmp12_ = _g_object_ref0 (_tmp11_);
#line 2082 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (self->priv->selected);
#line 2082 "../src/Widgets/SourceList/SourceList.vala"
		self->priv->selected = _tmp12_;
#line 2083 "../src/Widgets/SourceList/SourceList.vala"
		_tmp13_ = new_item;
#line 2083 "../src/Widgets/SourceList/SourceList.vala"
		g_signal_emit (self, code_widgets_source_list_tree_signals[CODE_WIDGETS_SOURCE_LIST_TREE_ITEM_SELECTED_SIGNAL], 0, _tmp13_);
#line 10484 "SourceList.c"
	}
#line 2066 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (new_item);
#line 2066 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (_tmp1_);
#line 2066 "../src/Widgets/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (path);
#line 10492 "SourceList.c"
}

static gboolean
code_widgets_source_list_tree_scroll_to_item (CodeWidgetsSourceListTree* self,
                                              CodeWidgetsSourceListItem* item,
                                              gboolean use_align,
                                              gfloat row_align)
{
	gboolean scrolled = FALSE;
	GtkTreePath* path = NULL;
	CodeWidgetsSourceListDataModel* _tmp0_;
	GtkTreePath* _tmp1_;
	GtkTreePath* _tmp2_;
	gboolean result;
#line 2087 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 2087 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (item != NULL, FALSE);
#line 2088 "../src/Widgets/SourceList/SourceList.vala"
	scrolled = FALSE;
#line 2090 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->_data_model;
#line 2090 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = code_widgets_source_list_data_model_get_item_path (_tmp0_, item);
#line 2090 "../src/Widgets/SourceList/SourceList.vala"
	path = _tmp1_;
#line 2091 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = path;
#line 2091 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp2_ != NULL) {
#line 10523 "SourceList.c"
		GtkTreePath* _tmp3_;
#line 2092 "../src/Widgets/SourceList/SourceList.vala"
		_tmp3_ = path;
#line 2092 "../src/Widgets/SourceList/SourceList.vala"
		gtk_tree_view_scroll_to_cell ((GtkTreeView*) self, _tmp3_, NULL, use_align, row_align, (gfloat) 0);
#line 2093 "../src/Widgets/SourceList/SourceList.vala"
		scrolled = TRUE;
#line 10531 "SourceList.c"
	}
#line 2096 "../src/Widgets/SourceList/SourceList.vala"
	result = scrolled;
#line 2096 "../src/Widgets/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (path);
#line 2096 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 10539 "SourceList.c"
}

static gboolean
code_widgets_source_list_tree_start_editing_item (CodeWidgetsSourceListTree* self,
                                                  CodeWidgetsSourceListItem* item)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	gboolean _tmp2_;
	gboolean _tmp3_;
	gboolean _tmp4_ = FALSE;
	gboolean _tmp5_;
	gboolean _tmp6_;
	GtkTreePath* path = NULL;
	CodeWidgetsSourceListDataModel* _tmp8_;
	GtkTreePath* _tmp9_;
	GtkTreePath* _tmp10_;
	gboolean _tmp17_;
	gboolean _tmp18_;
	gboolean result;
#line 2099 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 2099 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (item != NULL, FALSE);
#line 2099 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = code_widgets_source_list_item_get_editable (item);
#line 2099 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = _tmp0_;
#line 2099 "../src/Widgets/SourceList/SourceList.vala"
	_vala_return_val_if_fail (_tmp1_, "item.editable", FALSE);
#line 2099 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = code_widgets_source_list_item_get_selectable (item);
#line 2099 "../src/Widgets/SourceList/SourceList.vala"
	_tmp3_ = _tmp2_;
#line 2099 "../src/Widgets/SourceList/SourceList.vala"
	_vala_return_val_if_fail (_tmp3_, "item.selectable", FALSE);
#line 2100 "../src/Widgets/SourceList/SourceList.vala"
	_tmp5_ = code_widgets_source_list_tree_get_editing (self);
#line 2100 "../src/Widgets/SourceList/SourceList.vala"
	_tmp6_ = _tmp5_;
#line 2100 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp6_) {
#line 10582 "SourceList.c"
		CodeWidgetsSourceListItem* _tmp7_;
#line 2100 "../src/Widgets/SourceList/SourceList.vala"
		_tmp7_ = self->priv->edited;
#line 2100 "../src/Widgets/SourceList/SourceList.vala"
		_tmp4_ = item == _tmp7_;
#line 10588 "SourceList.c"
	} else {
#line 2100 "../src/Widgets/SourceList/SourceList.vala"
		_tmp4_ = FALSE;
#line 10592 "SourceList.c"
	}
#line 2100 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp4_) {
#line 2101 "../src/Widgets/SourceList/SourceList.vala"
		result = FALSE;
#line 2101 "../src/Widgets/SourceList/SourceList.vala"
		return result;
#line 10600 "SourceList.c"
	}
#line 2103 "../src/Widgets/SourceList/SourceList.vala"
	_tmp8_ = self->priv->_data_model;
#line 2103 "../src/Widgets/SourceList/SourceList.vala"
	_tmp9_ = code_widgets_source_list_data_model_get_item_path (_tmp8_, item);
#line 2103 "../src/Widgets/SourceList/SourceList.vala"
	path = _tmp9_;
#line 2104 "../src/Widgets/SourceList/SourceList.vala"
	_tmp10_ = path;
#line 2104 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp10_ != NULL) {
#line 10612 "SourceList.c"
		GtkCellRendererText* _tmp11_;
		GtkTreePath* _tmp12_;
		GtkTreeViewColumn* _tmp13_;
		GtkCellRendererText* _tmp14_;
#line 2105 "../src/Widgets/SourceList/SourceList.vala"
		self->priv->edited = item;
#line 2106 "../src/Widgets/SourceList/SourceList.vala"
		_tmp11_ = self->priv->text_cell;
#line 2106 "../src/Widgets/SourceList/SourceList.vala"
		g_object_set (_tmp11_, "editable", TRUE, NULL);
#line 2107 "../src/Widgets/SourceList/SourceList.vala"
		_tmp12_ = path;
#line 2107 "../src/Widgets/SourceList/SourceList.vala"
		_tmp13_ = gtk_tree_view_get_column ((GtkTreeView*) self, (gint) CODE_WIDGETS_SOURCE_LIST_TREE_COLUMN_ITEM);
#line 2107 "../src/Widgets/SourceList/SourceList.vala"
		_tmp14_ = self->priv->text_cell;
#line 2107 "../src/Widgets/SourceList/SourceList.vala"
		gtk_tree_view_set_cursor_on_cell ((GtkTreeView*) self, _tmp12_, _tmp13_, (GtkCellRenderer*) _tmp14_, TRUE);
#line 10631 "SourceList.c"
	} else {
		const gchar* _tmp15_;
		const gchar* _tmp16_;
#line 2109 "../src/Widgets/SourceList/SourceList.vala"
		_tmp15_ = code_widgets_source_list_item_get_name (item);
#line 2109 "../src/Widgets/SourceList/SourceList.vala"
		_tmp16_ = _tmp15_;
#line 2109 "../src/Widgets/SourceList/SourceList.vala"
		g_warning ("SourceList.vala:2109: Could not edit \"%s\": path not found", _tmp16_);
#line 10641 "SourceList.c"
	}
#line 2112 "../src/Widgets/SourceList/SourceList.vala"
	_tmp17_ = code_widgets_source_list_tree_get_editing (self);
#line 2112 "../src/Widgets/SourceList/SourceList.vala"
	_tmp18_ = _tmp17_;
#line 2112 "../src/Widgets/SourceList/SourceList.vala"
	result = _tmp18_;
#line 2112 "../src/Widgets/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (path);
#line 2112 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 10653 "SourceList.c"
}

static void
code_widgets_source_list_tree_stop_editing (CodeWidgetsSourceListTree* self)
{
	gboolean _tmp0_ = FALSE;
	gboolean _tmp1_;
	gboolean _tmp2_;
#line 2115 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 2116 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = code_widgets_source_list_tree_get_editing (self);
#line 2116 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = _tmp1_;
#line 2116 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp2_) {
#line 10670 "SourceList.c"
		CodeWidgetsSourceListItem* _tmp3_;
#line 2116 "../src/Widgets/SourceList/SourceList.vala"
		_tmp3_ = self->priv->edited;
#line 2116 "../src/Widgets/SourceList/SourceList.vala"
		_tmp0_ = _tmp3_ != NULL;
#line 10676 "SourceList.c"
	} else {
#line 2116 "../src/Widgets/SourceList/SourceList.vala"
		_tmp0_ = FALSE;
#line 10680 "SourceList.c"
	}
#line 2116 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp0_) {
#line 10684 "SourceList.c"
		GtkTreePath* path = NULL;
		CodeWidgetsSourceListDataModel* _tmp4_;
		CodeWidgetsSourceListItem* _tmp5_;
		GtkTreePath* _tmp6_;
		GtkTreePath* _tmp7_;
#line 2117 "../src/Widgets/SourceList/SourceList.vala"
		_tmp4_ = self->priv->_data_model;
#line 2117 "../src/Widgets/SourceList/SourceList.vala"
		_tmp5_ = self->priv->edited;
#line 2117 "../src/Widgets/SourceList/SourceList.vala"
		_tmp6_ = code_widgets_source_list_data_model_get_item_path (_tmp4_, _tmp5_);
#line 2117 "../src/Widgets/SourceList/SourceList.vala"
		path = _tmp6_;
#line 2121 "../src/Widgets/SourceList/SourceList.vala"
		_tmp7_ = path;
#line 2121 "../src/Widgets/SourceList/SourceList.vala"
		if (_tmp7_ != NULL) {
#line 10702 "SourceList.c"
			GtkTreePath* _tmp8_;
			GtkTreeViewColumn* _tmp9_;
			GtkCellRendererText* _tmp10_;
#line 2122 "../src/Widgets/SourceList/SourceList.vala"
			_tmp8_ = path;
#line 2122 "../src/Widgets/SourceList/SourceList.vala"
			_tmp9_ = gtk_tree_view_get_column ((GtkTreeView*) self, (gint) CODE_WIDGETS_SOURCE_LIST_TREE_COLUMN_ITEM);
#line 2122 "../src/Widgets/SourceList/SourceList.vala"
			_tmp10_ = self->priv->text_cell;
#line 2122 "../src/Widgets/SourceList/SourceList.vala"
			gtk_tree_view_set_cursor_on_cell ((GtkTreeView*) self, _tmp8_, _tmp9_, (GtkCellRenderer*) _tmp10_, FALSE);
#line 10714 "SourceList.c"
		}
#line 2116 "../src/Widgets/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (path);
#line 10718 "SourceList.c"
	}
}

static void
_code_widgets_source_list_tree_on_editing_done_gtk_cell_editable_editing_done (GtkCellEditable* _sender,
                                                                               gpointer self)
{
#line 2129 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_tree_on_editing_done ((CodeWidgetsSourceListTree*) self);
#line 10728 "SourceList.c"
}

static void
code_widgets_source_list_tree_on_editing_started (CodeWidgetsSourceListTree* self,
                                                  GtkCellEditable* editable,
                                                  const gchar* path)
{
	GtkEntry* _tmp0_;
	GtkEntry* _tmp1_;
#line 2126 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 2126 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (editable != NULL);
#line 2126 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (path != NULL);
#line 2127 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (editable, gtk_entry_get_type ()) ? ((GtkEntry*) editable) : NULL);
#line 2127 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->editable_entry);
#line 2127 "../src/Widgets/SourceList/SourceList.vala"
	self->priv->editable_entry = _tmp0_;
#line 2128 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = self->priv->editable_entry;
#line 2128 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp1_ != NULL) {
#line 10754 "SourceList.c"
		GtkEntry* _tmp2_;
		GtkEntry* _tmp3_;
#line 2129 "../src/Widgets/SourceList/SourceList.vala"
		_tmp2_ = self->priv->editable_entry;
#line 2129 "../src/Widgets/SourceList/SourceList.vala"
		g_signal_connect_object ((GtkCellEditable*) _tmp2_, "editing-done", (GCallback) _code_widgets_source_list_tree_on_editing_done_gtk_cell_editable_editing_done, self, 0);
#line 2130 "../src/Widgets/SourceList/SourceList.vala"
		_tmp3_ = self->priv->editable_entry;
#line 2130 "../src/Widgets/SourceList/SourceList.vala"
		g_object_set (_tmp3_, "editable", TRUE, NULL);
#line 10765 "SourceList.c"
	}
}

static void
code_widgets_source_list_tree_on_editing_canceled (CodeWidgetsSourceListTree* self)
{
	GtkEntry* _tmp0_;
	GtkCellRendererText* _tmp4_;
#line 2134 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 2135 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->editable_entry;
#line 2135 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp0_ != NULL) {
#line 10780 "SourceList.c"
		GtkEntry* _tmp1_;
		GtkEntry* _tmp2_;
		guint _tmp3_;
#line 2136 "../src/Widgets/SourceList/SourceList.vala"
		_tmp1_ = self->priv->editable_entry;
#line 2136 "../src/Widgets/SourceList/SourceList.vala"
		g_object_set (_tmp1_, "editable", FALSE, NULL);
#line 2137 "../src/Widgets/SourceList/SourceList.vala"
		_tmp2_ = self->priv->editable_entry;
#line 2137 "../src/Widgets/SourceList/SourceList.vala"
		g_signal_parse_name ("editing-done", gtk_cell_editable_get_type (), &_tmp3_, NULL, FALSE);
#line 2137 "../src/Widgets/SourceList/SourceList.vala"
		g_signal_handlers_disconnect_matched ((GtkCellEditable*) _tmp2_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp3_, 0, NULL, (GCallback) _code_widgets_source_list_tree_on_editing_done_gtk_cell_editable_editing_done, self);
#line 10794 "SourceList.c"
	}
#line 2140 "../src/Widgets/SourceList/SourceList.vala"
	_tmp4_ = self->priv->text_cell;
#line 2140 "../src/Widgets/SourceList/SourceList.vala"
	g_object_set (_tmp4_, "editable", FALSE, NULL);
#line 2141 "../src/Widgets/SourceList/SourceList.vala"
	self->priv->edited = NULL;
#line 10802 "SourceList.c"
}

static void
code_widgets_source_list_tree_on_editing_done (CodeWidgetsSourceListTree* self)
{
	gboolean _tmp0_ = FALSE;
	gboolean _tmp1_ = FALSE;
	CodeWidgetsSourceListItem* _tmp2_;
#line 2144 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 2145 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = self->priv->edited;
#line 2145 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp2_ != NULL) {
#line 10817 "SourceList.c"
		CodeWidgetsSourceListItem* _tmp3_;
		gboolean _tmp4_;
		gboolean _tmp5_;
#line 2145 "../src/Widgets/SourceList/SourceList.vala"
		_tmp3_ = self->priv->edited;
#line 2145 "../src/Widgets/SourceList/SourceList.vala"
		_tmp4_ = code_widgets_source_list_item_get_editable (_tmp3_);
#line 2145 "../src/Widgets/SourceList/SourceList.vala"
		_tmp5_ = _tmp4_;
#line 2145 "../src/Widgets/SourceList/SourceList.vala"
		_tmp1_ = _tmp5_;
#line 10829 "SourceList.c"
	} else {
#line 2145 "../src/Widgets/SourceList/SourceList.vala"
		_tmp1_ = FALSE;
#line 10833 "SourceList.c"
	}
#line 2145 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp1_) {
#line 10837 "SourceList.c"
		GtkEntry* _tmp6_;
#line 2145 "../src/Widgets/SourceList/SourceList.vala"
		_tmp6_ = self->priv->editable_entry;
#line 2145 "../src/Widgets/SourceList/SourceList.vala"
		_tmp0_ = _tmp6_ != NULL;
#line 10843 "SourceList.c"
	} else {
#line 2145 "../src/Widgets/SourceList/SourceList.vala"
		_tmp0_ = FALSE;
#line 10847 "SourceList.c"
	}
#line 2145 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp0_) {
#line 10851 "SourceList.c"
		CodeWidgetsSourceListItem* _tmp7_;
		GtkEntry* _tmp8_;
		const gchar* _tmp9_;
#line 2146 "../src/Widgets/SourceList/SourceList.vala"
		_tmp7_ = self->priv->edited;
#line 2146 "../src/Widgets/SourceList/SourceList.vala"
		_tmp8_ = self->priv->editable_entry;
#line 2146 "../src/Widgets/SourceList/SourceList.vala"
		_tmp9_ = gtk_entry_get_text (_tmp8_);
#line 2146 "../src/Widgets/SourceList/SourceList.vala"
		g_signal_emit (_tmp7_, code_widgets_source_list_item_signals[CODE_WIDGETS_SOURCE_LIST_ITEM_EDITED_SIGNAL], 0, _tmp9_);
#line 10863 "SourceList.c"
	}
#line 2149 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_tree_on_editing_canceled (self);
#line 10867 "SourceList.c"
}

static void
code_widgets_source_list_tree_on_activatable_activated (CodeWidgetsSourceListTree* self,
                                                        const gchar* item_path_str)
{
	CodeWidgetsSourceListItem* item = NULL;
	CodeWidgetsSourceListItem* _tmp0_;
	CodeWidgetsSourceListItem* _tmp1_;
#line 2152 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 2152 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (item_path_str != NULL);
#line 2153 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = code_widgets_source_list_tree_get_item_from_path_string (self, item_path_str);
#line 2153 "../src/Widgets/SourceList/SourceList.vala"
	item = _tmp0_;
#line 2154 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = item;
#line 2154 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp1_ != NULL) {
#line 10889 "SourceList.c"
		CodeWidgetsSourceListItem* _tmp2_;
#line 2155 "../src/Widgets/SourceList/SourceList.vala"
		_tmp2_ = item;
#line 2155 "../src/Widgets/SourceList/SourceList.vala"
		g_signal_emit (_tmp2_, code_widgets_source_list_item_signals[CODE_WIDGETS_SOURCE_LIST_ITEM_ACTION_ACTIVATED_SIGNAL], 0);
#line 10895 "SourceList.c"
	}
#line 2152 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (item);
#line 10899 "SourceList.c"
}

static CodeWidgetsSourceListItem*
code_widgets_source_list_tree_get_item_from_path_string (CodeWidgetsSourceListTree* self,
                                                         const gchar* item_path_str)
{
	GtkTreePath* item_path = NULL;
	GtkTreePath* _tmp0_;
	CodeWidgetsSourceListDataModel* _tmp1_;
	CodeWidgetsSourceListItem* _tmp2_;
	CodeWidgetsSourceListItem* result;
#line 2158 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 2158 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (item_path_str != NULL, NULL);
#line 2159 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = gtk_tree_path_new_from_string (item_path_str);
#line 2159 "../src/Widgets/SourceList/SourceList.vala"
	item_path = _tmp0_;
#line 2160 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = self->priv->_data_model;
#line 2160 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = code_widgets_source_list_data_model_get_item_from_path (_tmp1_, item_path);
#line 2160 "../src/Widgets/SourceList/SourceList.vala"
	result = _tmp2_;
#line 2160 "../src/Widgets/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (item_path);
#line 2160 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 10929 "SourceList.c"
}

static gboolean
code_widgets_source_list_tree_toggle_expansion (CodeWidgetsSourceListTree* self,
                                                CodeWidgetsSourceListExpandableItem* item)
{
	gboolean _tmp0_;
	gboolean _tmp1_;
	gboolean result;
#line 2163 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 2163 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (item != NULL, FALSE);
#line 2164 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = code_widgets_source_list_expandable_item_get_collapsible (item);
#line 2164 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = _tmp0_;
#line 2164 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp1_) {
#line 10949 "SourceList.c"
		gboolean _tmp2_;
		gboolean _tmp3_;
#line 2165 "../src/Widgets/SourceList/SourceList.vala"
		_tmp2_ = code_widgets_source_list_expandable_item_get_expanded (item);
#line 2165 "../src/Widgets/SourceList/SourceList.vala"
		_tmp3_ = _tmp2_;
#line 2165 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_expandable_item_set_expanded (item, !_tmp3_);
#line 2166 "../src/Widgets/SourceList/SourceList.vala"
		result = TRUE;
#line 2166 "../src/Widgets/SourceList/SourceList.vala"
		return result;
#line 10962 "SourceList.c"
	}
#line 2168 "../src/Widgets/SourceList/SourceList.vala"
	result = FALSE;
#line 2168 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 10968 "SourceList.c"
}

/**
         * Updates the tree to reflect the ''expanded'' property of expandable_item.
         */
static void
code_widgets_source_list_tree_update_expansion (CodeWidgetsSourceListTree* self,
                                                CodeWidgetsSourceListExpandableItem* expandable_item)
{
	GtkTreePath* path = NULL;
	CodeWidgetsSourceListDataModel* _tmp0_;
	GtkTreePath* _tmp1_;
	GtkTreePath* _tmp2_;
#line 2174 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 2174 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (expandable_item != NULL);
#line 2175 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->_data_model;
#line 2175 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = code_widgets_source_list_data_model_get_item_path (_tmp0_, (CodeWidgetsSourceListItem*) expandable_item);
#line 2175 "../src/Widgets/SourceList/SourceList.vala"
	path = _tmp1_;
#line 2177 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = path;
#line 2177 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp2_ != NULL) {
#line 10996 "SourceList.c"
		GtkTreePath* _tmp3_;
		gboolean _tmp4_;
		gboolean _tmp5_;
#line 2181 "../src/Widgets/SourceList/SourceList.vala"
		_tmp3_ = path;
#line 2181 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_tree_add_spacer_cell_for_level (self, gtk_tree_path_get_depth (_tmp3_) + 1, TRUE);
#line 2183 "../src/Widgets/SourceList/SourceList.vala"
		_tmp4_ = code_widgets_source_list_expandable_item_get_expanded (expandable_item);
#line 2183 "../src/Widgets/SourceList/SourceList.vala"
		_tmp5_ = _tmp4_;
#line 2183 "../src/Widgets/SourceList/SourceList.vala"
		if (_tmp5_) {
#line 11010 "SourceList.c"
			GtkTreePath* _tmp6_;
			gboolean _tmp7_ = FALSE;
			CodeWidgetsSourceListItem* _tmp8_;
#line 2184 "../src/Widgets/SourceList/SourceList.vala"
			_tmp6_ = path;
#line 2184 "../src/Widgets/SourceList/SourceList.vala"
			gtk_tree_view_expand_row ((GtkTreeView*) self, _tmp6_, FALSE);
#line 2192 "../src/Widgets/SourceList/SourceList.vala"
			_tmp8_ = self->priv->selected;
#line 2192 "../src/Widgets/SourceList/SourceList.vala"
			if (_tmp8_ != NULL) {
#line 11022 "SourceList.c"
				CodeWidgetsSourceListItem* _tmp9_;
				CodeWidgetsSourceListExpandableItem* _tmp10_;
				CodeWidgetsSourceListExpandableItem* _tmp11_;
#line 2192 "../src/Widgets/SourceList/SourceList.vala"
				_tmp9_ = self->priv->selected;
#line 2192 "../src/Widgets/SourceList/SourceList.vala"
				_tmp10_ = code_widgets_source_list_item_get_parent (_tmp9_);
#line 2192 "../src/Widgets/SourceList/SourceList.vala"
				_tmp11_ = _tmp10_;
#line 2192 "../src/Widgets/SourceList/SourceList.vala"
				_tmp7_ = _tmp11_ == expandable_item;
#line 11034 "SourceList.c"
			} else {
#line 2192 "../src/Widgets/SourceList/SourceList.vala"
				_tmp7_ = FALSE;
#line 11038 "SourceList.c"
			}
#line 2192 "../src/Widgets/SourceList/SourceList.vala"
			if (_tmp7_) {
#line 11042 "SourceList.c"
				CodeWidgetsSourceListItem* _tmp12_;
#line 2193 "../src/Widgets/SourceList/SourceList.vala"
				_tmp12_ = self->priv->selected;
#line 2193 "../src/Widgets/SourceList/SourceList.vala"
				code_widgets_source_list_tree_set_selected (self, _tmp12_, TRUE);
#line 11048 "SourceList.c"
			}
			{
				GeeIterator* _child_item_it = NULL;
				GeeCollection* _tmp13_;
				GeeCollection* _tmp14_;
				GeeCollection* _tmp15_;
				GeeIterator* _tmp16_;
				GeeIterator* _tmp17_;
#line 2198 "../src/Widgets/SourceList/SourceList.vala"
				_tmp13_ = code_widgets_source_list_expandable_item_get_children (expandable_item);
#line 2198 "../src/Widgets/SourceList/SourceList.vala"
				_tmp14_ = _tmp13_;
#line 2198 "../src/Widgets/SourceList/SourceList.vala"
				_tmp15_ = _tmp14_;
#line 2198 "../src/Widgets/SourceList/SourceList.vala"
				_tmp16_ = gee_iterable_iterator ((GeeIterable*) _tmp15_);
#line 2198 "../src/Widgets/SourceList/SourceList.vala"
				_tmp17_ = _tmp16_;
#line 2198 "../src/Widgets/SourceList/SourceList.vala"
				_g_object_unref0 (_tmp15_);
#line 2198 "../src/Widgets/SourceList/SourceList.vala"
				_child_item_it = _tmp17_;
#line 2198 "../src/Widgets/SourceList/SourceList.vala"
				while (TRUE) {
#line 11073 "SourceList.c"
					GeeIterator* _tmp18_;
					CodeWidgetsSourceListItem* child_item = NULL;
					GeeIterator* _tmp19_;
					gpointer _tmp20_;
					CodeWidgetsSourceListExpandableItem* child_expandable_item = NULL;
					CodeWidgetsSourceListItem* _tmp21_;
					CodeWidgetsSourceListExpandableItem* _tmp22_;
					CodeWidgetsSourceListExpandableItem* _tmp23_;
#line 2198 "../src/Widgets/SourceList/SourceList.vala"
					_tmp18_ = _child_item_it;
#line 2198 "../src/Widgets/SourceList/SourceList.vala"
					if (!gee_iterator_next (_tmp18_)) {
#line 2198 "../src/Widgets/SourceList/SourceList.vala"
						break;
#line 11088 "SourceList.c"
					}
#line 2198 "../src/Widgets/SourceList/SourceList.vala"
					_tmp19_ = _child_item_it;
#line 2198 "../src/Widgets/SourceList/SourceList.vala"
					_tmp20_ = gee_iterator_get (_tmp19_);
#line 2198 "../src/Widgets/SourceList/SourceList.vala"
					child_item = (CodeWidgetsSourceListItem*) _tmp20_;
#line 2199 "../src/Widgets/SourceList/SourceList.vala"
					_tmp21_ = child_item;
#line 2199 "../src/Widgets/SourceList/SourceList.vala"
					_tmp22_ = _g_object_ref0 (CODE_WIDGETS_SOURCE_LIST_IS_EXPANDABLE_ITEM (_tmp21_) ? ((CodeWidgetsSourceListExpandableItem*) _tmp21_) : NULL);
#line 2199 "../src/Widgets/SourceList/SourceList.vala"
					child_expandable_item = _tmp22_;
#line 2200 "../src/Widgets/SourceList/SourceList.vala"
					_tmp23_ = child_expandable_item;
#line 2200 "../src/Widgets/SourceList/SourceList.vala"
					if (_tmp23_ != NULL) {
#line 11106 "SourceList.c"
						CodeWidgetsSourceListExpandableItem* _tmp24_;
#line 2201 "../src/Widgets/SourceList/SourceList.vala"
						_tmp24_ = child_expandable_item;
#line 2201 "../src/Widgets/SourceList/SourceList.vala"
						code_widgets_source_list_tree_update_expansion (self, _tmp24_);
#line 11112 "SourceList.c"
					}
#line 2198 "../src/Widgets/SourceList/SourceList.vala"
					_g_object_unref0 (child_expandable_item);
#line 2198 "../src/Widgets/SourceList/SourceList.vala"
					_g_object_unref0 (child_item);
#line 11118 "SourceList.c"
				}
#line 2198 "../src/Widgets/SourceList/SourceList.vala"
				_g_object_unref0 (_child_item_it);
#line 11122 "SourceList.c"
			}
		} else {
			GtkTreePath* _tmp25_;
#line 2204 "../src/Widgets/SourceList/SourceList.vala"
			_tmp25_ = path;
#line 2204 "../src/Widgets/SourceList/SourceList.vala"
			gtk_tree_view_collapse_row ((GtkTreeView*) self, _tmp25_);
#line 11130 "SourceList.c"
		}
	}
#line 2174 "../src/Widgets/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (path);
#line 11135 "SourceList.c"
}

static void
code_widgets_source_list_tree_real_row_expanded (GtkTreeView* base,
                                                 GtkTreeIter* iter,
                                                 GtkTreePath* path)
{
	CodeWidgetsSourceListTree * self;
	CodeWidgetsSourceListExpandableItem* item = NULL;
	CodeWidgetsSourceListDataModel* _tmp0_;
	GtkTreeIter _tmp1_;
	CodeWidgetsSourceListItem* _tmp2_;
	CodeWidgetsSourceListExpandableItem* _tmp3_;
#line 2209 "../src/Widgets/SourceList/SourceList.vala"
	self = (CodeWidgetsSourceListTree*) base;
#line 2209 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (iter != NULL);
#line 2209 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (path != NULL);
#line 2210 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->_data_model;
#line 2210 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = *iter;
#line 2210 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = code_widgets_source_list_data_model_get_item (_tmp0_, &_tmp1_);
#line 2210 "../src/Widgets/SourceList/SourceList.vala"
	_tmp3_ = CODE_WIDGETS_SOURCE_LIST_IS_EXPANDABLE_ITEM (_tmp2_) ? ((CodeWidgetsSourceListExpandableItem*) _tmp2_) : NULL;
#line 2210 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp3_ == NULL) {
#line 2210 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (_tmp2_);
#line 11167 "SourceList.c"
	}
#line 2210 "../src/Widgets/SourceList/SourceList.vala"
	item = _tmp3_;
#line 2211 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (item != NULL);
#line 2213 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_tree_disable_item_property_monitor (self);
#line 2214 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_expandable_item_set_expanded (item, TRUE);
#line 2215 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_tree_enable_item_property_monitor (self);
#line 2209 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (item);
#line 11181 "SourceList.c"
}

static void
code_widgets_source_list_tree_real_row_collapsed (GtkTreeView* base,
                                                  GtkTreeIter* iter,
                                                  GtkTreePath* path)
{
	CodeWidgetsSourceListTree * self;
	CodeWidgetsSourceListExpandableItem* item = NULL;
	CodeWidgetsSourceListDataModel* _tmp0_;
	GtkTreeIter _tmp1_;
	CodeWidgetsSourceListItem* _tmp2_;
	CodeWidgetsSourceListExpandableItem* _tmp3_;
#line 2218 "../src/Widgets/SourceList/SourceList.vala"
	self = (CodeWidgetsSourceListTree*) base;
#line 2218 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (iter != NULL);
#line 2218 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (path != NULL);
#line 2219 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->_data_model;
#line 2219 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = *iter;
#line 2219 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = code_widgets_source_list_data_model_get_item (_tmp0_, &_tmp1_);
#line 2219 "../src/Widgets/SourceList/SourceList.vala"
	_tmp3_ = CODE_WIDGETS_SOURCE_LIST_IS_EXPANDABLE_ITEM (_tmp2_) ? ((CodeWidgetsSourceListExpandableItem*) _tmp2_) : NULL;
#line 2219 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp3_ == NULL) {
#line 2219 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (_tmp2_);
#line 11213 "SourceList.c"
	}
#line 2219 "../src/Widgets/SourceList/SourceList.vala"
	item = _tmp3_;
#line 2220 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (item != NULL);
#line 2222 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_tree_disable_item_property_monitor (self);
#line 2223 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_expandable_item_set_expanded (item, FALSE);
#line 2224 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_tree_enable_item_property_monitor (self);
#line 2218 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (item);
#line 11227 "SourceList.c"
}

static void
code_widgets_source_list_tree_real_row_activated (GtkTreeView* base,
                                                  GtkTreePath* path,
                                                  GtkTreeViewColumn* column)
{
	CodeWidgetsSourceListTree * self;
	GtkTreeViewColumn* _tmp0_;
#line 2227 "../src/Widgets/SourceList/SourceList.vala"
	self = (CodeWidgetsSourceListTree*) base;
#line 2227 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (path != NULL);
#line 2227 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (column != NULL);
#line 2228 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = gtk_tree_view_get_column ((GtkTreeView*) self, (gint) CODE_WIDGETS_SOURCE_LIST_TREE_COLUMN_ITEM);
#line 2228 "../src/Widgets/SourceList/SourceList.vala"
	if (column == _tmp0_) {
#line 11247 "SourceList.c"
		CodeWidgetsSourceListItem* item = NULL;
		CodeWidgetsSourceListDataModel* _tmp1_;
		CodeWidgetsSourceListItem* _tmp2_;
		CodeWidgetsSourceListItem* _tmp3_;
#line 2229 "../src/Widgets/SourceList/SourceList.vala"
		_tmp1_ = self->priv->_data_model;
#line 2229 "../src/Widgets/SourceList/SourceList.vala"
		_tmp2_ = code_widgets_source_list_data_model_get_item_from_path (_tmp1_, path);
#line 2229 "../src/Widgets/SourceList/SourceList.vala"
		item = _tmp2_;
#line 2230 "../src/Widgets/SourceList/SourceList.vala"
		_tmp3_ = item;
#line 2230 "../src/Widgets/SourceList/SourceList.vala"
		if (_tmp3_ != NULL) {
#line 11262 "SourceList.c"
			CodeWidgetsSourceListItem* _tmp4_;
#line 2231 "../src/Widgets/SourceList/SourceList.vala"
			_tmp4_ = item;
#line 2231 "../src/Widgets/SourceList/SourceList.vala"
			g_signal_emit (_tmp4_, code_widgets_source_list_item_signals[CODE_WIDGETS_SOURCE_LIST_ITEM_ACTIVATED_SIGNAL], 0);
#line 11268 "SourceList.c"
		}
#line 2228 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (item);
#line 11272 "SourceList.c"
	}
}

static gboolean
code_widgets_source_list_tree_real_key_release_event (GtkWidget* base,
                                                      GdkEventKey* event)
{
	CodeWidgetsSourceListTree * self;
	CodeWidgetsSourceListItem* _tmp0_;
	CodeWidgetsSourceListItem* _tmp1_;
	gboolean result;
#line 2235 "../src/Widgets/SourceList/SourceList.vala"
	self = (CodeWidgetsSourceListTree*) base;
#line 2235 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (event != NULL, FALSE);
#line 2236 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = code_widgets_source_list_tree_get_selected_item (self);
#line 2236 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = _tmp0_;
#line 2236 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp1_ != NULL) {
#line 2237 "../src/Widgets/SourceList/SourceList.vala"
		switch (event->keyval) {
#line 2237 "../src/Widgets/SourceList/SourceList.vala"
			case GDK_KEY_F2:
#line 11298 "SourceList.c"
			{
				GdkModifierType modifiers = 0U;
				gboolean _tmp2_ = FALSE;
#line 2239 "../src/Widgets/SourceList/SourceList.vala"
				modifiers = gtk_accelerator_get_default_mod_mask ();
#line 2241 "../src/Widgets/SourceList/SourceList.vala"
				if ((event->state & modifiers) == 0) {
#line 11306 "SourceList.c"
					CodeWidgetsSourceListItem* _tmp3_;
					CodeWidgetsSourceListItem* _tmp4_;
					gboolean _tmp5_;
					gboolean _tmp6_;
#line 2241 "../src/Widgets/SourceList/SourceList.vala"
					_tmp3_ = code_widgets_source_list_tree_get_selected_item (self);
#line 2241 "../src/Widgets/SourceList/SourceList.vala"
					_tmp4_ = _tmp3_;
#line 2241 "../src/Widgets/SourceList/SourceList.vala"
					_tmp5_ = code_widgets_source_list_item_get_editable (_tmp4_);
#line 2241 "../src/Widgets/SourceList/SourceList.vala"
					_tmp6_ = _tmp5_;
#line 2241 "../src/Widgets/SourceList/SourceList.vala"
					_tmp2_ = _tmp6_;
#line 11321 "SourceList.c"
				} else {
#line 2241 "../src/Widgets/SourceList/SourceList.vala"
					_tmp2_ = FALSE;
#line 11325 "SourceList.c"
				}
#line 2241 "../src/Widgets/SourceList/SourceList.vala"
				if (_tmp2_) {
#line 11329 "SourceList.c"
					CodeWidgetsSourceListItem* _tmp7_;
					CodeWidgetsSourceListItem* _tmp8_;
#line 2242 "../src/Widgets/SourceList/SourceList.vala"
					_tmp7_ = code_widgets_source_list_tree_get_selected_item (self);
#line 2242 "../src/Widgets/SourceList/SourceList.vala"
					_tmp8_ = _tmp7_;
#line 2242 "../src/Widgets/SourceList/SourceList.vala"
					code_widgets_source_list_tree_start_editing_item (self, _tmp8_);
#line 11338 "SourceList.c"
				}
#line 2243 "../src/Widgets/SourceList/SourceList.vala"
				break;
#line 11342 "SourceList.c"
			}
			default:
#line 2237 "../src/Widgets/SourceList/SourceList.vala"
			break;
#line 11347 "SourceList.c"
		}
	}
#line 2247 "../src/Widgets/SourceList/SourceList.vala"
	result = GTK_WIDGET_CLASS (code_widgets_source_list_tree_parent_class)->key_release_event ((GtkWidget*) G_TYPE_CHECK_INSTANCE_CAST (self, gtk_tree_view_get_type (), GtkTreeView), event);
#line 2247 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 11354 "SourceList.c"
}

static gboolean
code_widgets_source_list_tree_real_button_release_event (GtkWidget* base,
                                                         GdkEventButton* event)
{
	CodeWidgetsSourceListTree * self;
	gboolean _tmp0_ = FALSE;
	gboolean result;
#line 2250 "../src/Widgets/SourceList/SourceList.vala"
	self = (CodeWidgetsSourceListTree*) base;
#line 2250 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (event != NULL, FALSE);
#line 2251 "../src/Widgets/SourceList/SourceList.vala"
	if (self->priv->unselectable_item_clicked) {
#line 11370 "SourceList.c"
		GdkWindow* _tmp1_;
		GdkWindow* _tmp2_;
#line 2251 "../src/Widgets/SourceList/SourceList.vala"
		_tmp1_ = event->window;
#line 2251 "../src/Widgets/SourceList/SourceList.vala"
		_tmp2_ = gtk_tree_view_get_bin_window ((GtkTreeView*) self);
#line 2251 "../src/Widgets/SourceList/SourceList.vala"
		_tmp0_ = _tmp1_ == _tmp2_;
#line 11379 "SourceList.c"
	} else {
#line 2251 "../src/Widgets/SourceList/SourceList.vala"
		_tmp0_ = FALSE;
#line 11383 "SourceList.c"
	}
#line 2251 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp0_) {
#line 11387 "SourceList.c"
		GtkTreePath* path = NULL;
		GtkTreeViewColumn* column = NULL;
		gint x = 0;
		gint y = 0;
		gint cell_x = 0;
		gint cell_y = 0;
		GtkTreePath* _tmp3_ = NULL;
		GtkTreeViewColumn* _tmp4_ = NULL;
		gint _tmp5_ = 0;
		gint _tmp6_ = 0;
		gboolean _tmp7_;
		GtkTreeViewColumn* _tmp8_;
#line 2252 "../src/Widgets/SourceList/SourceList.vala"
		self->priv->unselectable_item_clicked = FALSE;
#line 2256 "../src/Widgets/SourceList/SourceList.vala"
		x = (gint) event->x;
#line 2256 "../src/Widgets/SourceList/SourceList.vala"
		y = (gint) event->y;
#line 2258 "../src/Widgets/SourceList/SourceList.vala"
		_tmp7_ = gtk_tree_view_get_path_at_pos ((GtkTreeView*) self, x, y, &_tmp3_, &_tmp4_, &_tmp5_, &_tmp6_);
#line 2258 "../src/Widgets/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (path);
#line 2258 "../src/Widgets/SourceList/SourceList.vala"
		path = _tmp3_;
#line 2258 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (column);
#line 2258 "../src/Widgets/SourceList/SourceList.vala"
		_tmp8_ = _g_object_ref0 (_tmp4_);
#line 2258 "../src/Widgets/SourceList/SourceList.vala"
		column = _tmp8_;
#line 2258 "../src/Widgets/SourceList/SourceList.vala"
		cell_x = _tmp5_;
#line 2258 "../src/Widgets/SourceList/SourceList.vala"
		cell_y = _tmp6_;
#line 2258 "../src/Widgets/SourceList/SourceList.vala"
		if (_tmp7_) {
#line 11424 "SourceList.c"
			CodeWidgetsSourceListExpandableItem* item = NULL;
			CodeWidgetsSourceListDataModel* _tmp9_;
			GtkTreePath* _tmp10_;
			CodeWidgetsSourceListItem* _tmp11_;
			CodeWidgetsSourceListExpandableItem* _tmp12_;
			CodeWidgetsSourceListExpandableItem* _tmp13_;
#line 2259 "../src/Widgets/SourceList/SourceList.vala"
			_tmp9_ = self->priv->_data_model;
#line 2259 "../src/Widgets/SourceList/SourceList.vala"
			_tmp10_ = path;
#line 2259 "../src/Widgets/SourceList/SourceList.vala"
			_tmp11_ = code_widgets_source_list_data_model_get_item_from_path (_tmp9_, _tmp10_);
#line 2259 "../src/Widgets/SourceList/SourceList.vala"
			_tmp12_ = CODE_WIDGETS_SOURCE_LIST_IS_EXPANDABLE_ITEM (_tmp11_) ? ((CodeWidgetsSourceListExpandableItem*) _tmp11_) : NULL;
#line 2259 "../src/Widgets/SourceList/SourceList.vala"
			if (_tmp12_ == NULL) {
#line 2259 "../src/Widgets/SourceList/SourceList.vala"
				_g_object_unref0 (_tmp11_);
#line 11443 "SourceList.c"
			}
#line 2259 "../src/Widgets/SourceList/SourceList.vala"
			item = _tmp12_;
#line 2261 "../src/Widgets/SourceList/SourceList.vala"
			_tmp13_ = item;
#line 2261 "../src/Widgets/SourceList/SourceList.vala"
			if (_tmp13_ != NULL) {
#line 11451 "SourceList.c"
				gboolean _tmp14_ = FALSE;
				CodeWidgetsSourceListExpandableItem* _tmp15_;
				gboolean _tmp16_;
				gboolean _tmp17_;
#line 2262 "../src/Widgets/SourceList/SourceList.vala"
				_tmp15_ = item;
#line 2262 "../src/Widgets/SourceList/SourceList.vala"
				_tmp16_ = code_widgets_source_list_item_get_selectable ((CodeWidgetsSourceListItem*) _tmp15_);
#line 2262 "../src/Widgets/SourceList/SourceList.vala"
				_tmp17_ = _tmp16_;
#line 2262 "../src/Widgets/SourceList/SourceList.vala"
				if (!_tmp17_) {
#line 2262 "../src/Widgets/SourceList/SourceList.vala"
					_tmp14_ = TRUE;
#line 11466 "SourceList.c"
				} else {
					CodeWidgetsSourceListDataModel* _tmp18_;
					CodeWidgetsSourceListExpandableItem* _tmp19_;
					GtkTreePath* _tmp20_;
#line 2262 "../src/Widgets/SourceList/SourceList.vala"
					_tmp18_ = self->priv->_data_model;
#line 2262 "../src/Widgets/SourceList/SourceList.vala"
					_tmp19_ = item;
#line 2262 "../src/Widgets/SourceList/SourceList.vala"
					_tmp20_ = path;
#line 2262 "../src/Widgets/SourceList/SourceList.vala"
					_tmp14_ = code_widgets_source_list_data_model_is_category (_tmp18_, (CodeWidgetsSourceListItem*) _tmp19_, NULL, _tmp20_);
#line 11479 "SourceList.c"
				}
#line 2262 "../src/Widgets/SourceList/SourceList.vala"
				if (_tmp14_) {
#line 11483 "SourceList.c"
					CodeWidgetsSourceListExpandableItem* _tmp21_;
#line 2263 "../src/Widgets/SourceList/SourceList.vala"
					_tmp21_ = item;
#line 2263 "../src/Widgets/SourceList/SourceList.vala"
					code_widgets_source_list_tree_toggle_expansion (self, _tmp21_);
#line 11489 "SourceList.c"
				}
			}
#line 2258 "../src/Widgets/SourceList/SourceList.vala"
			_g_object_unref0 (item);
#line 11494 "SourceList.c"
		}
#line 2251 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (column);
#line 2251 "../src/Widgets/SourceList/SourceList.vala"
		__vala_GtkTreePath_free0 (path);
#line 11500 "SourceList.c"
	}
#line 2268 "../src/Widgets/SourceList/SourceList.vala"
	result = GTK_WIDGET_CLASS (code_widgets_source_list_tree_parent_class)->button_release_event ((GtkWidget*) G_TYPE_CHECK_INSTANCE_CAST (self, gtk_tree_view_get_type (), GtkTreeView), event);
#line 2268 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 11506 "SourceList.c"
}

static gboolean
code_widgets_source_list_tree_real_button_press_event (GtkWidget* base,
                                                       GdkEventButton* event)
{
	CodeWidgetsSourceListTree * self;
	GdkWindow* _tmp0_;
	GdkWindow* _tmp1_;
	GtkTreePath* path = NULL;
	GtkTreeViewColumn* column = NULL;
	gint x = 0;
	gint y = 0;
	gint cell_x = 0;
	gint cell_y = 0;
	GtkTreePath* _tmp2_ = NULL;
	GtkTreeViewColumn* _tmp3_ = NULL;
	gint _tmp4_ = 0;
	gint _tmp5_ = 0;
	gboolean _tmp6_;
	GtkTreeViewColumn* _tmp7_;
	gboolean result;
#line 2271 "../src/Widgets/SourceList/SourceList.vala"
	self = (CodeWidgetsSourceListTree*) base;
#line 2271 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (event != NULL, FALSE);
#line 2272 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = event->window;
#line 2272 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = gtk_tree_view_get_bin_window ((GtkTreeView*) self);
#line 2272 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp0_ != _tmp1_) {
#line 2273 "../src/Widgets/SourceList/SourceList.vala"
		result = GTK_WIDGET_CLASS (code_widgets_source_list_tree_parent_class)->button_press_event ((GtkWidget*) G_TYPE_CHECK_INSTANCE_CAST (self, gtk_tree_view_get_type (), GtkTreeView), event);
#line 2273 "../src/Widgets/SourceList/SourceList.vala"
		return result;
#line 11543 "SourceList.c"
	}
#line 2277 "../src/Widgets/SourceList/SourceList.vala"
	x = (gint) event->x;
#line 2277 "../src/Widgets/SourceList/SourceList.vala"
	y = (gint) event->y;
#line 2279 "../src/Widgets/SourceList/SourceList.vala"
	_tmp6_ = gtk_tree_view_get_path_at_pos ((GtkTreeView*) self, x, y, &_tmp2_, &_tmp3_, &_tmp4_, &_tmp5_);
#line 2279 "../src/Widgets/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (path);
#line 2279 "../src/Widgets/SourceList/SourceList.vala"
	path = _tmp2_;
#line 2279 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (column);
#line 2279 "../src/Widgets/SourceList/SourceList.vala"
	_tmp7_ = _g_object_ref0 (_tmp3_);
#line 2279 "../src/Widgets/SourceList/SourceList.vala"
	column = _tmp7_;
#line 2279 "../src/Widgets/SourceList/SourceList.vala"
	cell_x = _tmp4_;
#line 2279 "../src/Widgets/SourceList/SourceList.vala"
	cell_y = _tmp5_;
#line 2279 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp6_) {
#line 11567 "SourceList.c"
		CodeWidgetsSourceListItem* item = NULL;
		CodeWidgetsSourceListDataModel* _tmp8_;
		GtkTreePath* _tmp9_;
		CodeWidgetsSourceListItem* _tmp10_;
		GdkRectangle start_cell_area = {0};
		GtkTreePath* _tmp11_;
		GtkTreeViewColumn* _tmp12_;
		GdkRectangle _tmp13_ = {0};
		GdkRectangle _tmp14_;
		gboolean _tmp15_ = FALSE;
		CodeWidgetsSourceListItem* _tmp16_;
#line 2280 "../src/Widgets/SourceList/SourceList.vala"
		_tmp8_ = self->priv->_data_model;
#line 2280 "../src/Widgets/SourceList/SourceList.vala"
		_tmp9_ = path;
#line 2280 "../src/Widgets/SourceList/SourceList.vala"
		_tmp10_ = code_widgets_source_list_data_model_get_item_from_path (_tmp8_, _tmp9_);
#line 2280 "../src/Widgets/SourceList/SourceList.vala"
		item = _tmp10_;
#line 2284 "../src/Widgets/SourceList/SourceList.vala"
		_tmp11_ = path;
#line 2284 "../src/Widgets/SourceList/SourceList.vala"
		_tmp12_ = gtk_tree_view_get_column ((GtkTreeView*) self, 0);
#line 2284 "../src/Widgets/SourceList/SourceList.vala"
		gtk_tree_view_get_cell_area ((GtkTreeView*) self, _tmp11_, _tmp12_, &_tmp13_);
#line 2284 "../src/Widgets/SourceList/SourceList.vala"
		start_cell_area = _tmp13_;
#line 2285 "../src/Widgets/SourceList/SourceList.vala"
		_tmp14_ = start_cell_area;
#line 2285 "../src/Widgets/SourceList/SourceList.vala"
		cell_x -= _tmp14_.x;
#line 2287 "../src/Widgets/SourceList/SourceList.vala"
		_tmp16_ = item;
#line 2287 "../src/Widgets/SourceList/SourceList.vala"
		if (_tmp16_ != NULL) {
#line 11603 "SourceList.c"
			GtkTreeViewColumn* _tmp17_;
			GtkTreeViewColumn* _tmp18_;
#line 2287 "../src/Widgets/SourceList/SourceList.vala"
			_tmp17_ = column;
#line 2287 "../src/Widgets/SourceList/SourceList.vala"
			_tmp18_ = gtk_tree_view_get_column ((GtkTreeView*) self, (gint) CODE_WIDGETS_SOURCE_LIST_TREE_COLUMN_ITEM);
#line 2287 "../src/Widgets/SourceList/SourceList.vala"
			_tmp15_ = _tmp17_ == _tmp18_;
#line 11612 "SourceList.c"
		} else {
#line 2287 "../src/Widgets/SourceList/SourceList.vala"
			_tmp15_ = FALSE;
#line 11616 "SourceList.c"
		}
#line 2287 "../src/Widgets/SourceList/SourceList.vala"
		if (_tmp15_) {
#line 2289 "../src/Widgets/SourceList/SourceList.vala"
			code_widgets_source_list_tree_stop_editing (self);
#line 2291 "../src/Widgets/SourceList/SourceList.vala"
			if (event->button == ((guint) GDK_BUTTON_SECONDARY)) {
#line 11624 "SourceList.c"
				CodeWidgetsSourceListItem* _tmp19_;
#line 2292 "../src/Widgets/SourceList/SourceList.vala"
				_tmp19_ = item;
#line 2292 "../src/Widgets/SourceList/SourceList.vala"
				code_widgets_source_list_tree_popup_context_menu (self, _tmp19_, event);
#line 2293 "../src/Widgets/SourceList/SourceList.vala"
				result = TRUE;
#line 2293 "../src/Widgets/SourceList/SourceList.vala"
				_g_object_unref0 (item);
#line 2293 "../src/Widgets/SourceList/SourceList.vala"
				_g_object_unref0 (column);
#line 2293 "../src/Widgets/SourceList/SourceList.vala"
				__vala_GtkTreePath_free0 (path);
#line 2293 "../src/Widgets/SourceList/SourceList.vala"
				return result;
#line 11640 "SourceList.c"
			} else {
#line 2294 "../src/Widgets/SourceList/SourceList.vala"
				if (event->button == ((guint) GDK_BUTTON_PRIMARY)) {
#line 11644 "SourceList.c"
					gboolean is_expandable = FALSE;
					CodeWidgetsSourceListItem* _tmp20_;
					gboolean _tmp21_ = FALSE;
					gboolean is_category = FALSE;
#line 2296 "../src/Widgets/SourceList/SourceList.vala"
					_tmp20_ = item;
#line 2296 "../src/Widgets/SourceList/SourceList.vala"
					is_expandable = CODE_WIDGETS_SOURCE_LIST_IS_EXPANDABLE_ITEM (_tmp20_);
#line 2297 "../src/Widgets/SourceList/SourceList.vala"
					if (is_expandable) {
#line 11655 "SourceList.c"
						CodeWidgetsSourceListDataModel* _tmp22_;
						CodeWidgetsSourceListItem* _tmp23_;
						GtkTreePath* _tmp24_;
#line 2297 "../src/Widgets/SourceList/SourceList.vala"
						_tmp22_ = self->priv->_data_model;
#line 2297 "../src/Widgets/SourceList/SourceList.vala"
						_tmp23_ = item;
#line 2297 "../src/Widgets/SourceList/SourceList.vala"
						_tmp24_ = path;
#line 2297 "../src/Widgets/SourceList/SourceList.vala"
						_tmp21_ = code_widgets_source_list_data_model_is_category (_tmp22_, _tmp23_, NULL, _tmp24_);
#line 11667 "SourceList.c"
					} else {
#line 2297 "../src/Widgets/SourceList/SourceList.vala"
						_tmp21_ = FALSE;
#line 11671 "SourceList.c"
					}
#line 2297 "../src/Widgets/SourceList/SourceList.vala"
					is_category = _tmp21_;
#line 2299 "../src/Widgets/SourceList/SourceList.vala"
					if (event->type == GDK_BUTTON_PRESS) {
#line 2300 "../src/Widgets/SourceList/SourceList.vala"
						if (is_expandable) {
#line 11679 "SourceList.c"
							gboolean _tmp25_ = FALSE;
							gboolean _tmp33_ = FALSE;
							gboolean _tmp34_ = FALSE;
#line 2306 "../src/Widgets/SourceList/SourceList.vala"
							if (is_category) {
#line 2306 "../src/Widgets/SourceList/SourceList.vala"
								_tmp25_ = TRUE;
#line 11687 "SourceList.c"
							} else {
								gboolean _tmp26_ = FALSE;
								CodeWidgetsSourceListItem* _tmp27_;
								gboolean _tmp28_;
								gboolean _tmp29_;
#line 2307 "../src/Widgets/SourceList/SourceList.vala"
								_tmp27_ = item;
#line 2307 "../src/Widgets/SourceList/SourceList.vala"
								_tmp28_ = code_widgets_source_list_item_get_selectable (_tmp27_);
#line 2307 "../src/Widgets/SourceList/SourceList.vala"
								_tmp29_ = _tmp28_;
#line 2307 "../src/Widgets/SourceList/SourceList.vala"
								if (!_tmp29_) {
#line 11701 "SourceList.c"
									GtkTreeViewColumn* _tmp30_;
									GtkTreePath* _tmp31_;
									CodeWidgetsSourceListCellRendererIcon* _tmp32_;
#line 2307 "../src/Widgets/SourceList/SourceList.vala"
									_tmp30_ = column;
#line 2307 "../src/Widgets/SourceList/SourceList.vala"
									_tmp31_ = path;
#line 2307 "../src/Widgets/SourceList/SourceList.vala"
									_tmp32_ = self->priv->activatable_cell;
#line 2307 "../src/Widgets/SourceList/SourceList.vala"
									_tmp26_ = !code_widgets_source_list_tree_over_cell (self, _tmp30_, _tmp31_, (GtkCellRenderer*) _tmp32_, cell_x);
#line 11713 "SourceList.c"
								} else {
#line 2307 "../src/Widgets/SourceList/SourceList.vala"
									_tmp26_ = FALSE;
#line 11717 "SourceList.c"
								}
#line 2307 "../src/Widgets/SourceList/SourceList.vala"
								_tmp25_ = _tmp26_;
#line 11721 "SourceList.c"
							}
#line 2306 "../src/Widgets/SourceList/SourceList.vala"
							self->priv->unselectable_item_clicked = _tmp25_;
#line 2309 "../src/Widgets/SourceList/SourceList.vala"
							if (!self->priv->unselectable_item_clicked) {
#line 11727 "SourceList.c"
								GtkTreeViewColumn* _tmp35_;
								GtkTreePath* _tmp36_;
#line 2310 "../src/Widgets/SourceList/SourceList.vala"
								_tmp35_ = column;
#line 2310 "../src/Widgets/SourceList/SourceList.vala"
								_tmp36_ = path;
#line 2310 "../src/Widgets/SourceList/SourceList.vala"
								_tmp34_ = code_widgets_source_list_tree_over_primary_expander (self, _tmp35_, _tmp36_, cell_x);
#line 11736 "SourceList.c"
							} else {
#line 2309 "../src/Widgets/SourceList/SourceList.vala"
								_tmp34_ = FALSE;
#line 11740 "SourceList.c"
							}
#line 2309 "../src/Widgets/SourceList/SourceList.vala"
							if (_tmp34_) {
#line 11744 "SourceList.c"
								CodeWidgetsSourceListItem* _tmp37_;
#line 2311 "../src/Widgets/SourceList/SourceList.vala"
								_tmp37_ = item;
#line 2311 "../src/Widgets/SourceList/SourceList.vala"
								_tmp33_ = code_widgets_source_list_tree_toggle_expansion (self, CODE_WIDGETS_SOURCE_LIST_IS_EXPANDABLE_ITEM (_tmp37_) ? ((CodeWidgetsSourceListExpandableItem*) _tmp37_) : NULL);
#line 11750 "SourceList.c"
							} else {
#line 2309 "../src/Widgets/SourceList/SourceList.vala"
								_tmp33_ = FALSE;
#line 11754 "SourceList.c"
							}
#line 2309 "../src/Widgets/SourceList/SourceList.vala"
							if (_tmp33_) {
#line 2312 "../src/Widgets/SourceList/SourceList.vala"
								result = TRUE;
#line 2312 "../src/Widgets/SourceList/SourceList.vala"
								_g_object_unref0 (item);
#line 2312 "../src/Widgets/SourceList/SourceList.vala"
								_g_object_unref0 (column);
#line 2312 "../src/Widgets/SourceList/SourceList.vala"
								__vala_GtkTreePath_free0 (path);
#line 2312 "../src/Widgets/SourceList/SourceList.vala"
								return result;
#line 11768 "SourceList.c"
							}
						}
					} else {
						gboolean _tmp38_ = FALSE;
						gboolean _tmp39_ = FALSE;
						gboolean _tmp40_ = FALSE;
						gboolean _tmp41_ = FALSE;
						gboolean _tmp42_ = FALSE;
#line 2315 "../src/Widgets/SourceList/SourceList.vala"
						if (event->type == GDK_2BUTTON_PRESS) {
#line 2316 "../src/Widgets/SourceList/SourceList.vala"
							_tmp42_ = !is_category;
#line 11781 "SourceList.c"
						} else {
#line 2315 "../src/Widgets/SourceList/SourceList.vala"
							_tmp42_ = FALSE;
#line 11785 "SourceList.c"
						}
#line 2315 "../src/Widgets/SourceList/SourceList.vala"
						if (_tmp42_) {
#line 11789 "SourceList.c"
							CodeWidgetsSourceListItem* _tmp43_;
							gboolean _tmp44_;
							gboolean _tmp45_;
#line 2317 "../src/Widgets/SourceList/SourceList.vala"
							_tmp43_ = item;
#line 2317 "../src/Widgets/SourceList/SourceList.vala"
							_tmp44_ = code_widgets_source_list_item_get_editable (_tmp43_);
#line 2317 "../src/Widgets/SourceList/SourceList.vala"
							_tmp45_ = _tmp44_;
#line 2317 "../src/Widgets/SourceList/SourceList.vala"
							_tmp41_ = _tmp45_;
#line 11801 "SourceList.c"
						} else {
#line 2315 "../src/Widgets/SourceList/SourceList.vala"
							_tmp41_ = FALSE;
#line 11805 "SourceList.c"
						}
#line 2315 "../src/Widgets/SourceList/SourceList.vala"
						if (_tmp41_) {
#line 11809 "SourceList.c"
							CodeWidgetsSourceListItem* _tmp46_;
							gboolean _tmp47_;
							gboolean _tmp48_;
#line 2318 "../src/Widgets/SourceList/SourceList.vala"
							_tmp46_ = item;
#line 2318 "../src/Widgets/SourceList/SourceList.vala"
							_tmp47_ = code_widgets_source_list_item_get_selectable (_tmp46_);
#line 2318 "../src/Widgets/SourceList/SourceList.vala"
							_tmp48_ = _tmp47_;
#line 2318 "../src/Widgets/SourceList/SourceList.vala"
							_tmp40_ = _tmp48_;
#line 11821 "SourceList.c"
						} else {
#line 2315 "../src/Widgets/SourceList/SourceList.vala"
							_tmp40_ = FALSE;
#line 11825 "SourceList.c"
						}
#line 2315 "../src/Widgets/SourceList/SourceList.vala"
						if (_tmp40_) {
#line 11829 "SourceList.c"
							GtkTreeViewColumn* _tmp49_;
							GtkTreePath* _tmp50_;
							GtkCellRendererText* _tmp51_;
#line 2319 "../src/Widgets/SourceList/SourceList.vala"
							_tmp49_ = column;
#line 2319 "../src/Widgets/SourceList/SourceList.vala"
							_tmp50_ = path;
#line 2319 "../src/Widgets/SourceList/SourceList.vala"
							_tmp51_ = self->priv->text_cell;
#line 2319 "../src/Widgets/SourceList/SourceList.vala"
							_tmp39_ = code_widgets_source_list_tree_over_cell (self, _tmp49_, _tmp50_, (GtkCellRenderer*) _tmp51_, cell_x);
#line 11841 "SourceList.c"
						} else {
#line 2315 "../src/Widgets/SourceList/SourceList.vala"
							_tmp39_ = FALSE;
#line 11845 "SourceList.c"
						}
#line 2315 "../src/Widgets/SourceList/SourceList.vala"
						if (_tmp39_) {
#line 11849 "SourceList.c"
							CodeWidgetsSourceListItem* _tmp52_;
#line 2320 "../src/Widgets/SourceList/SourceList.vala"
							_tmp52_ = item;
#line 2320 "../src/Widgets/SourceList/SourceList.vala"
							_tmp38_ = code_widgets_source_list_tree_start_editing_item (self, _tmp52_);
#line 11855 "SourceList.c"
						} else {
#line 2315 "../src/Widgets/SourceList/SourceList.vala"
							_tmp38_ = FALSE;
#line 11859 "SourceList.c"
						}
#line 2314 "../src/Widgets/SourceList/SourceList.vala"
						if (_tmp38_) {
#line 2323 "../src/Widgets/SourceList/SourceList.vala"
							result = TRUE;
#line 2323 "../src/Widgets/SourceList/SourceList.vala"
							_g_object_unref0 (item);
#line 2323 "../src/Widgets/SourceList/SourceList.vala"
							_g_object_unref0 (column);
#line 2323 "../src/Widgets/SourceList/SourceList.vala"
							__vala_GtkTreePath_free0 (path);
#line 2323 "../src/Widgets/SourceList/SourceList.vala"
							return result;
#line 11873 "SourceList.c"
						}
					}
				}
			}
		}
#line 2279 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (item);
#line 11881 "SourceList.c"
	}
#line 2329 "../src/Widgets/SourceList/SourceList.vala"
	result = GTK_WIDGET_CLASS (code_widgets_source_list_tree_parent_class)->button_press_event ((GtkWidget*) G_TYPE_CHECK_INSTANCE_CAST (self, gtk_tree_view_get_type (), GtkTreeView), event);
#line 2329 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (column);
#line 2329 "../src/Widgets/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (path);
#line 2329 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 11891 "SourceList.c"
}

static gboolean
code_widgets_source_list_tree_over_primary_expander (CodeWidgetsSourceListTree* self,
                                                     GtkTreeViewColumn* col,
                                                     GtkTreePath* path,
                                                     gint x)
{
	GtkTreeIter iter = {0};
	GtkTreeModel* _tmp0_;
	GtkTreeModel* _tmp1_;
	GtkTreeIter _tmp2_ = {0};
	gboolean _tmp3_;
	CodeWidgetsCellRendererExpander* _tmp4_;
	GtkTreeModel* _tmp5_;
	GtkTreeModel* _tmp6_;
	GtkTreeIter _tmp7_;
	CodeWidgetsCellRendererExpander* _tmp8_;
	gboolean _tmp9_;
	gboolean _tmp10_;
	GtkTreeModel* _tmp11_;
	GtkTreeModel* _tmp12_;
	GtkTreeIter _tmp13_;
	gint indentation_level = 0;
	CodeWidgetsSourceListCellRendererSpacer* last_spacer_cell = NULL;
	GeeHashMap* _tmp14_;
	gpointer _tmp15_;
	CodeWidgetsSourceListCellRendererSpacer* _tmp16_;
	gboolean result;
#line 2332 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 2332 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (col != NULL, FALSE);
#line 2332 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (path != NULL, FALSE);
#line 2334 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = gtk_tree_view_get_model ((GtkTreeView*) self);
#line 2334 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = _tmp0_;
#line 2334 "../src/Widgets/SourceList/SourceList.vala"
	_tmp3_ = gtk_tree_model_get_iter (_tmp1_, &_tmp2_, path);
#line 2334 "../src/Widgets/SourceList/SourceList.vala"
	iter = _tmp2_;
#line 2334 "../src/Widgets/SourceList/SourceList.vala"
	if (!_tmp3_) {
#line 2335 "../src/Widgets/SourceList/SourceList.vala"
		result = FALSE;
#line 2335 "../src/Widgets/SourceList/SourceList.vala"
		return result;
#line 11941 "SourceList.c"
	}
#line 2338 "../src/Widgets/SourceList/SourceList.vala"
	_tmp4_ = self->priv->primary_expander_cell;
#line 2338 "../src/Widgets/SourceList/SourceList.vala"
	_tmp5_ = gtk_tree_view_get_model ((GtkTreeView*) self);
#line 2338 "../src/Widgets/SourceList/SourceList.vala"
	_tmp6_ = _tmp5_;
#line 2338 "../src/Widgets/SourceList/SourceList.vala"
	_tmp7_ = iter;
#line 2338 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_tree_expander_cell_data_func (self, (GtkCellLayout*) col, (GtkCellRenderer*) _tmp4_, _tmp6_, &_tmp7_);
#line 2340 "../src/Widgets/SourceList/SourceList.vala"
	_tmp8_ = self->priv->primary_expander_cell;
#line 2340 "../src/Widgets/SourceList/SourceList.vala"
	_tmp9_ = gtk_cell_renderer_get_visible ((GtkCellRenderer*) _tmp8_);
#line 2340 "../src/Widgets/SourceList/SourceList.vala"
	_tmp10_ = _tmp9_;
#line 2340 "../src/Widgets/SourceList/SourceList.vala"
	if (!_tmp10_) {
#line 2341 "../src/Widgets/SourceList/SourceList.vala"
		result = FALSE;
#line 2341 "../src/Widgets/SourceList/SourceList.vala"
		return result;
#line 11965 "SourceList.c"
	}
#line 2344 "../src/Widgets/SourceList/SourceList.vala"
	_tmp11_ = gtk_tree_view_get_model ((GtkTreeView*) self);
#line 2344 "../src/Widgets/SourceList/SourceList.vala"
	_tmp12_ = _tmp11_;
#line 2344 "../src/Widgets/SourceList/SourceList.vala"
	_tmp13_ = iter;
#line 2344 "../src/Widgets/SourceList/SourceList.vala"
	if (gtk_tree_model_iter_n_children (_tmp12_, &_tmp13_) < 1) {
#line 2345 "../src/Widgets/SourceList/SourceList.vala"
		result = FALSE;
#line 2345 "../src/Widgets/SourceList/SourceList.vala"
		return result;
#line 11979 "SourceList.c"
	}
#line 2353 "../src/Widgets/SourceList/SourceList.vala"
	indentation_level = gtk_tree_path_get_depth (path);
#line 2354 "../src/Widgets/SourceList/SourceList.vala"
	_tmp14_ = self->priv->spacer_cells;
#line 2354 "../src/Widgets/SourceList/SourceList.vala"
	_tmp15_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp14_, (gpointer) ((gintptr) indentation_level));
#line 2354 "../src/Widgets/SourceList/SourceList.vala"
	last_spacer_cell = (CodeWidgetsSourceListCellRendererSpacer*) _tmp15_;
#line 2356 "../src/Widgets/SourceList/SourceList.vala"
	_tmp16_ = last_spacer_cell;
#line 2356 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp16_ != NULL) {
#line 11993 "SourceList.c"
		gint cell_x = 0;
		gint cell_width = 0;
		CodeWidgetsSourceListCellRendererSpacer* _tmp17_;
		gint _tmp18_ = 0;
		gint _tmp19_ = 0;
		gboolean _tmp20_;
#line 2359 "../src/Widgets/SourceList/SourceList.vala"
		_tmp17_ = last_spacer_cell;
#line 2359 "../src/Widgets/SourceList/SourceList.vala"
		_tmp20_ = gtk_tree_view_column_cell_get_position (col, (GtkCellRenderer*) _tmp17_, &_tmp18_, &_tmp19_);
#line 2359 "../src/Widgets/SourceList/SourceList.vala"
		cell_x = _tmp18_;
#line 2359 "../src/Widgets/SourceList/SourceList.vala"
		cell_width = _tmp19_;
#line 2359 "../src/Widgets/SourceList/SourceList.vala"
		if (_tmp20_) {
#line 12010 "SourceList.c"
			gint expander_width = 0;
			CodeWidgetsCellRendererExpander* _tmp21_;
			GtkTextDirection dir = 0;
			gboolean _tmp23_ = FALSE;
#line 2361 "../src/Widgets/SourceList/SourceList.vala"
			_tmp21_ = self->priv->primary_expander_cell;
#line 2361 "../src/Widgets/SourceList/SourceList.vala"
			expander_width = code_widgets_source_list_tree_get_cell_width (self, (GtkCellRenderer*) _tmp21_) + 1;
#line 2363 "../src/Widgets/SourceList/SourceList.vala"
			dir = gtk_widget_get_direction ((GtkWidget*) self);
#line 2364 "../src/Widgets/SourceList/SourceList.vala"
			if (dir == GTK_TEXT_DIR_NONE) {
#line 2365 "../src/Widgets/SourceList/SourceList.vala"
				dir = gtk_widget_get_default_direction ();
#line 12025 "SourceList.c"
			}
#line 2368 "../src/Widgets/SourceList/SourceList.vala"
			if (dir == GTK_TEXT_DIR_LTR) {
#line 12029 "SourceList.c"
				gint indentation_offset = 0;
				gboolean _tmp22_ = FALSE;
#line 2369 "../src/Widgets/SourceList/SourceList.vala"
				indentation_offset = cell_x + cell_width;
#line 2370 "../src/Widgets/SourceList/SourceList.vala"
				if (x >= indentation_offset) {
#line 2370 "../src/Widgets/SourceList/SourceList.vala"
					_tmp22_ = x <= (indentation_offset + expander_width);
#line 12038 "SourceList.c"
				} else {
#line 2370 "../src/Widgets/SourceList/SourceList.vala"
					_tmp22_ = FALSE;
#line 12042 "SourceList.c"
				}
#line 2370 "../src/Widgets/SourceList/SourceList.vala"
				result = _tmp22_;
#line 2370 "../src/Widgets/SourceList/SourceList.vala"
				_g_object_unref0 (last_spacer_cell);
#line 2370 "../src/Widgets/SourceList/SourceList.vala"
				return result;
#line 12050 "SourceList.c"
			}
#line 2373 "../src/Widgets/SourceList/SourceList.vala"
			if (x <= cell_x) {
#line 2373 "../src/Widgets/SourceList/SourceList.vala"
				_tmp23_ = x >= (cell_x - expander_width);
#line 12056 "SourceList.c"
			} else {
#line 2373 "../src/Widgets/SourceList/SourceList.vala"
				_tmp23_ = FALSE;
#line 12060 "SourceList.c"
			}
#line 2373 "../src/Widgets/SourceList/SourceList.vala"
			result = _tmp23_;
#line 2373 "../src/Widgets/SourceList/SourceList.vala"
			_g_object_unref0 (last_spacer_cell);
#line 2373 "../src/Widgets/SourceList/SourceList.vala"
			return result;
#line 12068 "SourceList.c"
		}
	}
#line 2377 "../src/Widgets/SourceList/SourceList.vala"
	result = FALSE;
#line 2377 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (last_spacer_cell);
#line 2377 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 12077 "SourceList.c"
}

static gboolean
code_widgets_source_list_tree_over_cell (CodeWidgetsSourceListTree* self,
                                         GtkTreeViewColumn* col,
                                         GtkTreePath* path,
                                         GtkCellRenderer* cell,
                                         gint x)
{
	gint cell_x = 0;
	gint cell_width = 0;
	gboolean found = FALSE;
	gint _tmp0_ = 0;
	gint _tmp1_ = 0;
	gboolean _tmp2_;
	gboolean _tmp3_ = FALSE;
	gboolean _tmp4_ = FALSE;
	gboolean result;
#line 2380 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 2380 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (col != NULL, FALSE);
#line 2380 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (path != NULL, FALSE);
#line 2380 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (cell != NULL, FALSE);
#line 2382 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = gtk_tree_view_column_cell_get_position (col, cell, &_tmp0_, &_tmp1_);
#line 2382 "../src/Widgets/SourceList/SourceList.vala"
	cell_x = _tmp0_;
#line 2382 "../src/Widgets/SourceList/SourceList.vala"
	cell_width = _tmp1_;
#line 2382 "../src/Widgets/SourceList/SourceList.vala"
	found = _tmp2_;
#line 2383 "../src/Widgets/SourceList/SourceList.vala"
	if (found) {
#line 2383 "../src/Widgets/SourceList/SourceList.vala"
		_tmp4_ = x > cell_x;
#line 12116 "SourceList.c"
	} else {
#line 2383 "../src/Widgets/SourceList/SourceList.vala"
		_tmp4_ = FALSE;
#line 12120 "SourceList.c"
	}
#line 2383 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp4_) {
#line 2383 "../src/Widgets/SourceList/SourceList.vala"
		_tmp3_ = x < (cell_x + cell_width);
#line 12126 "SourceList.c"
	} else {
#line 2383 "../src/Widgets/SourceList/SourceList.vala"
		_tmp3_ = FALSE;
#line 12130 "SourceList.c"
	}
#line 2383 "../src/Widgets/SourceList/SourceList.vala"
	result = _tmp3_;
#line 2383 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 12136 "SourceList.c"
}

static gint
code_widgets_source_list_tree_get_cell_width (CodeWidgetsSourceListTree* self,
                                              GtkCellRenderer* cell_renderer)
{
	GtkRequisition min_req = {0};
	GtkRequisition _tmp0_ = {0};
	GtkRequisition _tmp1_;
	gint result;
#line 2386 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 2386 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (cell_renderer != NULL, 0);
#line 2388 "../src/Widgets/SourceList/SourceList.vala"
	gtk_cell_renderer_get_preferred_size (cell_renderer, (GtkWidget*) self, &_tmp0_, NULL);
#line 2388 "../src/Widgets/SourceList/SourceList.vala"
	min_req = _tmp0_;
#line 2389 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = min_req;
#line 2389 "../src/Widgets/SourceList/SourceList.vala"
	result = _tmp1_.width;
#line 2389 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 12161 "SourceList.c"
}

static gboolean
code_widgets_source_list_tree_real_popup_menu (GtkWidget* base)
{
	CodeWidgetsSourceListTree * self;
	gboolean result;
#line 2392 "../src/Widgets/SourceList/SourceList.vala"
	self = (CodeWidgetsSourceListTree*) base;
#line 2393 "../src/Widgets/SourceList/SourceList.vala"
	result = code_widgets_source_list_tree_popup_context_menu (self, NULL, NULL);
#line 2393 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 12175 "SourceList.c"
}

static gboolean
code_widgets_source_list_tree_popup_context_menu (CodeWidgetsSourceListTree* self,
                                                  CodeWidgetsSourceListItem* item,
                                                  GdkEventButton* event)
{
	gboolean result;
#line 2396 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 2397 "../src/Widgets/SourceList/SourceList.vala"
	if (item == NULL) {
#line 12188 "SourceList.c"
		CodeWidgetsSourceListItem* _tmp0_;
		CodeWidgetsSourceListItem* _tmp1_;
#line 2398 "../src/Widgets/SourceList/SourceList.vala"
		_tmp0_ = code_widgets_source_list_tree_get_selected_item (self);
#line 2398 "../src/Widgets/SourceList/SourceList.vala"
		_tmp1_ = _tmp0_;
#line 2398 "../src/Widgets/SourceList/SourceList.vala"
		item = _tmp1_;
#line 12197 "SourceList.c"
	}
#line 2400 "../src/Widgets/SourceList/SourceList.vala"
	if (item != NULL) {
#line 12201 "SourceList.c"
		GtkMenu* menu = NULL;
		GtkMenu* _tmp2_;
		GtkMenu* _tmp3_;
#line 2401 "../src/Widgets/SourceList/SourceList.vala"
		_tmp2_ = code_widgets_source_list_item_get_context_menu (item);
#line 2401 "../src/Widgets/SourceList/SourceList.vala"
		menu = _tmp2_;
#line 2402 "../src/Widgets/SourceList/SourceList.vala"
		_tmp3_ = menu;
#line 2402 "../src/Widgets/SourceList/SourceList.vala"
		if (_tmp3_ != NULL) {
#line 12213 "SourceList.c"
			GtkMenu* _tmp4_;
			GtkMenu* _tmp5_;
#line 2403 "../src/Widgets/SourceList/SourceList.vala"
			_tmp4_ = menu;
#line 2403 "../src/Widgets/SourceList/SourceList.vala"
			g_object_set (_tmp4_, "attach-widget", (GtkWidget*) self, NULL);
#line 2404 "../src/Widgets/SourceList/SourceList.vala"
			_tmp5_ = menu;
#line 2404 "../src/Widgets/SourceList/SourceList.vala"
			gtk_menu_popup_at_pointer (_tmp5_, (GdkEvent*) event);
#line 2405 "../src/Widgets/SourceList/SourceList.vala"
			if (event == NULL) {
#line 12226 "SourceList.c"
				GtkMenu* _tmp6_;
#line 2406 "../src/Widgets/SourceList/SourceList.vala"
				_tmp6_ = menu;
#line 2406 "../src/Widgets/SourceList/SourceList.vala"
				gtk_menu_shell_select_first ((GtkMenuShell*) _tmp6_, FALSE);
#line 12232 "SourceList.c"
			}
#line 2409 "../src/Widgets/SourceList/SourceList.vala"
			result = TRUE;
#line 2409 "../src/Widgets/SourceList/SourceList.vala"
			_g_object_unref0 (menu);
#line 2409 "../src/Widgets/SourceList/SourceList.vala"
			return result;
#line 12240 "SourceList.c"
		}
#line 2400 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (menu);
#line 12244 "SourceList.c"
	}
#line 2413 "../src/Widgets/SourceList/SourceList.vala"
	result = FALSE;
#line 2413 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 12250 "SourceList.c"
}

static CodeWidgetsSourceListItem*
code_widgets_source_list_tree_get_item_from_model (GtkTreeModel* model,
                                                   GtkTreeIter* iter)
{
	CodeWidgetsSourceListDataModel* data_model = NULL;
	CodeWidgetsSourceListDataModel* _tmp0_;
	GtkTreeIter _tmp1_;
	CodeWidgetsSourceListItem* _tmp2_;
	CodeWidgetsSourceListItem* result;
#line 2416 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (model != NULL, NULL);
#line 2416 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (iter != NULL, NULL);
#line 2417 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = _g_object_ref0 (CODE_WIDGETS_SOURCE_LIST_IS_DATA_MODEL (model) ? ((CodeWidgetsSourceListDataModel*) model) : NULL);
#line 2417 "../src/Widgets/SourceList/SourceList.vala"
	data_model = _tmp0_;
#line 2418 "../src/Widgets/SourceList/SourceList.vala"
	_vala_assert (data_model != NULL, "data_model != null");
#line 2419 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = *iter;
#line 2419 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = code_widgets_source_list_data_model_get_item (data_model, &_tmp1_);
#line 2419 "../src/Widgets/SourceList/SourceList.vala"
	result = _tmp2_;
#line 2419 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (data_model);
#line 2419 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 12282 "SourceList.c"
}

static void
code_widgets_source_list_tree_spacer_cell_data_func (GtkCellLayout* layout,
                                                     GtkCellRenderer* renderer,
                                                     GtkTreeModel* model,
                                                     GtkTreeIter* iter)
{
	CodeWidgetsSourceListCellRendererSpacer* spacer = NULL;
	CodeWidgetsSourceListCellRendererSpacer* _tmp0_;
	CodeWidgetsSourceListCellRendererSpacer* _tmp1_;
	CodeWidgetsSourceListCellRendererSpacer* _tmp2_;
	gint _tmp3_;
	gint _tmp4_;
	GtkTreePath* path = NULL;
	GtkTreeIter _tmp5_;
	GtkTreePath* _tmp6_;
	gint level = 0;
	GtkTreePath* _tmp7_;
	CodeWidgetsSourceListCellRendererSpacer* _tmp9_;
	gint _tmp10_;
	gint _tmp11_;
#line 2422 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (layout != NULL);
#line 2422 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (renderer != NULL);
#line 2422 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (model != NULL);
#line 2422 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (iter != NULL);
#line 2428 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = _g_object_ref0 (CODE_WIDGETS_SOURCE_LIST_IS_CELL_RENDERER_SPACER (renderer) ? ((CodeWidgetsSourceListCellRendererSpacer*) renderer) : NULL);
#line 2428 "../src/Widgets/SourceList/SourceList.vala"
	spacer = _tmp0_;
#line 2429 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = spacer;
#line 2429 "../src/Widgets/SourceList/SourceList.vala"
	_vala_assert (_tmp1_ != NULL, "spacer != null");
#line 2430 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = spacer;
#line 2430 "../src/Widgets/SourceList/SourceList.vala"
	_tmp3_ = code_widgets_source_list_cell_renderer_spacer_get_level (_tmp2_);
#line 2430 "../src/Widgets/SourceList/SourceList.vala"
	_tmp4_ = _tmp3_;
#line 2430 "../src/Widgets/SourceList/SourceList.vala"
	_vala_assert (_tmp4_ > 0, "spacer.level > 0");
#line 2432 "../src/Widgets/SourceList/SourceList.vala"
	_tmp5_ = *iter;
#line 2432 "../src/Widgets/SourceList/SourceList.vala"
	_tmp6_ = gtk_tree_model_get_path (model, &_tmp5_);
#line 2432 "../src/Widgets/SourceList/SourceList.vala"
	path = _tmp6_;
#line 2434 "../src/Widgets/SourceList/SourceList.vala"
	level = -1;
#line 2435 "../src/Widgets/SourceList/SourceList.vala"
	_tmp7_ = path;
#line 2435 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp7_ != NULL) {
#line 12341 "SourceList.c"
		GtkTreePath* _tmp8_;
#line 2436 "../src/Widgets/SourceList/SourceList.vala"
		_tmp8_ = path;
#line 2436 "../src/Widgets/SourceList/SourceList.vala"
		level = gtk_tree_path_get_depth (_tmp8_);
#line 12347 "SourceList.c"
	}
#line 2438 "../src/Widgets/SourceList/SourceList.vala"
	_tmp9_ = spacer;
#line 2438 "../src/Widgets/SourceList/SourceList.vala"
	_tmp10_ = code_widgets_source_list_cell_renderer_spacer_get_level (_tmp9_);
#line 2438 "../src/Widgets/SourceList/SourceList.vala"
	_tmp11_ = _tmp10_;
#line 2438 "../src/Widgets/SourceList/SourceList.vala"
	gtk_cell_renderer_set_visible (renderer, _tmp11_ <= level);
#line 2422 "../src/Widgets/SourceList/SourceList.vala"
	__vala_GtkTreePath_free0 (path);
#line 2422 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (spacer);
#line 12361 "SourceList.c"
}

static void
code_widgets_source_list_tree_name_cell_data_func (CodeWidgetsSourceListTree* self,
                                                   GtkCellLayout* layout,
                                                   GtkCellRenderer* renderer,
                                                   GtkTreeModel* model,
                                                   GtkTreeIter* iter)
{
	GtkCellRendererText* text_renderer = NULL;
	GtkCellRendererText* _tmp0_;
	GtkCellRendererText* _tmp1_;
	GString* text = NULL;
	GString* _tmp2_;
	PangoWeight weight = 0;
	gboolean use_markup = FALSE;
	CodeWidgetsSourceListItem* item = NULL;
	GtkTreeIter _tmp3_;
	CodeWidgetsSourceListItem* _tmp4_;
	CodeWidgetsSourceListItem* _tmp5_;
	GtkCellRendererText* _tmp20_;
#line 2441 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 2441 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (layout != NULL);
#line 2441 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (renderer != NULL);
#line 2441 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (model != NULL);
#line 2441 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (iter != NULL);
#line 2447 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (renderer, gtk_cell_renderer_text_get_type ()) ? ((GtkCellRendererText*) renderer) : NULL);
#line 2447 "../src/Widgets/SourceList/SourceList.vala"
	text_renderer = _tmp0_;
#line 2448 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = text_renderer;
#line 2448 "../src/Widgets/SourceList/SourceList.vala"
	_vala_assert (_tmp1_ != NULL, "text_renderer != null");
#line 2450 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = g_string_new ("");
#line 2450 "../src/Widgets/SourceList/SourceList.vala"
	text = _tmp2_;
#line 2451 "../src/Widgets/SourceList/SourceList.vala"
	weight = PANGO_WEIGHT_NORMAL;
#line 2452 "../src/Widgets/SourceList/SourceList.vala"
	use_markup = FALSE;
#line 2454 "../src/Widgets/SourceList/SourceList.vala"
	_tmp3_ = *iter;
#line 2454 "../src/Widgets/SourceList/SourceList.vala"
	_tmp4_ = code_widgets_source_list_tree_get_item_from_model (model, &_tmp3_);
#line 2454 "../src/Widgets/SourceList/SourceList.vala"
	item = _tmp4_;
#line 2455 "../src/Widgets/SourceList/SourceList.vala"
	_tmp5_ = item;
#line 2455 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp5_ != NULL) {
#line 12419 "SourceList.c"
		CodeWidgetsSourceListItem* _tmp6_;
		const gchar* _tmp7_;
		const gchar* _tmp8_;
		CodeWidgetsSourceListDataModel* _tmp17_;
		CodeWidgetsSourceListItem* _tmp18_;
		GtkTreeIter _tmp19_;
#line 2456 "../src/Widgets/SourceList/SourceList.vala"
		_tmp6_ = item;
#line 2456 "../src/Widgets/SourceList/SourceList.vala"
		_tmp7_ = code_widgets_source_list_item_get_markup (_tmp6_);
#line 2456 "../src/Widgets/SourceList/SourceList.vala"
		_tmp8_ = _tmp7_;
#line 2456 "../src/Widgets/SourceList/SourceList.vala"
		if (_tmp8_ != NULL) {
#line 12434 "SourceList.c"
			GString* _tmp9_;
			CodeWidgetsSourceListItem* _tmp10_;
			const gchar* _tmp11_;
			const gchar* _tmp12_;
#line 2457 "../src/Widgets/SourceList/SourceList.vala"
			_tmp9_ = text;
#line 2457 "../src/Widgets/SourceList/SourceList.vala"
			_tmp10_ = item;
#line 2457 "../src/Widgets/SourceList/SourceList.vala"
			_tmp11_ = code_widgets_source_list_item_get_markup (_tmp10_);
#line 2457 "../src/Widgets/SourceList/SourceList.vala"
			_tmp12_ = _tmp11_;
#line 2457 "../src/Widgets/SourceList/SourceList.vala"
			g_string_append (_tmp9_, _tmp12_);
#line 2458 "../src/Widgets/SourceList/SourceList.vala"
			use_markup = TRUE;
#line 12451 "SourceList.c"
		} else {
			GString* _tmp13_;
			CodeWidgetsSourceListItem* _tmp14_;
			const gchar* _tmp15_;
			const gchar* _tmp16_;
#line 2460 "../src/Widgets/SourceList/SourceList.vala"
			_tmp13_ = text;
#line 2460 "../src/Widgets/SourceList/SourceList.vala"
			_tmp14_ = item;
#line 2460 "../src/Widgets/SourceList/SourceList.vala"
			_tmp15_ = code_widgets_source_list_item_get_name (_tmp14_);
#line 2460 "../src/Widgets/SourceList/SourceList.vala"
			_tmp16_ = _tmp15_;
#line 2460 "../src/Widgets/SourceList/SourceList.vala"
			g_string_append (_tmp13_, _tmp16_);
#line 12467 "SourceList.c"
		}
#line 2463 "../src/Widgets/SourceList/SourceList.vala"
		_tmp17_ = self->priv->_data_model;
#line 2463 "../src/Widgets/SourceList/SourceList.vala"
		_tmp18_ = item;
#line 2463 "../src/Widgets/SourceList/SourceList.vala"
		_tmp19_ = *iter;
#line 2463 "../src/Widgets/SourceList/SourceList.vala"
		if (code_widgets_source_list_data_model_is_category (_tmp17_, _tmp18_, &_tmp19_, NULL)) {
#line 2464 "../src/Widgets/SourceList/SourceList.vala"
			weight = PANGO_WEIGHT_BOLD;
#line 12479 "SourceList.c"
		}
	}
#line 2467 "../src/Widgets/SourceList/SourceList.vala"
	_tmp20_ = text_renderer;
#line 2467 "../src/Widgets/SourceList/SourceList.vala"
	g_object_set (_tmp20_, "weight", (gint) weight, NULL);
#line 2468 "../src/Widgets/SourceList/SourceList.vala"
	if (use_markup) {
#line 12488 "SourceList.c"
		GtkCellRendererText* _tmp21_;
		GString* _tmp22_;
		const gchar* _tmp23_;
#line 2469 "../src/Widgets/SourceList/SourceList.vala"
		_tmp21_ = text_renderer;
#line 2469 "../src/Widgets/SourceList/SourceList.vala"
		_tmp22_ = text;
#line 2469 "../src/Widgets/SourceList/SourceList.vala"
		_tmp23_ = _tmp22_->str;
#line 2469 "../src/Widgets/SourceList/SourceList.vala"
		g_object_set (_tmp21_, "markup", _tmp23_, NULL);
#line 12500 "SourceList.c"
	} else {
		GtkCellRendererText* _tmp24_;
		GString* _tmp25_;
		const gchar* _tmp26_;
#line 2471 "../src/Widgets/SourceList/SourceList.vala"
		_tmp24_ = text_renderer;
#line 2471 "../src/Widgets/SourceList/SourceList.vala"
		_tmp25_ = text;
#line 2471 "../src/Widgets/SourceList/SourceList.vala"
		_tmp26_ = _tmp25_->str;
#line 2471 "../src/Widgets/SourceList/SourceList.vala"
		g_object_set (_tmp24_, "text", _tmp26_, NULL);
#line 12513 "SourceList.c"
	}
#line 2441 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (item);
#line 2441 "../src/Widgets/SourceList/SourceList.vala"
	_g_string_free0 (text);
#line 2441 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (text_renderer);
#line 12521 "SourceList.c"
}

static void
code_widgets_source_list_tree_badge_cell_data_func (CodeWidgetsSourceListTree* self,
                                                    GtkCellLayout* layout,
                                                    GtkCellRenderer* renderer,
                                                    GtkTreeModel* model,
                                                    GtkTreeIter* iter)
{
	CodeWidgetsCellRendererBadge* badge_renderer = NULL;
	CodeWidgetsCellRendererBadge* _tmp0_;
	CodeWidgetsCellRendererBadge* _tmp1_;
	gchar* text = NULL;
	gchar* _tmp2_;
	gboolean visible = FALSE;
	CodeWidgetsSourceListItem* item = NULL;
	GtkTreeIter _tmp3_;
	CodeWidgetsSourceListItem* _tmp4_;
	CodeWidgetsSourceListItem* _tmp5_;
	CodeWidgetsCellRendererBadge* _tmp23_;
	CodeWidgetsCellRendererBadge* _tmp24_;
	const gchar* _tmp25_;
#line 2475 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 2475 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (layout != NULL);
#line 2475 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (renderer != NULL);
#line 2475 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (model != NULL);
#line 2475 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (iter != NULL);
#line 2481 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = _g_object_ref0 (CODE_WIDGETS_IS_CELL_RENDERER_BADGE (renderer) ? ((CodeWidgetsCellRendererBadge*) renderer) : NULL);
#line 2481 "../src/Widgets/SourceList/SourceList.vala"
	badge_renderer = _tmp0_;
#line 2482 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = badge_renderer;
#line 2482 "../src/Widgets/SourceList/SourceList.vala"
	_vala_assert (_tmp1_ != NULL, "badge_renderer != null");
#line 2484 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = g_strdup ("");
#line 2484 "../src/Widgets/SourceList/SourceList.vala"
	text = _tmp2_;
#line 2485 "../src/Widgets/SourceList/SourceList.vala"
	visible = FALSE;
#line 2487 "../src/Widgets/SourceList/SourceList.vala"
	_tmp3_ = *iter;
#line 2487 "../src/Widgets/SourceList/SourceList.vala"
	_tmp4_ = code_widgets_source_list_tree_get_item_from_model (model, &_tmp3_);
#line 2487 "../src/Widgets/SourceList/SourceList.vala"
	item = _tmp4_;
#line 2488 "../src/Widgets/SourceList/SourceList.vala"
	_tmp5_ = item;
#line 2488 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp5_ != NULL) {
#line 12578 "SourceList.c"
		gboolean _tmp6_ = FALSE;
		gboolean _tmp7_ = FALSE;
		CodeWidgetsSourceListDataModel* _tmp8_;
		CodeWidgetsSourceListItem* _tmp9_;
		GtkTreeIter _tmp10_;
#line 2490 "../src/Widgets/SourceList/SourceList.vala"
		_tmp8_ = self->priv->_data_model;
#line 2490 "../src/Widgets/SourceList/SourceList.vala"
		_tmp9_ = item;
#line 2490 "../src/Widgets/SourceList/SourceList.vala"
		_tmp10_ = *iter;
#line 2490 "../src/Widgets/SourceList/SourceList.vala"
		if (!code_widgets_source_list_data_model_is_category (_tmp8_, _tmp9_, &_tmp10_, NULL)) {
#line 12592 "SourceList.c"
			CodeWidgetsSourceListItem* _tmp11_;
			const gchar* _tmp12_;
			const gchar* _tmp13_;
#line 2491 "../src/Widgets/SourceList/SourceList.vala"
			_tmp11_ = item;
#line 2491 "../src/Widgets/SourceList/SourceList.vala"
			_tmp12_ = code_widgets_source_list_item_get_badge (_tmp11_);
#line 2491 "../src/Widgets/SourceList/SourceList.vala"
			_tmp13_ = _tmp12_;
#line 2491 "../src/Widgets/SourceList/SourceList.vala"
			_tmp7_ = _tmp13_ != NULL;
#line 12604 "SourceList.c"
		} else {
#line 2490 "../src/Widgets/SourceList/SourceList.vala"
			_tmp7_ = FALSE;
#line 12608 "SourceList.c"
		}
#line 2490 "../src/Widgets/SourceList/SourceList.vala"
		if (_tmp7_) {
#line 12612 "SourceList.c"
			CodeWidgetsSourceListItem* _tmp14_;
			const gchar* _tmp15_;
			const gchar* _tmp16_;
			gchar* _tmp17_;
			gchar* _tmp18_;
#line 2492 "../src/Widgets/SourceList/SourceList.vala"
			_tmp14_ = item;
#line 2492 "../src/Widgets/SourceList/SourceList.vala"
			_tmp15_ = code_widgets_source_list_item_get_badge (_tmp14_);
#line 2492 "../src/Widgets/SourceList/SourceList.vala"
			_tmp16_ = _tmp15_;
#line 2492 "../src/Widgets/SourceList/SourceList.vala"
			_tmp17_ = string_strip (_tmp16_);
#line 2492 "../src/Widgets/SourceList/SourceList.vala"
			_tmp18_ = _tmp17_;
#line 2492 "../src/Widgets/SourceList/SourceList.vala"
			_tmp6_ = g_strcmp0 (_tmp18_, "") != 0;
#line 2492 "../src/Widgets/SourceList/SourceList.vala"
			_g_free0 (_tmp18_);
#line 12632 "SourceList.c"
		} else {
#line 2490 "../src/Widgets/SourceList/SourceList.vala"
			_tmp6_ = FALSE;
#line 12636 "SourceList.c"
		}
#line 2490 "../src/Widgets/SourceList/SourceList.vala"
		visible = _tmp6_;
#line 2494 "../src/Widgets/SourceList/SourceList.vala"
		if (visible) {
#line 12642 "SourceList.c"
			CodeWidgetsSourceListItem* _tmp19_;
			const gchar* _tmp20_;
			const gchar* _tmp21_;
			gchar* _tmp22_;
#line 2495 "../src/Widgets/SourceList/SourceList.vala"
			_tmp19_ = item;
#line 2495 "../src/Widgets/SourceList/SourceList.vala"
			_tmp20_ = code_widgets_source_list_item_get_badge (_tmp19_);
#line 2495 "../src/Widgets/SourceList/SourceList.vala"
			_tmp21_ = _tmp20_;
#line 2495 "../src/Widgets/SourceList/SourceList.vala"
			_tmp22_ = g_strdup (_tmp21_);
#line 2495 "../src/Widgets/SourceList/SourceList.vala"
			_g_free0 (text);
#line 2495 "../src/Widgets/SourceList/SourceList.vala"
			text = _tmp22_;
#line 12659 "SourceList.c"
		}
	}
#line 2498 "../src/Widgets/SourceList/SourceList.vala"
	_tmp23_ = badge_renderer;
#line 2498 "../src/Widgets/SourceList/SourceList.vala"
	gtk_cell_renderer_set_visible ((GtkCellRenderer*) _tmp23_, visible);
#line 2499 "../src/Widgets/SourceList/SourceList.vala"
	_tmp24_ = badge_renderer;
#line 2499 "../src/Widgets/SourceList/SourceList.vala"
	_tmp25_ = text;
#line 2499 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_cell_renderer_badge_set_text (_tmp24_, _tmp25_);
#line 2475 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (item);
#line 2475 "../src/Widgets/SourceList/SourceList.vala"
	_g_free0 (text);
#line 2475 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (badge_renderer);
#line 12678 "SourceList.c"
}

static void
code_widgets_source_list_tree_icon_cell_data_func (CodeWidgetsSourceListTree* self,
                                                   GtkCellLayout* layout,
                                                   GtkCellRenderer* renderer,
                                                   GtkTreeModel* model,
                                                   GtkTreeIter* iter)
{
	CodeWidgetsSourceListCellRendererIcon* icon_renderer = NULL;
	CodeWidgetsSourceListCellRendererIcon* _tmp0_;
	CodeWidgetsSourceListCellRendererIcon* _tmp1_;
	gboolean visible = FALSE;
	GIcon* icon = NULL;
	CodeWidgetsSourceListItem* item = NULL;
	GtkTreeIter _tmp2_;
	CodeWidgetsSourceListItem* _tmp3_;
	CodeWidgetsSourceListItem* _tmp4_;
	gboolean _tmp20_ = FALSE;
	CodeWidgetsSourceListCellRendererIcon* _tmp22_;
	GIcon* _tmp23_ = NULL;
	CodeWidgetsSourceListCellRendererIcon* _tmp25_;
#line 2502 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 2502 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (layout != NULL);
#line 2502 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (renderer != NULL);
#line 2502 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (model != NULL);
#line 2502 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (iter != NULL);
#line 2507 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = _g_object_ref0 (CODE_WIDGETS_SOURCE_LIST_IS_CELL_RENDERER_ICON (renderer) ? ((CodeWidgetsSourceListCellRendererIcon*) renderer) : NULL);
#line 2507 "../src/Widgets/SourceList/SourceList.vala"
	icon_renderer = _tmp0_;
#line 2508 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = icon_renderer;
#line 2508 "../src/Widgets/SourceList/SourceList.vala"
	_vala_assert (_tmp1_ != NULL, "icon_renderer != null");
#line 2510 "../src/Widgets/SourceList/SourceList.vala"
	visible = FALSE;
#line 2511 "../src/Widgets/SourceList/SourceList.vala"
	icon = NULL;
#line 2513 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = *iter;
#line 2513 "../src/Widgets/SourceList/SourceList.vala"
	_tmp3_ = code_widgets_source_list_tree_get_item_from_model (model, &_tmp2_);
#line 2513 "../src/Widgets/SourceList/SourceList.vala"
	item = _tmp3_;
#line 2514 "../src/Widgets/SourceList/SourceList.vala"
	_tmp4_ = item;
#line 2514 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp4_ != NULL) {
#line 12733 "SourceList.c"
		CodeWidgetsSourceListDataModel* _tmp5_;
		CodeWidgetsSourceListItem* _tmp6_;
		GtkTreeIter _tmp7_;
#line 2516 "../src/Widgets/SourceList/SourceList.vala"
		_tmp5_ = self->priv->_data_model;
#line 2516 "../src/Widgets/SourceList/SourceList.vala"
		_tmp6_ = item;
#line 2516 "../src/Widgets/SourceList/SourceList.vala"
		_tmp7_ = *iter;
#line 2516 "../src/Widgets/SourceList/SourceList.vala"
		visible = !code_widgets_source_list_data_model_is_category (_tmp5_, _tmp6_, &_tmp7_, NULL);
#line 2518 "../src/Widgets/SourceList/SourceList.vala"
		if (visible) {
#line 12747 "SourceList.c"
			CodeWidgetsSourceListCellRendererIcon* _tmp8_;
			CodeWidgetsSourceListCellRendererIcon* _tmp9_;
#line 2519 "../src/Widgets/SourceList/SourceList.vala"
			_tmp8_ = icon_renderer;
#line 2519 "../src/Widgets/SourceList/SourceList.vala"
			_tmp9_ = self->priv->icon_cell;
#line 2519 "../src/Widgets/SourceList/SourceList.vala"
			if (_tmp8_ == _tmp9_) {
#line 12756 "SourceList.c"
				CodeWidgetsSourceListItem* _tmp10_;
				GIcon* _tmp11_;
				GIcon* _tmp12_;
				GIcon* _tmp13_;
#line 2520 "../src/Widgets/SourceList/SourceList.vala"
				_tmp10_ = item;
#line 2520 "../src/Widgets/SourceList/SourceList.vala"
				_tmp11_ = code_widgets_source_list_item_get_icon (_tmp10_);
#line 2520 "../src/Widgets/SourceList/SourceList.vala"
				_tmp12_ = _tmp11_;
#line 2520 "../src/Widgets/SourceList/SourceList.vala"
				_tmp13_ = _g_object_ref0 (_tmp12_);
#line 2520 "../src/Widgets/SourceList/SourceList.vala"
				_g_object_unref0 (icon);
#line 2520 "../src/Widgets/SourceList/SourceList.vala"
				icon = _tmp13_;
#line 12773 "SourceList.c"
			} else {
				CodeWidgetsSourceListCellRendererIcon* _tmp14_;
				CodeWidgetsSourceListCellRendererIcon* _tmp15_;
#line 2521 "../src/Widgets/SourceList/SourceList.vala"
				_tmp14_ = icon_renderer;
#line 2521 "../src/Widgets/SourceList/SourceList.vala"
				_tmp15_ = self->priv->activatable_cell;
#line 2521 "../src/Widgets/SourceList/SourceList.vala"
				if (_tmp14_ == _tmp15_) {
#line 12783 "SourceList.c"
					CodeWidgetsSourceListItem* _tmp16_;
					GIcon* _tmp17_;
					GIcon* _tmp18_;
					GIcon* _tmp19_;
#line 2522 "../src/Widgets/SourceList/SourceList.vala"
					_tmp16_ = item;
#line 2522 "../src/Widgets/SourceList/SourceList.vala"
					_tmp17_ = code_widgets_source_list_item_get_activatable (_tmp16_);
#line 2522 "../src/Widgets/SourceList/SourceList.vala"
					_tmp18_ = _tmp17_;
#line 2522 "../src/Widgets/SourceList/SourceList.vala"
					_tmp19_ = _g_object_ref0 (_tmp18_);
#line 2522 "../src/Widgets/SourceList/SourceList.vala"
					_g_object_unref0 (icon);
#line 2522 "../src/Widgets/SourceList/SourceList.vala"
					icon = _tmp19_;
#line 12800 "SourceList.c"
				} else {
#line 2524 "../src/Widgets/SourceList/SourceList.vala"
					g_assert_not_reached ();
#line 12804 "SourceList.c"
				}
			}
		}
	}
#line 2528 "../src/Widgets/SourceList/SourceList.vala"
	if (visible) {
#line 12811 "SourceList.c"
		GIcon* _tmp21_;
#line 2528 "../src/Widgets/SourceList/SourceList.vala"
		_tmp21_ = icon;
#line 2528 "../src/Widgets/SourceList/SourceList.vala"
		_tmp20_ = _tmp21_ != NULL;
#line 12817 "SourceList.c"
	} else {
#line 2528 "../src/Widgets/SourceList/SourceList.vala"
		_tmp20_ = FALSE;
#line 12821 "SourceList.c"
	}
#line 2528 "../src/Widgets/SourceList/SourceList.vala"
	visible = _tmp20_;
#line 2530 "../src/Widgets/SourceList/SourceList.vala"
	_tmp22_ = icon_renderer;
#line 2530 "../src/Widgets/SourceList/SourceList.vala"
	gtk_cell_renderer_set_visible ((GtkCellRenderer*) _tmp22_, visible);
#line 2531 "../src/Widgets/SourceList/SourceList.vala"
	if (visible) {
#line 12831 "SourceList.c"
		GIcon* _tmp24_;
#line 2531 "../src/Widgets/SourceList/SourceList.vala"
		_tmp24_ = icon;
#line 2531 "../src/Widgets/SourceList/SourceList.vala"
		_tmp23_ = _tmp24_;
#line 12837 "SourceList.c"
	} else {
#line 2531 "../src/Widgets/SourceList/SourceList.vala"
		_tmp23_ = NULL;
#line 12841 "SourceList.c"
	}
#line 2531 "../src/Widgets/SourceList/SourceList.vala"
	_tmp25_ = icon_renderer;
#line 2531 "../src/Widgets/SourceList/SourceList.vala"
	g_object_set ((GtkCellRendererPixbuf*) _tmp25_, "gicon", _tmp23_, NULL);
#line 2502 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (item);
#line 2502 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (icon);
#line 2502 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (icon_renderer);
#line 12853 "SourceList.c"
}

/**
         * Controls expander visibility.
         */
static void
code_widgets_source_list_tree_expander_cell_data_func (CodeWidgetsSourceListTree* self,
                                                       GtkCellLayout* layout,
                                                       GtkCellRenderer* renderer,
                                                       GtkTreeModel* model,
                                                       GtkTreeIter* iter)
{
	CodeWidgetsSourceListItem* item = NULL;
	GtkTreeIter _tmp0_;
	CodeWidgetsSourceListItem* _tmp1_;
	CodeWidgetsSourceListItem* _tmp2_;
	CodeWidgetsCellRendererExpander* _tmp12_;
#line 2537 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 2537 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (layout != NULL);
#line 2537 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (renderer != NULL);
#line 2537 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (model != NULL);
#line 2537 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (iter != NULL);
#line 2543 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = *iter;
#line 2543 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = code_widgets_source_list_tree_get_item_from_model (model, &_tmp0_);
#line 2543 "../src/Widgets/SourceList/SourceList.vala"
	item = _tmp1_;
#line 2544 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = item;
#line 2544 "../src/Widgets/SourceList/SourceList.vala"
	if (_tmp2_ != NULL) {
#line 12891 "SourceList.c"
		CodeWidgetsSourceListExpandableItem* expandable_item = NULL;
		CodeWidgetsSourceListItem* _tmp3_;
		CodeWidgetsSourceListExpandableItem* _tmp4_;
		CodeWidgetsSourceListExpandableItem* _tmp5_;
#line 2548 "../src/Widgets/SourceList/SourceList.vala"
		_tmp3_ = item;
#line 2548 "../src/Widgets/SourceList/SourceList.vala"
		_tmp4_ = _g_object_ref0 (CODE_WIDGETS_SOURCE_LIST_IS_EXPANDABLE_ITEM (_tmp3_) ? ((CodeWidgetsSourceListExpandableItem*) _tmp3_) : NULL);
#line 2548 "../src/Widgets/SourceList/SourceList.vala"
		expandable_item = _tmp4_;
#line 2549 "../src/Widgets/SourceList/SourceList.vala"
		_tmp5_ = expandable_item;
#line 2549 "../src/Widgets/SourceList/SourceList.vala"
		if (_tmp5_ != NULL) {
#line 12906 "SourceList.c"
			gboolean _tmp6_ = FALSE;
			gboolean _tmp7_;
			gboolean _tmp8_;
#line 2550 "../src/Widgets/SourceList/SourceList.vala"
			g_object_get (renderer, "is-expander", &_tmp7_, NULL);
#line 2550 "../src/Widgets/SourceList/SourceList.vala"
			_tmp8_ = _tmp7_;
#line 2550 "../src/Widgets/SourceList/SourceList.vala"
			if (_tmp8_) {
#line 12916 "SourceList.c"
				CodeWidgetsSourceListExpandableItem* _tmp9_;
				gboolean _tmp10_;
				gboolean _tmp11_;
#line 2550 "../src/Widgets/SourceList/SourceList.vala"
				_tmp9_ = expandable_item;
#line 2550 "../src/Widgets/SourceList/SourceList.vala"
				_tmp10_ = code_widgets_source_list_expandable_item_get_collapsible (_tmp9_);
#line 2550 "../src/Widgets/SourceList/SourceList.vala"
				_tmp11_ = _tmp10_;
#line 2550 "../src/Widgets/SourceList/SourceList.vala"
				_tmp6_ = _tmp11_;
#line 12928 "SourceList.c"
			} else {
#line 2550 "../src/Widgets/SourceList/SourceList.vala"
				_tmp6_ = FALSE;
#line 12932 "SourceList.c"
			}
#line 2550 "../src/Widgets/SourceList/SourceList.vala"
			g_object_set (renderer, "is-expander", _tmp6_, NULL);
#line 12936 "SourceList.c"
		}
#line 2544 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (expandable_item);
#line 12940 "SourceList.c"
	}
#line 2553 "../src/Widgets/SourceList/SourceList.vala"
	_tmp12_ = self->priv->primary_expander_cell;
#line 2553 "../src/Widgets/SourceList/SourceList.vala"
	if (renderer == G_TYPE_CHECK_INSTANCE_CAST (_tmp12_, gtk_cell_renderer_get_type (), GtkCellRenderer)) {
#line 12946 "SourceList.c"
		CodeWidgetsSourceListDataModel* _tmp13_;
		GtkTreeIter _tmp14_;
#line 2554 "../src/Widgets/SourceList/SourceList.vala"
		_tmp13_ = self->priv->_data_model;
#line 2554 "../src/Widgets/SourceList/SourceList.vala"
		_tmp14_ = *iter;
#line 2554 "../src/Widgets/SourceList/SourceList.vala"
		gtk_cell_renderer_set_visible (renderer, !code_widgets_source_list_data_model_is_iter_at_root_level (_tmp13_, &_tmp14_));
#line 12955 "SourceList.c"
	} else {
		CodeWidgetsCellRendererExpander* _tmp15_;
#line 2555 "../src/Widgets/SourceList/SourceList.vala"
		_tmp15_ = self->priv->secondary_expander_cell;
#line 2555 "../src/Widgets/SourceList/SourceList.vala"
		if (renderer == G_TYPE_CHECK_INSTANCE_CAST (_tmp15_, gtk_cell_renderer_get_type (), GtkCellRenderer)) {
#line 12962 "SourceList.c"
			CodeWidgetsSourceListDataModel* _tmp16_;
			CodeWidgetsSourceListItem* _tmp17_;
			GtkTreeIter _tmp18_;
#line 2556 "../src/Widgets/SourceList/SourceList.vala"
			_tmp16_ = self->priv->_data_model;
#line 2556 "../src/Widgets/SourceList/SourceList.vala"
			_tmp17_ = item;
#line 2556 "../src/Widgets/SourceList/SourceList.vala"
			_tmp18_ = *iter;
#line 2556 "../src/Widgets/SourceList/SourceList.vala"
			gtk_cell_renderer_set_visible (renderer, code_widgets_source_list_data_model_is_category (_tmp16_, _tmp17_, &_tmp18_, NULL));
#line 12974 "SourceList.c"
		} else {
#line 2558 "../src/Widgets/SourceList/SourceList.vala"
			g_assert_not_reached ();
#line 12978 "SourceList.c"
		}
	}
#line 2537 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (item);
#line 12983 "SourceList.c"
}

static CodeWidgetsSourceListDataModel*
code_widgets_source_list_tree_get_data_model (CodeWidgetsSourceListTree* self)
{
	CodeWidgetsSourceListDataModel* result;
	CodeWidgetsSourceListDataModel* _tmp0_;
#line 1542 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 1542 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->_data_model;
#line 1542 "../src/Widgets/SourceList/SourceList.vala"
	result = _tmp0_;
#line 1542 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 12999 "SourceList.c"
}

static void
code_widgets_source_list_tree_set_data_model (CodeWidgetsSourceListTree* self,
                                              CodeWidgetsSourceListDataModel* value)
{
	CodeWidgetsSourceListDataModel* old_value;
#line 1542 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 1542 "../src/Widgets/SourceList/SourceList.vala"
	old_value = code_widgets_source_list_tree_get_data_model (self);
#line 1542 "../src/Widgets/SourceList/SourceList.vala"
	if (old_value != value) {
#line 13013 "SourceList.c"
		CodeWidgetsSourceListDataModel* _tmp0_;
#line 1542 "../src/Widgets/SourceList/SourceList.vala"
		_tmp0_ = _g_object_ref0 (value);
#line 1542 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (self->priv->_data_model);
#line 1542 "../src/Widgets/SourceList/SourceList.vala"
		self->priv->_data_model = _tmp0_;
#line 1542 "../src/Widgets/SourceList/SourceList.vala"
		g_object_notify_by_pspec ((GObject *) self, code_widgets_source_list_tree_properties[CODE_WIDGETS_SOURCE_LIST_TREE_DATA_MODEL_PROPERTY]);
#line 13023 "SourceList.c"
	}
}

static CodeWidgetsSourceListItem*
code_widgets_source_list_tree_get_selected_item (CodeWidgetsSourceListTree* self)
{
	CodeWidgetsSourceListItem* result;
	CodeWidgetsSourceListItem* _tmp0_;
#line 1547 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 1547 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->selected;
#line 1547 "../src/Widgets/SourceList/SourceList.vala"
	result = _tmp0_;
#line 1547 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 13040 "SourceList.c"
}

static void
code_widgets_source_list_tree_set_selected_item (CodeWidgetsSourceListTree* self,
                                                 CodeWidgetsSourceListItem* value)
{
#line 1548 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 1548 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_tree_set_selected (self, value, TRUE);
#line 1548 "../src/Widgets/SourceList/SourceList.vala"
	g_object_notify_by_pspec ((GObject *) self, code_widgets_source_list_tree_properties[CODE_WIDGETS_SOURCE_LIST_TREE_SELECTED_ITEM_PROPERTY]);
#line 13053 "SourceList.c"
}

static gboolean
code_widgets_source_list_tree_get_editing (CodeWidgetsSourceListTree* self)
{
	gboolean result;
	GtkCellRendererText* _tmp0_;
	gboolean _tmp1_;
	gboolean _tmp2_;
#line 1552 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 1552 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->text_cell;
#line 1552 "../src/Widgets/SourceList/SourceList.vala"
	g_object_get ((GtkCellRenderer*) _tmp0_, "editing", &_tmp1_, NULL);
#line 1552 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = _tmp1_;
#line 1552 "../src/Widgets/SourceList/SourceList.vala"
	result = _tmp2_;
#line 1552 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 13075 "SourceList.c"
}

static PangoEllipsizeMode
code_widgets_source_list_tree_get_ellipsize_mode (CodeWidgetsSourceListTree* self)
{
	PangoEllipsizeMode result;
	GtkCellRendererText* _tmp0_;
	PangoEllipsizeMode _tmp1_;
	PangoEllipsizeMode _tmp2_;
#line 1556 "../src/Widgets/SourceList/SourceList.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 1556 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->text_cell;
#line 1556 "../src/Widgets/SourceList/SourceList.vala"
	g_object_get (_tmp0_, "ellipsize", &_tmp1_, NULL);
#line 1556 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = _tmp1_;
#line 1556 "../src/Widgets/SourceList/SourceList.vala"
	result = _tmp2_;
#line 1556 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 13097 "SourceList.c"
}

static void
code_widgets_source_list_tree_set_ellipsize_mode (CodeWidgetsSourceListTree* self,
                                                  PangoEllipsizeMode value)
{
	GtkCellRendererText* _tmp0_;
#line 1557 "../src/Widgets/SourceList/SourceList.vala"
	g_return_if_fail (self != NULL);
#line 1557 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->text_cell;
#line 1557 "../src/Widgets/SourceList/SourceList.vala"
	g_object_set (_tmp0_, "ellipsize", value, NULL);
#line 1557 "../src/Widgets/SourceList/SourceList.vala"
	g_object_notify_by_pspec ((GObject *) self, code_widgets_source_list_tree_properties[CODE_WIDGETS_SOURCE_LIST_TREE_ELLIPSIZE_MODE_PROPERTY]);
#line 13113 "SourceList.c"
}

static void
_code_widgets_source_list_tree_expander_cell_data_func_gtk_cell_layout_data_func (GtkCellLayout* cell_layout,
                                                                                  GtkCellRenderer* cell,
                                                                                  GtkTreeModel* tree_model,
                                                                                  GtkTreeIter* iter,
                                                                                  gpointer self)
{
#line 1668 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_tree_expander_cell_data_func ((CodeWidgetsSourceListTree*) self, cell_layout, cell, tree_model, iter);
#line 13125 "SourceList.c"
}

static void
_code_widgets_source_list_tree_on_activatable_activated_code_widgets_source_list_cell_renderer_icon_activated (CodeWidgetsSourceListCellRendererIcon* _sender,
                                                                                                               const gchar* path,
                                                                                                               gpointer self)
{
#line 1672 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_tree_on_activatable_activated ((CodeWidgetsSourceListTree*) self, path);
#line 13135 "SourceList.c"
}

static void
_code_widgets_source_list_tree_icon_cell_data_func_gtk_cell_layout_data_func (GtkCellLayout* cell_layout,
                                                                              GtkCellRenderer* cell,
                                                                              GtkTreeModel* tree_model,
                                                                              GtkTreeIter* iter,
                                                                              gpointer self)
{
#line 1674 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_tree_icon_cell_data_func ((CodeWidgetsSourceListTree*) self, cell_layout, cell, tree_model, iter);
#line 13147 "SourceList.c"
}

static void
_code_widgets_source_list_tree_badge_cell_data_func_gtk_cell_layout_data_func (GtkCellLayout* cell_layout,
                                                                               GtkCellRenderer* cell,
                                                                               GtkTreeModel* tree_model,
                                                                               GtkTreeIter* iter,
                                                                               gpointer self)
{
#line 1680 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_tree_badge_cell_data_func ((CodeWidgetsSourceListTree*) self, cell_layout, cell, tree_model, iter);
#line 13159 "SourceList.c"
}

static void
_code_widgets_source_list_tree_on_editing_started_gtk_cell_renderer_editing_started (GtkCellRenderer* _sender,
                                                                                     GtkCellEditable* editable,
                                                                                     const gchar* path,
                                                                                     gpointer self)
{
#line 1685 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_tree_on_editing_started ((CodeWidgetsSourceListTree*) self, editable, path);
#line 13170 "SourceList.c"
}

static void
_code_widgets_source_list_tree_on_editing_canceled_gtk_cell_renderer_editing_canceled (GtkCellRenderer* _sender,
                                                                                       gpointer self)
{
#line 1686 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_tree_on_editing_canceled ((CodeWidgetsSourceListTree*) self);
#line 13179 "SourceList.c"
}

static void
_code_widgets_source_list_tree_name_cell_data_func_gtk_cell_layout_data_func (GtkCellLayout* cell_layout,
                                                                              GtkCellRenderer* cell,
                                                                              GtkTreeModel* tree_model,
                                                                              GtkTreeIter* iter,
                                                                              gpointer self)
{
#line 1690 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_tree_name_cell_data_func ((CodeWidgetsSourceListTree*) self, cell_layout, cell, tree_model, iter);
#line 13191 "SourceList.c"
}

static gboolean
_code_widgets_source_list_tree_select_func_gtk_tree_selection_func (GtkTreeSelection* selection,
                                                                    GtkTreeModel* model,
                                                                    GtkTreePath* path,
                                                                    gboolean path_currently_selected,
                                                                    gpointer self)
{
	gboolean result;
	result = code_widgets_source_list_tree_select_func ((CodeWidgetsSourceListTree*) self, selection, model, path, path_currently_selected);
#line 1710 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 13205 "SourceList.c"
}

static gboolean
_code_widgets_source_list_tree_on_query_tooltip_gtk_widget_query_tooltip (GtkWidget* _sender,
                                                                          gint x,
                                                                          gint y,
                                                                          gboolean keyboard_tooltip,
                                                                          GtkTooltip* tooltip,
                                                                          gpointer self)
{
	gboolean result;
	result = code_widgets_source_list_tree_on_query_tooltip ((CodeWidgetsSourceListTree*) self, x, y, keyboard_tooltip, tooltip);
#line 1722 "../src/Widgets/SourceList/SourceList.vala"
	return result;
#line 13220 "SourceList.c"
}

static GObject *
code_widgets_source_list_tree_constructor (GType type,
                                           guint n_construct_properties,
                                           GObjectConstructParam * construct_properties)
{
	GObject * obj;
	GObjectClass * parent_class;
	CodeWidgetsSourceListTree * self;
	GtkStyleContext* style_context = NULL;
	GtkStyleContext* _tmp0_;
	GtkStyleContext* _tmp1_;
	GtkStyleContext* _tmp2_;
	GtkCssProvider* css_provider = NULL;
	GtkCssProvider* _tmp3_;
	CodeWidgetsSourceListDataModel* _tmp9_;
	GtkTreeViewColumn* item_column = NULL;
	GtkTreeViewColumn* _tmp10_;
	GtkTreeViewColumn* _tmp11_;
	GtkTreeViewColumn* _tmp12_;
	CodeWidgetsCellRendererExpander* _tmp13_;
	CodeWidgetsCellRendererExpander* _tmp14_;
	CodeWidgetsCellRendererExpander* _tmp15_;
	GtkTreeViewColumn* _tmp16_;
	CodeWidgetsCellRendererExpander* _tmp17_;
	GtkTreeViewColumn* _tmp18_;
	CodeWidgetsCellRendererExpander* _tmp19_;
	CodeWidgetsSourceListCellRendererIcon* _tmp20_;
	CodeWidgetsSourceListCellRendererIcon* _tmp21_;
	CodeWidgetsSourceListCellRendererIcon* _tmp22_;
	GtkTreeViewColumn* _tmp23_;
	CodeWidgetsSourceListCellRendererIcon* _tmp24_;
	GtkTreeViewColumn* _tmp25_;
	CodeWidgetsSourceListCellRendererIcon* _tmp26_;
	CodeWidgetsCellRendererBadge* _tmp27_;
	CodeWidgetsCellRendererBadge* _tmp28_;
	CodeWidgetsCellRendererBadge* _tmp29_;
	GtkTreeViewColumn* _tmp30_;
	CodeWidgetsCellRendererBadge* _tmp31_;
	GtkTreeViewColumn* _tmp32_;
	CodeWidgetsCellRendererBadge* _tmp33_;
	GtkCellRendererText* _tmp34_;
	GtkCellRendererText* _tmp35_;
	GtkCellRendererText* _tmp36_;
	GtkCellRendererText* _tmp37_;
	GtkCellRendererText* _tmp38_;
	GtkCellRendererText* _tmp39_;
	GtkCellRendererText* _tmp40_;
	GtkTreeViewColumn* _tmp41_;
	GtkCellRendererText* _tmp42_;
	GtkTreeViewColumn* _tmp43_;
	GtkCellRendererText* _tmp44_;
	CodeWidgetsSourceListCellRendererIcon* _tmp45_;
	CodeWidgetsSourceListCellRendererIcon* _tmp46_;
	GtkTreeViewColumn* _tmp47_;
	CodeWidgetsSourceListCellRendererIcon* _tmp48_;
	GtkTreeViewColumn* _tmp49_;
	CodeWidgetsSourceListCellRendererIcon* _tmp50_;
	CodeWidgetsCellRendererExpander* _tmp51_;
	gint expander_spacing = 0;
	gint _tmp52_ = 0;
	CodeWidgetsCellRendererExpander* _tmp53_;
	GtkTreeViewColumn* _tmp54_;
	CodeWidgetsCellRendererExpander* _tmp55_;
	GtkTreeViewColumn* _tmp56_;
	CodeWidgetsCellRendererExpander* _tmp57_;
	GtkTreeSelection* selection = NULL;
	GtkTreeSelection* _tmp58_;
	GtkTreeSelection* _tmp59_;
	GtkTreeSelection* _tmp60_;
	GtkTreeSelection* _tmp61_;
	GError* _inner_error0_ = NULL;
#line 1627 "../src/Widgets/SourceList/SourceList.vala"
	parent_class = G_OBJECT_CLASS (code_widgets_source_list_tree_parent_class);
#line 1627 "../src/Widgets/SourceList/SourceList.vala"
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
#line 1627 "../src/Widgets/SourceList/SourceList.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, CODE_WIDGETS_SOURCE_LIST_TYPE_TREE, CodeWidgetsSourceListTree);
#line 1628 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = gtk_widget_get_style_context ((GtkWidget*) self);
#line 1628 "../src/Widgets/SourceList/SourceList.vala"
	style_context = _tmp0_;
#line 1629 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = style_context;
#line 1629 "../src/Widgets/SourceList/SourceList.vala"
	gtk_style_context_add_class (_tmp1_, GTK_STYLE_CLASS_SIDEBAR);
#line 1630 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = style_context;
#line 1630 "../src/Widgets/SourceList/SourceList.vala"
	gtk_style_context_add_class (_tmp2_, "source-list");
#line 1632 "../src/Widgets/SourceList/SourceList.vala"
	_tmp3_ = gtk_css_provider_new ();
#line 1632 "../src/Widgets/SourceList/SourceList.vala"
	css_provider = _tmp3_;
#line 13316 "SourceList.c"
	{
		GtkCssProvider* _tmp4_;
		GtkStyleContext* _tmp5_;
		GtkCssProvider* _tmp6_;
#line 1634 "../src/Widgets/SourceList/SourceList.vala"
		_tmp4_ = css_provider;
#line 1634 "../src/Widgets/SourceList/SourceList.vala"
		gtk_css_provider_load_from_data (_tmp4_, CODE_WIDGETS_SOURCE_LIST_TREE_DEFAULT_STYLESHEET, (gssize) -1, &_inner_error0_);
#line 1634 "../src/Widgets/SourceList/SourceList.vala"
		if (G_UNLIKELY (_inner_error0_ != NULL)) {
#line 13327 "SourceList.c"
			goto __catch0_g_error;
		}
#line 1635 "../src/Widgets/SourceList/SourceList.vala"
		_tmp5_ = style_context;
#line 1635 "../src/Widgets/SourceList/SourceList.vala"
		_tmp6_ = css_provider;
#line 1635 "../src/Widgets/SourceList/SourceList.vala"
		gtk_style_context_add_provider (_tmp5_, (GtkStyleProvider*) _tmp6_, (guint) GTK_STYLE_PROVIDER_PRIORITY_FALLBACK);
#line 13336 "SourceList.c"
	}
	goto __finally0;
	__catch0_g_error:
	{
		GError* e = NULL;
		GError* _tmp7_;
		const gchar* _tmp8_;
#line 1633 "../src/Widgets/SourceList/SourceList.vala"
		e = _inner_error0_;
#line 1633 "../src/Widgets/SourceList/SourceList.vala"
		_inner_error0_ = NULL;
#line 1637 "../src/Widgets/SourceList/SourceList.vala"
		_tmp7_ = e;
#line 1637 "../src/Widgets/SourceList/SourceList.vala"
		_tmp8_ = _tmp7_->message;
#line 1637 "../src/Widgets/SourceList/SourceList.vala"
		g_warning ("SourceList.vala:1637: Could not create CSS Provider: %s\n" \
"Stylesheet:\n" \
"%s", _tmp8_, CODE_WIDGETS_SOURCE_LIST_TREE_DEFAULT_STYLESHEET);
#line 1633 "../src/Widgets/SourceList/SourceList.vala"
		_g_error_free0 (e);
#line 13356 "SourceList.c"
	}
	__finally0:
#line 1633 "../src/Widgets/SourceList/SourceList.vala"
	if (G_UNLIKELY (_inner_error0_ != NULL)) {
#line 1633 "../src/Widgets/SourceList/SourceList.vala"
		_g_object_unref0 (css_provider);
#line 1633 "../src/Widgets/SourceList/SourceList.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error0_->message, g_quark_to_string (_inner_error0_->domain), _inner_error0_->code);
#line 1633 "../src/Widgets/SourceList/SourceList.vala"
		g_clear_error (&_inner_error0_);
#line 13367 "SourceList.c"
	}
#line 1640 "../src/Widgets/SourceList/SourceList.vala"
	_tmp9_ = self->priv->_data_model;
#line 1640 "../src/Widgets/SourceList/SourceList.vala"
	gtk_tree_view_set_model ((GtkTreeView*) self, (GtkTreeModel*) _tmp9_);
#line 1642 "../src/Widgets/SourceList/SourceList.vala"
	gtk_widget_set_valign ((GtkWidget*) self, GTK_ALIGN_FILL);
#line 1642 "../src/Widgets/SourceList/SourceList.vala"
	gtk_widget_set_halign ((GtkWidget*) self, GTK_ALIGN_FILL);
#line 1643 "../src/Widgets/SourceList/SourceList.vala"
	g_object_set ((GtkWidget*) self, "expand", TRUE, NULL);
#line 1645 "../src/Widgets/SourceList/SourceList.vala"
	gtk_tree_view_set_enable_search ((GtkTreeView*) self, FALSE);
#line 1646 "../src/Widgets/SourceList/SourceList.vala"
	gtk_tree_view_set_headers_visible ((GtkTreeView*) self, FALSE);
#line 1647 "../src/Widgets/SourceList/SourceList.vala"
	g_object_set ((GtkTreeView*) self, "enable-grid-lines", GTK_TREE_VIEW_GRID_LINES_NONE, NULL);
#line 1650 "../src/Widgets/SourceList/SourceList.vala"
	gtk_tree_view_set_expander_column ((GtkTreeView*) self, NULL);
#line 1651 "../src/Widgets/SourceList/SourceList.vala"
	gtk_tree_view_set_show_expanders ((GtkTreeView*) self, FALSE);
#line 1653 "../src/Widgets/SourceList/SourceList.vala"
	_tmp10_ = gtk_tree_view_column_new ();
#line 1653 "../src/Widgets/SourceList/SourceList.vala"
	g_object_ref_sink (_tmp10_);
#line 1653 "../src/Widgets/SourceList/SourceList.vala"
	item_column = _tmp10_;
#line 1654 "../src/Widgets/SourceList/SourceList.vala"
	_tmp11_ = item_column;
#line 1654 "../src/Widgets/SourceList/SourceList.vala"
	gtk_tree_view_column_set_expand (_tmp11_, TRUE);
#line 1656 "../src/Widgets/SourceList/SourceList.vala"
	_tmp12_ = item_column;
#line 1656 "../src/Widgets/SourceList/SourceList.vala"
	gtk_tree_view_insert_column ((GtkTreeView*) self, _tmp12_, (gint) CODE_WIDGETS_SOURCE_LIST_TREE_COLUMN_ITEM);
#line 1664 "../src/Widgets/SourceList/SourceList.vala"
	_tmp13_ = code_widgets_cell_renderer_expander_new ();
#line 1664 "../src/Widgets/SourceList/SourceList.vala"
	g_object_ref_sink (_tmp13_);
#line 1664 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->secondary_expander_cell);
#line 1664 "../src/Widgets/SourceList/SourceList.vala"
	self->priv->secondary_expander_cell = _tmp13_;
#line 1665 "../src/Widgets/SourceList/SourceList.vala"
	_tmp14_ = self->priv->secondary_expander_cell;
#line 1665 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_cell_renderer_expander_set_is_category_expander (_tmp14_, TRUE);
#line 1666 "../src/Widgets/SourceList/SourceList.vala"
	_tmp15_ = self->priv->secondary_expander_cell;
#line 1666 "../src/Widgets/SourceList/SourceList.vala"
	g_object_set ((GtkCellRenderer*) _tmp15_, "xpad", (guint) 10, NULL);
#line 1667 "../src/Widgets/SourceList/SourceList.vala"
	_tmp16_ = item_column;
#line 1667 "../src/Widgets/SourceList/SourceList.vala"
	_tmp17_ = self->priv->secondary_expander_cell;
#line 1667 "../src/Widgets/SourceList/SourceList.vala"
	gtk_cell_layout_pack_end ((GtkCellLayout*) _tmp16_, (GtkCellRenderer*) _tmp17_, FALSE);
#line 1668 "../src/Widgets/SourceList/SourceList.vala"
	_tmp18_ = item_column;
#line 1668 "../src/Widgets/SourceList/SourceList.vala"
	_tmp19_ = self->priv->secondary_expander_cell;
#line 1668 "../src/Widgets/SourceList/SourceList.vala"
	gtk_cell_layout_set_cell_data_func ((GtkCellLayout*) _tmp18_, (GtkCellRenderer*) _tmp19_, _code_widgets_source_list_tree_expander_cell_data_func_gtk_cell_layout_data_func, g_object_ref (self), g_object_unref);
#line 1670 "../src/Widgets/SourceList/SourceList.vala"
	_tmp20_ = code_widgets_source_list_cell_renderer_icon_new ();
#line 1670 "../src/Widgets/SourceList/SourceList.vala"
	g_object_ref_sink (_tmp20_);
#line 1670 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->activatable_cell);
#line 1670 "../src/Widgets/SourceList/SourceList.vala"
	self->priv->activatable_cell = _tmp20_;
#line 1671 "../src/Widgets/SourceList/SourceList.vala"
	_tmp21_ = self->priv->activatable_cell;
#line 1671 "../src/Widgets/SourceList/SourceList.vala"
	g_object_set ((GtkCellRenderer*) _tmp21_, "xpad", (guint) 6, NULL);
#line 1672 "../src/Widgets/SourceList/SourceList.vala"
	_tmp22_ = self->priv->activatable_cell;
#line 1672 "../src/Widgets/SourceList/SourceList.vala"
	g_signal_connect_object (_tmp22_, "activated", (GCallback) _code_widgets_source_list_tree_on_activatable_activated_code_widgets_source_list_cell_renderer_icon_activated, self, 0);
#line 1673 "../src/Widgets/SourceList/SourceList.vala"
	_tmp23_ = item_column;
#line 1673 "../src/Widgets/SourceList/SourceList.vala"
	_tmp24_ = self->priv->activatable_cell;
#line 1673 "../src/Widgets/SourceList/SourceList.vala"
	gtk_cell_layout_pack_end ((GtkCellLayout*) _tmp23_, (GtkCellRenderer*) _tmp24_, FALSE);
#line 1674 "../src/Widgets/SourceList/SourceList.vala"
	_tmp25_ = item_column;
#line 1674 "../src/Widgets/SourceList/SourceList.vala"
	_tmp26_ = self->priv->activatable_cell;
#line 1674 "../src/Widgets/SourceList/SourceList.vala"
	gtk_cell_layout_set_cell_data_func ((GtkCellLayout*) _tmp25_, (GtkCellRenderer*) _tmp26_, _code_widgets_source_list_tree_icon_cell_data_func_gtk_cell_layout_data_func, g_object_ref (self), g_object_unref);
#line 1676 "../src/Widgets/SourceList/SourceList.vala"
	_tmp27_ = code_widgets_cell_renderer_badge_new ();
#line 1676 "../src/Widgets/SourceList/SourceList.vala"
	g_object_ref_sink (_tmp27_);
#line 1676 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->badge_cell);
#line 1676 "../src/Widgets/SourceList/SourceList.vala"
	self->priv->badge_cell = _tmp27_;
#line 1677 "../src/Widgets/SourceList/SourceList.vala"
	_tmp28_ = self->priv->badge_cell;
#line 1677 "../src/Widgets/SourceList/SourceList.vala"
	g_object_set ((GtkCellRenderer*) _tmp28_, "xpad", (guint) 1, NULL);
#line 1678 "../src/Widgets/SourceList/SourceList.vala"
	_tmp29_ = self->priv->badge_cell;
#line 1678 "../src/Widgets/SourceList/SourceList.vala"
	g_object_set ((GtkCellRenderer*) _tmp29_, "xalign", (gfloat) 1, NULL);
#line 1679 "../src/Widgets/SourceList/SourceList.vala"
	_tmp30_ = item_column;
#line 1679 "../src/Widgets/SourceList/SourceList.vala"
	_tmp31_ = self->priv->badge_cell;
#line 1679 "../src/Widgets/SourceList/SourceList.vala"
	gtk_cell_layout_pack_end ((GtkCellLayout*) _tmp30_, (GtkCellRenderer*) _tmp31_, FALSE);
#line 1680 "../src/Widgets/SourceList/SourceList.vala"
	_tmp32_ = item_column;
#line 1680 "../src/Widgets/SourceList/SourceList.vala"
	_tmp33_ = self->priv->badge_cell;
#line 1680 "../src/Widgets/SourceList/SourceList.vala"
	gtk_cell_layout_set_cell_data_func ((GtkCellLayout*) _tmp32_, (GtkCellRenderer*) _tmp33_, _code_widgets_source_list_tree_badge_cell_data_func_gtk_cell_layout_data_func, g_object_ref (self), g_object_unref);
#line 1682 "../src/Widgets/SourceList/SourceList.vala"
	_tmp34_ = (GtkCellRendererText*) gtk_cell_renderer_text_new ();
#line 1682 "../src/Widgets/SourceList/SourceList.vala"
	g_object_ref_sink (_tmp34_);
#line 1682 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->text_cell);
#line 1682 "../src/Widgets/SourceList/SourceList.vala"
	self->priv->text_cell = _tmp34_;
#line 1683 "../src/Widgets/SourceList/SourceList.vala"
	_tmp35_ = self->priv->text_cell;
#line 1683 "../src/Widgets/SourceList/SourceList.vala"
	g_object_set (_tmp35_, "editable-set", TRUE, NULL);
#line 1684 "../src/Widgets/SourceList/SourceList.vala"
	_tmp36_ = self->priv->text_cell;
#line 1684 "../src/Widgets/SourceList/SourceList.vala"
	g_object_set (_tmp36_, "editable", FALSE, NULL);
#line 1685 "../src/Widgets/SourceList/SourceList.vala"
	_tmp37_ = self->priv->text_cell;
#line 1685 "../src/Widgets/SourceList/SourceList.vala"
	g_signal_connect_object ((GtkCellRenderer*) _tmp37_, "editing-started", (GCallback) _code_widgets_source_list_tree_on_editing_started_gtk_cell_renderer_editing_started, self, 0);
#line 1686 "../src/Widgets/SourceList/SourceList.vala"
	_tmp38_ = self->priv->text_cell;
#line 1686 "../src/Widgets/SourceList/SourceList.vala"
	g_signal_connect_object ((GtkCellRenderer*) _tmp38_, "editing-canceled", (GCallback) _code_widgets_source_list_tree_on_editing_canceled_gtk_cell_renderer_editing_canceled, self, 0);
#line 1687 "../src/Widgets/SourceList/SourceList.vala"
	_tmp39_ = self->priv->text_cell;
#line 1687 "../src/Widgets/SourceList/SourceList.vala"
	g_object_set (_tmp39_, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
#line 1688 "../src/Widgets/SourceList/SourceList.vala"
	_tmp40_ = self->priv->text_cell;
#line 1688 "../src/Widgets/SourceList/SourceList.vala"
	g_object_set ((GtkCellRenderer*) _tmp40_, "xalign", (gfloat) 0, NULL);
#line 1689 "../src/Widgets/SourceList/SourceList.vala"
	_tmp41_ = item_column;
#line 1689 "../src/Widgets/SourceList/SourceList.vala"
	_tmp42_ = self->priv->text_cell;
#line 1689 "../src/Widgets/SourceList/SourceList.vala"
	gtk_cell_layout_pack_end ((GtkCellLayout*) _tmp41_, (GtkCellRenderer*) _tmp42_, TRUE);
#line 1690 "../src/Widgets/SourceList/SourceList.vala"
	_tmp43_ = item_column;
#line 1690 "../src/Widgets/SourceList/SourceList.vala"
	_tmp44_ = self->priv->text_cell;
#line 1690 "../src/Widgets/SourceList/SourceList.vala"
	gtk_cell_layout_set_cell_data_func ((GtkCellLayout*) _tmp43_, (GtkCellRenderer*) _tmp44_, _code_widgets_source_list_tree_name_cell_data_func_gtk_cell_layout_data_func, g_object_ref (self), g_object_unref);
#line 1692 "../src/Widgets/SourceList/SourceList.vala"
	_tmp45_ = code_widgets_source_list_cell_renderer_icon_new ();
#line 1692 "../src/Widgets/SourceList/SourceList.vala"
	g_object_ref_sink (_tmp45_);
#line 1692 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->icon_cell);
#line 1692 "../src/Widgets/SourceList/SourceList.vala"
	self->priv->icon_cell = _tmp45_;
#line 1693 "../src/Widgets/SourceList/SourceList.vala"
	_tmp46_ = self->priv->icon_cell;
#line 1693 "../src/Widgets/SourceList/SourceList.vala"
	g_object_set ((GtkCellRenderer*) _tmp46_, "xpad", (guint) 2, NULL);
#line 1694 "../src/Widgets/SourceList/SourceList.vala"
	_tmp47_ = item_column;
#line 1694 "../src/Widgets/SourceList/SourceList.vala"
	_tmp48_ = self->priv->icon_cell;
#line 1694 "../src/Widgets/SourceList/SourceList.vala"
	gtk_cell_layout_pack_end ((GtkCellLayout*) _tmp47_, (GtkCellRenderer*) _tmp48_, FALSE);
#line 1695 "../src/Widgets/SourceList/SourceList.vala"
	_tmp49_ = item_column;
#line 1695 "../src/Widgets/SourceList/SourceList.vala"
	_tmp50_ = self->priv->icon_cell;
#line 1695 "../src/Widgets/SourceList/SourceList.vala"
	gtk_cell_layout_set_cell_data_func ((GtkCellLayout*) _tmp49_, (GtkCellRenderer*) _tmp50_, _code_widgets_source_list_tree_icon_cell_data_func_gtk_cell_layout_data_func, g_object_ref (self), g_object_unref);
#line 1698 "../src/Widgets/SourceList/SourceList.vala"
	_tmp51_ = code_widgets_cell_renderer_expander_new ();
#line 1698 "../src/Widgets/SourceList/SourceList.vala"
	g_object_ref_sink (_tmp51_);
#line 1698 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->primary_expander_cell);
#line 1698 "../src/Widgets/SourceList/SourceList.vala"
	self->priv->primary_expander_cell = _tmp51_;
#line 1701 "../src/Widgets/SourceList/SourceList.vala"
	gtk_widget_style_get ((GtkWidget*) self, CODE_WIDGETS_SOURCE_LIST_TREE_STYLE_PROP_EXPANDER_SPACING, &_tmp52_, NULL);
#line 1701 "../src/Widgets/SourceList/SourceList.vala"
	expander_spacing = _tmp52_;
#line 1702 "../src/Widgets/SourceList/SourceList.vala"
	_tmp53_ = self->priv->primary_expander_cell;
#line 1702 "../src/Widgets/SourceList/SourceList.vala"
	g_object_set ((GtkCellRenderer*) _tmp53_, "xpad", (guint) (expander_spacing / 2), NULL);
#line 1704 "../src/Widgets/SourceList/SourceList.vala"
	_tmp54_ = item_column;
#line 1704 "../src/Widgets/SourceList/SourceList.vala"
	_tmp55_ = self->priv->primary_expander_cell;
#line 1704 "../src/Widgets/SourceList/SourceList.vala"
	gtk_cell_layout_pack_end ((GtkCellLayout*) _tmp54_, (GtkCellRenderer*) _tmp55_, FALSE);
#line 1705 "../src/Widgets/SourceList/SourceList.vala"
	_tmp56_ = item_column;
#line 1705 "../src/Widgets/SourceList/SourceList.vala"
	_tmp57_ = self->priv->primary_expander_cell;
#line 1705 "../src/Widgets/SourceList/SourceList.vala"
	gtk_cell_layout_set_cell_data_func ((GtkCellLayout*) _tmp56_, (GtkCellRenderer*) _tmp57_, _code_widgets_source_list_tree_expander_cell_data_func_gtk_cell_layout_data_func, g_object_ref (self), g_object_unref);
#line 1708 "../src/Widgets/SourceList/SourceList.vala"
	_tmp58_ = gtk_tree_view_get_selection ((GtkTreeView*) self);
#line 1708 "../src/Widgets/SourceList/SourceList.vala"
	_tmp59_ = _g_object_ref0 (_tmp58_);
#line 1708 "../src/Widgets/SourceList/SourceList.vala"
	selection = _tmp59_;
#line 1709 "../src/Widgets/SourceList/SourceList.vala"
	_tmp60_ = selection;
#line 1709 "../src/Widgets/SourceList/SourceList.vala"
	gtk_tree_selection_set_mode (_tmp60_, GTK_SELECTION_BROWSE);
#line 1710 "../src/Widgets/SourceList/SourceList.vala"
	_tmp61_ = selection;
#line 1710 "../src/Widgets/SourceList/SourceList.vala"
	gtk_tree_selection_set_select_function (_tmp61_, _code_widgets_source_list_tree_select_func_gtk_tree_selection_func, g_object_ref (self), g_object_unref);
#line 1713 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_tree_enable_item_property_monitor (self);
#line 1716 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_tree_add_spacer_cell_for_level (self, 1, TRUE);
#line 1719 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_tree_configure_drag_source (self, NULL, (gint) 0);
#line 1720 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_tree_configure_drag_dest (self, NULL, (gint) 0, 0);
#line 1722 "../src/Widgets/SourceList/SourceList.vala"
	g_signal_connect_object ((GtkWidget*) self, "query-tooltip", (GCallback) _code_widgets_source_list_tree_on_query_tooltip_gtk_widget_query_tooltip, self, G_CONNECT_AFTER);
#line 1723 "../src/Widgets/SourceList/SourceList.vala"
	gtk_widget_set_has_tooltip ((GtkWidget*) self, TRUE);
#line 1627 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (selection);
#line 1627 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (item_column);
#line 1627 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (css_provider);
#line 1627 "../src/Widgets/SourceList/SourceList.vala"
	return obj;
#line 13617 "SourceList.c"
}

static void
code_widgets_source_list_tree_class_init (CodeWidgetsSourceListTreeClass * klass,
                                          gpointer klass_data)
{
	GParamSpecInt* _tmp0_;
	GParamSpecInt* _tmp1_;
	GParamSpecInt* _tmp2_;
	GParamSpecInt* _tmp3_;
	GParamSpecInt* _tmp4_;
	GParamSpecInt* _tmp5_;
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_tree_parent_class = g_type_class_peek_parent (klass);
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
	g_type_class_adjust_private_offset (klass, &CodeWidgetsSourceListTree_private_offset);
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
	((GtkWidgetClass *) klass)->drag_motion = (gboolean (*) (GtkWidget*, GdkDragContext*, gint, gint, guint)) code_widgets_source_list_tree_real_drag_motion;
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
	((GtkWidgetClass *) klass)->drag_data_received = (void (*) (GtkWidget*, GdkDragContext*, gint, gint, GtkSelectionData*, guint, guint)) code_widgets_source_list_tree_real_drag_data_received;
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
	((GtkTreeViewClass *) klass)->cursor_changed = (void (*) (GtkTreeView*)) code_widgets_source_list_tree_real_cursor_changed;
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
	((GtkTreeViewClass *) klass)->row_expanded = (void (*) (GtkTreeView*, GtkTreeIter*, GtkTreePath*)) code_widgets_source_list_tree_real_row_expanded;
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
	((GtkTreeViewClass *) klass)->row_collapsed = (void (*) (GtkTreeView*, GtkTreeIter*, GtkTreePath*)) code_widgets_source_list_tree_real_row_collapsed;
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
	((GtkTreeViewClass *) klass)->row_activated = (void (*) (GtkTreeView*, GtkTreePath*, GtkTreeViewColumn*)) code_widgets_source_list_tree_real_row_activated;
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
	((GtkWidgetClass *) klass)->key_release_event = (gboolean (*) (GtkWidget*, GdkEventKey*)) code_widgets_source_list_tree_real_key_release_event;
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
	((GtkWidgetClass *) klass)->button_release_event = (gboolean (*) (GtkWidget*, GdkEventButton*)) code_widgets_source_list_tree_real_button_release_event;
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
	((GtkWidgetClass *) klass)->button_press_event = (gboolean (*) (GtkWidget*, GdkEventButton*)) code_widgets_source_list_tree_real_button_press_event;
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
	((GtkWidgetClass *) klass)->popup_menu = (gboolean (*) (GtkWidget*)) code_widgets_source_list_tree_real_popup_menu;
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_code_widgets_source_list_tree_get_property;
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_code_widgets_source_list_tree_set_property;
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->constructor = code_widgets_source_list_tree_constructor;
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->finalize = code_widgets_source_list_tree_finalize;
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), CODE_WIDGETS_SOURCE_LIST_TREE_DATA_MODEL_PROPERTY, code_widgets_source_list_tree_properties[CODE_WIDGETS_SOURCE_LIST_TREE_DATA_MODEL_PROPERTY] = g_param_spec_object ("data-model", "data-model", "data-model", CODE_WIDGETS_SOURCE_LIST_TYPE_DATA_MODEL, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), CODE_WIDGETS_SOURCE_LIST_TREE_SELECTED_ITEM_PROPERTY, code_widgets_source_list_tree_properties[CODE_WIDGETS_SOURCE_LIST_TREE_SELECTED_ITEM_PROPERTY] = g_param_spec_object ("selected-item", "selected-item", "selected-item", CODE_WIDGETS_SOURCE_LIST_TYPE_ITEM, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), CODE_WIDGETS_SOURCE_LIST_TREE_EDITING_PROPERTY, code_widgets_source_list_tree_properties[CODE_WIDGETS_SOURCE_LIST_TREE_EDITING_PROPERTY] = g_param_spec_boolean ("editing", "editing", "editing", FALSE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), CODE_WIDGETS_SOURCE_LIST_TREE_ELLIPSIZE_MODE_PROPERTY, code_widgets_source_list_tree_properties[CODE_WIDGETS_SOURCE_LIST_TREE_ELLIPSIZE_MODE_PROPERTY] = g_param_spec_enum ("ellipsize-mode", "ellipsize-mode", "ellipsize-mode", pango_ellipsize_mode_get_type (), 0, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_tree_signals[CODE_WIDGETS_SOURCE_LIST_TREE_ITEM_SELECTED_SIGNAL] = g_signal_new ("item-selected", CODE_WIDGETS_SOURCE_LIST_TYPE_TREE, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, CODE_WIDGETS_SOURCE_LIST_TYPE_ITEM);
#line 1592 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = (GParamSpecInt*) g_param_spec_int (CODE_WIDGETS_SOURCE_LIST_TREE_STYLE_PROP_LEVEL_INDENTATION, "Level Indentation", "Space to add at the beginning of every indentation level. Must be an e" \
"ven number.", 1, 50, 6, G_PARAM_READABLE);
#line 1592 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = _tmp0_;
#line 1592 "../src/Widgets/SourceList/SourceList.vala"
	gtk_widget_class_install_style_property (GTK_WIDGET_CLASS (klass), (GParamSpec*) _tmp1_);
#line 1592 "../src/Widgets/SourceList/SourceList.vala"
	_g_param_spec_unref0 (_tmp1_);
#line 1602 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = (GParamSpecInt*) g_param_spec_int (CODE_WIDGETS_SOURCE_LIST_TREE_STYLE_PROP_LEFT_PADDING, "Left Padding", "Padding added to the left side of the tree. Must be an even number.", 1, 50, 4, G_PARAM_READABLE);
#line 1602 "../src/Widgets/SourceList/SourceList.vala"
	_tmp3_ = _tmp2_;
#line 1602 "../src/Widgets/SourceList/SourceList.vala"
	gtk_widget_class_install_style_property (GTK_WIDGET_CLASS (klass), (GParamSpec*) _tmp3_);
#line 1602 "../src/Widgets/SourceList/SourceList.vala"
	_g_param_spec_unref0 (_tmp3_);
#line 1612 "../src/Widgets/SourceList/SourceList.vala"
	_tmp4_ = (GParamSpecInt*) g_param_spec_int (CODE_WIDGETS_SOURCE_LIST_TREE_STYLE_PROP_EXPANDER_SPACING, "Expander Spacing", "Space added between an item and its expander. Must be an even number.", 1, 50, 4, G_PARAM_READABLE);
#line 1612 "../src/Widgets/SourceList/SourceList.vala"
	_tmp5_ = _tmp4_;
#line 1612 "../src/Widgets/SourceList/SourceList.vala"
	gtk_widget_class_install_style_property (GTK_WIDGET_CLASS (klass), (GParamSpec*) _tmp5_);
#line 1612 "../src/Widgets/SourceList/SourceList.vala"
	_g_param_spec_unref0 (_tmp5_);
#line 13696 "SourceList.c"
}

static void
code_widgets_source_list_tree_instance_init (CodeWidgetsSourceListTree * self,
                                             gpointer klass)
{
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
	self->priv = code_widgets_source_list_tree_get_instance_private (self);
#line 1576 "../src/Widgets/SourceList/SourceList.vala"
	self->priv->unselectable_item_clicked = FALSE;
#line 13707 "SourceList.c"
}

static void
code_widgets_source_list_tree_finalize (GObject * obj)
{
	CodeWidgetsSourceListTree * self;
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, CODE_WIDGETS_SOURCE_LIST_TYPE_TREE, CodeWidgetsSourceListTree);
#line 1727 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_tree_disable_item_property_monitor (self);
#line 1542 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->_data_model);
#line 1565 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->selected);
#line 1568 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->editable_entry);
#line 1569 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->text_cell);
#line 1570 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->icon_cell);
#line 1571 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->activatable_cell);
#line 1572 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->badge_cell);
#line 1573 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->primary_expander_cell);
#line 1574 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->secondary_expander_cell);
#line 1575 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->spacer_cells);
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
	G_OBJECT_CLASS (code_widgets_source_list_tree_parent_class)->finalize (obj);
#line 13740 "SourceList.c"
}

/**
     * The tree that actually displays the items.
     *
     * All the user interaction happens here.
     */
static GType
code_widgets_source_list_tree_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (CodeWidgetsSourceListTreeClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) code_widgets_source_list_tree_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (CodeWidgetsSourceListTree), 0, (GInstanceInitFunc) code_widgets_source_list_tree_instance_init, NULL };
	GType code_widgets_source_list_tree_type_id;
	code_widgets_source_list_tree_type_id = g_type_register_static (gtk_tree_view_get_type (), "CodeWidgetsSourceListTree", &g_define_type_info, 0);
	CodeWidgetsSourceListTree_private_offset = g_type_add_instance_private (code_widgets_source_list_tree_type_id, sizeof (CodeWidgetsSourceListTreePrivate));
	return code_widgets_source_list_tree_type_id;
}

static GType
code_widgets_source_list_tree_get_type (void)
{
	static volatile gsize code_widgets_source_list_tree_type_id__once = 0;
	if (g_once_init_enter (&code_widgets_source_list_tree_type_id__once)) {
		GType code_widgets_source_list_tree_type_id;
		code_widgets_source_list_tree_type_id = code_widgets_source_list_tree_get_type_once ();
		g_once_init_leave (&code_widgets_source_list_tree_type_id__once, code_widgets_source_list_tree_type_id);
	}
	return code_widgets_source_list_tree_type_id__once;
}

static void
_vala_code_widgets_source_list_tree_get_property (GObject * object,
                                                  guint property_id,
                                                  GValue * value,
                                                  GParamSpec * pspec)
{
	CodeWidgetsSourceListTree * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, CODE_WIDGETS_SOURCE_LIST_TYPE_TREE, CodeWidgetsSourceListTree);
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
	switch (property_id) {
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_TREE_DATA_MODEL_PROPERTY:
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
		g_value_set_object (value, code_widgets_source_list_tree_get_data_model (self));
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_TREE_SELECTED_ITEM_PROPERTY:
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
		g_value_set_object (value, code_widgets_source_list_tree_get_selected_item (self));
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_TREE_EDITING_PROPERTY:
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
		g_value_set_boolean (value, code_widgets_source_list_tree_get_editing (self));
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_TREE_ELLIPSIZE_MODE_PROPERTY:
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
		g_value_set_enum (value, code_widgets_source_list_tree_get_ellipsize_mode (self));
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 13804 "SourceList.c"
		default:
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 13810 "SourceList.c"
	}
}

static void
_vala_code_widgets_source_list_tree_set_property (GObject * object,
                                                  guint property_id,
                                                  const GValue * value,
                                                  GParamSpec * pspec)
{
	CodeWidgetsSourceListTree * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, CODE_WIDGETS_SOURCE_LIST_TYPE_TREE, CodeWidgetsSourceListTree);
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
	switch (property_id) {
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_TREE_DATA_MODEL_PROPERTY:
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_tree_set_data_model (self, g_value_get_object (value));
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_TREE_SELECTED_ITEM_PROPERTY:
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_tree_set_selected_item (self, g_value_get_object (value));
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_TREE_ELLIPSIZE_MODE_PROPERTY:
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_tree_set_ellipsize_mode (self, g_value_get_enum (value));
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 13842 "SourceList.c"
		default:
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 1540 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 13848 "SourceList.c"
	}
}

static void
_code_widgets_source_list___lambda5_ (CodeWidgetsSourceList* self,
                                      CodeWidgetsSourceListItem* item)
{
#line 2669 "../src/Widgets/SourceList/SourceList.vala"
	g_signal_emit (self, code_widgets_source_list_signals[CODE_WIDGETS_SOURCE_LIST_ITEM_SELECTED_SIGNAL], 0, item);
#line 13858 "SourceList.c"
}

static void
__code_widgets_source_list___lambda5__code_widgets_source_list_tree_item_selected (CodeWidgetsSourceListTree* _sender,
                                                                                   CodeWidgetsSourceListItem* item,
                                                                                   gpointer self)
{
#line 2669 "../src/Widgets/SourceList/SourceList.vala"
	_code_widgets_source_list___lambda5_ ((CodeWidgetsSourceList*) self, item);
#line 13868 "SourceList.c"
}

static GObject *
code_widgets_source_list_constructor (GType type,
                                      guint n_construct_properties,
                                      GObjectConstructParam * construct_properties)
{
	GObject * obj;
	GObjectClass * parent_class;
	CodeWidgetsSourceList * self;
	CodeWidgetsSourceListDataModel* _tmp0_;
	CodeWidgetsSourceListTree* _tmp1_;
	CodeWidgetsSourceListTree* _tmp2_;
	CodeWidgetsSourceListTree* _tmp3_;
#line 2662 "../src/Widgets/SourceList/SourceList.vala"
	parent_class = G_OBJECT_CLASS (code_widgets_source_list_parent_class);
#line 2662 "../src/Widgets/SourceList/SourceList.vala"
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
#line 2662 "../src/Widgets/SourceList/SourceList.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, CODE_WIDGETS_TYPE_SOURCE_LIST, CodeWidgetsSourceList);
#line 2663 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = self->priv->data_model;
#line 2663 "../src/Widgets/SourceList/SourceList.vala"
	_tmp1_ = code_widgets_source_list_tree_new (_tmp0_);
#line 2663 "../src/Widgets/SourceList/SourceList.vala"
	g_object_ref_sink (_tmp1_);
#line 2663 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->tree);
#line 2663 "../src/Widgets/SourceList/SourceList.vala"
	self->priv->tree = _tmp1_;
#line 2665 "../src/Widgets/SourceList/SourceList.vala"
	gtk_scrolled_window_set_policy ((GtkScrolledWindow*) self, GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
#line 2666 "../src/Widgets/SourceList/SourceList.vala"
	_tmp2_ = self->priv->tree;
#line 2666 "../src/Widgets/SourceList/SourceList.vala"
	gtk_container_add ((GtkContainer*) self, (GtkWidget*) _tmp2_);
#line 2667 "../src/Widgets/SourceList/SourceList.vala"
	gtk_widget_show_all ((GtkWidget*) self);
#line 2669 "../src/Widgets/SourceList/SourceList.vala"
	_tmp3_ = self->priv->tree;
#line 2669 "../src/Widgets/SourceList/SourceList.vala"
	g_signal_connect_object (_tmp3_, "item-selected", (GCallback) __code_widgets_source_list___lambda5__code_widgets_source_list_tree_item_selected, self, 0);
#line 2662 "../src/Widgets/SourceList/SourceList.vala"
	return obj;
#line 13913 "SourceList.c"
}

static void
code_widgets_source_list_class_init (CodeWidgetsSourceListClass * klass,
                                     gpointer klass_data)
{
#line 220 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_parent_class = g_type_class_peek_parent (klass);
#line 220 "../src/Widgets/SourceList/SourceList.vala"
	g_type_class_adjust_private_offset (klass, &CodeWidgetsSourceList_private_offset);
#line 220 "../src/Widgets/SourceList/SourceList.vala"
	((CodeWidgetsSourceListClass *) klass)->item_selected = code_widgets_source_list_real_item_selected;
#line 220 "../src/Widgets/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_code_widgets_source_list_get_property;
#line 220 "../src/Widgets/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_code_widgets_source_list_set_property;
#line 220 "../src/Widgets/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->constructor = code_widgets_source_list_constructor;
#line 220 "../src/Widgets/SourceList/SourceList.vala"
	G_OBJECT_CLASS (klass)->finalize = code_widgets_source_list_finalize;
#line 13934 "SourceList.c"
	/**
	     * Root-level expandable item.
	     *
	     * This item contains the first-level source list items. It //only serves as an item container//.
	     * It is used to add and remove items to/from the widget.
	     *
	     * Internally, it allows the source list to connect to its {@link Code.Widgets.SourceList.ExpandableItem.child_added}
	     * and {@link Code.Widgets.SourceList.ExpandableItem.child_removed} signals in order to monitor
	     * new children additions/removals.
	     *
	     * @since 0.2
	     */
#line 220 "../src/Widgets/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), CODE_WIDGETS_SOURCE_LIST_ROOT_PROPERTY, code_widgets_source_list_properties[CODE_WIDGETS_SOURCE_LIST_ROOT_PROPERTY] = g_param_spec_object ("root", "root", "root", CODE_WIDGETS_SOURCE_LIST_TYPE_EXPANDABLE_ITEM, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 13949 "SourceList.c"
	/**
	     * The current selected item.
	     *
	     * Setting it to //null// un-selects the previously selected item, if there was any.
	     * {@link Code.Widgets.SourceList.ExpandableItem.expand_with_parents} is called on the
	     * item's parent to make sure it's possible to select it.
	     *
	     * @since 0.2
	     */
#line 220 "../src/Widgets/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), CODE_WIDGETS_SOURCE_LIST_SELECTED_PROPERTY, code_widgets_source_list_properties[CODE_WIDGETS_SOURCE_LIST_SELECTED_PROPERTY] = g_param_spec_object ("selected", "selected", "selected", CODE_WIDGETS_SOURCE_LIST_TYPE_ITEM, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 13961 "SourceList.c"
	/**
	     * Text ellipsize mode.
	     *
	     * @since 0.2
	     */
#line 220 "../src/Widgets/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), CODE_WIDGETS_SOURCE_LIST_ELLIPSIZE_MODE_PROPERTY, code_widgets_source_list_properties[CODE_WIDGETS_SOURCE_LIST_ELLIPSIZE_MODE_PROPERTY] = g_param_spec_enum ("ellipsize-mode", "ellipsize-mode", "ellipsize-mode", pango_ellipsize_mode_get_type (), 0, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 13969 "SourceList.c"
	/**
	     * Whether an item is being edited.
	     *
	     * @see Code.Widgets.SourceList.start_editing_item
	     * @since 0.2
	     */
#line 220 "../src/Widgets/SourceList/SourceList.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), CODE_WIDGETS_SOURCE_LIST_EDITING_PROPERTY, code_widgets_source_list_properties[CODE_WIDGETS_SOURCE_LIST_EDITING_PROPERTY] = g_param_spec_boolean ("editing", "editing", "editing", FALSE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
#line 13978 "SourceList.c"
	/**
	     * Emitted when the source list selection changes.
	     *
	     * @param item Selected item; //null// if nothing is selected.
	     * @since 0.2
	     */
#line 220 "../src/Widgets/SourceList/SourceList.vala"
	code_widgets_source_list_signals[CODE_WIDGETS_SOURCE_LIST_ITEM_SELECTED_SIGNAL] = g_signal_new ("item-selected", CODE_WIDGETS_TYPE_SOURCE_LIST, G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (CodeWidgetsSourceListClass, item_selected), NULL, NULL, g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, CODE_WIDGETS_SOURCE_LIST_TYPE_ITEM);
#line 13987 "SourceList.c"
}

static void
code_widgets_source_list_instance_init (CodeWidgetsSourceList * self,
                                        gpointer klass)
{
	CodeWidgetsSourceListDataModel* _tmp0_;
#line 220 "../src/Widgets/SourceList/SourceList.vala"
	self->priv = code_widgets_source_list_get_instance_private (self);
#line 2650 "../src/Widgets/SourceList/SourceList.vala"
	_tmp0_ = code_widgets_source_list_data_model_new ();
#line 2650 "../src/Widgets/SourceList/SourceList.vala"
	self->priv->data_model = _tmp0_;
#line 14001 "SourceList.c"
}

static void
code_widgets_source_list_finalize (GObject * obj)
{
	CodeWidgetsSourceList * self;
#line 220 "../src/Widgets/SourceList/SourceList.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, CODE_WIDGETS_TYPE_SOURCE_LIST, CodeWidgetsSourceList);
#line 2649 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->tree);
#line 2650 "../src/Widgets/SourceList/SourceList.vala"
	_g_object_unref0 (self->priv->data_model);
#line 220 "../src/Widgets/SourceList/SourceList.vala"
	G_OBJECT_CLASS (code_widgets_source_list_parent_class)->finalize (obj);
#line 14016 "SourceList.c"
}

/**
 * A widget that can display a list of items organized in categories.
 *
 * The source list widget consists of a collection of items, some of which are also expandable (and
 * thus can contain more items). All the items displayed in the source list are children of the widget's
 * root item. The API is meant to be used as follows:
 *
 * 1. Create the items you want to display in the source list, setting the appropriate values for their
 * properties. The desired hierarchy is achieved by creating expandable items and adding items to them.
 * These will be displayed as descendants in the widget's tree structure. The expandable items that are
 * not nested inside any other item are considered to be at root level, and should be added to
 * the widget's root item.<<BR>>
 *
 * Expandable items located at the root level are treated as categories, and only support text.
 *
 * ''Example''<<BR>>
 * The final tree will have the following structure:
 * {{{
 * Libraries
 *   Music
 * Stores
 *   My Store
 *      Music
 *      Podcasts
 * Devices
 *   Player 1
 *   Player 2
 * }}}
 *
 * {{{
 * var library_category = new Code.Widgets.SourceList.ExpandableItem ("Libraries");
 * var store_category = new Code.Widgets.SourceList.ExpandableItem ("Stores");
 * var device_category = new Code.Widgets.SourceList.ExpandableItem ("Devices");
 *
 * var music_item = new Code.Widgets.SourceList.Item ("Music");
 *
 * // "Libraries" will be the parent category of "Music"
 * library_category.add (music_item);
 *
 * // We plan to add sub-items to the store, so let's use an expandable item
 * var my_store_item = new Code.Widgets.SourceList.ExpandableItem ("My Store");
 * store_category.add (my_store_item);
 *
 * var my_store_podcast_item = new Code.Widgets.SourceList.Item ("Podcasts");
 * var my_store_music_item = new Code.Widgets.SourceList.Item ("Music");
 *
 * my_store_item.add (my_store_music_item);
 * my_store_item.add (my_store_podcast_item);
 *
 * var player1_item = new Code.Widgets.SourceList.Item ("Player 1");
 * var player2_item = new Code.Widgets.SourceList.Item ("Player 2");
 *
 * device_category.add (player1_item);
 * device_category.add (player2_item);
 * }}}
 *
 * 2. Create a source list widget.<<BR>>
 * {{{
 * var source_list = new Code.Widgets.SourceList ();
 * }}}
 *
 * 3. Add root-level items to the {@link Code.Widgets.SourceList.root} item.
 * This item only serves as a container, and all its properties are ignored by the widget.
 *
 * {{{
 * // This will add the main categories (including their children) to the source list. After
 * // having being added to be widget, any other item added to any of these items
 * // (or any other child item in a deeper level) will be automatically added too.
 * // There's no need to deal with the source list widget directly.
 *
 * var root = source_list.root;
 *
 * root.add (library_category);
 * root.add (store_category);
 * root.add (device_category);
 * }}}
 *
 * The steps mentioned above are enough for initializing the source list. Future changes to the items'
 * properties are ''automatically'' reflected by the widget.
 *
 * Final steps would involve connecting handlers to the source list events, being
 * {@link Code.Widgets.SourceList.item_selected} the most important, as it indicates that
 * the selection was modified.
 *
 * Pack the source list into the GUI using the {@link Gtk.Paned} widget.
 * This is usually done as follows:
 * {{{
 * var pane = new Gtk.Paned (Gtk.Orientation.HORIZONTAL);
 * pane.pack1 (source_list, false, false);
 * pane.pack2 (content_area, true, false);
 * }}}
 *
 * @since 0.2
 * @see Gtk.Paned
 */
static GType
code_widgets_source_list_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (CodeWidgetsSourceListClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) code_widgets_source_list_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (CodeWidgetsSourceList), 0, (GInstanceInitFunc) code_widgets_source_list_instance_init, NULL };
	GType code_widgets_source_list_type_id;
	code_widgets_source_list_type_id = g_type_register_static (gtk_scrolled_window_get_type (), "CodeWidgetsSourceList", &g_define_type_info, 0);
	CodeWidgetsSourceList_private_offset = g_type_add_instance_private (code_widgets_source_list_type_id, sizeof (CodeWidgetsSourceListPrivate));
	return code_widgets_source_list_type_id;
}

GType
code_widgets_source_list_get_type (void)
{
	static volatile gsize code_widgets_source_list_type_id__once = 0;
	if (g_once_init_enter (&code_widgets_source_list_type_id__once)) {
		GType code_widgets_source_list_type_id;
		code_widgets_source_list_type_id = code_widgets_source_list_get_type_once ();
		g_once_init_leave (&code_widgets_source_list_type_id__once, code_widgets_source_list_type_id);
	}
	return code_widgets_source_list_type_id__once;
}

static void
_vala_code_widgets_source_list_get_property (GObject * object,
                                             guint property_id,
                                             GValue * value,
                                             GParamSpec * pspec)
{
	CodeWidgetsSourceList * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, CODE_WIDGETS_TYPE_SOURCE_LIST, CodeWidgetsSourceList);
#line 220 "../src/Widgets/SourceList/SourceList.vala"
	switch (property_id) {
#line 220 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_ROOT_PROPERTY:
#line 220 "../src/Widgets/SourceList/SourceList.vala"
		g_value_set_object (value, code_widgets_source_list_get_root (self));
#line 220 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 220 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_SELECTED_PROPERTY:
#line 220 "../src/Widgets/SourceList/SourceList.vala"
		g_value_set_object (value, code_widgets_source_list_get_selected (self));
#line 220 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 220 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_ELLIPSIZE_MODE_PROPERTY:
#line 220 "../src/Widgets/SourceList/SourceList.vala"
		g_value_set_enum (value, code_widgets_source_list_get_ellipsize_mode (self));
#line 220 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 220 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_EDITING_PROPERTY:
#line 220 "../src/Widgets/SourceList/SourceList.vala"
		g_value_set_boolean (value, code_widgets_source_list_get_editing (self));
#line 220 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 14170 "SourceList.c"
		default:
#line 220 "../src/Widgets/SourceList/SourceList.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 220 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 14176 "SourceList.c"
	}
}

static void
_vala_code_widgets_source_list_set_property (GObject * object,
                                             guint property_id,
                                             const GValue * value,
                                             GParamSpec * pspec)
{
	CodeWidgetsSourceList * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, CODE_WIDGETS_TYPE_SOURCE_LIST, CodeWidgetsSourceList);
#line 220 "../src/Widgets/SourceList/SourceList.vala"
	switch (property_id) {
#line 220 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_ROOT_PROPERTY:
#line 220 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_set_root (self, g_value_get_object (value));
#line 220 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 220 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_SELECTED_PROPERTY:
#line 220 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_set_selected (self, g_value_get_object (value));
#line 220 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 220 "../src/Widgets/SourceList/SourceList.vala"
		case CODE_WIDGETS_SOURCE_LIST_ELLIPSIZE_MODE_PROPERTY:
#line 220 "../src/Widgets/SourceList/SourceList.vala"
		code_widgets_source_list_set_ellipsize_mode (self, g_value_get_enum (value));
#line 220 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 14208 "SourceList.c"
		default:
#line 220 "../src/Widgets/SourceList/SourceList.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 220 "../src/Widgets/SourceList/SourceList.vala"
		break;
#line 14214 "SourceList.c"
	}
}

static inline gpointer
_vala_memdup2 (gconstpointer mem,
               gsize byte_size)
{
	gpointer new_mem;
#line 181 "../src/Services/TemplateManager.vala"
	if (mem && byte_size != 0) {
#line 181 "../src/Services/TemplateManager.vala"
		new_mem = g_malloc (byte_size);
#line 181 "../src/Services/TemplateManager.vala"
		memcpy (new_mem, mem, byte_size);
#line 14229 "SourceList.c"
	} else {
#line 181 "../src/Services/TemplateManager.vala"
		new_mem = NULL;
#line 14233 "SourceList.c"
	}
#line 181 "../src/Services/TemplateManager.vala"
	return new_mem;
#line 14237 "SourceList.c"
}

